<?php


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

/**
 * Installer Class
 */
class schlixInstaller {

    //_________________________________________________________//
    public function __construct() {
        date_default_timezone_set('UTC');
        session_name('gusrinstall');
        session_start();
    }

    //_________________________________________________________//
    function execsql($sql, $SystemDB, $echo_line = true) {
        $SystemDB->query("SET SQL_MODE=''"); // fix most errors reported by microsoft tester

        foreach ($sql as $line) {
            ///		$line = str_replace('"/images','"'.$_SESSION['site_httpbase'].'/images',$line);
            if (strpos($line, '-- Table structure for table') !== false)
            {
                if ($echo_line)
                    echo __Html::P($line);
            }
            // Skip it if it's a comment
            if (substr($line, 0, 2) == '--' || $line == '')
                continue;

            // Add this line to the current segment
            $templine .= $line;
            // If it has a semicolon at the end, it's the end of the query
            if (substr(trim($line), -1, 1) == ';') {
                // Perform the query

                $SystemDB->query($templine);
                // Reset temp variable to empty
                $templine = '';
            }
        }
    }

    //_________________________________________________________//
    public function stepRequirementAndDBSetup() {
        global $__html_title;
        
        $__html_title = '2. System Requirement Check';        
        if ($_POST['submit'])
            $_SESSION['agree'] = $_POST['agreement'];
        if ($_SESSION['agree'] != 'yes') {
            echo __Html::DIV_start(array('class'=>'alert alert-danger')).__Html::P('User has not agreed to license term & conditions').__Html::DIV_end();
            return false;
        }
        
        $req = $this->checkSystemRequirement();

        $_SESSION['met_requirement'] = $req['met_requirement'];
        if (!$_SESSION['db_host'])
            $_SESSION['db_host'] = 'localhost';

        include('step_requirement.template.php');
    }

    //_________________________________________________________//
    public function stepSiteVariablesSetup() {
        global $__html_title;
        
        $__html_title = '3. Initialize Configuration';        
        $can_continue = ($_SESSION['agree'] && $_SESSION['met_requirement'] && $_SESSION['req_database_ok']);
        if (!$can_continue) {
            
            echo __Html::DIV_start(array('class'=>'alert alert-danger')).__Html::P('Cannot continue - either you have not agreed to the license or your system does not meet the requirements').__Html::DIV_end();
            return;
        }

        $site_path = str_replace('\\', '/', dirname(realpath('../index.php')));
        $is_not_normal_port = ($_SERVER['SERVER_PORT'] != 80) && ($_SERVER['SERVER_PORT'] != 443);
        $_SESSION['site_domain_name_and_port'] = $_SERVER['SERVER_NAME'] . ($is_not_normal_port ? ':' . $_SERVER['SERVER_PORT'] : '');
        $_SESSION['site_scheme'] = 'http';

        $curpath = pathinfo(parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH), PATHINFO_DIRNAME);
        $pos = strrpos($curpath, '/install');
        $the_httpbase = substr($curpath, 0, $pos);

        if ($the_httpbase == '/')
            $the_httpbase = '';
        $_SESSION['site_httpbase'] = $the_httpbase;

        if (!$_SESSION['mail_default_sender'])
            $_SESSION['mail_default_sender'] = 'Webmaster';
        if (!$_SESSION['short_site_name'])
            $_SESSION['short_site_name'] = 'main';
        if (!$_SESSION['mail_default_email'])
            $_SESSION['mail_default_email'] = 'info@' . str_replace('www.', '', $_SERVER['SERVER_NAME']);
        if (!$_SESSION['admin_login_time'])
            $_SESSION['admin_login_time'] = 1209600;
        if (!$_SESSION['schlix_js_mode'])
            $_SESSION['schlix_js_mode'] = 3;
        if (!$_SESSION['admin_username'])
            $_SESSION['admin_username'] = 'admin';
        include('step_variables.template.php');
    }

    /**
     * Not using original addslahes
     * @param string $str
     * @return type
     */
    private function addSlashes($str) {
        return str_replace("'", "\\'", $str);
    }

    /**
     * Generate the content of config.inc.php
     * @param string $site_name
     * @param string $db_host
     * @param string $db_database
     * @param string $db_username
     * @param string $db_password
     * @param string $admin_login_time
     * @param string $mail_default_sender
     * @param string $mail_default_email
     * @param int $system_timezone
     * @param string $record_404_errors
     * @return string
     */
    private function generateSiteConfigFileContent($site_name, $db_host, $db_database, $db_username, $db_password, $admin_login_time, $mail_default_sender, $mail_default_email, $system_timezone, $record_404_errors, $enable_ssl)
    {
        $str_enable_ssl = $enable_ssl ? 'true' : 'false';
        $data = '<?php' . "\n";
        $data.= "/* Auto generated by installer on " . date('Y-m-d H:j:s') . " */\n";
        $data.="define('SCHLIX_SITE_NAME','{$site_name}');\n";
        if ($enable_ssl)
        {
            $data.="define('SCHLIX_SITE_HTTP_URL','https://'.\$_SERVER['HTTP_HOST'] . ((\$_SERVER['SERVER_PORT'] != 80 && \$_SERVER['SERVER_PORT'] != 443) ? ':' . \$_SERVER['SERVER_PORT'] : '')); \n";
             $data.="define('SCHLIX_SITE_HTTPS_URL','https://'.\$_SERVER['HTTP_HOST'] . ((\$_SERVER['SERVER_PORT'] != 80 && \$_SERVER['SERVER_PORT'] != 443) ? ':' . \$_SERVER['SERVER_PORT'] : '')); \n";
        } else
        {
            $data.="define('SCHLIX_SITE_HTTP_URL','http://'.\$_SERVER['HTTP_HOST'] . ((\$_SERVER['SERVER_PORT'] != 80 && \$_SERVER['SERVER_PORT'] != 443) ? ':' . \$_SERVER['SERVER_PORT'] : '')); /* Automatic URL */ \n";
            $data.="define('SCHLIX_SITE_HTTPS_URL','');\n";            
        }
        $data.="define('SCHLIX_SITE_PATH',str_replace('\\\','/',dirname(__FILE__)));\n";
        $data.="define('SCHLIX_DEFAULT_USER_CLASS','users'); /* Default user class - e.g: LDAP, smf forum, opencart, etc */\n";
        $data.="define('SCHLIX_DEFAULT_LANGUAGE','en_us'); /* Default Language */ \n";
        $data.="define('SCHLIX_DEFAULT_ADMIN_GROUP','Administrators');\n";
        //$data.="define('SCHLIX_ADMIN_LANGUAGE','en_us');\n";
        $data.="define('SCHLIX_SITE_OFFLINE',false); /* Set this to yes if you want to make your site offline for visitors */ \n";
        $data.="define('SCHLIX_SEF_ENABLED', true);\n";
        $data.="define('SCHLIX_SITE_SSL_ENABLED',{$str_enable_ssl});\n";
        $data.="define('SCHLIX_FORCE_SSL_ADMIN_LOGIN',{$str_enable_ssl});\n"; // 
        $data.="define('SCHLIX_DB_HOST','{$db_host}');\n";
        $data.="define('SCHLIX_DB_DATABASE','{$db_database}');\n";
        $data.="define('SCHLIX_DB_USERNAME','{$db_username}');\n";
        $data.="define('SCHLIX_DB_PASSWORD','{$db_password}');\n";
        $data.="define('SCHLIX_PAGE_CACHE_ENABLED', false);\n";
        $data.="define('SCHLIX_SQL_ENFORCE_ROW_LIMIT', true);\n";
        $data.="define('SCHLIX_SQL_CACHE_ENABLED', false);\n";
        $data.="define('SCHLIX_SQL_CACHE_TIME', 0);\n";
        $data.="define('SCHLIX_SESSION_LOGIN_TIME',{$admin_login_time}); /* in seconds */ \n";
        $data.="define('SCHLIX_ADMIN_THEME','admindefault');\n";
        $data.="define('SCHLIX_USER_IMAGE_DIRECTORY',SCHLIX_SITE_PATH.'/media/images');\n";
        $data.="define('SCHLIX_MAIL_DEFAULT_SENDER','{$mail_default_sender}');\n";
        $data.="define('SCHLIX_MAIL_DEFAULT_EMAIL','{$mail_default_email}');\n";
        $data.="define('SCHLIX_SMTP_HOST', '');\n";
        $data.="define('SCHLIX_SMTP_PORT', 25);\n";
        $data.="define('SCHLIX_SMTP_USE_AUTH', false);\n";
        $data.="define('SCHLIX_SMTP_USERNAME', '');\n";
        $data.="define('SCHLIX_SMTP_PASSWORD', '');\n";
        $data.="define('SCHLIX_SMTP_SECURITY', '');\n";
        $data.="define('SCHLIX_SYSTEM_TIMEZONE', '{$system_timezone}');\n";
        //$data.="define('SCHLIX_SITE_META_KEYWORDS',''); /* Default meta keywords if there is none defined in the app/item/category */\n";
        //$data.="define('SCHLIX_SITE_META_DESCRIPTION','{$site_name}'); /* Default meta keywords if there is none defined in the app/item/category */ \n";
        //$data.="define('SCHLIX_JS_MODE','{$schlix_js_mode}'); /* Javascript mode. 1 = Debug, 2 = Minified, 3 = Minified and Gzipped */ \n";
        $data.="define('SCHLIX_JS_MODE',3); /* Javascript mode. 1 = Debug, 2 = Minified, 3 = Minified and Gzipped */ \n";
        $data.="define('SCHLIX_RECORD_404_ERRORS','{$record_404_errors}'); /* Record 404 errors in the database */ \n";

        $data.="/**********************************************************************/\n";
        $data.="/* Do not modify below this line - this is for auto http/https switch */\n";
        $data.='define(\'SCHLIX_SITE_URL\', (defined(\'SCHLIX_SITE_HTTPS_URL\') && SCHLIX_SITE_SSL_ENABLED && SCHLIX_SITE_HTTPS_URL !=\'\' && ($_SERVER[\'HTTPS\']===\'on\' || $_SERVER[\'HTTPS\']===1 || $_SERVER[\'SERVER_PORT\']===443)) ? SCHLIX_SITE_HTTPS_URL : SCHLIX_SITE_HTTP_URL);';
        $data.="\n/**********************************************************************/\n";

        $data.='?>';        
        return $data;
    }
    
    /**
     * Generate Multisite-config.inc.php file content. If $use_wildcard is specified, 
     * it invalidates the $use_www. If $use_www is unspecified, only the specified domain
     * name 
     * @param string $site_id
     * @param string $server_name
     * @param bool $use_wildcard
     * @param bool $use_www
     * @return string
     */
    private function generateMultisiteConfigFileContent($site_id, $server_name, $use_wildcard, $use_www) 
    {
        if ($use_wildcard)
        {        
            $trans_www['*'] = $site_id; // this means all domain, including parked domain will go to this site_id
        } else
        {
            if ($use_www)
            {
                $site_host = $server_name;echo remove_prefix_from_string('www.', $server_name);
                $secondary_host = str_starts_with($site_host, 'www.' ) ? remove_prefix_from_string($server_name, 'www.') : 'www.'.$server_name;
                $trans_www[$site_host] = $site_id;
                $trans_www[$secondary_host] = $site_id;
            } else
            {
                $site_host = $server_name;
                $trans_www[$site_host] = $site_id;            
            }
        }


        $data = '<?php' . "\n";
        $data.='// Automatically generated on ' . get_current_datetime() . "\n\n";
        $data.= '/* 
 * Define multisites here. You **MUST EXPLICITLY** define all domains (including with or without www).
 * If you want to enable all sites pointing to a certain subsite, use the wildcard. Only one wildcard is allowed.*/' . "\n\n";
        $data.='global $SchlixWebsites;' . "\n\n";
        $data.='$SchlixWebsites =' . var_export($trans_www, TRUE) . ';';

        return $data;
    }
    
    /**
     * Return a result array with key: (status, messages), where messages is a
     * list of messages
     * @param string $db_host
     * @param string $db_database
     * @param string $db_username
     * @param string $db_password
     * @return array
     */
    protected function initializeDatabase($db_host, $db_database, $db_username, $db_password)
    {
        $init_result = array();
        $messages = array();
        // now init
        @$link = mysqli_connect($db_host, $db_username, $db_password);
        if ($link) {
            // check if database already exists
            $sql = 'SHOW DATABASES';
            $result = array();
            $last_query_result = mysqli_query($link, $sql);
            while ($row = mysqli_fetch_assoc($last_query_result))
                $result[] = $row;
            mysqli_free_result($last_query_result);
            if ($result) {
                $dbs = array();
                foreach ($result as $db) {
                    $dbs[] = $db['Database'];
                }
                if (in_array($db_database, $dbs)) {
                    $messages[] = "Database {$db_database} exists.";
                    $init_result['status'] = true;
                } else {// if not exist then create
                    $messages[] = "Database {$db_database} does not exist. Trying to create one.";
                    $sql = "CREATE DATABASE {$db_database} DEFAULT CHARACTER SET = utf8 DEFAULT COLLATE utf8_unicode_ci;";
                    $last_query_result = mysqli_query($link, $sql);
                    if (!$last_query_result) {
                        $messages[] = ('Cannot create database. Installation failed. Please create the database first (e.g: from phpMyAdmin) then proceed with this installation step. You do not need to close this window.');
                        $init_result['status'] = false;
                    } else {
                        $messages[] = ('Database creation successful');
                        $init_result['status'] = true;
                    }
                }
            } else {
                $messages[] = ('No Show Database permission - ok but continuing install');
                $init_result['status'] = true;                
            }            
             mysqli_close($link);              
             
        } else
        {
            $init_result['status'] = false;
        }
        $init_result['messages'] = $messages;
        return $init_result;
    }
    //_________________________________________________________//
    public function stepDBDataSampleAndUsersSetup() {
        global $__html_title;
        
        $__html_title = '4. Database Installation';
        $can_continue = ($_SESSION['agree'] && $_SESSION['met_requirement'] && $_SESSION['req_database_ok'] && ($_SESSION['site_name'] || $_POST['site_name'] ) );
        if (!$can_continue) {
            echo __Html::DIV_start(array('class'=>'alert alert-danger')).__Html::P('Cannot continue - please complete the previous steps first').__Html::DIV_end();
            return false;
        }
        if ($_POST['submit']) {
            $_SESSION['use_wildcard'] = (bool) $_POST['use_wildcard'];
            $_SESSION['use_www'] = (bool) $_POST['use_www'];
            $_SESSION['short_site_name'] = convert_into_sef_friendly_title($_POST['short_site_name']);
            $_SESSION['site_name'] = $this->addSlashes(htmlspecialchars($_POST['site_name']));
            $_SESSION['site_scheme'] = $_POST['site_scheme'];
            $_SESSION['site_domain_name_and_port'] = $_POST['site_domain_name_and_port'];
            $_SESSION['site_url'] = $_SESSION['site_scheme'].'://'.$_SESSION['site_domain_name_and_port'];
            $_SESSION['mail_default_sender'] = $this->addSlashes($_POST['mail_default_sender']);
            $_SESSION['mail_default_email'] = $this->addSlashes($_POST['mail_default_email']);
            $_SESSION['admin_login_time'] = $_POST['admin_login_time'];
            $_SESSION['admin_username'] = $this->addSlashes(trim($_POST['admin_username']));
            $_SESSION['admin_password'] = $this->addSlashes(trim($_POST['admin_password']));
            $_SESSION['system_timezone'] = $_POST['system_timezone'];
            //$_SESSION['site_httpbase'] = $_POST['site_httpbase'];
            $_SESSION['req_configinit_ok'] = 1;
        }
        // Save Multisite config file
        $data = $this->generateMultisiteConfigFileContent($_SESSION['short_site_name'], $_SERVER['SERVER_NAME'], $_SESSION['use_wildcard'], $_SESSION['use_www']);
        $result = file_put_contents(INSTALLER_ROOT_PATH . '/multisite-config.inc.php', $data);
        if (!$result) {
            echo __Html::H3("Cannot save main multi-site configuration file");
            return;
        }
        // create site skeleton
        $site_folder = $_SESSION['short_site_name'];
        if (!create_site_skeleton(INSTALLER_ROOT_PATH, $site_folder)) {
            echo __Html::H3("Cannot create subsite folder " . $site_folder);
            return;
        }       
        $use_https_fulltime = ($_SESSION['site_scheme'] == 'https');
        // Save site config file
        $data = $this->generateSiteConfigFileContent($_SESSION['site_name'], $_SESSION['db_host'], $_SESSION['db_database'], $_SESSION['db_username'], $_SESSION['db_password'], $_SESSION['admin_login_time'], $_SESSION['mail_default_sender'], $_SESSION['mail_default_email'], $_SESSION['system_timezone'], $_SESSION['record_404_errors'], $use_https_fulltime);
            
        $site_folder = $_SESSION['short_site_name'];
        $config_file_location = INSTALLER_ROOT_PATH . '/web/' . $site_folder . '/config.inc.php';
        $config_save_result = file_put_contents($config_file_location, $data);
        if (!$config_save_result) {
            $_SESSION['settings_save_ok'] = 0;
            echo __Html::H3("Cannot save subsite configuration file");
            return;
        } else {
            $_SESSION['settings_save_ok'] = 1;
        }

       $db_init_result =  $this->initializeDatabase($_SESSION['db_host'], $_SESSION['db_database'], $_SESSION['db_username'], $_SESSION['db_password']);
       $_SESSION['db_init_ok'] = $db_init_result['status'];
       if ($db_init_result['messages'])
       {
           foreach ($db_init_result['messages'] as $message)
           {
               echo __Html::H3($message);
           }
       }
       
        if ($_SESSION['db_init_ok']) {
            echo __Html::H3('Creating tables...');

            $install_sql = file(INSTALLER_ROOT_PATH . '/system/apps/sitemanager/sql/install_step1.sql', true);
            if (!$install_sql) {
                $_SESSION['db_tables_ok'] = 0;
                echo __Html::H3('Cannot load installation SQL');
                return false;
            }
            $install_sql_step2 = file(INSTALLER_ROOT_PATH . '/system/apps/sitemanager/sql/install_step2.sql', true);
            if (!$install_sql_step2) {
                $_SESSION['db_tables_ok'] = 0;
                echo __Html::H3('Cannot load installation SQL Step 2');
                return false;
            }

            $SystemDB = new \SCHLIX\cmsDatabase($_SESSION['db_host'], $_SESSION['db_database'], $_SESSION['db_username'], $_SESSION['db_password']);
            global $SystemLog;
            $SystemLog = new \SCHLIX\cmsLogger('gk_log_items');

            $this->execsql($install_sql, $SystemDB);
            $this->execsql($install_sql_step2, $SystemDB); // separate it. Install.SQL is run on upgrade
            $salt = $this->generateRandomPasswordSalt('blowfish');
            $hashed_password = crypt($_SESSION['admin_password'], $salt);
            
                
            $SystemDB->query("UPDATE gk_user_items set password = '{$hashed_password}', salt='{$salt}', email_address = '{$_SESSION['mail_default_email']}' WHERE id = 1"); // change password, add salt
            
            $sanitized_username = $SystemDB->real_escape_string($_SESSION['admin_username']);
            if ($_SESSION['admin_username'] != 'admin')
                $SystemDB->query("UPDATE gk_user_items set username = '{$sanitized_username}' WHERE id = 1"); // change password, add salt
            echo __Html::H3('Done');
            $_SESSION['db_tables_ok'] = 1;
            $_SESSION['db_blog_ok'] = 0;
        }
        include('step_database.template.php');
    }

    //_________________________________________________________//
    public function stepFinishSetup() {
        global $__html_title;
        
        
        $__html_title = '5. Finish';        
        $can_continue = ($_SESSION['agree'] && $_SESSION['met_requirement'] && $_SESSION['req_database_ok'] && $_SESSION['db_init_ok'] && $_SESSION['db_tables_ok'] && $_POST['submit'] && $_SESSION['db_blog_ok'] == 0);

        if (!$can_continue) {
            echo __Html::DIV_start(array('class'=>'alert alert-danger')).__Html::P('Please complete the previous steps before attempting to finish the installation').__Html::DIV_end();
            return false;
        }


        $sqlfname = ($_POST['installsample'] == 1) ? INSTALLER_ROOT_PATH . '/system/apps/sitemanager/sql/install_sample_magazine.sql' : INSTALLER_ROOT_PATH . '/system/apps/sitemanager/sql/install_sample_empty.sql';
        $_SESSION['db_blog_ok'] = 0;
        echo __Html::H3('Installing sample data...');

        $install_sql = file($sqlfname, true);
        if (!$install_sql) {
            $_SESSION['db_tables_ok'] = 0;
            echo __Html::H3('Cannot load sample data SQL');
            return false;
        }

        $new_images_path = $_SESSION['site_httpbase'] . '/web/' . $_SESSION['short_site_name'] . '/media/images/';
        $new_images_replacement = 'src="' . $new_images_path;
        //$install_sql = str_replace('src="system/images/', $new_images_replacement, $install_sql);
        //$install_sql = str_replace('src="/src/web/test/media/images/', $new_images_replacement, $install_sql);

        $SystemDB = new SCHLIX\cmsDatabase($_SESSION['db_host'], $_SESSION['db_database'], $_SESSION['db_username'], $_SESSION['db_password']);
        global $SystemLog;
        $SystemLog = new SCHLIX\cmsLogger('gk_log_items');


        $this->execsql($install_sql, $SystemDB);

        
        $menu_site_httpbase = $_SESSION['site_httpbase'];
        if ($menu_site_httpbase == '/')
            $menu_site_httpbase = ''; // reset it, so just / will be ignored
        if ($menu_site_httpbase != '')
        {
            if (!str_ends_with($menu_site_httpbase, '/'))
                $menu_site_httpbase.='/';
            // March 2, 2017 - in reponse to Softaculous tester. 
            // Menu will be reset after first run again
            $SystemDB->query("update `gk_menu_items` set sefurl = concat(:menu_site_httpbase,sefurl) where `application` NOT IN ('menu_divider','menu_placeholder','external_link')", ['menu_site_httpbase' => $menu_site_httpbase]);
            $SystemDB->query("update `gk_menu_items` set sefurl = :menu_site_httpbase where `application` = 'home'", ['menu_site_httpbase' =>  $_SESSION['site_httpbase']]); 
        }
        
        $SystemDB->query("update `gk_contact_items` set email = '{$_SESSION['mail_default_email']}'");
        echo __Html::H3('Database sample installation is done');

        $salt = $this->generateRandomPasswordSalt('blowfish');
        $hashed_password = crypt('testuser', $salt);
        $SystemDB->query("UPDATE gk_user_items set password = '{$hashed_password}', salt='{$salt}' WHERE id = 2"); // change password, add salt
        $uuid = new_uuid_v4();
        $SystemDB->query("INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('system', 'str_system_id', '{$uuid}', NOW(), 0)"); 
        include (realpath('../runtime.inc.php'));
        $schlix_version = SCHLIX_VERSION;
        $SystemDB->query("INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('system', 'str_database_version', '{$schlix_version}', NOW(), 0)");

        $_SESSION['db_blog_ok'] = 1;
        /*copy_recursive(INSTALLER_ROOT_PATH . '/system/themes/html5demo', INSTALLER_ROOT_PATH . '/web/' . $_SESSION['short_site_name'] . '/themes/html5demo');
        copy_recursive(INSTALLER_ROOT_PATH . '/system/images/demo', INSTALLER_ROOT_PATH . '/web/' . $_SESSION['short_site_name'] . '/media/images/demo');
        copy_recursive(INSTALLER_ROOT_PATH . '/system/images/slideshow', INSTALLER_ROOT_PATH . '/web/' . $_SESSION['short_site_name'] . '/media/slideshow');*/
        $this->copySampleData(INSTALLER_ROOT_PATH, $_SESSION['short_site_name'] );
        include('step_finish.template.php');
    }

    //_________________________________________________________//
    /**
     * Test database requirement with ajax
     */
    public function ajaxTestDatabase() {
        $_GET['ajax'] = 1;
        $ajax_result = '';
        if ($_POST['db_host'] && $_POST['db_database'] && $_POST['db_username'] && $_POST['db_password']) {
            $db_host = trim($_POST['db_host']);
            $db_username = trim($_POST['db_username']);
            $db_password = trim($_POST['db_password']);
            $db_database = trim($_POST['db_database']);
            $link = @mysqli_connect($db_host, $db_username, $db_password);
            if ($link) {
                $_SESSION['db_host'] = $db_host;
                $_SESSION['db_username'] = $db_username;
                $_SESSION['db_password'] = $db_password;

                //  $select_db_ok = mysqli_select_db($db_database, $link);
                //$last_query_result = mysqli_query("select 'abcde' as 'ok'", $link);
                $select_db_ok = mysqli_select_db($link, $db_database);
                $last_query_result = mysqli_query($link, "SELECT VERSION()");
                if (!$last_query_result) {
                    $error1 = 'Cannot run query: ' . mysqli_error($link);
                    //if (!$select_db_ok) $error1 = "Failed to select database";
                    $ajax_result = ajax_reply(200, $error1);
                } else {
                    $result = array();
                    while ($row = mysqli_fetch_assoc($last_query_result)) {
                        $result[] = $row;
                    }
                    mysqli_free_result($last_query_result);
                    if ($result) {
                        $_SESSION['db_database'] = $db_database;
                        $_SESSION['req_database_ok'] = 1;
                        $ajax_result =ajax_reply(200, 'OK');
                    } else {
                        $_SESSION['req_database_ok'] = 0;
                        $ajax_result = ajax_reply('300', 'Strange error ' . mysqli_error($link));
                    }
                }
                if ($link)
                    mysqli_close($link);
            } else {
                $_SESSION['req_database_ok'] = 0;
                $ajax_result = ajax_reply(200, 'Cannot connect using the entered hostname, username, and password. ' . mysqli_connect_error());
            }
        } else {
            $_SESSION['req_database_ok'] = 0;
            $ajax_result =ajax_reply(200, 'Parameters incomplete');
        }
        ajax_echo($ajax_result);
    }

    //_________________________________________________________//
    private function indicateYesNoWithPicture($req) {
        $active_icon = "fa fa-check";
        $inactive_icon = "fa fa-times";
        $active_color = "green";
        $inactive_color= "red";
        $which_icon = ($req) ? $active_icon : $inactive_icon;
        $which_color = ($req) ? $active_color : $inactive_color;
        return '<i class="' . $which_icon . '" style="color:'.$which_color.'"></i>';
    }

    //_________________________________________________________//
    private function generateRandomPasswordSalt($hash_type, $work_factor = 9) {
        // copy pasted from users.class.php - yes this is messy but the code for install is going to be deleted anyway.

        if ($work_factor < 4 || $work_factor > 31)
            $work_factor = 9;
        $rand = function_exists('openssl_random_pseudo_bytes') ? openssl_random_pseudo_bytes(16) : sha1(mt_rand(0, 32768) . microtime(true), true);
        $silly_string = substr(strtr(base64_encode($rand), '+', '.'), 0, 22);
        switch ($hash_type) {
            case 'sha256': $salt = '$5$rounds=' . str_pad($work_factor * 10000, 6, '0', STR_PAD_LEFT) . '$' . substr($silly_string, 0, 16);
                break;
            case 'sha512': $salt = '$6$rounds=' . str_pad($work_factor * 10000, 6, '0', STR_PAD_LEFT) . '$' . substr($silly_string, 0, 16);
                break;
            case 'blowfish':
            default: $salt = '$2y$' . str_pad($work_factor, 2, '0', STR_PAD_LEFT) . '$' . $silly_string;
                break;
        }
        return $salt;
    }

    //_________________________________________________________//
    public function probeHttpBase() {
        $curpath = pathinfo(parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH), PATHINFO_DIRNAME);
        $pos = strrpos($curpath, '/install');
        $the_httpbase = substr($curpath, 0, $pos);
        if ($the_httpbase == '/')
            $the_httpbase = '';
        return $the_httpbase;
    }

    //_________________________________________________________//		
    public function isUserInstallingSchlixOnDefaultFolder() {
        $my_httpbase = $this->probeHttpBase();
        $pos = strpos($my_httpbase, '/schlix');
        return ($pos !== FALSE);
    }

    //_________________________________________________________//	
    public function step1WeclomePage() {
        global $__html_title;
        
        $__html_title = '1. Welcome';        
        include('step_welcome.template.php');
    }

    /**
     * Check requirement
     * @return array
     */
    protected function checkSystemRequirement()
    {
        $php_extensions = get_loaded_extensions();
        ob_start();
        phpinfo(INFO_MODULES);
        $phpinfostr = ob_get_contents();
        ob_end_clean();
        $result = array();
        $result['session'] = in_array('session', $php_extensions);
        $result['mbstring'] = in_array('mbstring', $php_extensions);
        $result['gd'] = in_array('gd', $php_extensions);
        $result['mysql'] = in_array('mysqli', $php_extensions);
        $result['php50'] = version_compare(PHP_VERSION, '5.5.0');
        $result['bcrypt'] = (CRYPT_BLOWFISH == 1);
        //$result['curl'] = function_exists('curl_init');
        
        $prev_dir = remove_multiple_slashes(realpath(__DIR__.'/..'));
        $config_inc_php = remove_multiple_slashes($prev_dir.'/multisite-config.inc.php');
        $htaccess_real_file = remove_multiple_slashes($prev_dir.'/.htaccess');
        $htaccess_txt_file = remove_multiple_slashes($prev_dir.'/htaccess.txt');
        if (!file_exists($htaccess_real_file))
        {
            @copy($htaccess_txt_file, $htaccess_real_file);
        }
        //$result['configincphp'] = ( is_writable('../') && !(file_exists('../config.inc.php')) ) || ( is_writable('../config.inc.php') && (file_exists('../config.inc.php')) );
        
        $result['configincphp'] = ( is_writable($prev_dir) && !(file_exists($config_inc_php)) ) || ( is_writable($config_inc_php) && (file_exists($config_inc_php)) );
        $result['mod_rewrite'] = (strpos($phpinfostr, 'mod_rewrite') >= 0);
        $result['htaccess'] = file_exists($htaccess_real_file);
        
        $result['met_requirement'] = ($result['htaccess'] && $result['php50'] && $result['mysql'] && $result['mod_rewrite'] && $result['configincphp'] && $result['gd'] && $result['bcrypt']);
        $error_list = array();
        
        if (!$result['htaccess'])
            $error_list[] = '.htaccess file does not exist and the backup file htaccess.txt could not be copied either';        
        if (!$result['php50'])
            $error_list[] = 'PHP 5.5 or greator is required';
        if (!$result['mysql'])
            $error_list[] = 'MySQLi extension is required';
        if (!$result['mod_rewrite'])
            $error_list[] = 'Mod Rewrite is required for Apache-based installation';
        if (!$result['configincphp'])
            $error_list[] = 'Config file multisite-config.inc.php must be writable. Try chmod 0755 the whole install dir and change the script owner to the same as the website owner';
        if (!$result['gd'])
            $error_list[] = 'GD2 image library is required';
        if (!$result['bcrypt'])
            $error_list[] = 'BCrypt (CRYPT_BLOWFISH) is required';        
        /*if (!$result['curl'])
            $error_list[] = 'CURL extension is required'; */
        if ($error_list)
            $result['error_list'] = $error_list;
        return $result;
    }
    
    /**
     * Returns true if database check is okay, otherwise return an array of error list
     * @param string $db_host
     * @param string $db_database
     * @param string $db_username
     * @param string $db_password
     * @return string|array
     */
    public function testDatabase($db_host, $db_database, $db_username, $db_password)
    {
        $testresult = false;
        $error_msg = null;
        $link = @mysqli_connect($db_host, $db_username, $db_password);
        if ($link) {
            //  $select_db_ok = mysqli_select_db($db_database, $link);
            //$last_query_result = mysqli_query("select 'abcde' as 'ok'", $link);
            $select_db_ok = mysqli_select_db($link, $db_database);
            $last_query_result = mysqli_query($link, "SELECT VERSION()");
            if ($last_query_result) {
                $result = array();
                while ($row = mysqli_fetch_assoc($last_query_result)) {
                    $result[] = $row;
                }
                mysqli_free_result($last_query_result);
                if ($result) {
                    $testresult = true; 
                } else {
                    $testresult = false;
                }
            } else
                $testresult = false;
            if ($link)
                mysqli_close($link);
        } else {
            $error_msg = mysqli_connect_error();
        }        
        return array('status' => $testresult, 'error' => $error_msg);
    }
    
    private function copySampleData($site_root_file_path, $short_site_name)
    {
            copy_recursive($site_root_file_path . '/system/themes/html5demo', $site_root_file_path . '/web/' . $short_site_name. '/themes/html5demo');
            copy_recursive($site_root_file_path . '/system/themes/samplemagazine', $site_root_file_path . '/web/' . $short_site_name. '/themes/samplemagazine');
            copy_recursive($site_root_file_path . '/system/images/articles', $site_root_file_path . '/web/' . $short_site_name. '/media/articles');
            copy_recursive($site_root_file_path . '/system/images/demo', $site_root_file_path . '/web/' . $short_site_name. '/media/images/demo');
            copy_recursive($site_root_file_path . '/system/images/slideshow', $site_root_file_path . '/web/' . $short_site_name. '/media/slideshow');
            copy_recursive($site_root_file_path . '/system/images/avatars', $site_root_file_path . '/web/' . $short_site_name. '/data/user/avatars');
            copy($site_root_file_path . '/system/images/schlixlogo.png', $site_root_file_path . '/web/' . $short_site_name. '/media/images/schlixlogo.png');
    }
    /**
     * Perform automatic installation
     * @param bool $agree_term
     * @param string $db_host
     * @param string $db_database
     * @param string $db_username
     * @param string $db_password
     * @param string $site_name
     * @param string $site_id
     * @param string $site_httpbase
     * @param string $email_from
     * @param string $email_address
     * @param string $admin_username
     * @param string $admin_password
     * @param string $sampledata
     * @param string $use_wildcard
     * @param string $timezone
     * @return bool
     */
    public function automatedInstall($agree_term, $db_host, $db_database, $db_username, $db_password, $site_domain_name, $site_use_https, $site_httpbase, $site_name, $site_root_file_path, $site_id, $email_from, $email_address, $admin_username, $admin_password, $sampledata = true, $use_wildcard = false, $timezone = 'UTC', $use_www = false) 
    {
            
            $site_httpbase = remove_multiple_slashes($site_httpbase);
            
            $error_list = array();
            ///////////////////////////////////////////////////////////////////////////////
            //1. License Agreement
            ///////////////////////////////////////////////////////////////////////////////
            if (!$agree_term) {
                throw new Exception("You must agree to SCHLIX Shared Source license before continuing");
            }
            ///////////////////////////////////////////////////////////////////////////////
            //2. Requirement
            ///////////////////////////////////////////////////////////////////////////////
            $requirement = $this->checkSystemRequirement();
            if ($requirement['error_list']) {
                $error_list = $requirement['error_list'];
                $str_error_list = implode(',', $error_list);
                throw new Exception("Requirement not met: ".$str_error_list);
            }
            ///////////////////////////////////////////////////////////////////////////////
            //3. Database
            ///////////////////////////////////////////////////////////////////////////////
            $db_test_result = $this->testDatabase($db_host, $db_database, $db_username, $db_password);
            if ($db_test_result['status'] === false) {
                throw new Exception('Cannot connect to the database :'.$db_test_result['error']);
            } 
            ///////////////////////////////////////////////////////////////////////////////
            // 4. check SiteName, Site ID, Database
            ///////////////////////////////////////////////////////////////////////////////
            $lowercase_site_id = strtolower($site_id);
            if (alpha_numeric_only($lowercase_site_id) != $lowercase_site_id)
                throw new Exception('Site ID must not contain spaces or any special character. Only alphanumeric characters are accepted');
            if (filter_var($email_address, FILTER_VALIDATE_EMAIL) === false) {
                throw new Exception('Invalid email address');
            }
            ///////////////////////////////////////////////////////////////////////////////
            // 5. Write multisite-config.inc.php
            ///////////////////////////////////////////////////////////////////////////////
            $multisite_config_data = $this->generateMultisiteConfigFileContent($lowercase_site_id, $site_domain_name, $use_wildcard, $use_www);
            $multisite_config_write_result = file_put_contents($site_root_file_path . '/multisite-config.inc.php', $multisite_config_data);
            if ($multisite_config_write_result === false) {
                throw new Exception('Cannot write to multisite-config.inc.php');
            }
            ///////////////////////////////////////////////////////////////////////////////
            // 6. create site skeleton
            ///////////////////////////////////////////////////////////////////////////////
            if (!create_site_skeleton($site_root_file_path, $lowercase_site_id)) {
                throw new Exception("Cannot create subsite folder " . $lowercase_site_id);
            }
            ///////////////////////////////////////////////////////////////////////////////
            // 7. Write config.inc.php
            ///////////////////////////////////////////////////////////////////////////////
            $config_data = $this->generateSiteConfigFileContent($site_name, $db_host, $db_database, $db_username, $db_password, 86400*2, $email_from, $email_address, $timezone, 0, $site_use_https);
            $config_file_location = $site_root_file_path . '/web/' . $lowercase_site_id . '/config.inc.php';

            $config_save_result = file_put_contents($config_file_location, $config_data);
            if (!$config_save_result) {
                throw new Exception("Cannot save subsite configuration file");
            }
            /////////////////////////////////////////////////////////////////////////////////
            // 8. Initialize database
            ///////////////////////////////////////////////////////////////////////////////
            $db_init_result = $this->initializeDatabase($db_host, $db_database, $db_username, $db_password);
            if ($db_init_result['status'] == false) {
                throw new Exception("Error while initializing database: ". implode(',', $db_init_result['messages']));
            }
            ///////////////////////////////////////////////////////////////////////////////
            // 9. Run install_step1.sql
            ///////////////////////////////////////////////////////////////////////////////
            $install_sql = file($site_root_file_path . '/system/apps/sitemanager/sql/install_step1.sql', true);
            if (!$install_sql) {
                throw new Exception('Cannot load installation SQL');
            }
            $install_sql_step2 = file($site_root_file_path . '/system/apps/sitemanager/sql/install_step2.sql', true);
            if (!$install_sql_step2) {
                throw new Exception('Cannot load installation SQL Step 2');
            }

            global $SystemLog;
            
            $SystemDB = new \SCHLIX\cmsDatabase($db_host, $db_database, $db_username, $db_password);
            $SystemLog = new \SCHLIX\cmsLogger('gk_log_items');

            $this->execsql($install_sql, $SystemDB, FALSE);
            $this->execsql($install_sql_step2, $SystemDB, FALSE); // separate it. Install.SQL is run on upgrade
            $salt = $this->generateRandomPasswordSalt('blowfish');
            $hashed_password = crypt($admin_password, $salt);
            $sanitized_email = $SystemDB->sanitizeString($email_address);
            $sanitized_admin_username = $SystemDB->sanitizeString($admin_username);
            $sanitized_password = $SystemDB->sanitizeString($hashed_password);
            $sanitized_salt = $SystemDB->sanitizeString($salt);
            // change password, add salt
            $SystemDB->query("UPDATE gk_user_items set `username` = {$sanitized_admin_username}, `password` = {$sanitized_password}, `salt`={$sanitized_salt}, email_address = {$sanitized_email} WHERE id = 1"); 
            ///////////////////////////////////////////////////////////////////////////////
            // 10. Sample Data, yes or no? In the future we should have several sample data
            ///////////////////////////////////////////////////////////////////////////////
            $which_sample_data = ($sampledata) ? 'install_sample_magazine.sql' : 'install_sample_empty.sql';
            $sqlfname =  $site_root_file_path.'/system/apps/sitemanager/sql/'.$which_sample_data;

            $install_sql = file($sqlfname, true);
            if (!$install_sql) {
                throw new Exception('Cannot load sample data SQL');
            }

            $new_images_path = remove_multiple_slashes($site_httpbase . '/web/' . $lowercase_site_id. '/media/images/');
            $new_images_replacement = 'src="' . $new_images_path;
            
            $install_sql = str_replace('src="system/images/', $new_images_replacement, $install_sql);
            $install_sql = str_replace('src="/src/web/test/media/images/', $new_images_replacement, $install_sql);


            $this->execsql($install_sql, $SystemDB, FALSE);

            $menu_site_httpbase = $site_httpbase;
            if ($menu_site_httpbase == '/')
                $menu_site_httpbase = ''; // reset it, so just / will be ignored
            if ($menu_site_httpbase != '')
            {
                if (!str_ends_with($menu_site_httpbase, '/'))
                    $menu_site_httpbase.='/'; // add trailing slash for menu only
                // March 2, 2017 - in reponse to Softaculous tester. 
                // Menu will be reset after first run again
                $SystemDB->query("update `gk_menu_items` set sefurl = concat(:menu_site_httpbase,sefurl) where `application` NOT IN ('menu_divider','menu_placeholder','external_link')", ['menu_site_httpbase' => $menu_site_httpbase]);
                $SystemDB->query("update `gk_menu_items` set sefurl = :menu_site_httpbase where `application` = 'home'", ['menu_site_httpbase' => $site_httpbase]);
            }
            $SystemDB->query("update `gk_contact_items` set email = {$sanitized_email}");

            $salt = $this->generateRandomPasswordSalt('blowfish');
            $hashed_password = crypt('testuser', $salt);
            $sanitized_salt = $SystemDB->sanitizeString($salt);
            $sanitized_password = $SystemDB->sanitizeString($hashed_password);
            $SystemDB->query("UPDATE gk_user_items set `password` = {$sanitized_password}, `salt`={$sanitized_salt} WHERE id = 2"); // change password, add salt
            $uuid = new_uuid_v4();
            $SystemDB->query("INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('system', 'str_system_id', '{$uuid}', NOW(), 0)"); 
            include ( $site_root_file_path.'/runtime.inc.php');
            $schlix_version = SCHLIX_VERSION;
            $SystemDB->query("INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('system', 'str_database_version', '{$schlix_version}', NOW(), 0)");

            $this->copySampleData($site_root_file_path, $lowercase_site_id);            
            
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        return true;
    }

    //_________________________________________________________//
    public function Run($command) {
        switch ($command['action']) {
            case 'testdatabase': $this->ajaxTestDatabase();
                break;
            case 'requirement': $this->stepRequirementAndDBSetup();
                break;
            case 'variables': $this->stepSiteVariablesSetup();
                break;
            case 'database': $this->stepDBDataSampleAndUsersSetup();
                break;
            case 'finish': $this->stepFinishSetup();
                break;
            case 'welcome':
            default: $this->step1WeclomePage();
        }
    }

}

?>
