<?php
/* 
 * Upgrade function - for the auto upgrader (installed separately)
 * 
 * Copyright (C) 2016 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.txt for details
 */

/**
 * Upgrade
 * @ignore
 * @param string $from_version
 * @param string $db_host
 * @param string $db_database
 * @param string $db_username
 * @param string $db_password
 */
function upgrade_schlix ($from_version, $db_host, $db_database, $db_username, $db_password)
{    
    $messages = array();
    $errors = array();
    
    $mydb = new \Schlix\cmsDatabase($db_host,$db_database, $db_username,$db_password);    
    
    $mydb->query("SET SQL_MODE=''");
    // v2.0.0 BETA 5
    if (!$mydb->tableColumnExists('gk_user_items', 'avatar'))
        $mydb->query('ALTER TABLE `gk_user_items` ADD `avatar` VARCHAR(255) NULL DEFAULT NULL AFTER `display_name`;');
    if (!$mydb->tableColumnExists('gk_menu_items', 'subtemplate_mobile'))
        $mydb->query('ALTER TABLE `gk_menu_items` ADD `subtemplate_mobile` VARCHAR(64) NULL DEFAULT NULL AFTER `subtemplate`;');
    
    // v2.0.0 RC1
    // SCHLIX updater
    $result = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_app_items` WHERE `title` = 'schlixupdater'");
    if ($result)
    {
        $sql = "UPDATE `gk_app_items` SET `app_alias` =  'schlixupdater', show_in_app_menu = '0' WHERE  `title` = 'schlixupdater'";
    } else
    {
        $sql = "INSERT INTO `gk_app_items` (`title`, `app_alias`, `app_description`, `description`, `has_frontend`, `has_backend`, `show_left_to_adminmenu`, `show_right_to_adminmenu`, `show_in_app_menu`, `show_in_main`, `status`, `sort_order`, `date_modified`) VALUES ( 'schlixupdater', 'schlixupdater', 'SCHLIX Updater', NULL, 1, 1, 0, 0, 0, NULL, 0, 0, NOW())";
    }
    $mydb->query($sql);
    
    
    // v2.0.0 STABLE
    if ($mydb->tableColumnExists('gk_menu_items', 'right_font_icon_css_class'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` CHANGE `right_font_icon_css_class` `use_font_icon_only_no_text` TINYINT(1) NULL DEFAULT NULL; ');
    }
    if ($mydb->tableColumnExists('gk_menu_items', 'left_font_icon_css_class'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` CHANGE `left_font_icon_css_class` `font_icon` VARCHAR(63) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL; ');    
    }
    
    if ($mydb->tableColumnExists('gk_menu_items', 'right_glyph_icon_css_class'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` CHANGE `right_glyph_icon_css_class` `use_font_icon_only_no_text` TINYINT(1) NULL DEFAULT NULL; ');
    }
    if ($mydb->tableColumnExists('gk_menu_items', 'left_glyph_icon_css_class'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` CHANGE `left_glyph_icon_css_class` `font_icon` VARCHAR(63) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL; ');    
    }
    
    //quirk
    if (!$mydb->tableColumnExists('gk_menu_items', 'use_font_icon_only_no_text'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` ADD `use_font_icon_only_no_text` TINYINT(1) NULL DEFAULT NULL ;');    
    }
    if (!$mydb->tableColumnExists('gk_menu_items', 'font_icon'))
    {
        $mydb->query('ALTER TABLE `gk_menu_items` ADD `font_icon` VARCHAR(63) CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL;');    
    }
    // v2.0.1
    if (!$mydb->tableColumnExists('gk_email_queue_items', 'store_folder'))
    {
        $mydb->query('ALTER TABLE `gk_email_queue_items` ADD `store_folder` VARCHAR(255) NOT NULL AFTER `attachments`; ');
    }
    // v2.0.1 fix cron
    $mydb->query("update`gk_cronscheduler_items` set command_internal='\\SCHLIX\\cmsLogger::processRunCleanupLog' WHERE command_internal = '\\cmsLogger::processRunCleanupLog'");
    
    // v2.0.3
    $mydb->query("update`gk_theme_items` set status = 1");
    
    if (!$mydb->tableColumnExists('gk_theme_items', 'options'))
    {
        $mydb->query('ALTER TABLE `gk_theme_items` ADD `options` MEDIUMTEXT NULL DEFAULT NULL AFTER `subtemplates`; ');
    }
    
    // Frontend Edit Control
    $result = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_macro_items` WHERE `title`='frontendeditcontrol' ");
    if (!$result)
    {
        $mydb->query("INSERT INTO `gk_macro_items` (`title`, `description`, `status`, `sort_order`, `display_in_apps`, `display_in_blocks`, `date_modified`, `enabled_apps`, `enabled_blocks`) VALUES  ( 'frontendeditcontrol', '', 1, 0, 1, 1, NOW(),NULL,NULL)");
        
        $macro_config_exists = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_macro_config` WHERE `section`='frontendeditcontrol' AND `key` ='array_groups_access' ");
        if (!$macro_config_exists)
        {
            $mydb->query("INSERT INTO `gk_macro_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('frontendeditcontrol', 'array_groups_access', 'a:1:{i:0;s:1:\"1\";}', NOW(), 0);");
        }        
    }
    /// end v2.0.3
    
    // v2.0.5
    
    
    if (!$mydb->tableColumnExists('gk_sites_websites', 'db_host'))
    {
        $mydb->query('ALTER TABLE `gk_sites_websites` ADD `mail_default_sender` VARCHAR(255) NULL DEFAULT NULL AFTER `mail_default_email`, ADD `db_host` VARCHAR(255) NULL DEFAULT NULL AFTER `mail_default_sender`, ADD `db_database` VARCHAR(255) NULL DEFAULT NULL AFTER `db_host`, ADD `system_timezone` VARCHAR(255) NULL DEFAULT NULL AFTER `db_database`;');
    }
    
    // v2.0.6
    if (!$mydb->tableExists('gk_theme_config'))
    {
        $mydb->query("CREATE TABLE `gk_theme_config` (
          `section` varchar(32) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
          `key` varchar(64) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
          `value` text COLLATE utf8_unicode_ci,
          `date_modified` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
          `modified_by_id` bigint(20) NOT NULL DEFAULT '0',
          UNIQUE KEY `primary_key` (`section`,`key`) 
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");
        
    }
    ////////////////////////////////////////
    // v2.0.9 (no public release, and v2.1.0
    ////////////////////////////////////////
    // Drop unusued table columns
    if ($mydb->tableColumnExists('gk_blog_items', 'close_commenting'))
        $mydb->query("ALTER TABLE `gk_blog_items` DROP `close_commenting`;");
    if ($mydb->tableColumnExists('gk_blog_items', 'close_pingback'))
        $mydb->query("ALTER TABLE `gk_blog_items` DROP `close_pingback`;");
    if ($mydb->tableColumnExists('gk_blog_items', 'hide_comments'))
        $mydb->query("ALTER TABLE `gk_blog_items` DROP `hide_comments`;");
    // 2.1.0 - add gk_landing_categories
    if (!$mydb->tableExists('gk_landing_categories'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_landing_categories` (
                    `cid` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                    `guid` varchar(36) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `title` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
                    `virtual_filename` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `date_created` datetime DEFAULT NULL,
                    `date_modified` datetime DEFAULT NULL,
                    `date_available` datetime DEFAULT NULL,
                    `date_expiry` datetime DEFAULT NULL,
                    `sort_order` int(3) unsigned DEFAULT NULL,
                    `status` tinyint(1) NOT NULL DEFAULT '1',
                    `featured` tinyint(1) NOT NULL DEFAULT '0',
                    `version` decimal(20,2) NOT NULL DEFAULT '1.00',
                    `pageview` bigint(20) NOT NULL DEFAULT '0',
                    `items_per_page` int(11) unsigned NOT NULL DEFAULT '10',
                    `created_by_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `modified_by_id` bigint(20) unsigned DEFAULT NULL,
                    `locked_by_id` bigint(20) DEFAULT NULL,
                    `meta_key` text COLLATE utf8_unicode_ci,
                    `meta_description` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `custom_css` mediumtext COLLATE utf8_unicode_ci,
                    `custom_javascript` mediumtext COLLATE utf8_unicode_ci,
                    `custom_header` mediumtext COLLATE utf8_unicode_ci,
                    `permission_write` text COLLATE utf8_unicode_ci,
                    `permission_read` text COLLATE utf8_unicode_ci,
                    `options` text COLLATE utf8_unicode_ci,
                    `tags` text COLLATE utf8_unicode_ci,

                    PRIMARY KEY (`cid`),
                    KEY `date_created` (`date_created`),
                    KEY `date_modified` (`date_modified`),
                    KEY `sort_order` (`sort_order`),
                    KEY `status` (`status`),
                    KEY `created_by_id` (`created_by_id`),
                    KEY `modified_by_id` (`modified_by_id`),
                    UNIQUE KEY `guid` (`guid`)
                  ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");
    }
    // 2.1.0 - add gk_landing_items
    if (!$mydb->tableExists('gk_landing_items'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_landing_items` (
                    `id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                    `guid` varchar(36) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `category_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `title` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
                    `virtual_filename` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
                    `summary` mediumtext COLLATE utf8_unicode_ci,
                    `date_created` datetime DEFAULT NULL,
                    `date_modified` datetime DEFAULT NULL,
                    `date_available` datetime DEFAULT NULL,
                    `date_expiry` datetime DEFAULT NULL,
                    `created_by_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `modified_by_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `locked_by_id` bigint(20) DEFAULT NULL,
                    `permission_write` text COLLATE utf8_unicode_ci,
                    `permission_read` text COLLATE utf8_unicode_ci,
                    `meta_key` text COLLATE utf8_unicode_ci,
                    `meta_description` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `pageview` bigint(20) NOT NULL DEFAULT '0',
                    `status` int(11) NOT NULL DEFAULT '0',
                    `featured` tinyint(1) NOT NULL DEFAULT '0',
                    `version` decimal(20,2) NOT NULL DEFAULT '1.00',
                    `sort_order` int(11) NOT NULL DEFAULT '0',
                    `options` text COLLATE utf8_unicode_ci,
                    `tags` text COLLATE utf8_unicode_ci,

                    PRIMARY KEY (`id`),
                    KEY `category_id` (`category_id`),
                    KEY `title` (`title`),
                    KEY `id_category_id` (`id`,`category_id`),
                    KEY `id_category_id_status` (`id`,`category_id`,`status`),
                    KEY `id_category_id_status_sort_order` (`id`,`category_id`,`status`,`sort_order`),
                    KEY `virtual_filename` (`virtual_filename`),
                    KEY `date_created` (`date_created`),
                    KEY `date_modified` (`date_modified`),
                    KEY `created_by_id` (`created_by_id`),
                    KEY `modified_by_id` (`modified_by_id`),
                    KEY `status` (`status`),
                    KEY `sort_order` (`sort_order`),
                    UNIQUE KEY `guid` (`guid`)
                  ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");    
    }
    // 2.1.0 - add table  gk_blog_categories_items
    if (!$mydb->tableExists('gk_blog_categories_items'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_blog_categories_items` (
                      `id` bigint(20) unsigned NOT NULL DEFAULT '0',
                      `cid` bigint(20) unsigned NOT NULL DEFAULT '0',
                      `is_primary` tinyint(1) unsigned NOT NULL DEFAULT '0',
                      PRIMARY KEY (`id`,`cid`),
                      KEY `is_primary` (`is_primary`),
                      UNIQUE KEY `primary_category` (`id`,`cid`,`is_primary`)
                    ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
                  ");        
        // now upgrade blog and migrate it to multi-category
        $mydb->query("INSERT INTO gk_blog_categories_items (id, cid, is_primary)
            SELECT id, category_id as cid, 1 as is_primary
            FROM gk_blog_items;");
        // now drop category_id after migration
        if ($mydb->tableColumnExists('gk_blog_items', 'category_id'))
            $mydb->query("ALTER TABLE `gk_blog_items` DROP `category_id`;");
        
    }    
    // 2.1.0 - gk_blog_items - add new columns - summary
    if (!$mydb->tableColumnExists('gk_blog_items', 'summary_secondary_headline'))
        $mydb->query("ALTER TABLE `gk_blog_items` ADD `summary_secondary_headline` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER `summary`; ");
    if (!$mydb->tableColumnExists('gk_blog_items', 'summary_intro_image'))
        $mydb->query('ALTER TABLE `gk_blog_items` ADD `summary_intro_image` VARCHAR(255) NULL DEFAULT NULL AFTER `summary_secondary_headline`;');
    if (!$mydb->tableColumnExists('gk_blog_items', 'summary_intro_image_caption'))
        $mydb->query("ALTER TABLE `gk_blog_items` ADD `summary_intro_image_caption` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER `summary_intro_image`; ");
    // 2.1.0  - gk_blog_items - add new columns - description    
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_alternative_title'))
        $mydb->query("ALTER TABLE `gk_blog_items` ADD `description_alternative_title` VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `description`; ");    
    
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_secondary_headline'))
        $mydb->query("ALTER TABLE `gk_blog_items` ADD `description_secondary_headline` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER `description_alternative_title`; ");
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_intro_image'))
        $mydb->query('ALTER TABLE `gk_blog_items` ADD `description_intro_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description_secondary_headline`;');
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_background_image'))
        $mydb->query('ALTER TABLE `gk_blog_items` ADD `description_background_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description_intro_image`;');    
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_intro_image_caption'))
        $mydb->query('ALTER TABLE `gk_blog_items` ADD `description_intro_image_caption` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER `description_background_image`;');
    if (!$mydb->tableColumnExists('gk_blog_items', 'description_custom_html_code'))
        $mydb->query('ALTER TABLE `gk_blog_items` ADD `description_custom_html_code` TEXT CHARACTER SET utf8 COLLATE utf8_unicode_ci NULL DEFAULT NULL AFTER `description_intro_image_caption`;');

    // 2.1.0 - gk_blog_categories - add new columns - summary
    if (!$mydb->tableColumnExists('gk_blog_categories', 'summary_intro_image'))
        $mydb->query('ALTER TABLE `gk_blog_categories` ADD `summary_intro_image` VARCHAR(255) NULL DEFAULT NULL AFTER `summary`;');
    // 2.1.0 - gk_blog_categories - add new columns - summary    
    if (!$mydb->tableColumnExists('gk_blog_categories', 'description_intro_image'))
        $mydb->query('ALTER TABLE `gk_blog_categories` ADD `description_intro_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description`;');
    if (!$mydb->tableColumnExists('gk_blog_categories', 'description_background_image'))
        $mydb->query('ALTER TABLE `gk_blog_categories` ADD `description_background_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description_intro_image`;');    
    
    
    // 2.1.0 - gk_html_categories - add new columns - description
    if (!$mydb->tableColumnExists('gk_html_categories', 'description_background_image'))
        $mydb->query('ALTER TABLE `gk_html_categories` ADD `description_background_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description`;');    
    // 2.1.0 - gk_html_items - add new columns - description
    if (!$mydb->tableColumnExists('gk_html_items', 'description_background_image'))
        $mydb->query('ALTER TABLE `gk_html_items` ADD `description_background_image` VARCHAR(255) NULL DEFAULT NULL AFTER `description`;');    
    
    // 2.1.0 - add table  gk_tag_categories
    if (!$mydb->tableExists('gk_tag_categories'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_tag_categories` (
                    `cid` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                    `guid` varchar(36) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `title` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
                    `virtual_filename` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `date_created` datetime DEFAULT NULL,
                    `date_modified` datetime DEFAULT NULL,
                    `date_available` datetime DEFAULT NULL,
                    `date_expiry` datetime DEFAULT NULL,
                    `sort_order` int(3) unsigned DEFAULT NULL,
                    `status` tinyint(1) NOT NULL DEFAULT '1',
                    `featured` tinyint(1) NOT NULL DEFAULT '0',
                    `pageview` bigint(20) NOT NULL DEFAULT '0',
                    `items_per_page` int(11) unsigned NOT NULL DEFAULT '10',
                    `created_by_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `modified_by_id` bigint(20) unsigned DEFAULT NULL,
                    `locked_by_id` bigint(20) DEFAULT NULL,
                    `meta_key` text COLLATE utf8_unicode_ci,
                    `meta_description` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `summary` mediumtext COLLATE utf8_unicode_ci,
                    `description` mediumtext COLLATE utf8_unicode_ci,
                    `permission_write` text COLLATE utf8_unicode_ci,
                    `permission_read` text COLLATE utf8_unicode_ci,
                    `options` text COLLATE utf8_unicode_ci,

                    PRIMARY KEY (`cid`),
                    KEY `date_created` (`date_created`),
                    KEY `date_modified` (`date_modified`),
                    KEY `sort_order` (`sort_order`),
                    KEY `status` (`status`),
                    KEY `created_by_id` (`created_by_id`),
                    KEY `modified_by_id` (`modified_by_id`),
                    UNIQUE KEY `guid` (`guid`)
                  ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");        
    }    
    
    // 2.1.0 - add table  gk_tag_categories
    if (!$mydb->tableExists('gk_tag_items'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_tag_items` (
                    `id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
                    `guid` varchar(36) COLLATE utf8_unicode_ci DEFAULT NULL,
                    `category_id` bigint(20) unsigned NOT NULL DEFAULT '0',
                    `date_created` datetime DEFAULT NULL,
                    `internalurl` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
                    `application` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',

                    PRIMARY KEY (`id`),
                    KEY `category_id` (`category_id`),
                    KEY `application` (`application`),
                    KEY `guid` (`guid`),
                    UNIQUE KEY `guid_category_id` (`guid`, `category_id`)
                  ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");        
    }
    // 2.1.0 - add gk_menu_items -  subtemplate_inner_pages
    
    if (!$mydb->tableColumnExists('gk_menu_items', 'subtemplate_inner_pages'))
        $mydb->query('ALTER TABLE `gk_menu_items` ADD `subtemplate_inner_pages` VARCHAR(64) NULL DEFAULT NULL AFTER `subtemplate`; ');    
    
    // Tag
    $result = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_app_items` WHERE `title` = 'tag'");
    if (!$result)
    {        
        $mydb->query("INSERT INTO `gk_app_items` (`title`, `app_alias`, `app_description`, `description`, `has_frontend`, `has_backend`, `show_left_to_adminmenu`, `show_right_to_adminmenu`, `show_in_app_menu`, `show_in_main`, `status`, `sort_order`, `date_modified`) VALUES ( 'tag', 'tag', 'Tags', NULL, 1, 1, 0, 0, 0, NULL, 0, 0, NOW())");
    }
    
    // Landing
    $result = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_app_items` WHERE `title` = 'landing'");
    if (!$result)    
    {        
        $mydb->query("INSERT INTO `gk_app_items` (`title`, `app_alias`, `app_description`, `description`, `has_frontend`, `has_backend`, `show_left_to_adminmenu`, `show_right_to_adminmenu`, `show_in_app_menu`, `show_in_main`, `status`, `sort_order`, `date_modified`) VALUES ( 'landing', 'lp', 'Landing Pages', NULL, 1, 1, 0, 0, 1, NULL, 0, 0, NOW())");
    }
    
    // 2.1.0 Fix menus that don't suppose to be listed
    $mydb->query("UPDATE `gk_app_items` SET show_in_app_menu = '0' WHERE  `title` IN ('schlixupdater','simplesitemap','tag');");
    // end 
    /*
     * Not necessary
    if (!$mydb->tableColumnExists('gk_cronscheduler_items', 'date_lastrun_for'))
    {
        $mydb->query('ALTER TABLE `gk_cronscheduler_items` ADD `date_lastrun_for` DATETIME NULL DEFAULT NULL AFTER `date_lastrun_end`; ');
    }*/
    // v2.1.1-1 - WYSIWYG profiles
    if (!$mydb->tableExists('gk_wysiwyg_profiles'))
    {
        $mydb->query("CREATE TABLE `gk_wysiwyg_profiles` (
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `editor_id` bigint(20) NOT NULL,
            `title` varchar(255) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
            `virtual_filename` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
            `sort_order` int(11) NOT NULL DEFAULT '0',            
            `date_created` datetime NOT NULL,
            `date_modified` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            `permission_read` text COLLATE utf8_unicode_ci,
            `permission_write` text COLLATE utf8_unicode_ci,
            `options` text COLLATE utf8_unicode_ci,
            `is_system` tinyint(1) NOT NULL DEFAULT '0',
            PRIMARY KEY (`id`),
            KEY `editor_id` (`editor_id`),
            KEY `title` (`title`),
            KEY `is_system` (`is_system`)
          ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;
          ");
        
        $mydb->query("
        INSERT INTO `gk_wysiwyg_profiles` (`id`, `editor_id`, `title`, `virtual_filename`, `sort_order`, `date_created`, `date_modified`, `permission_read`, `permission_write`, `options`, `is_system`) VALUES
        (1, 1, 'Blank Text Area', 'blank', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (2, 2, 'TinyMCE4 Full Featured', 'tinymce4-full', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (3, 2, 'TinyMCE4 Simple', 'tinymce4-simple', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (4, 2, 'TinyMCE4 Limited (No Media Manager)', 'tinymce4-limited', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (5, 3, 'CKEditor4 Full Featured', 'ckeditor4-full', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (6, 3, 'CKEditor4 Simple', 'ckeditor4-simple', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (7, 3, 'CKEditor4 Limited (No Media Manager)', 'ckeditor4-limited', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (8, 4, 'CodeMirror PHP', 'codemirror-php', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (9, 4, 'CodeMirror HTML', 'codemirror-html', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (10, 4, 'CodeMirror Javascript', 'codemirror-js', 0, NOW(), NOW(), NULL, NULL, NULL, 1),
        (11, 4, 'CodeMirror CSS', 'codemirror-css', 0, NOW(), NOW(), NULL, NULL, NULL, 1);
        ");        
    }
    
    // v2.1.1-1 - WYSIWYG Config and profiles
    if (!$mydb->tableExists('gk_wysiwyg_config'))
    {
        $mydb->query("CREATE TABLE IF NOT EXISTS `gk_wysiwyg_config` (
            `section` varchar(32) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
            `key` varchar(64) COLLATE utf8_unicode_ci NOT NULL DEFAULT 'untitled',
            `value` text COLLATE utf8_unicode_ci,
            `date_modified` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            `modified_by_id` bigint(16) NOT NULL DEFAULT '0',
            UNIQUE KEY `primary_key` (`section`,`key`)
          ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");
            // TinyMCE4 full
            $mydb->query("INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES
            ('tinymce4-full', 'array_menubar_top', 'a:7:{i:0;s:4:\"file\";i:1;s:4:\"edit\";i:2;s:6:\"insert\";i:3;s:4:\"view\";i:4;s:6:\"format\";i:5;s:5:\"table\";i:6;s:5:\"tools\";}', NOW(), 0),
            ('tinymce4-full', 'array_plugins', 'a:37:{i:0;s:7:\"advlist\";i:1;s:6:\"anchor\";i:2;s:8:\"autolink\";i:3;s:10:\"autoresize\";i:4;s:7:\"charmap\";i:5;s:4:\"code\";i:6;s:10:\"codemirror\";i:7;s:10:\"codesample\";i:8;s:11:\"colorpicker\";i:9;s:11:\"contextmenu\";i:10;s:8:\"compat3x\";i:11;s:14:\"directionality\";i:12;s:9:\"emoticons\";i:13;s:11:\"fontawesome\";i:14;s:10:\"fullscreen\";i:15;s:2:\"hr\";i:16;s:5:\"image\";i:17;s:10:\"imagetools\";i:18;s:14:\"insertdatetime\";i:19;s:4:\"link\";i:20;s:5:\"lists\";i:21;s:5:\"media\";i:22;s:11:\"nonbreaking\";i:23;s:11:\"noneditable\";i:24;s:9:\"pagebreak\";i:25;s:5:\"paste\";i:26;s:7:\"preview\";i:27;s:5:\"print\";i:28;s:4:\"save\";i:29;s:13:\"searchreplace\";i:30;s:5:\"table\";i:31;s:8:\"template\";i:32;s:9:\"textcolor\";i:33;s:11:\"textpattern\";i:34;s:12:\"visualblocks\";i:35;s:11:\"visualchars\";i:36;s:9:\"wordcount\";}', NOW(), 0),
            ('tinymce4-full', 'array_toolbar1', 'a:22:{i:0;s:4:\"undo\";i:1;s:4:\"redo\";i:2;s:1:\"|\";i:3;s:11:\"styleselect\";i:4;s:9:\"forecolor\";i:5;s:9:\"backcolor\";i:6;s:1:\"|\";i:7;s:6:\"insert\";i:8;s:1:\"|\";i:9;s:4:\"link\";i:10;s:6:\"unlink\";i:11;s:6:\"anchor\";i:12;s:5:\"image\";i:13;s:5:\"media\";i:14;s:5:\"table\";i:15;s:1:\"|\";i:16;s:7:\"charmap\";i:17;s:11:\"fontawesome\";i:18;s:1:\"|\";i:19;s:12:\"visualblocks\";i:20;s:1:\"|\";i:21;s:10:\"codesample\";}', NOW(), 0),
            ('tinymce4-full', 'array_toolbar2', 'a:22:{i:0;s:4:\"bold\";i:1;s:6:\"italic\";i:2;s:9:\"underline\";i:3;s:13:\"strikethrough\";i:4;s:9:\"subscript\";i:5;s:11:\"superscript\";i:6;s:1:\"|\";i:7;s:12:\"removeformat\";i:8;s:1:\"|\";i:9;s:9:\"alignleft\";i:10;s:11:\"aligncenter\";i:11;s:10:\"alignright\";i:12;s:12:\"alignjustify\";i:13;s:1:\"|\";i:14;s:7:\"bullist\";i:15;s:7:\"numlist\";i:16;s:7:\"outdent\";i:17;s:6:\"indent\";i:18;s:10:\"blockquote\";i:19;s:2:\"hr\";i:20;s:1:\"|\";i:21;s:4:\"code\";}', NOW(), 0),
            ('tinymce4-full', 'bool_config_allow_conditional_comments', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_autoresize_on_init', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_elementpath', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_end_container_on_empty_block', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_fix_list_elements', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_force_hex_style_colors', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_image_advtab', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_image_dimensions', '', NOW(), 0),
            ('tinymce4-full', 'bool_config_importcss_append', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_statusbar', '1', NOW(), 0),
            ('tinymce4-full', 'bool_config_typeahead_urls', '1', NOW(), 0),
            ('tinymce4-full', 'bool_forced_root_block', '1', NOW(), 0),
            ('tinymce4-full', 'str_config_autoresize_max_height', '800', NOW(), 0),
            ('tinymce4-full', 'str_config_autoresize_min_height', '400', NOW(), 0),
            ('tinymce4-full', 'str_config_extended_valid_elements', 'strike,span[class|style|id],i[class],div[class],div,div[id|class|style],script[language|type|src]', NOW(), 0),
            ('tinymce4-full', 'str_config_importcss_groups', '[ {title: \'Image styles\', filter: /^(img)\\./},\n       {title: \'Heading styles\', filter: /^(h1|h2|h3|h4|h5|h6)\\./},\n       {title: \'Table styles\', filter: /^(table|td|tr)\\./},\n       {title: \'List styles\', filter: /^(ul|li)\\./},\n       {title: \'Block styles\', filter: /^(div|p)\\./},\n       {title: \'Link styles\', filter: /^(a)\\./},\n       {title: \'Generic styles\'} ]', NOW(), 0),
            ('tinymce4-full', 'str_config_init_instance_callback', 'function (inst) { \n           inst.execCommand(\'mceAutoResize\'); // please enable the autoresize plugin\n         }', NOW(), 0),
            ('tinymce4-full', 'str_config_schema', 'html5', NOW(), 0),
            ('tinymce4-full', 'str_config_selector', 'textarea.wysiwyg', NOW(), 0),
            ('tinymce4-full', 'str_config_setup', 'function (editor) {\n           editor.on(\'change\', function () {\n               tinymce.triggerSave(); // this function is necessary for SCHLIX CMS - DO NOT REMOVE THIS LINE!\n           });\n       }', NOW(), 0),
            ('tinymce4-full', 'str_config_width', '100%', NOW(), 0),
            ('tinymce4-full', 'str_media_manager', 'schlix', NOW(), 0),
            ('tinymce4-full', 'str_media_manager_image_list', 'schlix', NOW(), 0),
            ('tinymce4-full', 'str_media_manager_link_list', 'schlix', NOW(), 0),
            ('tinymce4-full', 'str_media_manager_media_list', 'schlix', NOW(), 0),
            ('tinymce4-full', 'str_media_permission', 'full', NOW(), 0);");

            //TinyMCE4-simple
            $mydb->query("
            INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES
            ('tinymce4-simple', 'bool_config_menubar', '', NOW(), 0),
            ('tinymce4-simple', 'array_plugins', 'a:17:{i:0;s:7:\"advlist\";i:1;s:8:\"autolink\";i:2;s:5:\"lists\";i:3;s:4:\"link\";i:4;s:5:\"image\";i:5;s:7:\"charmap\";i:6;s:5:\"print\";i:7;s:7:\"preview\";i:8;s:6:\"anchor\";i:9;s:13:\"searchreplace\";i:10;s:12:\"visualblocks\";i:11;s:14:\"insertdatetime\";i:12;s:5:\"media\";i:13;s:5:\"table\";i:14;s:11:\"contextmenu\";i:15;s:5:\"paste\";i:16;s:10:\"codemirror\";}', NOW(), 0),
            ('tinymce4-simple', 'array_toolbar1', 'a:22:{i:0;s:4:\"undo\";i:1;s:4:\"redo\";i:2;s:1:\"|\";i:3;s:6:\"insert\";i:4;s:1:\"|\";i:5;s:11:\"styleselect\";i:6;s:1:\"|\";i:7;s:4:\"bold\";i:8;s:6:\"italic\";i:9;s:1:\"|\";i:10;s:9:\"alignleft\";i:11;s:11:\"aligncenter\";i:12;s:10:\"alignright\";i:13;s:12:\"alignjustify\";i:14;s:1:\"|\";i:15;s:7:\"bullist\";i:16;s:7:\"numlist\";i:17;s:7:\"outdent\";i:18;s:6:\"indent\";i:19;s:1:\"|\";i:20;s:4:\"link\";i:21;s:5:\"image\";}', NOW(), 0),
            ('tinymce4-simple', 'bool_config_allow_conditional_comments', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_autoresize_on_init', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_elementpath', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_end_container_on_empty_block', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_fix_list_elements', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_force_hex_style_colors', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_image_advtab', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_image_dimensions', '', NOW(), 0),
            ('tinymce4-simple', 'bool_config_importcss_append', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_statusbar', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_config_typeahead_urls', '1', NOW(), 0),
            ('tinymce4-simple', 'bool_forced_root_block', '1', NOW(), 0),
            ('tinymce4-simple', 'str_config_autoresize_max_height', '800', NOW(), 0),
            ('tinymce4-simple', 'str_config_autoresize_min_height', '400', NOW(), 0),
            ('tinymce4-simple', 'str_config_extended_valid_elements', 'strike,span[class|style|id],i[class],div[class],div,div[id|class|style],script[language|type|src]', NOW(), 0),
            ('tinymce4-simple', 'str_config_importcss_groups', '[ {title: \'Image styles\', filter: /^(img)\\./},\n       {title: \'Heading styles\', filter: /^(h1|h2|h3|h4|h5|h6)\\./},\n       {title: \'Table styles\', filter: /^(table|td|tr)\\./},\n       {title: \'List styles\', filter: /^(ul|li)\\./},\n       {title: \'Block styles\', filter: /^(div|p)\\./},\n       {title: \'Link styles\', filter: /^(a)\\./},\n       {title: \'Generic styles\'} ]', NOW(), 0),
            ('tinymce4-simple', 'str_config_init_instance_callback', 'function (inst) { \n           inst.execCommand(\'mceAutoResize\'); // please enable the autoresize plugin\n         }', NOW(), 0),
            ('tinymce4-simple', 'str_config_schema', 'html5', NOW(), 0),
            ('tinymce4-simple', 'str_config_selector', 'textarea.wysiwyg', NOW(), 0),
            ('tinymce4-simple', 'str_config_setup', 'function (editor) {\n           editor.on(\'change\', function () {\n               tinymce.triggerSave(); // this function is necessary for SCHLIX CMS - DO NOT REMOVE THIS LINE!\n           });\n       }', NOW(), 0),
            ('tinymce4-simple', 'str_config_width', '100%', NOW(), 0),
            ('tinymce4-simple', 'str_media_manager', 'schlix', NOW(), 0),
            ('tinymce4-simple', 'str_media_manager_image_list', 'schlix', NOW(), 0),
            ('tinymce4-simple', 'str_media_manager_link_list', 'schlix', NOW(), 0),
            ('tinymce4-simple', 'str_media_manager_media_list', 'schlix', NOW(), 0),
            ('tinymce4-simple', 'str_media_permission', 'limited', NOW(), 0);");
            // TinyMCE4-limited
            $mydb->query("INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES            
            ('tinymce4-limited', 'bool_config_menubar', '', NOW(), 0),
            ('tinymce4-limited', 'array_plugins', 'a:16:{i:0;s:7:\"advlist\";i:1;s:8:\"autolink\";i:2;s:5:\"lists\";i:3;s:4:\"link\";i:4;s:5:\"image\";i:5;s:7:\"charmap\";i:6;s:5:\"print\";i:7;s:7:\"preview\";i:8;s:6:\"anchor\";i:9;s:13:\"searchreplace\";i:10;s:12:\"visualblocks\";i:11;s:14:\"insertdatetime\";i:12;s:5:\"table\";i:13;s:11:\"contextmenu\";i:14;s:5:\"paste\";i:15;s:10:\"codemirror\";}', NOW(), 0),
            ('tinymce4-limited', 'array_toolbar1', 'a:22:{i:0;s:4:\"undo\";i:1;s:4:\"redo\";i:2;s:1:\"|\";i:3;s:6:\"insert\";i:4;s:1:\"|\";i:5;s:11:\"styleselect\";i:6;s:1:\"|\";i:7;s:4:\"bold\";i:8;s:6:\"italic\";i:9;s:1:\"|\";i:10;s:9:\"alignleft\";i:11;s:11:\"aligncenter\";i:12;s:10:\"alignright\";i:13;s:12:\"alignjustify\";i:14;s:1:\"|\";i:15;s:7:\"bullist\";i:16;s:7:\"numlist\";i:17;s:7:\"outdent\";i:18;s:6:\"indent\";i:19;s:1:\"|\";i:20;s:4:\"link\";i:21;s:5:\"image\";}', NOW(), 0),
            ('tinymce4-limited', 'bool_config_allow_conditional_comments', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_autoresize_on_init', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_elementpath', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_end_container_on_empty_block', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_fix_list_elements', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_force_hex_style_colors', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_image_advtab', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_image_dimensions', '', NOW(), 0),
            ('tinymce4-limited', 'bool_config_importcss_append', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_statusbar', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_config_typeahead_urls', '1', NOW(), 0),
            ('tinymce4-limited', 'bool_forced_root_block', '1', NOW(), 0),
            ('tinymce4-limited', 'str_config_autoresize_max_height', '800', NOW(), 0),
            ('tinymce4-limited', 'str_config_autoresize_min_height', '400', NOW(), 0),
            ('tinymce4-limited', 'str_config_extended_valid_elements', 'strike,span[class|style|id],i[class],div[class],div,div[id|class|style],script[language|type|src]', NOW(), 0),
            ('tinymce4-limited', 'str_config_init_instance_callback', 'function (inst) { \n           inst.execCommand(\'mceAutoResize\'); // please enable the autoresize plugin\n         }', NOW(), 0),
            ('tinymce4-limited', 'str_config_schema', 'html5', NOW(), 0),
            ('tinymce4-limited', 'str_config_selector', 'textarea.wysiwyg', NOW(), 0),
            ('tinymce4-limited', 'str_config_setup', 'function (editor) {\n           editor.on(\'change\', function () {\n               tinymce.triggerSave(); // this function is necessary for SCHLIX CMS - DO NOT REMOVE THIS LINE!\n           });\n       }', NOW(), 0),
            ('tinymce4-limited', 'str_config_width', '100%', NOW(), 0);");

            // CKEditor4-full
            $mydb->query("INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES
            ('ckeditor4-full', 'array_plugins', 'a:53:{i:0;s:11:\"basicstyles\";i:1;s:10:\"blockquote\";i:2;s:9:\"clipboard\";i:3;s:11:\"codesnippet\";i:4;s:11:\"colorbutton\";i:5;s:11:\"colordialog\";i:6;s:11:\"contextmenu\";i:7;s:14:\"copyformatting\";i:8;s:12:\"dialogadvtab\";i:9;s:3:\"div\";i:10;s:12:\"elementspath\";i:11;s:8:\"enterkey\";i:12;s:8:\"entities\";i:13;s:11:\"filebrowser\";i:14;s:4:\"find\";i:15;s:13:\"floatingspace\";i:16;s:4:\"font\";i:17;s:6:\"format\";i:18;s:14:\"horizontalrule\";i:19;s:10:\"htmlwriter\";i:20;s:6:\"iframe\";i:21;s:12:\"iframedialog\";i:22;s:5:\"image\";i:23;s:11:\"indentblock\";i:24;s:10:\"indentlist\";i:25;s:7:\"justify\";i:26;s:8:\"language\";i:27;s:4:\"link\";i:28;s:4:\"list\";i:29;s:9:\"liststyle\";i:30;s:9:\"magicline\";i:31;s:8:\"maximize\";i:32;s:8:\"onchange\";i:33;s:9:\"pagebreak\";i:34;s:13:\"pastefromword\";i:35;s:9:\"pastetext\";i:36;s:7:\"preview\";i:37;s:12:\"removeformat\";i:38;s:6:\"resize\";i:39;s:4:\"save\";i:40;s:9:\"selectall\";i:41;s:10:\"showblocks\";i:42;s:11:\"showborders\";i:43;s:10:\"sourcearea\";i:44;s:11:\"specialchar\";i:45;s:11:\"stylescombo\";i:46;s:3:\"tab\";i:47;s:5:\"table\";i:48;s:10:\"tabletools\";i:49;s:7:\"toolbar\";i:50;s:4:\"undo\";i:51;s:6:\"widget\";i:52;s:11:\"wysiwygarea\";}', NOW(), 0),
            ('ckeditor4-full', 'array_extra_plugins', 'a:3:{i:0;s:10:\"codemirror\";i:1;s:11:\"fontawesome\";i:2;s:16:\"schlixcmsbrowser\";}', NOW(), 0),
            ('ckeditor4-full', 'array_toolbar1', 'a:25:{i:0;s:3:\"Cut\";i:1;s:4:\"Copy\";i:2;s:5:\"Paste\";i:3;s:9:\"PasteText\";i:4;s:13:\"PasteFromWord\";i:5;s:1:\"|\";i:6;s:4:\"Find\";i:7;s:7:\"Replace\";i:8;s:4:\"Undo\";i:9;s:4:\"Redo\";i:10;s:1:\"|\";i:11;s:6:\"Format\";i:12;s:6:\"Styles\";i:13;s:9:\"TextColor\";i:14;s:7:\"BGColor\";i:15;s:1:\"|\";i:16;s:9:\"CreateDiv\";i:17;s:5:\"Table\";i:18;s:1:\"|\";i:19;s:11:\"SpecialChar\";i:20;s:11:\"FontAwesome\";i:21;s:1:\"|\";i:22;s:10:\"ShowBlocks\";i:23;s:1:\"|\";i:24;s:11:\"CodeSnippet\";}', NOW(), 0),
            ('ckeditor4-full', 'array_toolbar2', 'a:28:{i:0;s:4:\"Bold\";i:1;s:6:\"Italic\";i:2;s:9:\"Underline\";i:3;s:6:\"Strike\";i:4;s:9:\"Subscript\";i:5;s:11:\"Superscript\";i:6;s:1:\"|\";i:7;s:12:\"RemoveFormat\";i:8;s:1:\"|\";i:9;s:11:\"JustifyLeft\";i:10;s:13:\"JustifyCenter\";i:11;s:12:\"JustifyRight\";i:12;s:12:\"JustifyBlock\";i:13;s:1:\"|\";i:14;s:12:\"NumberedList\";i:15;s:12:\"BulletedList\";i:16;s:1:\"|\";i:17;s:7:\"Outdent\";i:18;s:6:\"Indent\";i:19;s:10:\"Blockquote\";i:20;s:14:\"HorizontalRule\";i:21;s:1:\"|\";i:22;s:4:\"Link\";i:23;s:6:\"Unlink\";i:24;s:6:\"Anchor\";i:25;s:5:\"Image\";i:26;s:1:\"|\";i:27;s:6:\"Source\";}', NOW(), 0),
            ('ckeditor4-full', 'str_media_permission', 'full', NOW(), 0),
            ('ckeditor4-full', 'str_file_browser', 'schlix', NOW(), 0),
            ('ckeditor4-full', 'str_image_browser', 'schlix', NOW(), 0),
            ('ckeditor4-full', 'str_donot_remove_these_elements', 'strike,span,b,i,div,script', NOW(), 0),
            ('ckeditor4-full', 'str_config_filebrowserWindowHeight', '500', NOW(), 0),
            ('ckeditor4-full', 'str_config_filebrowserWindowWidth', '800', NOW(), 0),
            ('ckeditor4-full', 'str_config_autoGrow_bottomSpace', '', NOW(), 0),
            ('ckeditor4-full', 'str_config_autoGrow_maxHeight', '800', NOW(), 0),
            ('ckeditor4-full', 'str_config_autoGrow_minHeight', '400', NOW(), 0),
            ('ckeditor4-full', 'bool_config_resizeEnabled', '1', NOW(), 0),
            ('ckeditor4-full', 'str_config_height', '200', NOW(), 0),
            ('ckeditor4-full', 'str_config_width', 'auto', NOW(), 0),
            ('ckeditor4-full', 'str_config_removeDialogTabs', 'link:upload;image:upload', NOW(), 0),
            ('ckeditor4-full', 'bool_config_disableNativeTableHandles', '1', NOW(), 0),
            ('ckeditor4-full', 'bool_config_autoGrow_onStartup', '1', NOW(), 0),
            ('ckeditor4-full', 'bool_config_toolbarStartupExpanded', '1', NOW(), 0),
            ('ckeditor4-full', 'bool_config_toolbarCanCollapse', '', NOW(), 0),
            ('ckeditor4-full', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('ckeditor4-full', 'str_schlix_function_init', '// please keep this function so you can save with Apply button\nfor (var instance in CKEDITOR.instances )\n{  \n    CKEDITOR.instances[instance].on( \'change\', SCHLIX.CMS.WysiwygHelper.fixAutoUpdateForm);\n}', NOW(), 0);");

            // CKEditor4-simple
            $mydb->query("INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES
            ('ckeditor4-simple', 'array_plugins', 'a:52:{i:0;s:11:\"basicstyles\";i:1;s:10:\"blockquote\";i:2;s:9:\"clipboard\";i:3;s:11:\"colorbutton\";i:4;s:11:\"colordialog\";i:5;s:11:\"contextmenu\";i:6;s:14:\"copyformatting\";i:7;s:12:\"dialogadvtab\";i:8;s:3:\"div\";i:9;s:12:\"elementspath\";i:10;s:8:\"enterkey\";i:11;s:8:\"entities\";i:12;s:11:\"filebrowser\";i:13;s:4:\"find\";i:14;s:13:\"floatingspace\";i:15;s:4:\"font\";i:16;s:6:\"format\";i:17;s:14:\"horizontalrule\";i:18;s:10:\"htmlwriter\";i:19;s:6:\"iframe\";i:20;s:12:\"iframedialog\";i:21;s:5:\"image\";i:22;s:11:\"indentblock\";i:23;s:10:\"indentlist\";i:24;s:7:\"justify\";i:25;s:8:\"language\";i:26;s:4:\"link\";i:27;s:4:\"list\";i:28;s:9:\"liststyle\";i:29;s:9:\"magicline\";i:30;s:8:\"maximize\";i:31;s:8:\"onchange\";i:32;s:9:\"pagebreak\";i:33;s:13:\"pastefromword\";i:34;s:9:\"pastetext\";i:35;s:7:\"preview\";i:36;s:12:\"removeformat\";i:37;s:6:\"resize\";i:38;s:4:\"save\";i:39;s:9:\"selectall\";i:40;s:10:\"showblocks\";i:41;s:11:\"showborders\";i:42;s:10:\"sourcearea\";i:43;s:11:\"specialchar\";i:44;s:11:\"stylescombo\";i:45;s:3:\"tab\";i:46;s:5:\"table\";i:47;s:10:\"tabletools\";i:48;s:7:\"toolbar\";i:49;s:4:\"undo\";i:50;s:6:\"widget\";i:51;s:11:\"wysiwygarea\";}', NOW(), 0),
            ('ckeditor4-simple', 'array_extra_plugins', 'a:2:{i:0;s:10:\"codemirror\";i:1;s:16:\"schlixcmsbrowser\";}', NOW(), 0),
            ('ckeditor4-simple', 'array_toolbar1', 'a:26:{i:0;s:4:\"Bold\";i:1;s:6:\"Italic\";i:2;s:9:\"Underline\";i:3;s:6:\"Strike\";i:4;s:9:\"Subscript\";i:5;s:11:\"Superscript\";i:6;s:1:\"|\";i:7;s:12:\"RemoveFormat\";i:8;s:1:\"|\";i:9;s:11:\"JustifyLeft\";i:10;s:13:\"JustifyCenter\";i:11;s:12:\"JustifyRight\";i:12;s:12:\"JustifyBlock\";i:13;s:1:\"|\";i:14;s:12:\"NumberedList\";i:15;s:12:\"BulletedList\";i:16;s:1:\"|\";i:17;s:7:\"Outdent\";i:18;s:6:\"Indent\";i:19;s:10:\"Blockquote\";i:20;s:14:\"HorizontalRule\";i:21;s:1:\"|\";i:22;s:4:\"Link\";i:23;s:6:\"Unlink\";i:24;s:6:\"Anchor\";i:25;s:5:\"Image\";}', NOW(), 0),
            ('ckeditor4-simple', 'array_toolbar2', 'a:0:{}', NOW(), 0),
            ('ckeditor4-simple', 'array_toolbar3', 'a:0:{}', NOW(), 0),
            ('ckeditor4-simple', 'array_toolbar4', 'a:0:{}', NOW(), 0),
            ('ckeditor4-simple', 'str_media_permission', 'limited', NOW(), 0),
            ('ckeditor4-simple', 'str_file_browser', 'schlix', NOW(), 0),
            ('ckeditor4-simple', 'str_image_browser', 'schlix', NOW(), 0),
            ('ckeditor4-simple', 'str_donot_remove_these_elements', 'strike,span,b,i,div', NOW(), 0),
            ('ckeditor4-simple', 'str_config_filebrowserWindowHeight', '500', NOW(), 0),
            ('ckeditor4-simple', 'str_config_filebrowserWindowWidth', '800', NOW(), 0),
            ('ckeditor4-simple', 'str_config_autoGrow_bottomSpace', '', NOW(), 0),
            ('ckeditor4-simple', 'str_config_autoGrow_maxHeight', '800', NOW(), 0),
            ('ckeditor4-simple', 'str_config_autoGrow_minHeight', '400', NOW(), 0),
            ('ckeditor4-simple', 'bool_config_resizeEnabled', '1', NOW(), 0),
            ('ckeditor4-simple', 'str_config_height', '200', NOW(), 0),
            ('ckeditor4-simple', 'str_config_width', 'auto', NOW(), 0),
            ('ckeditor4-simple', 'str_config_removeDialogTabs', 'link:upload;image:upload', NOW(), 0),
            ('ckeditor4-simple', 'bool_config_disableNativeTableHandles', '1', NOW(), 0),
            ('ckeditor4-simple', 'bool_config_autoGrow_onStartup', '1', NOW(), 0),
            ('ckeditor4-simple', 'bool_config_toolbarStartupExpanded', '1', NOW(), 0),
            ('ckeditor4-simple', 'bool_config_toolbarCanCollapse', '', NOW(), 0),
            ('ckeditor4-simple', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('ckeditor4-simple', 'str_schlix_function_init', '// please keep this function so you can save with Apply button\nfor (var instance in CKEDITOR.instances )\n{  \n    CKEDITOR.instances[instance].on( \'change\', SCHLIX.CMS.WysiwygHelper.fixAutoUpdateForm);\n}', NOW(), 0);");
            // CKEditor4-limited
            $mydb->query("INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES            
            ('ckeditor4-limited', 'array_plugins', 'a:50:{i:0;s:11:\"basicstyles\";i:1;s:10:\"blockquote\";i:2;s:9:\"clipboard\";i:3;s:11:\"colorbutton\";i:4;s:11:\"colordialog\";i:5;s:11:\"contextmenu\";i:6;s:14:\"copyformatting\";i:7;s:12:\"dialogadvtab\";i:8;s:3:\"div\";i:9;s:12:\"elementspath\";i:10;s:8:\"enterkey\";i:11;s:8:\"entities\";i:12;s:11:\"filebrowser\";i:13;s:4:\"find\";i:14;s:13:\"floatingspace\";i:15;s:4:\"font\";i:16;s:6:\"format\";i:17;s:14:\"horizontalrule\";i:18;s:10:\"htmlwriter\";i:19;s:6:\"iframe\";i:20;s:12:\"iframedialog\";i:21;s:11:\"indentblock\";i:22;s:10:\"indentlist\";i:23;s:7:\"justify\";i:24;s:8:\"language\";i:25;s:4:\"list\";i:26;s:9:\"liststyle\";i:27;s:9:\"magicline\";i:28;s:8:\"maximize\";i:29;s:8:\"onchange\";i:30;s:9:\"pagebreak\";i:31;s:13:\"pastefromword\";i:32;s:9:\"pastetext\";i:33;s:7:\"preview\";i:34;s:12:\"removeformat\";i:35;s:6:\"resize\";i:36;s:4:\"save\";i:37;s:9:\"selectall\";i:38;s:10:\"showblocks\";i:39;s:11:\"showborders\";i:40;s:10:\"sourcearea\";i:41;s:11:\"specialchar\";i:42;s:11:\"stylescombo\";i:43;s:3:\"tab\";i:44;s:5:\"table\";i:45;s:10:\"tabletools\";i:46;s:7:\"toolbar\";i:47;s:4:\"undo\";i:48;s:6:\"widget\";i:49;s:11:\"wysiwygarea\";}', NOW(), 0),
            ('ckeditor4-limited', 'array_extra_plugins', 'a:2:{i:0;s:11:\"fontawesome\";i:1;s:16:\"schlixcmsbrowser\";}', NOW(), 0),
            ('ckeditor4-limited', 'array_toolbar1', 'a:23:{i:0;s:4:\"Bold\";i:1;s:6:\"Italic\";i:2;s:9:\"Underline\";i:3;s:6:\"Strike\";i:4;s:9:\"Subscript\";i:5;s:11:\"Superscript\";i:6;s:1:\"|\";i:7;s:12:\"RemoveFormat\";i:8;s:1:\"|\";i:9;s:11:\"JustifyLeft\";i:10;s:13:\"JustifyCenter\";i:11;s:12:\"JustifyRight\";i:12;s:12:\"JustifyBlock\";i:13;s:1:\"|\";i:14;s:12:\"NumberedList\";i:15;s:12:\"BulletedList\";i:16;s:1:\"|\";i:17;s:7:\"Outdent\";i:18;s:6:\"Indent\";i:19;s:1:\"|\";i:20;s:6:\"Anchor\";i:21;s:1:\"|\";i:22;s:6:\"Source\";}', NOW(), 0),
            ('ckeditor4-limited', 'array_toolbar2', 'a:0:{}', NOW(), 0),
            ('ckeditor4-limited', 'array_toolbar3', 'a:0:{}', NOW(), 0),
            ('ckeditor4-limited', 'array_toolbar4', 'a:0:{}', NOW(), 0),
            ('ckeditor4-limited', 'str_media_permission', '', NOW(), 0),
            ('ckeditor4-limited', 'str_file_browser', 'none', NOW(), 0),
            ('ckeditor4-limited', 'str_image_browser', 'none', NOW(), 0),
            ('ckeditor4-limited', 'str_config_filebrowserWindowHeight', '500', NOW(), 0),
            ('ckeditor4-limited', 'str_config_filebrowserWindowWidth', '800', NOW(), 0),
            ('ckeditor4-limited', 'str_config_autoGrow_bottomSpace', '', NOW(), 0),
            ('ckeditor4-limited', 'str_config_autoGrow_maxHeight', '800', NOW(), 0),
            ('ckeditor4-limited', 'str_config_autoGrow_minHeight', '400', NOW(), 0),
            ('ckeditor4-limited', 'bool_config_resizeEnabled', '1', NOW(), 0),
            ('ckeditor4-limited', 'str_config_height', '200', NOW(), 0),
            ('ckeditor4-limited', 'str_config_width', 'auto', NOW(), 0),
            ('ckeditor4-limited', 'str_config_removeDialogTabs', 'link:upload;image:upload', NOW(), 0),
            ('ckeditor4-limited', 'bool_config_disableNativeTableHandles', '1', NOW(), 0),
            ('ckeditor4-limited', 'bool_config_autoGrow_onStartup', '1', NOW(), 0),
            ('ckeditor4-limited', 'bool_config_toolbarStartupExpanded', '1', NOW(), 0),
            ('ckeditor4-limited', 'bool_config_toolbarCanCollapse', '', NOW(), 0),
            ('ckeditor4-limited', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('ckeditor4-limited', 'str_schlix_function_init', '// please keep this function so you can save with Apply button\nfor (var instance in CKEDITOR.instances )\n{  \n    CKEDITOR.instances[instance].on( \'change\', SCHLIX.CMS.WysiwygHelper.fixAutoUpdateForm);\n}', NOW(), 0);");

            // CodeMirror
            $mydb->query("
            INSERT INTO `gk_wysiwyg_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES
            ('codemirror-php', 'bool_config_autoCloseBrackets', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_autoCloseTags', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_autoFormatOnStart', '', NOW(), 0),
            ('codemirror-php', 'bool_config_autoFormatOnUncomment', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_continueComments', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_enableCodeFolding', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_enableCodeFormatting', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_enableSearchTools', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_highlightMatches', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_indentWithTabs', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_lineNumbers', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_lineWrapping', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_matchBrackets', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_matchTags', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_showAutoCompleteButton', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_showSearchButton', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_showTrailingSpace', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_showUncommentButton', '1', NOW(), 0),
            ('codemirror-php', 'bool_config_styleActiveLine', '1', NOW(), 0),
            ('codemirror-php', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('codemirror-php', 'str_config_theme', 'mbo', NOW(), 0),
            ('codemirror-php', 'str_config_mode', 'php', NOW(), 0),
            ('codemirror-html', 'bool_config_autoCloseBrackets', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_autoCloseTags', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_autoFormatOnStart', '', NOW(), 0),
            ('codemirror-html', 'bool_config_autoFormatOnUncomment', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_continueComments', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_enableCodeFolding', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_enableCodeFormatting', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_enableSearchTools', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_highlightMatches', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_indentWithTabs', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_lineNumbers', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_lineWrapping', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_matchBrackets', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_matchTags', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_showAutoCompleteButton', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_showSearchButton', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_showTrailingSpace', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_showUncommentButton', '1', NOW(), 0),
            ('codemirror-html', 'bool_config_styleActiveLine', '1', NOW(), 0),
            ('codemirror-html', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('codemirror-html', 'str_config_theme', 'mbo', NOW(), 0),
            ('codemirror-html', 'str_config_mode', 'htmlmixed', NOW(), 0),
            ('codemirror-js', 'bool_config_autoCloseBrackets', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_autoCloseTags', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_autoFormatOnStart', '', NOW(), 0),
            ('codemirror-js', 'bool_config_autoFormatOnUncomment', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_continueComments', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_enableCodeFolding', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_enableCodeFormatting', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_enableSearchTools', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_highlightMatches', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_indentWithTabs', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_lineNumbers', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_lineWrapping', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_matchBrackets', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_matchTags', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_showAutoCompleteButton', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_showSearchButton', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_showTrailingSpace', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_showUncommentButton', '1', NOW(), 0),
            ('codemirror-js', 'bool_config_styleActiveLine', '1', NOW(), 0),
            ('codemirror-js', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('codemirror-js', 'str_config_theme', 'mbo', NOW(), 0),
            ('codemirror-js', 'str_config_mode', 'js', NOW(), 0),
            ('codemirror-css', 'bool_config_autoCloseBrackets', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_autoCloseTags', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_autoFormatOnStart', '', NOW(), 0),
            ('codemirror-css', 'bool_config_autoFormatOnUncomment', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_continueComments', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_enableCodeFolding', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_enableCodeFormatting', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_enableSearchTools', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_highlightMatches', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_indentWithTabs', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_lineNumbers', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_lineWrapping', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_matchBrackets', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_matchTags', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_showAutoCompleteButton', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_showSearchButton', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_showTrailingSpace', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_showUncommentButton', '1', NOW(), 0),
            ('codemirror-css', 'bool_config_styleActiveLine', '1', NOW(), 0),
            ('codemirror-css', 'str_config_schlix_selector', 'wysiwyg', NOW(), 0),
            ('codemirror-css', 'str_config_theme', 'mbo', NOW(), 0),
            ('codemirror-css', 'str_config_mode', 'css', NOW(), 0);");
            
        
    }
    // v2.1.1-4 - set default WYSIWYG
    
    $result = $mydb->getQueryResultSingleRow("SELECT * FROM `gk_config` WHERE `section` = 'wysiwygeditors' AND `key` = 'int_default_full_privilege_wysiwyg_profile_id'");
    if (!$result)
    {
        $mydb->query("INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('wysiwygeditors', 'int_default_full_privilege_wysiwyg_profile_id', '2', NOW(), 1)");
    }
    
    // v2.1.3
    if (!$mydb->tableColumnExists('gk_email_queue_items', 'hourly_marker'))
    {
        $mydb->query('ALTER TABLE `gk_email_queue_items` ADD `hourly_marker` VARCHAR(13) NOT NULL AFTER `queue_marker`; ');
        $mydb->query('ALTER TABLE `gk_email_queue_items` ADD INDEX `status_hourly_marker` (`status`, `hourly_marker`)');
        $mydb->query('ALTER TABLE `gk_email_queue_items` ADD INDEX `hourly_marker` ( `hourly_marker`)');
    }
    
    // v2.1.6
    // Fix upgrade error from v2.1.0.x Tag
    $result = $mydb->getQueryResultArray("SELECT * FROM `gk_app_items` WHERE `title` = 'tag'");
    if (is_array($result) && count($result) > 1)
    {        
        $preserve_id = $result[0]['id'];
        $mydb->query("DELETE FROM `gk_app_items` WHERE `title` ='tag' AND id <> {$preserve_id}");
    } 
    ///////////////////////////////////
    set_db_to_current_version($mydb);
    $messages[] = ___('Upgrade completed');
    
    
    // Always delete table cache columns
    $mydb->query("DELETE FROM `gk_cache_items` WHERE cache_key = 'table_columns' OR cache_key = 'table_pks'");
    
    return array('messages' => $messages, 'errors' => $errors);
}
/**
 * Update version
 * @param \SCHLIX\cmsDatabase $mydb
 * @ignore
 */
function set_db_to_current_version($mydb)
{
    $schlix_version = sanitize_string(SCHLIX_VERSION);
    $sql = "SELECT * FROM `gk_config` WHERE `section` = 'system' AND `key` = 'str_database_version'";
    $result = $mydb->getQueryResultSingleRow($sql);
    if ($result)
    {
        $sql = "UPDATE `gk_config` SET `value` =  {$schlix_version} WHERE  `section` = 'system' AND `key` = 'str_database_version'";
    } else
    {
        $sql = "INSERT INTO `gk_config` (`section`, `key`, `value`, `date_modified`, `modified_by_id`) VALUES ('system', 'str_database_version', {$schlix_version}, NOW(), 0)";
    }
    $mydb->query($sql);
}