<?php
/* 
 * Copyright (C) 2016 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.txt for details
 */

/**
 * SCHLIX Version definition
 */
define ('SCHLIX_VERSION','2.1.8-7');
 
if (version_compare(PHP_VERSION, '5.5.0') < 0)
    die('PHP v5.5.0+ required. This system is running on PHP v'.PHP_VERSION.'. If this is a CPanel-based server, change it in MultiPHP');
/**
 * Initializes SCHLIX runtime
 */

function setupSchlixRuntime()
{
    global $CurrentWebSiteThemeName;

    define('SCHLIX_ROOT_PATH', str_replace('\\', '/', dirname(__FILE__)));    
    require_once (SCHLIX_ROOT_PATH.'/system/libs/schlix/translation.inc.php');
    ini_set('pcre.jit', false); // May 1, 2017 - fix for PHP7 PREG_JIT_STACKLIMIT_ERROR
    set_error_handler("_schlix_main_error_handler");
    set_exception_handler("_schlix_exception_handler");
    register_shutdown_function('schlix_shutdown_handler');
    spl_autoload_register('schlix_autoloader');
    define('SCHLIX_SERVER_ROOT_PATH',str_replace('\\', '/', realpath($_SERVER['DOCUMENT_ROOT'])));    
    define('SCHLIX_SYSTEM_PATH', SCHLIX_ROOT_PATH.'/system');
    define('SCHLIX_SYSTEM_URL_PATH_NO_BASE', '/system');
    // 2. Initialize Multisit
    if (!file_exists(SCHLIX_ROOT_PATH.'/multisite-config.inc.php'))
        die('Multisite configuration file cannot be found. Please <a href="install">install</a> the CMS first.');
    require (SCHLIX_ROOT_PATH.'/multisite-config.inc.php');
    
    // 2a. You can force SITE_HTTPBASE in multisite-config.inc.php
    // Jan 17, 2017 - fix for ISPConfig since it uses smylink
    // Jan 19, 2017 - fix for Odin automation with custom config
    if (file_exists(SCHLIX_ROOT_PATH.'/custom-config.inc.php'))
    {
        include (SCHLIX_ROOT_PATH.'/custom-config.inc.php');
    }
    if (!defined('SCHLIX_SITE_HTTPBASE'))
    {
        $site_httpbase = '';
        if (strlen(SCHLIX_SERVER_ROOT_PATH) != strlen (SCHLIX_ROOT_PATH))
        {
            $s = strpos(SCHLIX_ROOT_PATH, SCHLIX_SERVER_ROOT_PATH);
            if (($s == 0) && ($s !== false)) {
                $prefix_length = strlen(SCHLIX_SERVER_ROOT_PATH);
                $site_httpbase = substr(SCHLIX_ROOT_PATH, $prefix_length, strlen(SCHLIX_ROOT_PATH) - $prefix_length);
            }
            else 
            {
                //The only time that this custom config is generated automatically
                //is when the site is visited and no custom-config.inc.php found
                //The visit must be SCHLIX CMS root URL without any other parameters
                if ($_SERVER['REQUEST_URI'].'index.php' === $_SERVER['SCRIPT_NAME'])
                {
                    $site_httpbase = rtrim($_SERVER['REQUEST_URI'],'/');
                    $custom_config_content = "<?php define ('SCHLIX_SITE_HTTPBASE','{$site_httpbase}');\n";
                    $result = file_put_contents(SCHLIX_ROOT_PATH.'/custom-config.inc.php', $custom_config_content);
                    if ($result === FALSE)
                        die ('Error while trying to define SCHLIX_SITE_HTTPBASE automatically and write to /custom-config.inc.php');
                } else
                die ('Could not determine SCHLIX_SITE_HTTPBASE automatically since SCHLIX seems to be installed outside of server document root. Please define SCHLIX_SITE_HTTPBASE manually in custom-config.inc.php and change the line in .htaccess that contains index.php to /{SCHLIX_SITE_HTTPBASE}/index.php');
            }
        }
        define('SCHLIX_SITE_HTTPBASE', $site_httpbase); // auto path (but it's slower ... change this when you're in production mode    
    }
    define('SCHLIX_SYSTEM_URL_PATH', SCHLIX_SITE_HTTPBASE.'/system');
    
    // 3. Initialize Current Subsite 
    verifySchlixSubsites();    
    define('CURRENT_SUBSITE_PATH', SCHLIX_ROOT_PATH.'/web/'.CURRENT_SUBSITE_NAME);
    define('SCHLIX_FS_CACHE_FOLDER', CURRENT_SUBSITE_PATH.'/cache');


    if (!file_exists(CURRENT_SUBSITE_PATH.'/config.inc.php'))
      die('Configuration file for ['.CURRENT_SUBSITE_NAME.'] cannot be found. Please <a href="install/">re-install</a> or configure config.inc.php for subsite: ');
    
    
    define('CURRENT_SUBSITE_URL_PATH', SCHLIX_SITE_HTTPBASE.'/web/'.CURRENT_SUBSITE_NAME);
    define('CURRENT_SUBSITE_URL_PATH_NO_BASE', '/web/'.CURRENT_SUBSITE_NAME);
    
    require (CURRENT_SUBSITE_PATH.'/config.inc.php');
    //. Begin the rest 
    require (SCHLIX_ROOT_PATH.'/system/libs/schlix/definitions.inc.php');
    //. Load language
    if (SCHLIX_DEFAULT_LANGUAGE !== 'en_us')
    {
        $lang_file = SCHLIX_ROOT_PATH.'/system/languages/'. SCHLIX_DEFAULT_LANGUAGE . '.lang.php';
        if (file_exists($lang_file)) require $lang_file;
    }
    // end load language

    require (SCHLIX_ROOT_PATH.'/system/libs/schlix/app_interface.php');
    require_once(SCHLIX_ROOT_PATH.'/system/libs/schlix/app_basic.class.php');
    //***** INTERNAL *****
    require_once (SCHLIX_ROOT_PATH.'/system/libs/schlix/html.inc.php');
    require_once (SCHLIX_ROOT_PATH.'/system/libs/schlix/schlix_util.inc.php');
    require_once (SCHLIX_ROOT_PATH.'/system/libs/schlix/util.inc.php');
    require_once (SCHLIX_ROOT_PATH.'/system/libs/schlix/db.inc.php');
    
    //***** END INTERNAL INIT *****
    initializeSchlixSystem();
    $CurrentWebSiteThemeName = \App\Themes::getCurrentFrontendWebsiteTemplateName();
    define ('CURRENT_THEME_PATH', SCHLIX_SITE_PATH . "/themes/{$CurrentWebSiteThemeName}");
    define ('CURRENT_THEME_URL_PATH', SCHLIX_SITE_HTTPBASE.'/web/'.CURRENT_SUBSITE_NAME."/themes/{$CurrentWebSiteThemeName}");
    define ('CURRENT_THEME_URL_PATH_NO_BASE', '/web/'.CURRENT_SUBSITE_NAME."/themes/{$CurrentWebSiteThemeName}");
}

function initializeSchlixSystem()
{
    
    global $CurrentPage;
    /**
     * Global view data
     * @var array
     */
    global $ViewData;
    /**
     * Database
     * @var \SCHLIX\cmsDBCache
     */
    global $SystemDB;
    /**
     * Versioning
     * @var \App\versioning
     */
    global $Versioning;
    /**
     * Tag Manager
     * @var \App\Tag
     */    
    global $TagManager;
    /**
     * Current User
     * @var \App\Users
     */    
    global $CurrentUser;
    /**
     * System configuration registry
     * @var \SCHLIX\cmsConfigRegistry
     */        
    global $SystemConfig;
    /**
     * @var \SCHLIX\cmsLogger
     */        
    global $SystemLog;
    /**
     * @var \SCHLIX\cmsSessionHandler
     */            
    global $SchlixSessionHandler;
    // internal - don't access directly
    global $__schlix_table_columns, $__schlix_table_pks;

    $__timezone = (defined('SCHLIX_SYSTEM_TIMEZONE') && SCHLIX_SYSTEM_TIMEZONE != '') ? SCHLIX_SYSTEM_TIMEZONE : 'GMT';
    date_default_timezone_set($__timezone);
    $CurrentPage = filter_input(INPUT_SERVER, 'PHP_SELF', FILTER_SANITIZE_URL);
    $SystemDB	= new Schlix\cmsDatabase(SCHLIX_DB_HOST,SCHLIX_DB_DATABASE, SCHLIX_DB_USERNAME,SCHLIX_DB_PASSWORD);

    $SystemConfig = new Schlix\cmsConfigRegistry('gk_config');
    $SystemLog = new Schlix\cmsLogger('gk_log_items');

    $SchlixSessionHandler = new Schlix\cmsSessionHandler();
    session_name(SCHLIX_SESSION_NAME);
    session_start();

    $SystemDB->query("SET SQL_MODE=''");
    
    // 1. Cache table names
    $result = $SystemDB->getQueryResultArray( "SHOW TABLES");
    $all_tables = array_map('array_values', $result);    
    foreach ($all_tables as $tbl)
        \SCHLIX\cmsContextCache::set('db_table_exists', $tbl[0], 1);
    // 2. Cache application alias
    $sql = "SELECT title, app_alias, app_description FROM gk_app_items";
    $app_info = $SystemDB->getQueryResultArray($sql);
    foreach ($app_info as $info)
    {
        \SCHLIX\cmsContextCache::set('app_alias', $info['title'], $info['app_alias']);
        \SCHLIX\cmsContextCache::set('app_description', $info['title'], $info['app_description']);
    }    
    
    // 3. Cache table columns
    $__schlix_table_columns = SCHLIX\cmsDBCache::get('table_columns');
    $__schlix_table_pks = SCHLIX\cmsDBCache::get('table_pks');
    if ($__schlix_table_columns == null || $__schlix_table_pks == null )
    {
        $__schlix_table_columns = array();
        $__schlix_table_pks = array();
        foreach ($all_tables as $tbl)
        {
            $ticked_table_name = "`{$tbl[0]}`";
            $sql = "SHOW COLUMNS FROM {$ticked_table_name}";
            $result = $SystemDB->getQueryResultArray($sql);
            $__schlix_table_columns[$tbl[0]] = array_column($result,'Field');        
            
            $sql = "SHOW KEYS FROM {$ticked_table_name} WHERE key_name = 'PRIMARY'";
            $result = $SystemDB->getQueryResultArray($sql);
            $__schlix_table_pks[$tbl[0]] = array_column($result,'Column_name');        
        }
        \SCHLIX\cmsDBCache::set('table_columns', $__schlix_table_columns,0,5,0); // set for 5 minutes
        \SCHLIX\cmsDBCache::set('table_pks', $__schlix_table_pks,0,5,0); // set for 5 minutes
        
    }  
    // setup versioning (for later version - frontend editing)
    $Versioning = new App\versioning();
    // setup tagging (for later version - frontend editing)
    $TagManager = new App\Tag();    
}
 
/**
 * Returns true if current request is through SSL, whether it is through a proxy load balancer or not
 * @return boolean
 */
function isCurrentRequestSSL()
{
    return ( (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || 
         (!empty( $_SERVER['HTTP_X_FORWARDED_PROTO'] ) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https' )
       );
}
/**
 * Verify the subsites configuration
 * @global array $SchlixWebsites
 * @global String $CurrentSubSite
 */
function verifySchlixSubsites()
{
    global $SchlixWebsites, $CurrentSubSite;


    $_site_request = strip_tags($_SERVER['HTTP_HOST']);
    $_pport = strpos($_site_request, ':');
    if ($_pport > 0) {
        $_site_request = substr($_site_request, 0, $_pport);
    }
// The HTTP_HOST is obtained from the HTTP request header and this is what the client actually used as "target host" of the request.
    if (array_key_exists($_site_request, $SchlixWebsites)) {
        $CurrentSubSite = $SchlixWebsites[$_site_request];
    } else {
        if (array_key_exists('*', $SchlixWebsites)) {
            $CurrentSubSite = $SchlixWebsites['*'];
        } else {
            // $_SERVER['HTTP_HOST'] has already been validated earlier with $_SERVER['HTTP_HOST'] = strip_tags($_SERVER['HTTP_HOST']);
            $SiteRequestAttempt = (isCurrentRequestSSL() ? 'https' : 'http') . '://' . $_site_request . (($_SERVER['SERVER_PORT'] != 80 && $_SERVER['SERVER_PORT'] != 443) ? ':' . $_SERVER['SERVER_PORT'] : '');
            die('There is no website configured at this address: ' . $SiteRequestAttempt);
        }
    }
    define('CURRENT_SUBSITE_NAME', '' . $CurrentSubSite);
}




//______________________________________________________________________________________//

function _schlix_exception_handler($exc)
{
    echo "Uncaught exception: " , $exc->getMessage().' on '. $exc->getFile().' line '. $exc->getLine(), "\n";
}

/**
 * SCHLIX Error Handler
 * @param type $type
 * @param type $message
 * @param type $file
 * @param type $line
 * @return boolean
 */

function _schlix_main_error_handler($type, $message, $file, $line)
{
    
    $_ERRORS = array(
        E_ERROR => 'Error', // E_ERROR
        E_WARNING => 'Warning', // E_WARNING
        E_PARSE => 'Parse error', // E_PARSE
        E_NOTICE => 'Notice', // E_NOTICE
        E_CORE_ERROR => 'Core error', // E_CORE_ERROR
        E_CORE_WARNING => 'Core warning', // E_CORE_WARNING
        E_COMPILE_ERROR => 'Compile error', // E_COMPILE_ERROR
        E_COMPILE_WARNING => 'Compiler warning', // E_COMPILE_WARNING
        E_USER_ERROR => 'User-defined error', // E_USER_ERROR
        E_USER_WARNING => 'User-defined warning', // E_USER_WARNING
        E_USER_NOTICE => 'User-defined notice', // E_USER_NOTICE
        E_STRICT => 'Strict PHP', // E_STRICT
        E_RECOVERABLE_ERROR => 'Recoverable Error', // E_RECOVERABLE_ERROR
        E_DEPRECATED => 'Deprecated', // E_DEPRECATED
        E_USER_DEPRECATED => 'User-defined deprecation' // E_USER_DEPRECATED
    );

    if ( $type != 1 && (($type === E_NOTICE) || !(error_reporting() & $type))) return;
     
    
    if (!@is_string($name = @array_search($type, @array_flip($_ERRORS)))) {
        $name = 'E_UNKNOWN';
    }
    

    if (isset($_SERVER['HTTP_USER_AGENT']))
    {
        //if (is_ajax_request()) 2017 Sept 12 - In case the error happens before it is initialized
        if ((!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest'))
        $err_format = ajax_reply (500, "%s in [%s] at [line %d]: %s\n");
            else
        $err_format = '<div style="background:pink;padding:3px">%s in <strong>%s</strong> at <strong>line %d</strong>: <span style="color:red">%s</span></div><br />';
    } else
    {
        $err_format = "%s in [%s] at [line %d]: %s\n";
    }
    /*
    $err_format = isset($_SERVER['HTTP_USER_AGENT']) ? 
            '<div style="background:pink;padding:3px">%s in <strong>%s</strong> at <strong>line %d</strong>: <span style="color:red">%s</span></div><br />':
            "%s in [%s] at [line %d]: %s\n";
     
     */
    return(print(@sprintf($err_format, $name, @basename($file), $line, $message)));
}


function schlix_shutdown_handler()
{
    if (is_array($error = error_get_last())) {

        return(call_user_func_array('_schlix_main_error_handler', $error));
    }

    return(TRUE);
}


/**
 * This is the autoloader for Schlix as of June 2013.
 *
 * @param string $class
 */


/**
 * This is the autoloader for Schlix as of June 2013.
 *
 * @param string $class
 */
function schlix_autoloader($class)
{
    
    $valid_schlix_namespaces = array('app','block','macro','wysiwygeditor');
    $possibilities = array (SCHLIX_SITE_PATH, SCHLIX_SYSTEM_PATH);
    $class = ltrim($class, '\\');    
    // if using namespace then it can be an App, Block, or Filter (or Libs)
    if (strpos($class, '\\'))
    {
        $last_ns_pos = strrpos($class, '\\');
        $namespace = substr($class, 0, $last_ns_pos);
        $class_name_only = substr($class, $last_ns_pos + 1);
        
        $lowered_namespace =  strtolower($namespace);
        $main_directory =  $lowered_namespace.'s';
        $desired_file = str_replace(array('\\', '_'),array( '/','.'), strtolower($class_name_only));
        $parts = explode('.', $desired_file);
        $class_directory = $parts[0];
        $alternative_class_file_name = '';
        if (in_array($lowered_namespace, $valid_schlix_namespaces) && is_dir(SCHLIX_SYSTEM_PATH.'/'.$main_directory))
        {
            if ($lowered_namespace === 'app')
            {
                if (strpos($desired_file,'.') !== FALSE)
                {
                    $arr_classes = explode('.',$desired_file);
                    $count_classes = ___c($arr_classes);
                    $valid_count_classes =  ($arr_classes[$count_classes-1] != 'admin')  ? $count_classes : $count_classes - 1;
                    if ($valid_count_classes == 2)
                    {
                        $alternative_class_file_name = "/{$main_directory}/{$class_directory}/{$arr_classes[1]}/{$desired_file}.class.php" ;
                    }
                } 
            }
            $default_class_file_name = "/{$main_directory}/{$class_directory}/{$desired_file}.class.php" ;
            
        } else
        {
            $last_fns_pos = strpos($lowered_namespace, '\\');
            if ($last_fns_pos > 0)
                $lowered_namespace = substr($lowered_namespace, 0, $last_fns_pos);
            $default_class_file_name = str_replace('\\','/',"/libs/{$lowered_namespace}/{$class_name_only}.php");
            $alternative_class_file_name = str_replace('\\','/',"/libs/{$lowered_namespace}/__load__.php");
            
        }
    } else // if there's no namespace
    {
            $class_name_only = strtolower($class);
            $default_class_file_name =  '/libs/' . $class_name_only . '/' . $class_name_only . '.class.php';
    } 
    // Now begin loading
    foreach ($possibilities as $possible_dir)
    {
        $possible_class_file = $possible_dir.$default_class_file_name;
        if ($alternative_class_file_name)
        {
            $possible_alternative_class_file = $possible_dir.$alternative_class_file_name;
            if (is_readable($possible_alternative_class_file)) {
                require_once $possible_alternative_class_file;
                return;
            }
            
        }
        if (is_readable($possible_class_file)) {
            require_once $possible_class_file;
            return;
        }
    }  
    
}
