<?php
namespace App;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
include_once(SCHLIX_ROOT_PATH.'/system/libs/schlix/admin_basic.class.php');
include_once(SCHLIX_ROOT_PATH.'/system/libs/schlix/admin_basic_manager.class.php');

class Admin extends \SCHLIX\cmsApplication_Basic  {
                
    protected $report_folder  = '';
    protected $malicious_files = [];
    protected $fixed_malicious_files = [];
    protected $data_directories = array(
      'syslangpack' => '/data/private/syslangpack',      
      'quarantine' => '/data/private/quarantine',
      'tmp' => '/data/private/syslangpack/tmp',
    );
    
    /**
     * FS Cache
     * @var \SCHLIX\cmsFSCache 
     */
    protected $fs_cache;
    
    public function __construct()
    {
            parent::__construct('Admin');
            $this->fs_cache = new \SCHLIX\cmsFSCache('generic');
            
     }

    //_______________________________________________________________________________________________________________//
    private function __setTitlePage($title)
    {
        global $HTMLHeader;
        $str = ($this->app) ? $this->app->getApplicationDescription() : '';
        $HTMLHeader->ADD(\__Html::TITLE($title.' '.$str . ' ' . ___('Administration') . ' | SCHLIX'));
    }
                
    /**
     * View current active sessions
     * @global \App\Users $CurrentUser
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @return boolean
     */
    protected function viewActiveSessions() {

        global $CurrentUser, $SystemDB;
        $admadm = new Admin_Admin;
        if (!$admadm->CheckIfCurrentUserAllowedAccess())
        {
            $CurrentUser->logout();            
            die (___('Access to the administration is not allowed for your user group'));
        }

        $seconds = 180;
        $sql = "SELECT id, ip_address, user_agent  FROM `gk_session_handler` WHERE TIMESTAMPDIFF(SECOND, `timestamp`,NOW()) < {$seconds}";
        $active_sessions = $SystemDB->getQueryResultArray($sql);
        
        $this->loadTemplateFile('view.sessions', compact(array_keys(get_defined_vars())));
        $this->__setTitlePage(___('Active Sessions'));
        return true;
    }    
    /**
     * Ensure directory exists
     */
    public function ensureDataDirectoryExists()
    {
        $this->initializeDataDirectories();
    }
    
    
    public function viewMainPage() {

        global $CurrentUser;
        $admadm = new Admin_Admin;
        if (!$admadm->CheckIfCurrentUserAllowedAccess())
        {
            $CurrentUser->logout();            
            die (___('Access to the administration is not allowed for your user group'));
        }
        $_SESSION['backend_login'] = true;
        $latest_version = $this->checkLatestVersion();
        $this->loadTemplateFile('view.main.admin', compact(array_keys(get_defined_vars())));
        $this->__setTitlePage('');
    }

    public static function getAdminBackendMenuArray()
    {
        global $SystemConfig;
        
        $third_party_app_list = self::get3rdPartyApplicationList();
        $main_app_replacement = $SystemConfig->get('system','str_main_app');
        $frontend_app = [];
        $submenu_third_parth_apps =  [
                  ['app'=>'applications', 'icon' => 'fa-arrow-right', 'label' => ___('View All')],
                  ['app'=>'','type' => 'divider'],
                ];
        foreach ($third_party_app_list as $app)
        {
            $submenu_third_parth_apps[] = ['extracss' => 'color-apps', 'app'=> $app['title'], 'label' => $app['app_description'], 'image' => \App\Applications::getApplicationIcon($app['title'])];
            if ($app['title'] == $main_app_replacement)
            {
                $frontend_app[] = ['extracss' => 'color-frontend-app', 'id' => 'frontend-app', 'app'=> $app['title'], 'label' => $app['app_description'], 'image' => \App\Applications::getApplicationIcon($app['title'])];
            }
        }
        $menu = [
          ['app'=>'html', 'icon' => 'fa-file', 'label' => ___('Web Pages'), 'extracss' => 'color-apps'],
          ['app'=>'blog', 'icon' => 'fa-newspaper-o', 'label' => ___('Blog'), 'extracss' => 'color-apps'],
          ['app' => '_folder_apps', 'icon' => 'fa-puzzle-piece', 'label' => ___('Applications'), 'extracss' => 'color-apps', 
            // app submenus
            'submenus' => $submenu_third_parth_apps
            ],
           // end submenu
          ['app'=>'blocks', 'icon' => 'fa-th-large', 'label' => ___('Blocks'), 'extracss' => 'color-blocks'],
          ['app'=>'macros', 'icon' => 'fa-indent', 'label' => ___('Macros'), 'extracss' => 'color-macros'],
          ['app'=>'menus', 'icon' => 'fa-th-list', 'label' => ___('Menus'), 'extracss' => 'color-menus'],            
          ['app'=>'users', 'icon' => 'fa-users', 'label' => ___('Users'), 'extracss' => 'color-users'],
          ['app' => '_folder_tools', 'icon' => 'fa-wrench', 'label' => ___('Tools'), 'extracss' => 'color-tools', 
            // tools submenus
            'submenus' => 
                [
                  ['app'=>'mediamanager', 'icon' => 'fa-picture-o', 'label' => ___('Media Manager')],
                  ['app'=>'tag', 'icon' => 'fa-tags', 'label' => ___('Tags')],
                  ['app'=>'simplesitemap', 'icon' => 'fa-map-signs', 'label' => ___('Site Map')],
                  ['app'=>'logviewer', 'icon' => 'fa-info-circle', 'label' => ___('Log Viewer')],
                  ['app'=>'emailqueue', 'icon' => 'fa-envelope', 'label' => ___('Email Queue')],
                ]
            
            ],
           // end submenu
          ['app' => '_folder_settings', 'icon' => 'fa-gears', 'label' => ___('Settings'),  'extracss' => 'color-settings', 
            // settings submenus
            'submenus' => 
                [
                  ['app'=>'schlixupdater', 'icon' => 'fa-shield', 'label' => ___('System Update')],
                  ['app'=>'sitemanager', 'icon' => 'fa-sitemap', 'label' => ___('Site Manager')],
                  ['app'=>'themes', 'icon' => 'fa-television', 'label' => ___('Themes')],
                  ['app'=>'wysiwygeditors', 'icon' => 'fa-pencil-square', 'label' => ___('WYSIWYG Editors')],
                  ['app'=>'emailtemplates', 'icon' => 'fa-envelope-square', 'label' => ___('Email Templates')],
                  ['app'=>'cronscheduler', 'icon' => 'fa-clock-o', 'label' => ___('System Scheduler')],
                  ['app'=>'httperror', 'icon' => 'fa-exclamation-circle', 'label' => ___('HTTP Error')],
                  ['app'=>'bannedip', 'icon' => 'fa-hand-stop-o', 'label' => ___('Banned IP')],
                ]
            ],
            // end submenu
          ['app'=>'help', 'icon' => 'fa-question-circle', 'label' => ___('Help'), 'extracss' => 'color-help'],
          ['app'=>'', 'icon' => 'fa-sign-out', 'label' => ___('Logout'), 'url' => SCHLIX_SITE_HTTPBASE.'/admin/logout', 'extracss' => 'color-logout' ]
        ];
        
        
        return ($main_app_replacement != 'html' && $main_app_replacement != 'blog') ? array_merge($frontend_app, $menu) : $menu;
    }
    

    public static function renderBackendMenu()
    {
        global $CurrentAdminAppName, $SystemConfig;
        
        $main_app_replacement = $SystemConfig->get('system','str_main_app');
        
        
        $menu_items = self::getAdminBackendMenuArray();
        $menu_str = '<ul class="sidebar-menu" id="top_admin_menu">';
        foreach ($menu_items as $m)
        {
            $submenus = '';
            $sub_has_active = false;
            $url =  $m['url'] ? $m['url'] : self::getAdminApp($m['app']);
            if ($m['submenus'])
            {
                $url = 'javascript:void(0)';
                $submenus = "<ul class=\"treeview-menu\" id=\"sub_{$m['app']}\">";

                foreach ($m['submenus'] as $sub)
                {
                    $subli_class = [];
                    if ($sub['type'] == 'divider')
                    {
                        $submenus.= '<li class="menu-divider"><span></span></li>';
                    } else
                    {
                        $suburl = self::getAdminApp($sub['app']);
                        $subicon = $sub['icon'] ? "<i class=\"fa {$sub['icon']}\"></i>" : '';
                        $subimg = $sub['image'] ? "<img src=\"{$sub['image']}\" alt=\"{$sub['app']}\" />" : '';
                        if ($sub['image'] )
                            $subli_class[] = 'app_list';
                        // also include sub-applications - May 28, 2017
                        if (($sub['app'] == $CurrentAdminAppName || str_starts_with($CurrentAdminAppName.'.', $sub['app']) ) && $main_app_replacement != $sub['app'])
                        {
                            $subli_class[] = 'active';
                            $sub_has_active = true;
                        }
                        if ($sub['extracss'])
                            $subli_class[] = $sub['extracss'];
                        $subli_class_all = implode(' ', $subli_class);
                        $subli_class_str = $subli_class_all ? " class=\"{$subli_class_all}\"" : '';
                        $submenus.= "<li id=\"menu_{$sub['app']}\"{$subli_class_str}><a href=\"{$suburl}\">{$subicon}{$subimg}&nbsp;{$sub['label']}</a></li>\n";
                    }
                }
                $submenus.= '</ul>';
            }
            
            $root_li_class = [];
            if ($submenus)
                $root_li_class[] = 'treeview';
            if ( $m['app'] && (($CurrentAdminAppName == $m['app']) || $sub_has_active))
                $root_li_class[] = 'active'; 

            
            if ($m['icon'])
            {
                $image_or_icon = "<i class=\"fa {$m['icon']}\"></i>";
            } else
            {
                $root_li_class[] = 'app_list';                 
                $image_or_icon = "<img src=\"{$m['image']}\" alt=\"{$m['app']}\" />";
                
            }
            
            if ($m['extracss'])
                $root_li_class[] = $m['extracss'];            
            $rootli_class_all = implode(' ', $root_li_class);     
            $rootli_class_str = $rootli_class_all ? " class=\"{$rootli_class_all}\"" : '';
            
            
            $li_id_main = $m['id'] ? $m['id'] : "menu_{$m['app']}";
            $menu_str.="<li id=\"{$li_id_main}\"{$rootli_class_str}><a href=\"{$url}\">{$image_or_icon}&nbsp;{$m['label']}</a>{$submenus}</li>\n";            
        }
        $menu_str .= '</ul>';
        return $menu_str;
    }
    
    //_________________________________________________________________________//
    /**
     * Returns an array containing the name of 3rd party applications list
     * @return type
     */
    public static function get3rdPartyApplicationList()
    {
        $app_manager = new \App\applications;
        return $app_manager->get3rdPartyApplicationList();
    }
    
    public function getDataModelURL()
    {
        return '/'.get_application_alias('admin').'/app/admin';
        // return SCHLIX_SITE_HTTPBASE . "/admin/app/{$this->app_name}";
    }    
    
    public static function getAdminApp($str)
    {
        global $Application;
        return $Application->createFriendlyURL('app='.$str);
    }
    
    
//_______________________________________________________________________________________________________________//
    public function createFriendlyURL($str)
    {
        $final_url = '';
        if (SCHLIX_SEF_ENABLED) {
            $command_array = [];
            parse_str($str, $command_array); // output to $command_array
            if ((is_array($command_array)) && (array_key_exists('app', $command_array)) && ($command_array['action'] !== '')) {
                foreach ($command_array as $key => $value)
                {
                    $final_url.= "/{$key}/{$value}";
                }
                $final_url = SCHLIX_SITE_HTTPBASE . '/' . get_application_alias($this->app_name) . $final_url;
            }
            else return parent::createFriendlyURL($str);
        }
        return remove_multiple_slashes($final_url);
    }
    
//_______________________________________________________________________________________________________________//
    public function  interpretFriendlyURL($urlpath) {
                
        if (SCHLIX_SEF_ENABLED && !$_GET['app']) {
            $parsedurl = $this->probeFriendlyURLDestination($urlpath);
            /* $url_array = explode('/',$urlpath);
              $appname = $url_array[1];
              array_splice($url_array,0,2); */
            $url = $parsedurl['url'];
            $url_array = $parsedurl['url_array'];

            switch ($url_array[0]) {
                case 'active-sessions':
                case 'readnews':
                case 'login':
                case 'logout': $command['action'] = $url_array[0];
                    break;
                case 'app':
                    $command['action'] = 'load_admin_app';
                    $command['appname'] = $url_array[1];
                    break;
                case 'config':$command['action'] = 'config';
                    break;
                default:
                    if (empty($url_array[0])) {
                        $command['action'] = 'main';
                    } else if ($url_array[0] === 'action')
                    {
                        if ($url_array[1] == 'deleteinstalldir')
                        {
                            $command['action'] = 'deleteinstalldir';
                        }
                        
                    } else
                    {
                        return parent::interpretFriendlyURL($url);
                    }
            }
            return $command;
        } else {
            return parent::interpretFriendlyURL($url);
        }
    }
                
    //_______________________________________________________________________________________________________________//
    protected function translateOptions($the_options) {
        //echo HTML_WARNING_OLD_VERSION;
        return parent::translateMetaOptions($the_options);
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Load admin app
     * @global \SCHLIX\cmsAdmin_Basic $Administration
     * @global \App\wysiwygEditors $WYSIWYGEditor
     * @global \App\Users $CurrentUser
     * @global string $CurrentAdminAppName
     * @param string $appname
     * @return bool
     */
    /**
     * Load admin app
     * @global \SCHLIX\cmsAdmin_Basic $Administration
     * @global \App\wysiwygEditors $WYSIWYGEditor
     * @global \App\Users $CurrentUser
     * @global string $CurrentAdminAppName
     * @param string $appname
     * @return bool
     */
    public function loadAdminApp($appname) {

        global $Administration;
        global $WYSIWYGEditor;
        global $CurrentUser;
        global $CurrentAdminAppName;
        $pos = strpos( $appname, '.');
        $appname = convert_to_safe_filename($appname);
        $possible_dirs = array (SCHLIX_SITE_PATH, SCHLIX_SYSTEM_PATH);
        
        if ($pos > 0) // It means there's a submodule
        {
            $classname = str_replace('.','_', $appname) . '_Admin';
            $file_name = $appname;    
            $folder = substr($appname, 0, $pos);
        } else
        {
            $classname = str_replace('.','_', $appname) . '_Admin';
            $file_name = $folder = $appname;
        } 
                
        $classname = 'App\\'.$classname;
        
        $possible_files = [];
        if (strpos($file_name,'.') > 0)
        {
            
            list ($main_app, $sub_app) = explode('.', $file_name);
            $possible_files[] =  "/apps/{$folder}/{$sub_app}/{$file_name}.admin.class.php";
        }
        $possible_files[] =  "/apps/{$folder}/{$file_name}.admin.class.php";
        $possible_file_found = false;
        foreach ($possible_dirs as $possible_dir)
        {
            foreach ($possible_files as $possible_file)
            {
                if (file_exists($possible_dir.$possible_file))
                {
                    $possible_file_found = true;
                    break;
                }
            }
        }
        if (!$possible_file_found) {
            $errormsg = 'Administration class file cannot be found - ' .convert_to_safe_filename($file_name);
            die ($errormsg);
        }
        $Administration = new $classname($errormsg);                
        $Administration->loadApplicationJavascript();
	if ($Administration->CheckIfCurrentUserAllowedAccess())        
            $running_app = $Administration->Run();
        else 
        {
            $CurrentUser->logout();            
            die (___('Access not allowed for your user group'));
        }
        $WYSIWYGEditor->runDefaultFullPrivilegeWysiwygEditor();
        $CurrentAdminAppName = $appname;
        return $running_app;
    }
    
    private function setIntendedAdminAppBeforeLogin()
    {
        $_SESSION['admin_intention'] = filter_input(INPUT_SERVER, 'REQUEST_URI', FILTER_SANITIZE_URL);        
        return $this->viewLogin();
    }
    //_________________________________________________________________________//
    function getIntendedAdminApplication() {
        /* This is just for the convenience if admin session times out, then it will filter the request and forward it to the new location */
        /* Filtered against XSS attack */
        $allowable_actions = array('edititem', 'editcategory', 'newitem', 'newcategory');
        $the_url = (defined('SCHLIX_SITE_HTTP_URL')) ? SCHLIX_SITE_HTTP_URL : SCHLIX_SITE_URL; // for backward compatibility
        //if (isset($_SESSION['admin_intention'])	)
        if (isset($_SESSION['admin_intention']) && $_SESSION['admin_intention'] != SCHLIX_SITE_HTTPBASE . '/admin/' && $_SESSION['admin_intention'] != SCHLIX_SITE_HTTPBASE . '/admin/login/') {
            
            $intended_url = $_SESSION['admin_intention'];
            unset($_SESSION['admin_intention']);
            redirect_url($intended_url);
        } else            
        {
            redirect_url($this->createFriendlyURL(''));            
            
        }
    }
    
    /**
     * Checks the latest version
     * @global \SCHLIX\cmsConfigRegistry $SystemConfig
     * @return array|NULL
     */
    public function checkLatestVersion()
    {
        global $SystemConfig;
        
        $UpdaterAdmin = new \App\SCHLIXUpdater_Admin();
        $latest_version = NULL;
        $last_update_check = $this->getConfig('str_last_update_check');
        $current_time = get_current_datetime();
        $this->setConfig('str_last_update_check', $current_time);
        $update_frequency_days = $SystemConfig->get('system','int_system_update_frequency');
        if ($update_frequency_days == 0) 
            $update_frequency_days = 3;
        if (is_date($last_update_check))
        {
            $how_many_days_since_last_check = days_difference($last_update_check, $current_time);
            if ($how_many_days_since_last_check > $update_frequency_days)
            {
                $latest_version = $UpdaterAdmin->getLatestCMSUniversalVersion();
            }
        } else
        {
            $latest_version = $UpdaterAdmin->getLatestCMSUniversalVersion();
        }
        if (version_compare(strtoupper($latest_version['version']), strtoupper(SCHLIX_VERSION)) <= 0) 
        {
            return NULL;
        }
        
        return $latest_version;
    } 
    
    /**
     * Returns RSS cache
     * @param string $source
     * @param int $cachetime
     * @return string
     */
    private function getCachedSCHLIXNews($source, $cachetime = 24) {
                
        $cache_key = 'rss'.md5($source);
        $cached_content = $this->fs_cache->get($cache_key);
        if (!$cached_content) {
            $fcontent = get_remote_file_content($source);
            $this->fs_cache->set($cache_key, $fcontent,$cachetime);
        } else {
            $fcontent = $cached_content;
        }
        return $fcontent;
    }

    /**
     * Load Iframe ext url
     * @param type $source
     */
    public function loadExternalRSS($source) {
        global $__forceOverrideSubTemplate;

        
        $__forceOverrideSubTemplate = "blank.template";
        switch ($source) {
            case 'news-ext': $rss_url = 'https://www.schlix.com/cmsservices/action/rss-extensions';
                break;
            case 'news-themes': $rss_url = 'https://www.schlix.com/cmsservices/action/rss-themes';
                break;            
            case 'news-main':
            default: $rss_url = 'https://www.schlix.com/cmsservices/action/rss-news';
                break;
        }
        $rss_content = $this->getCachedSCHLIXNews($rss_url);
        if (empty($rss_content))
            return RETURN_FULLPAGE;
        
        $simplerss = simplexml_load_string($rss_content);
        $this->loadTemplateFile('view.externalrss', compact(array_keys(get_defined_vars())));
        return RETURN_FULLPAGE;
    }

    /**
     * View the login box
     * @global string $__forceOverrideSubTemplate
     * @global Users $CurrentUser
     * @return type
     */
    public function viewLogin() {
        
        global $__forceOverrideSubTemplate;
        global $CurrentUser;
        global $SystemConfig;

        $__forceOverrideSubTemplate = "login.template";
        $this->__setTitlePage('Login');
        $force_ssl_admin_login = $SystemConfig->get(SCHLIX_DEFAULT_USER_CLASS,'bool_force_ssl_admin_authentication');
        $post_location = ($force_ssl_admin_login == 1) ? force_https_url().$this->createFriendlyUrl('login') : $this->createFriendlyUrl('login');

        if (is_postback())
        {
            check_csrf_halt_on_error();            
            $CurrentUser->performAuthentication(fpost_string('username'),fpost_string('password'), fpost_bool('remember'));
            if ($CurrentUser->adminAuthenticated())
            {
                //redirectURL($this->createFriendlyURL(''));
                $this->getIntendedAdminApplication();
            } else 
            {
                $error = ___('Invalid username/password');
            }
            //if ($_SESSION['login_error']) sleep(2);
        }
        $this->loadTemplateFile('view.login', compact(array_keys(get_defined_vars())));
        return RETURN_FULLPAGE;
    }
    
    /**
     * Display a box with info
     * @param int $number
     * @param string $title
     * @param url $link
     */
    public function viewBox($number, $title, $link, $bg_color_class, $icon_class) {
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.box', $local_variables, false);
    }

    /**
     * View incoming Messages
     */
    public function viewIncomingMessages() {
        global $SystemDB;

        $sql = "SELECT * FROM gk_contact_messages ORDER BY date_created DESC LIMIT 0,5";
        $incoming_messages = $SystemDB->getQueryResultArray($sql);

        $this->loadTemplateFile('view.inbox', array('incoming_messages' => $incoming_messages), false);
    }

    /**
     * View incoming Messages
     */
    public function getCountOfIncomingMessagesWithinDays($days) {
        global $SystemDB;

        $days = (int) $days;
        $sql = "SELECT COUNT(*) AS total_count FROM gk_contact_messages WHERE DATEDIFF(NOW(),`date_created`) <= {$days} ORDER BY date_created DESC";
        $incoming_messages = $SystemDB->getQueryResultSingleRow($sql);
        return $incoming_messages['total_count'];
    }    
    /**
     * Display a box with info
     * @param int $number
     * @param string $title
     * @param url $link
     */
    public function viewSystemInfo() {

        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.systeminfo', $local_variables, false);
    }

    //_________________________________________________________________________//

    public function getApplicationDescription() {
        return 'Main';
    }

    public function checkIfInstallationDirectoryStillExists()
    {
        $local_variables = compact(array_keys(get_defined_vars()));
        
        if (is_dir(SCHLIX_ROOT_PATH.'/install')) 
        {

            $this->loadTemplateFile('view.installdir.warning', $local_variables, false);
            return true;
        }
        return false;
    }
    
    
    private function disableMaliciousUser($id)
    {
        global $SystemDB, $SystemLog;
        
        $id = (int) $id;
        $SystemDB->query("UPDATE gk_user_items SET status = -10 WHERE id = :id",['id' => $id]);
        $SystemLog->record("Disabling user ID # {$id} because of a malicious action");
    }
    
    private function getMaliciousUsers()
    {
        global $SystemDB, $CurrentUser;
        
        $mal_users_final = [];
        $sql = "SELECT id, guid, avatar, username, email_address, last_ip_address FROM gk_user_items WHERE `avatar` IS NOT NULL";
        $all_users = $SystemDB->getQueryResultArray($sql);
        $c = ___c($all_users);
        for ($i = 0;$i < $c;$i++)
        {
            $mal_fn = $all_users[$i]['guid'].'.php';
            $all_users[$i]['malicious'] = false;
            $possible_mals = 
                    [
                        $CurrentUser->getDataFileFullPath('avatar_original', $mal_fn),
                        $CurrentUser->getDataFileFullPath('avatar_large', $mal_fn),
                        $CurrentUser->getDataFileFullPath('avatar_medium', $mal_fn),
                        $CurrentUser->getDataFileFullPath('avatar_small', $mal_fn),
                    ];
            foreach ($possible_mals as $mal)
            {
                if (file_exists($mal))
                {
                    
                    $all_users[$i]['malicious'] = true;
                    //$this->malicious_files[] = $mal;
                    $this->addMFBeta($mal);
                }
            }
            
            $mal_avatar_fn = $all_users[$i]['avatar'];
            $possible_mal_avatars = 
                    [
                        $CurrentUser->getDataFileFullPath('avatar_original', $mal_avatar_fn),
                        $CurrentUser->getDataFileFullPath('avatar_large', $mal_avatar_fn),
                        $CurrentUser->getDataFileFullPath('avatar_medium', $mal_avatar_fn),
                        $CurrentUser->getDataFileFullPath('avatar_small', $mal_avatar_fn),
                    ];
            foreach ($possible_mal_avatars as $avt)
            {
                if (file_exists($avt))
                {
                    $str_content = file_get_contents($avt);
                    if (str_contains($str_content, 'php'))
                    {
                        $all_users[$i]['malicious'] = true;
                        //$this->malicious_files[] = $avt;
                        $this->addMFBeta($avt);
                    }
                }
            }
            if ($all_users[$i]['malicious'] === true)
            {
                $mal_users_final[] = $all_users[$i];
                $this->disableMaliciousUser($all_users[$i]['id']);
            }
        }
        return $mal_users_final;
    }
    
    
    public function getWorldWritableFiles()
    {
        global $CurrentUser, $SystemLog;
        $result = [];
        $unfixable = [];
        $phpscript_owner_id = getmyuid();
        $items = \SCHLIX\cmsDirectoryFilter::getRecursiveDirectoryIterator(SCHLIX_ROOT_PATH, \SCHLIX\cmsDirectoryFilter::FILTER_NONE, ['..','.']);

        foreach ($items as $item) {
            $is_dir = $item->isDir();
            $is_file = $item->isFile() ;
           if ($is_file||$is_dir) {
               $ext = strtolower($item->getExtension());
               $path =  $item->getPath();
               $full_fn = remove_multiple_slashes($path.'/'.$item->getFileName());

               $perm = fileperms($full_fn);
               $world_wrtb = ($perm & 0x0002);
               
               if ($world_wrtb && is_writable($full_fn))
               {
                   $owner = fileowner($full_fn);
                   if ($owner == $phpscript_owner_id)
                   {
                        $new_perm = $perm & ~0x0002  & ~0x0010; // go-w
                        //
                        $fix_result = @chmod($full_fn, $new_perm); 
                        // quick fix - Dec 4 - scenario for non-suphp
                        if ($fix_result)
                            $result[] = ['filename' => $full_fn, 'fixed' => $fix_result];
                   }
               }
           } 
       }
       return $result;
     }
    
    /**
     * Quick check - BETA
     * @global \App\Users $CurrentUser
     * @global \SCHLIX\cmsLogger $SystemLog
     */
    public function getImproperPHPFiles()
    {
        global $CurrentUser, $SystemLog;
        $result = [];
        $folders = [
            CURRENT_SUBSITE_PATH.'/media',
            CURRENT_SUBSITE_PATH.'/data',
            ];
        
        foreach ($folders as $folder)
        {
            $items = \SCHLIX\cmsDirectoryFilter::getRecursiveDirectoryIterator($folder, \SCHLIX\cmsDirectoryFilter::FILTER_FILE_ONLY, ['web.config','.htaccess','README.txt']);

            foreach ($items as $item) {
                if ($item->isFile()) {
                    $ext = strtolower($item->getExtension());
                    $path =  $item->getPath();
                    $full_fn =remove_multiple_slashes($path.'/'.$item->getFileName());
                    $is_malicious = false;
                    if ($ext == 'php' || $ext == 'phtml')
                    {
                        if ($folder !== CURRENT_SUBSITE_PATH.'/data')
                        {
                            $is_malicious = true;
                        } else 
                        if (str_contains($folder, '/languages'))
                            $is_malicious = (!str_contains($full_fn, 'lang.php'));
                        else
                        {
                            $fc = file_get_contents($full_fn);
                            if (str_contains($fc, 'bin2hex') || str_contains($fc, 'gzinflate'))
                                $is_malicious = true;
                        }
                    } else if ($ext == 'gif' || $ext == 'png' || $ext == 'jpg' || $ext == 'jpeg')
                    {
                        $fc = file_get_contents($full_fn);
                        if (str_contains($fc, '?php'))
                            $is_malicious = true;                            
                    }
                    
                    if ($is_malicious)
                    {
                        $this->addMFBeta($full_fn);
                    }
                }
            }
        }
    }
    
    private function addMFBeta($full_fn)
    {        
            //$sha1 = sha1_file($full_fn);
            //$qfn = remove_multiple_slashes($this->report_folder.'/'.$sha1);
            //$fixed_result = @rename($full_fn, $qfn);
            $this->malicious_files[] = ['filename' => $full_fn, 'fixed' => false];//,'quarantine_filename' => $qfn];
    }
    
    private function fixMF()
    {        
        $c = ___c($this->malicious_files);
        for ($i = 0; $i < $c; $i++)
        {
            $full_fn = $this->malicious_files[$i]['filename'];
            $sha1 = sha1_file($full_fn);
            $qfn = remove_multiple_slashes($this->report_folder.'/'.$sha1);            
            $fixed_result = @rename($full_fn, $qfn);
            
            $this->malicious_files[$i]['fixed'] = $fixed_result;
            $this->malicious_files[$i]['quarantine_filename'] = $qfn;
        }
    }
    
    
    public function performSecurityCheckBETA()
    {
        global $CurrentUser, $SystemDB;
        
        $this->initializeDataDirectories();
        $is_windows = strtoupper(substr(PHP_OS, 0, 3)) === 'WIN' ;
        // 1. malicious users
        $mal_users = $this->getMaliciousUsers();
        // 2. world writable
        $world_writables = $is_windows ? NULL : $this->getWorldWritableFiles();
        //PHP Script Owner
        
        
        // 3. check php files in /media
        $this->getImproperPHPFiles();
        
        $ist_mu = ___c($mal_users) > 0;
        $ist_ww = ___c($world_writables) > 0;
        $ist_mf = ___c($this->malicious_files) > 0;
        $maybe_vulnerable = $ist_mu || $ist_ww || $ist_mf ;
        if ($maybe_vulnerable)
        {
            
            $this->report_folder = $this->getDataFileFullPath('quarantine',date('Y-m-d_H_i_s'));
            $main_report_filename = remove_multiple_slashes($this->report_folder.'/report.txt');
            $error = '';
            if (!is_dir($this->report_folder)) 
            if (!mkdir($this->report_folder, 0755, true))
                $error = "Fatal error: Quarantine storage folder {$this->report_folder} cannot be created";
            
            if ($ist_mu)
            {
                $report_content.= ___('Malicious users')."\n";
                $report_content.= var_export($mal_users, true);
            }
            if ($ist_ww)
            {
                $report_content.= ___('World writable files')."\n";
                $report_content.= var_export($world_writables, true);
            }
            if ($ist_mf)
            {
                $this->fixMF();
                $report_content.= ___('Potentially malicious files')."\n";
                $report_content.= var_export($this->malicious_files, true);
            }
            
            file_put_contents($main_report_filename, $report_content);
            
            $local_variables = compact(array_keys(get_defined_vars()));
            $this->loadTemplateFile('view.security.warning', $local_variables, false);
            return true;
        }
        return false;
        
    }
    
    
    public function checkBasicSecurityIssueBETA()
    {
        global $SystemConfig;
        
        
        $last_security_check = $this->getConfig('str_last_basic_security_check');
        $current_time = get_current_datetime();
        $this->setConfig('str_last_basic_security_check',  $current_time);
        $security_check = $SystemConfig->get('system','int_system_securitycheck_frequency');
        if ($security_check == 0) 
            $security_check = 7;
        //$last_security_check = NULL;
        if (is_date($last_security_check))
        {
            $how_many_days_since_last_check = days_difference($last_security_check, $current_time);
            if ($how_many_days_since_last_check > $security_check)
            {
                return $this->performSecurityCheckBETA();
            }
        } else
        {
             return $this->performSecurityCheckBETA();
        }
        return false;
    }
    
    
    //_________________________________________________________________________//

    function deleteInstallDirectory($ok) {
        if ($ok == 1 && is_dir(SCHLIX_ROOT_PATH . '/install')) {
            $newname = SCHLIX_ROOT_PATH . '/install' . mt_rand();
            $result = __del_tree(SCHLIX_ROOT_PATH . '/install'); // @rename (SCHLIX_SITE_PATH.DIRECTORY_SEPARATOR.'install',$newname);

            if ($result)
                $this->errormsg = ___('Installation directory has been removed completely');
            else
                $this->errormsg = 'Cannot rename installation directory. Please check the permission on /install and try again or delete it manually - ' . SCHLIX_ROOT_PATH . '/install';
        }include ('view.deleteinstalldir.template.php');
    }

    //_________________________________________________________________________//

    public function checkIfWindowsAndNoSMTP() {
        if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN' && SCHLIX_SMTP_HOST == '')
            echo \__Html::DIV_start(array('class' => 'alert alert-warning')) 
            . \__Html::P(___('Notice - your SMTP server is not configured. This is okay, but system message/contact mail/registration will not be sent to users'))
            . \__Html::DIV_end() ;
    }

    //This function transforms the php.ini notation for numbers (like '2M') to an integer (2*1024*1024 in this case)   

    function getMaximumFileUploadSize() {
        return min($this->convert_ini_php_memsize(@ini_get('post_max_size')), $this->convert_ini_php_memsize(@ini_get('upload_max_filesize')));
    }

    //_________________________________________________________________________//

    public function convert_ini_php_memsize($size_str) {
        switch (substr(strtolower($size_str), -1)) {
            case 'm': return (int) $size_str * 1048576;
            case 'k': return (int) $size_str * 1024;
            case 'g': return (int) $size_str * 1073741824;
            default: return $size_str;
        }
    }
    /**
     * Logout the current user
     * @global type $CurrentUser
     */
    public function logOut()
    {
        global $CurrentUser;
        
	$CurrentUser->logout();
        redirect_url($this->createFriendlyURL('login'));
    }
    //_______________________________________________________________________________________________________________//
    /**
     * Main controller
     * @param type $command
     * @return boolean
     */
    public function Run($command) {
        global $CurrentUser, $CurrentUser;
        
        $CurrentUser = $CurrentUser;
                
	if ($CurrentUser->adminAuthenticated() === true && $CurrentUser->getCurrentUserID() > 0)
	{
            
            if ($command['action'] != 'deleteinstalldir')
            {                
                 if ($this->checkIfInstallationDirectoryStillExists()) return true;
                 // quick fix - TODO REMOVE - Nov 20 2018
                 if ($command['action']  == 'main')
                    if ($this->checkBasicSecurityIssueBETA()) return true;
            }
            
            switch ($command['action']) {
		case 'deleteinstalldir': $this->deleteInstallDirectory(fpost_int('deleteinstalldir'));break;
                
                case 'login':
                    redirect_url($this->createFriendlyURL(''));
                    break;
                case 'logout':
                    return $this->logout();
                case 'readnews':
                    return $this->loadExternalRSS($_GET['source']);
                    break;
                case 'active-sessions': 
                    return $this->viewActiveSessions();
                case 'load_admin_app':
                    return $this->loadAdminApp($command['appname']);
                    break;

                default: return parent::Run($command);
            }

        } else
        // if not authenticated
        {
            switch ($command['action']) {
                case 'login':
                    return $this->viewLogin();
                    break;
                case 'load_admin_app':
                    return $this->setIntendedAdminAppBeforeLogin();
                    break;                    
                
                default:  return $this->viewLogin();
            }
        }

        return true;
    }

}
                