<?php
if (!defined('SCHLIX_VERSION'))
    die('No Access');
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
global $SystemConfig, $CurrentUser, $SchlixSupportedLanguages;

$app_list = $this->getListofApplications();
$main_frontend_app = $SystemConfig->get('system', 'str_main_app');
if (empty($main_frontend_app))
    $main_frontend_app = 'html';
$forbidden_frontend = ['cronscheduler', 'wysiwygeditors', 'mediamanager'];

$languages = array();
?> 
<schlix-data-editor id="schlix-<?= $this->app_name ?>-admin" class="schlix-data-editor" data-app="<?= $this->app->getApplicationNameOnly() ?>" data-main-app-description="<?= $this->app->getApplicationDescription() . ': ' . ___('Global System Configuration') ?>"  data-schlix-controller="SCHLIX.CMS.AdminAdmin" data-model="<?= $this->getDataModelURL() ?>" data-field-id="id" data-cancel-warning="true">
    <div class="content">
        <!-- content -->
        <?php
        ///////////////////////////////
        // Display save error, if any
        $this->displayLastConfigSaveResult()
        ///////////////////////////////
        ?>
        <form method="post" action="<?= $this->createFriendlyAdminURL('action=saveglobalconfig') ?>" autocomplete="off" >
            <?= \INPUT::CSRF() ?>
            <div class="row">
                <div class="col-sm-offset-9 col-sm-3 text-right" schlix-id="data-editor-toolbar">
                    <button type="submit" class="btn btn-success schlix-command-button" data-schlix-command="save-item" name="btn_save"><i class="fa fa-check"></i> <?= ___('Save'); ?></button>
                    <button type="submit" class="btn btn-danger schlix-command-button" data-schlix-command="cancel-item" name="btn_cancel"><i class="fa fa-times"></i> <?= ___('Cancel'); ?></button>
                </div>            
                
            </div>

            <div class="row">
                <div class="col-sm-6">
                    <!-- config goes here -->
        <schlix-config:textbox config-key='str_site_name' config-default-value="<?= SCHLIX_SITE_NAME ?>" class="form-control" config-fonticon="fas fa-university" label="<?= ___('Site Name') ?>" />             
                    <div class="row">
                        <div class="col-sm-12">
                            <label for="str_interface_language"><?= ___('Language') ?> (<?= ___('BETA') ?>)</label>
                            <div class="input-group">
                                <span class="input-group-addon"><i class="fa fa-language"></i> </span>
                                <schlix-config:dropdownlist class="form-control" config-key='str_interface_language'>
                                    <schlix-config:option value="en_us"><?= ___('English') ?> (<?= ___('default') ?>)</schlix-config:option>
                                    <?php foreach ($SchlixSupportedLanguages as $lang => $desc): ?>
                                        <schlix-config:option value="<?= $lang ?>"><?= $desc ?> </schlix-config:option>
                                    <?php endforeach ?>
                                </schlix-config:dropdownlist>
                                <p id="schlix-downloadlang-notice"></p>
                            </div>
                        </div>
                    </div>
                    <!-- note -->
                    <p><em><?= ___('When you change the language, SCHLIX CMS will attempt to download the system language pack if it has not been installed. Installation may take up to 1 minute.') ?></em></p>
                    <div class="row">
                        <div class="col-sm-12">
                            <label for="str_main_app"><?= ___('Main application for the frontpage') ?></label>
                            <div class="input-group">
                                <span class="input-group-addon"><i class="fa fa-puzzle-piece"></i> </span>
                                <schlix-config:dropdownlist class="form-control" config-key='str_main_app' >
                                    <schlix-config:option value="">(<?= ___('Default') ?>)</schlix-config:option>
                                    <?php foreach ($app_list as $app_info): ?>
                                        <?php if (!in_array($app_info['value'], $forbidden_frontend)): ?>
                                            <schlix-config:option value="<?= $app_info['value'] ?>"><?= ___h($app_info['label']) ?> </schlix-config:option>
                                        <?php endif ?>
                                    <?php endforeach ?>
                                </schlix-config:dropdownlist>
                            </div>

                        </div>
                    </div>
                    <!-- note -->                    
                    <p><em><?= ___('You can change which application to be called when the homepage is called. You can use this feature to mimic other software, e.g. blog in the front page. The standard and the recommended one is Web Pages (html).') ?></em></p>
                    <!-- row -->
                    <div class="row">
                        <div class="col-sm-12">
                            <label for="int_system_update_frequency"><?= ___('System update check frequency') ?></label>
                            <div class="input-group">
                                <span class="input-group-addon"><i class="far fa-clock"></i> </span>

                                <schlix-config:dropdownlist class="form-control" config-key='int_system_update_frequency' >
                                    <schlix-config:option value=""><?= ___('(Default - 3 days)') ?> </schlix-config:option>
                                    <schlix-config:option value="7"><?= ___('Every week') ?> </schlix-config:option>
                                    <schlix-config:option value="14"><?= ___('Every two weeks') ?> </schlix-config:option>
                                    <schlix-config:option value="30"><?= ___('Once a month') ?> </schlix-config:option>
                                    <schlix-config:option value="180"><?= ___('Every 6 months') ?> </schlix-config:option>
                                </schlix-config:dropdownlist>
                            </div>
                        </div>
                    </div>

                    <!-- row -->
                    <div class="row">
                        <div class="col-sm-12">
                            <label for="int_system_update_frequency"><?= ___('System security check frequency') ?></label>
                            <div class="input-group">
                                <span class="input-group-addon"><i class="far fa-clock"></i> </span>

                                <schlix-config:dropdownlist class="form-control" config-key='int_system_securitycheck_frequency' >
                                    
                                    <schlix-config:option value=""><?= ___('Every week') ?> </schlix-config:option>
                                    <schlix-config:option value="14"><?= ___('Every two weeks') ?> </schlix-config:option>
                                    <schlix-config:option value="30"><?= ___('Once a month') ?> </schlix-config:option>
                                    <schlix-config:option value="180"><?= ___('Every 6 months') ?> </schlix-config:option>
                                </schlix-config:dropdownlist>
                            </div>
                        </div>
                    </div>

                    <!-- end config -->
                </div>
                <div class="col-sm-6">
                    <!-- e -->
                     
                </div>
            </div>
            
            
        </form>

        <!-- end content -->
    </div>
</schlix-data-editor>    