<?php
namespace App;

class Applications extends \SCHLIX\cmsApplication_CategorizedList
{
    public function __construct()
    {
		parent::__construct( ___('Applications'), 'gk_app_items','');
                $this->disable_frontend_runtime = true;
    }
    //_________________________________________________________________________//

    public function viewMainPage()
	{
		return false;
	}
    /**
     * Returns full application info by app name
     * @param string $app_name
     * @return array
     */
    public function getItemByAppName($app_name)
    {
        $sanitized_app_name = sanitize_string($app_name);

        $items = $this->getAllItems('*', "title = {$sanitized_app_name}");
        return (is_array($items) && (___c($items) === 1)) ? $items[0] : null;
    }
    /**
     * Returns application list which can be shown in the backend
     * @return array
     */
    public function get3rdPartyApplicationList()
    {
            return $this->getAllItems('*','has_backend = 1 AND show_in_app_menu = 1', 0, 1000, 'app_description');
    }
    
    /**
     * Returns the application icon URL path
     * @param string $app
     * @return string
     */
    public static function getApplicationIcon($app)
    {
        $app_icon = 'apps/'.$app.'/'.strtolower($app).'_logo.png';
        if (file_exists (CURRENT_SUBSITE_PATH.'/'.$app_icon))
                $app_icon_url_path = CURRENT_SUBSITE_URL_PATH.'/'.$app_icon;
        elseif (file_exists (SCHLIX_SYSTEM_PATH.'/'.$app_icon))
                $app_icon_url_path = SCHLIX_SYSTEM_URL_PATH.'/'.$app_icon;
        else
                $app_icon_url_path = SCHLIX_SYSTEM_URL_PATH.'/images/icon_plugins.png';
        return $app_icon_url_path;
    }
    
    /**
     * Returns package info for a package, where type can be macro, block, or app
     * @param string $type
     * @param string $app
     * @return array
     */
    public static function getPackageInfo($type,$app)
    {
        $app_package_info = $type.'s/'.$app.'/packageinfo.inc.php';
        if (file_exists (CURRENT_SUBSITE_PATH.'/'.$app_package_info))
                $app_package_info_file = CURRENT_SUBSITE_PATH.'/'.$app_package_info;
        elseif (file_exists (SCHLIX_SYSTEM_PATH.'/'.$app_package_info))
                $app_package_info_file = SCHLIX_SYSTEM_PATH.'/'.$app_package_info;
        else
            return NULL;
        // file size must be small, no more than 2KB
        if (filesize($app_package_info_file) <= 2048)
        {
            $package_info = [];
            $file_content = file ($app_package_info_file);
            foreach ($file_content as $line)
            {
                if ($line)
                {
                    preg_match_all('#\$((?:[a-z][a-z0-9_]*))\s?=\s?\'(.*)\';#iU', $line, $matches);
                    $match_count = ___c($matches[0]);                    
                    if ($match_count > 0) 
                    {
                        $package_info[$matches[1][0]] = $matches[2][0];
                    }                    
                }
            }
            return $package_info;
        }
            
        return null;
    }
    //_________________________________________________________________________//
    public function getAllApplicationList()
    {
            return $this->getAllItems('*','has_backend = 1 ');
    }

	
    //_________________________________________________________________________//
    public function setApplicationItemField($app_name, $field, $value)
    {
        global $SystemDB;

        $app_item = $this->getItemByAppName($app_name);
        $sanitized_app_name = sanitize_string($app_name);
        $sanitized_value= sanitize_string($value);
        $sanitized_field = alpha_numeric_with_dash_underscore($field);
        if ($app_item != null)
        {
            // update
            $sql = "UPDATE {$this->table_items} SET `{$sanitized_field}` = {$sanitized_value} WHERE title = {$sanitized_app_name}";
            $SystemDB->query($sql);
        } else
        {
            // insert
            if ( get_system_or_user_file("/apps/{$app_name}/{$app_name}.class.php") )
            {
                // update
                $sql = "INSERT INTO {$this->table_items} (title,`{$sanitized_field}`) VALUES ({$sanitized_app_name},{$sanitized_value})";
                $SystemDB->query($sql);
            }
        }

    }
    //_________________________________________________________________________//
    public function getApplicationAliasThatDoesNotConflictWithFileSystem($app_name,$alias)
    {
		$all_files_in_main_directory = scandir(SCHLIX_SITE_PATH);
        $i = 1;
        $final_alias = $alias;
        while (in_array($final_alias, $all_files_in_main_directory))
        {
            $final_alias = $alias.$i;
        }
        return ($app_name === $alias) ? $alias : $final_alias;
    }

    //_________________________________________________________________________//
    public function setApplicationAlias($app_name, $alias)
    {
        $alias = trim ($alias);
        $app_name = trim ($app_name);
        if (empty($alias)) $alias = $app_name;
        $final_name = $this->getApplicationAliasThatDoesNotConflictWithFileSystem($app_name,$alias);
        $this->setApplicationItemField($app_name,'app_alias', $final_name);
        return $final_name;
    }
    //_________________________________________________________________________//
    public function setApplicationDescription($app_name, $description)
    {
        $description = trim ($description);
        $app_name = trim ($app_name);
        if (empty($description)) $description = ucwords ($app_name);
        $this->setApplicationItemField($app_name,'app_description', $description);
    }

}

?>