<?php

namespace App;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
class BootstrapSlideshow_Admin extends \SCHLIX\cmsAdmin_CategorizedList {

    //_________________________________________________________________________//    
    public function __construct() {
        
        $datatype = 'basicnestedcategory';
        $methods = []; /* array('standard_main_app' => 'Main Application Page',
          'standard_browse' => 'View a specific item/category',
          'rss' => 'View RSS 2.0'
        );*/

       parent::__construct($datatype, $methods);
        $this->setItemFieldNamesForAjaxListing('id', 'date_created', 'date_modified', 'title', 'virtual_filename', 'image', 'status', 'sort_order');
    }

    /**
     * You can customize the response schema field here
     * @param array $response_schema
     * @return array
     */
    public function modifyItemResponseSchemaFields(array $response_schema) {
        $response_schema = parent::modifyItemResponseSchemaFields($response_schema);

        $response_schema[] = array('key' => 'file_exists', 'parser' => 'string');
        $response_schema[] = array('key' => 'thumb_url', 'parser' => 'string');
        $response_schema[] = array('key' => 'xl_url', 'parser' => 'string');

        return $response_schema;
    }

//_________________________________________________________________________//    	
    public function ajaxGetItemsByCategoryID($id, $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC') {
        $parent_output = parent::ajaxGetItemsByCategoryID($id, $start, $end, $sortby, $sortdirection);
        $reply = json_decode($parent_output, true);
        $totalcount = ___c($reply['data']);
        
        for ($i = 0; $i < $totalcount; $i++) {
            $image_filename = $reply['data'][$i]['image'];
            $reply['data'][$i]['file_exists'] = is_file( $this->app->getDataFileFullPath('image_thumbs', $image_filename));
            $reply['data'][$i]['thumb_url'] = $this->app->getDataFileURLPathWithHash('image_thumbs', $image_filename);
            $reply['data'][$i]['xl_url'] = $this->app->getDataFileURLPathWithHash('image_large', $image_filename);
        }
        return json_encode($reply);
    }
    //_________________________________________________________________________//	
    /**
     * Hook Before save Item
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues)
    {
        
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        if ( (int) $datavalues['delete_picture'] == 1)
        {
            $datavalues['image'] = '';
        }
        return $datavalues;
    }


    //_________________________________________________________________________//    
    public function ajaxDeleteObjects($mixed_items_to_delete) {
        global $SystemDB;
        $mixed_items_array = explode(',', $mixed_items_to_delete); // e.g: i4,i14
        // Process sub-folders first
        foreach ($mixed_items_array as $mixed_item) {
            $item_id = substr($mixed_item, 1);
            $items_to_delete[] = $item_id;
        }
        $items_to_delete_str = implode(",", $items_to_delete);
        if ($items_to_delete_str) {
            $sql = "SELECT * from {$this->table_items} WHERE {$this->field_id} in ({$items_to_delete_str})";
            $result_array = $SystemDB->getQueryResultArray($sql);
            foreach ($result_array as $item) {
                $image_fn_original = $this->app->getDataFileFullPath('image_original', $item['image']);
                $image_fn_thumbs = $this->app->getDataFileFullPath('image_thumbs', $item['image']);
                $image_fn_large = $this->app->getDataFileFullPath('image_large', $item['image']);
                if (file_exists($image_fn_original))
                    @unlink($image_fn_original);
                if (file_exists($image_fn_thumbs))
                    @unlink($image_fn_thumbs);
                if (file_exists($image_fn_large))
                    @unlink($image_fn_large);
            }
            $items_to_delete_str = implode(",", $items_to_delete);
        }
        return parent::ajaxDeleteObjects($mixed_items_to_delete);
        //	echo ajaxReply(200,'Delete OK');
    }

    //_________________________________________________________________________// 	
    public function ajaxUploadFiles() {

        $upload_failed = false;
        $int_thumb_width = $this->app->getConfig('int_thumb_width',256);
        $int_thumb_height = $this->app->getConfig('int_thumb_height',256);
        $int_xl_width = $this->app->getConfig('int_xl_width', 1024);
        $int_xl_height = $this->app->getConfig('int_xl_height', 1024);
        
        $error_list = $this->app->initializeDataDirectories();
        if ($error_list)
        {
            return ajax_reply(300,___('Error while trying to create data directories. Maybe you can check the permission on the /web folder'));
        }
        $multiplefile_names = $_FILES['filedata']['name'];
        for ($i = 0, $count = ___c($multiplefile_names); $i < $count; $i++) {
            $base_filename = strtolower(convert_into_sef_friendly_title(strip_tags(basename(pathinfo($multiplefile_names[$i], PATHINFO_FILENAME)))) . '.' . pathinfo($multiplefile_names[$i], PATHINFO_EXTENSION));
            $full_filename = $this->app->getDataFileFullPath('image_original', $base_filename);
            
            $valid_image = is_valid_uploaded_image_file($_FILES['filedata']['tmp_name'][$i], $full_filename);
            if ($valid_image)
            {

                $move_result = move_uploaded_file($_FILES['filedata']['tmp_name'][$i], $full_filename);
                if ($move_result) {
                    create_image_thumbnail($full_filename, $this->app->getDataFileFullPath('image_large', $base_filename), $int_xl_width, $int_xl_height);
                    create_image_thumbnail($full_filename, $this->app->getDataFileFullPath('image_thumbs', $base_filename), $int_thumb_width, $int_thumb_height);
                    $title = pathinfo(basename($multiplefile_names[$i]), PATHINFO_FILENAME);

                    $this->insertImageIntoSQL($base_filename, $title);
                } else
                    $upload_failed = true;
            } else
                    $upload_failed = true;
            //echo "Moving {$_FILES['filedata']['tmp_name'][$i]} to {$placeholder}{$multiplefile_names[$i]}\n";
        }
        if ($upload_failed)
            return ajax_reply(500, 'Upload Error - Cannot Upload');
        else
            return ajax_reply(200, 'Upload OK');
    }

    //_________________________________________________________________________//	

    private function insertImageIntoSQL($base_filename, $title = '') {
        global $SystemDB;

        $datavalues = [];
        $datavalues['title'] = empty($title) ? $base_filename : $title;
        $datavalues['virtual_filename'] = convert_into_sef_friendly_title($datavalues['title']);
        $datavalues['image'] = $base_filename;
        $datavalues ['category_id'] = (int) $_COOKIE[$this->app_name . '_currentCategory'];
        $datavalues['date_created'] = $datavalues['date_modified'] = $datavalues['date_available'] = get_current_datetime();
        $datavalues['status'] = 1;

        $SystemDB->simpleInsertInto($this->table_items, $datavalues);
        $new_id = $SystemDB->getLastInsertID();
        return true;
    }

    //_________________________________________________________________________//	

    public function RunInstallScript($sample_data = false) {
        parent::RunInstallScript($sample_data);

        $this->app->setConfig('xl_width', '1600');
        $this->app->setConfig('xl_height', '500');
        $this->app->setConfig('thumb_width', '256');
        $this->app->setConfig('thumb_height', '256');
    }

    //_________________________________________________________________________//	
    public function RunUninstallScript($uninstall_db, $uninstall_everything) {
        parent::RunUninstallScript($uninstall_db, $uninstall_everything);
        if ($uninstall_everything) {
            __del_tree($this->app->getDataDirectoryFullPath('image_original'));
            __del_tree($this->app->getDataDirectoryFullPath('image_thumbs'));
            __del_tree($this->app->getDataDirectoryFullPath('image_large'));
        }
    }
    
    public function viewMainPage()
    {
        // Fix - v2.1.7-2 (Dec 2, 2017) - ensure data directory exists before upload
        $this->app->ensureDataDirectoryExists();
        parent::viewMainPage();
    }

    //_________________________________________________________________________//	

    public function Run() {
        switch (fget_alphanumeric('action')) {
            case 'upload':
                return ajax_echo($this->ajaxUploadFiles());
                break;
            default:
                return parent::Run();
        }
    }

}

?>
