//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
/**
 * View Controller class
 */
SCHLIX.CMS.BootstrapSlideshowAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("bootstrapslideshow");
    };
    


    /**
     * Datatable row format: item title
     * @static
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_Image (elCell, oRecord, oColumn, oData) {

        var field_title = SCHLIX.Util.isUndefined(this.parentControl.element_data["field-item_title"]) ? "title" : this.parentControl.element_data["field-item_title"];
        var odata_id = oRecord.getData("id");
        var odata_cid = oRecord.getData("cid");
        var odata_ext = oRecord.getData("extension");
        var file_exists = oRecord.getData("file_exists");
        var itemTitle = oRecord.getData(field_title); // oRecord.getData("title");
        var itemLink = site_httpbase;
        if (itemTitle == null)
        {
            SCHLIX.log("formatDataTableCell_DefaultTitleColumn cannot process a null title", 'warn');
            return;
        }
        if (itemTitle.length > 40)
            itemTitle = itemTitle.substr(0, 40) + '...';

        var app_name = this.parentControl.app_name;
        if (odata_cid > 0)
        {
            SCHLIX.CMS.__default_formatFolderInDataTable(app_name, elCell, oRecord, itemTitle, site_httpbase + this.parentControl.schlix_application_url + 'action=editcategory&id=' + odata_cid);
        } else
        {
            var icon = file_exists== 'true' ?  '<img src="' + oRecord.getData('thumb_url') + '" /><br />' : '<i class="far fa-image"></i> [No image]';

            var the_id = oRecord.getData("id");
            var theValue = 'i' + the_id;
            if (itemTitle === '')
                itemTitle = '(Untitled)';
            //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
            // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            

            itemLink += this.parentControl.schlix_application_url + 'action=edititem&id=' + the_id;
            var item_class = 'dragdrop';
            if (itemTitle)
                itemTitle = SCHLIX.Util.escapeHTML(itemTitle);            

            elCell.innerHTML = '<a class="' + item_class + '" id="' + app_name + "-lnk-" + theValue + '" data-dragdrop-id="' + theValue + '" title="Click here to edit this item" href="' + itemLink + '">' + " " + icon + " " + itemTitle + '</a>';

        }
    };

    onShowImageUploadDialog ()
    {
        
        var currentCategory = this.cms_control.currentCategory;
        console.log(currentCategory);
        if (currentCategory === 'c0' || currentCategory === 0 || isNaN(currentCategory))
        {
            SCHLIX.Alert.warning('Please navigate to a category before trying to upload a file');
        } else
        {
            
        var obj = SCHLIX.CMS.getObject('schlix-cms-upload-image-dialog');
            obj.show();
        }
        
    }; // end func
    
    /**
     * SCHLIX item editor - preview image
     */
    previewImage () {
        var input = document.getElementById('image');
        if (input.files && input.files[0]) {
            var reader = new FileReader();

            reader.onload = function (e) {
                var preview = document.getElementById('preview_image');
                preview.src = e.target.result;
                preview.style.display = '';
            };

            reader.readAsDataURL(input.files[0]);
        }
    };
    
    onSubmitImageUploadDialog (event)
    {
        var prefix_file_size = SCHLIX.Dom.get('filelist-prefix-totalsize');
        var upload_form = SCHLIX.Dom.get('frm_upload_file');
        var btn_upload = SCHLIX.Dom.get('btn-media-upload');
        var btn_cancel = SCHLIX.Dom.get('btn-media-cancel');
        var total_file_size = parseInt(SCHLIX.Dom.get('__total_file_size').value, 10);
        var max_file_size = parseInt(SCHLIX.Dom.get('__max_file_size').value, 10);
        if (total_file_size > 0)
        {
            if (total_file_size > max_file_size)
            {
                SCHLIX.Alert.error('Total file size (' + this.formatFileSize(total_file_size) + ') is too large for this server. Max = ' + this.formatFileSize(max_file_size));
                event.preventDefault();
                event.stopPropagation();
                
                return false;
            }
            btn_upload.disabled = true;
            prefix_file_size.innerHTML = '<i class="fa fa-spinner fa-spin fa-2x fa-fw"></i>Uploading';
            btn_cancel.style.display = 'none';
        }
    };    
    
    onDOMReady (event)
    {
        SCHLIX.Event.on('filedata', 'change', this.updateFileListToUpload, this, true);
        SCHLIX.Event.on('image', 'change', this.previewImage, this, true);                
        SCHLIX.Event.on('btn-media-upload','click', this.onSubmitImageUploadDialog, this, true);
        
    }; // end func
 
    formatFileSize(size)
    {
        var result = size;
        if (size >= 1024 && size <= 1024*1024)
        {
            result = Math.round(size / 1024.0, 2) + ' Kb';
        } 
        else if (size > 1024*1024 && size <= 1024*1024*1024)
        {
            result = Math.round(size / (1024.0 * 1024.0), 2) + ' Mb';
        }
        else if (size > 1024*1024*1024)
        {
            result = Math.round(size / (1024.0*1024.0*1024.0), 2) + ' Gb';
        }
        return result;
    };
////////////////////////////////////////////////////////////
    updateFileListToUpload ()
    {
        var input = document.getElementById('filedata');
        var ul = document.getElementById('filelist');
        var prefix_file_size = SCHLIX.Dom.get('filelist-prefix-totalsize');
        var display_file_size = SCHLIX.Dom.get('filelist-display-totalsize');
        var total_file_size_input = SCHLIX.Dom.get('__total_file_size');
        
        var total_size = 0;
        
        while (ul.hasChildNodes()) {
            ul.removeChild(ul.firstChild);
        }
        // You've selected input.files.length files
        for (var i = 0; i < input.files.length; i++) {
            // input.files[i] is a file object
            var li = document.createElement("li");
            li.innerHTML = '<i class="far fa-file-image"></i> ' + input.files[i].name + ' (' + this.formatFileSize(input.files[i].size) + ')';
            total_size += input.files[i].size;            
            ul.appendChild(li);
        }
        total_file_size_input.value = total_size;
        prefix_file_size.innerHTML = "Total: ";
        display_file_size.innerHTML = this.formatFileSize(total_size);
        
    }; // end func

    
    runCommand (command, evt)
    {
        switch (command)
        {
            case 'new-item':
                this.redirectToCMSCommand("newitem");
                return true;
                break;
            case 'new-category':
                this.redirectToCMSCommand("newcategory");
                return true;
                break;
            case 'edit-current-category':
                var target = evt.target;                
                window.location = target.href;
                return true;
                break;                                
            case 'upload':
                this.onShowImageUploadDialog();
                break;
                
            case 'config':
                this.redirectToCMSCommand("editconfig");
                return true;
                break;
            case 'refresh':
                this.cms_control.refreshControls();
                return true;
                break;
            default:
                return super.runCommand(command, evt);
                break;
        }
        //alert("Received Command: " + command);
    }
};
