//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
/**
 * View Controller class
 */
SCHLIX.CMS.ContactsAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("contacts");
        this.map= null;
        this.geocoder= null;
        this.marker= null;

    };
/**
 * Datatable row format: action
 * @param {type} elCell
 * @param {type} oRecord
 * @param {type} oColumn
 * @param {type} oData
 * @param {type} oDataTable
 * @returns {undefined}
 */

    static formatAction (elCell, oRecord, oColumn, oData)
    {
        var id = oRecord.getData("id");
        var message = oRecord.getData("message");

        var icon = '<i class="fa fa-eye"></i> ';
        var message = '<div id="contact_msg_' + id + '" style="display:none">' + message + '</div>';
        elCell.innerHTML = message + '<a class="contact_view_message"  data-message-email="' + oRecord.getData('email') + '" data-message-from="' + oRecord.getData('name') + '" data-message-id="' + id + '">' + icon + ' View</a>';
    };
    /**
     * Automatically called when the DOM is ready
     * @param {event} event
     */
    onDOMReady (event)
    {
        SCHLIX.Event.on('btn_update_map', 'click', this.tryToCodeAddress, this, true);
        SCHLIX.Event.on('tab_map', 'click', this.initMap, this, true);
        SCHLIX.Event.delegate('schlix-contacts-messages-admin', 'click', this.showMessageByID, 'a.contact_view_message', this, true);
        this.initMap();
    }; // end func
///////////////////////////////////////////////////////////
    showMessageByID (event)
    {

        var el = SCHLIX.Dom.get(event.target.id);
        var tgt_message_id = el.getAttribute('data-message-id');
        var tgt_message_from = el.getAttribute('data-message-from');
        var tgt_message_email = el.getAttribute('data-message-email');
        var tgt_message_ip_address = el.getAttribute('data-message-ipaddress');
        var message = SCHLIX.Dom.get('contact_msg_' + tgt_message_id);
        var message_detail_placeholder = SCHLIX.Dom.get('view_detail_message_placeholder');
        message_detail_placeholder.innerHTML = '<p>From: <strong>' + tgt_message_from + " &lt;" + tgt_message_email + '&gt;</strong></p>';
        message_detail_placeholder.innerHTML += message.innerHTML;
        message_detail_placeholder.style.display = 'block';
        return false;
    }; // end func
    tryToCodeAddress ()
    {
        var map_zoom = document.getElementById('map_zoom');
        map_zoom.value = parseInt(map_zoom.value);
        if (map_zoom.value < 0 || map_zoom.value > 19)
            map_zoom.value = this.map.getZoom();

        var fields = ["street", "city", "province", "country"];
        var address = '';
        var address_array = new Array();
        for (var i = 0; i < fields.length; i++)
        {
            if (document.getElementById(fields[i]) != null)
            {
                address_array.push(document.getElementById(fields[i]).value);
            }
        }
        address = address_array.join(',');
           var schlix_app =  this; //SCHLIX.CMS.initializedController['SCHLIX.CMS.ContactsAdmin'];
        
        this.geocoder.geocode({'address': address}, function (results, status) {
        
        
        if (status === google.maps.GeocoderStatus.OK)
        {

            schlix_app.map.setCenter(results[0].geometry.location);
            //marker = null;
            //marker = new google.maps.Marker({ map: map, position: results[0].geometry.location });
            schlix_app.marker.setPosition(results[0].geometry.location);
            var latitude = document.getElementById('latitude');
            var longitude = document.getElementById('longitude');
            latitude.value = results[0].geometry.location.lat();
            longitude.value = results[0].geometry.location.lng();
//		alert(results[0].geometry.location);
        } else {
            alert("Geocode was not successful for the following reason: " + status);
        }
    });
    }; // end func 
    initMap ()
    {
        var map_canvas = document.getElementById("map_canvas");
        var latitude = document.getElementById('latitude');
        var longitude = document.getElementById('longitude');
        var map_zoom = document.getElementById('map_zoom');
        if (latitude != null && longitude != null)
        {
            var map_zoom_value = (map_zoom != null) ? parseInt(map_zoom.value) : 10;
            var latlng = (parseInt(latitude.value) != 0 && parseInt(longitude.value) != 0) ? new google.maps.LatLng(latitude.value, longitude.value) : new google.maps.LatLng(53.581092, -113.389893);
            var myOptions = {zoom: map_zoom_value, center: latlng, mapTypeId: google.maps.MapTypeId.ROADMAP};
            this.map = new google.maps.Map(map_canvas, myOptions);
            this.marker = new google.maps.Marker({map: this.map, position: latlng});
            this.marker.setDraggable(true);
            this.geocoder = new google.maps.Geocoder();
            
           var schlix_app =  this; //SCHLIX.CMS.initializedController['SCHLIX.CMS.ContactsAdmin'];
            
            google.maps.event.addListener(this.marker, 'dragend', function () {
                schlix_app.getCurrentMarkerPosition();
            });
            google.maps.event.addListener(this.map, 'dblclick', function (event) {
                var map_zoom = document.getElementById('map_zoom');
                map_zoom.value = schlix_app.map.getZoom();
                schlix_app.marker.setPosition(event.latLng);
            });
            google.maps.event.addListener(this.map, 'zoom_changed', function () {
                var map_zoom = document.getElementById('map_zoom');
                map_zoom.value = schlix_app.map.getZoom();
            });
        }

    }; // end func
///////////////////////////////////////////////////////////

    resizeMap ()
    {
        var map_canvas = document.getElementById('map_canvas');
        map_canvas.style.display = 'block';
           var schlix_app =  this; //SCHLIX.CMS.initializedController['SCHLIX.CMS.ContactsAdmin'];
        
        setTimeout(function () {
            var latitude = document.getElementById('latitude');
            var longitude = document.getElementById('longitude');
            var latlng = (latitude.value != 0 && longitude.value != 0) ? new google.maps.LatLng(latitude.value, longitude.value) : new google.maps.LatLng(53.581092, -113.389893);
            google.maps.event.trigger(schlix_app.map, 'resize');
            schlix_app.marker.setPosition(latlng);
            schlix_app.map.setCenter(latlng);
            //google.maps.event.trigger( map_canvas, 'resize' );
        }
        , 200);
    }; // end func
    getCurrentMarkerPosition ()
    {
        var map_zoom = document.getElementById('map_zoom');
        var latitude = document.getElementById('latitude');
        var longitude = document.getElementById('longitude');
        latitude.value = this.marker.getPosition().lat();
        longitude.value = this.marker.getPosition().lng();
        map_zoom.value = this.map.getZoom();

    }; // end func
    runCommand (command, evt)
    {
        switch (command)
        {
            case 'new-item':
                this.redirectToCMSCommand("newitem");
                return true;
                break;
            case 'new-category':
                this.redirectToCMSCommand("newcategory");
                return true;
                break;
            case 'config':
                this.redirectToCMSCommand("editconfig");
                return true;
                break;
            case 'refresh':
                this.cms_control.refreshControls();
                return true;
                break;
            default:
                return super.runCommand(command, evt);
                break;
        }
        //alert("Received Command: " + command);
    };
};
