<?php

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

namespace App;

//error_reporting(E_ALL);
class Contacts extends \SCHLIX\cmsApplication_HierarchicalTree_List {

    protected $Captcha = null;
    protected $enable_captcha = false;
    protected $table_messages = 'gk_contact_messages';
    protected $wait_time_in_seconds_before_next_contact;

    public function __construct() {
        parent::__construct(___('Contacts'), 'gk_contact_items',  'gk_contact_categories');
        $this->enable_captcha = $this->getConfig('bool_enable_captcha');

        $this->wait_time_in_seconds_before_next_contact = $this->getConfig('int_seconds_before_next_contact');
        // this is so that the sql query in getLastContactCountByIPAddressWithinLimit works
        if ($this->wait_time_in_seconds_before_next_contact == 0)
            $this->wait_time_in_seconds_before_next_contact = strval('0');
        $this->has_versioning = true;
    }

    //_______________________________________________________________________________________________________________//
    public function getLastContactCountByIPAddressWithinLimit($ip) {
        global $SystemDB;

        $sanitized_ip_address = sanitize_string($ip);
        // don't use mysql time
        $current_date_time = sanitize_string(get_current_datetime());
        $sql = "SELECT COUNT(`id`) as total_count FROM gk_contact_messages WHERE ip_address={$sanitized_ip_address} AND (date_created > {$current_date_time} - INTERVAL {$this->wait_time_in_seconds_before_next_contact} SECOND)";
        $result = $SystemDB->getQueryResultSingleRow($sql, false);
        return $result['total_count'];
    }
     

    //_______________________________________________________________________________________________________________//
    public function viewItemByID($id = 1, $from_cache = false) {

        parent::viewItemByID($id, $from_cache);
        
        $this->viewContactFormByID($id);
    }

    //_______________________________________________________________________________________________________________//
    public function viewMainPage() {
        global $SystemDB;

        $item_count = $this->getTotalItemCount('', $this->cache);
        if ($item_count == 0) {
            echo \__Html::P(CONTACTS_NO_CONTACTS);
        }
        if ($item_count == 1) {
            $sql = "SELECT id FROM {$this->table_items}";
            $reply = $SystemDB->getQueryResultArray($sql, true);
            $this->viewItemByID($reply[0]['id']);
        } else if ($item_count > 1) {
            $category_sort_by = $this->getConfig('str_mainpage_category_sortby', 'title');
            $category_sort_dir = $this->getConfig('str_mainpage_category_sortdirection', 'ASC');
            
            $item_sort_by = $this->getConfig('str_mainpage_item_sortby', 'title');
            $item_sort_dir = $this->getConfig('str_mainpage_item_sortdirection', 'ASC');

            $categories = $this->getAllCategories('*', 'status > 0 AND parent_id = 0', 0, 100, $category_sort_by, $category_sort_dir); 
            $items = $this->getAllItems('*', 'status > 0 AND category_id = 0', 0, 100, $item_sort_by, $item_sort_dir); 
            //display single category
            if (___c($categories) == 1)
                $this->viewCategoryByID($categories[0]['cid'], fget_int('pg'), 'title', 'ASC');
            else {
                $this->setPageTitle($this->app_description);
                $this->loadTemplateFile('view.main', get_defined_vars());
            }
        }
    }

    //_______________________________________________________________________________________________________________//
    protected function isWindowsServerAndSMTPHasNotBeenSetUp() {
        return (SCHLIX_SYSTEM_MSFTWIN  && SCHLIX_SMTP_HOST == '');
    }

    
    //_______________________________________________________________________________________________________________//
    public function viewContactFormByID($id) {
        $item = $this->getItemByID($id);
        if ($item)
        {
            if ($this->enable_captcha) {
                $this->Captcha = new securimage();
                $this->Captcha->session_name = SCHLIX_SESSION_NAME;
            }
            
            $captchaurl = SCHLIX_SITE_HTTPBASE . '/captcha.php';
            $codecheck = (($this->enable_captcha) && ($this->Captcha != null)) ? $this->Captcha->check( fpost_string_noquotes_notags('verification_code')) : true;
            if (!$this->getConfig('bool_disable_contact_form')) {
                $variables = compact(array_keys(get_defined_vars()));                
                $this->loadTemplateFile('view.item.contact_form', $variables);
            }
            
        } else
        {
            echo ___("Error: Contact cannot be found");
        }
    }

    
    /**
     * Process the AJAX message submission
     * @return array
     */
    public function ajaxSubmitMessage()
    {
        global $SystemDB, $SystemMail;
        $error_list = [];

        check_csrf_halt_on_error(true);
        $id = fpost_int('contact_id');
        $item = $this->getItemByID($id);
        $is_contact_form_disabled = $this->getConfig('bool_disable_contact_form');
        // in case posted with other stuff
        $submit_message = fpost_int('submitmessage');
        if ($submit_message != 1)
            return NULL;
        if (!$is_contact_form_disabled)
        {
             // if the item exists and it's not inactive
            if ($item && $item['status'] > 0)
            {
                $input_email = fpost_string('email',255);
                $input_name = fpost_string('name', 255);
                $input_subject = fpost_string('subject', 255);
                $input_phone = fpost_string('phone', 63);
                $input_message = fpost_string('message', 4095);
                $codecheck = (($this->enable_captcha) && ($this->Captcha != null)) ? $this->Captcha->check( fpost_string_noquotes_notags('verification_code')) : true;

                $captcha_validated = ($codecheck !== false);

                $flood_control = $this->getLastContactCountByIPAddressWithinLimit(get_user_real_ip_address());
                $flood_control_validated = ($flood_control == 0 || $this->wait_time_in_seconds_before_next_contact == 0);
                $valid_email_address = filter_var($input_email, FILTER_VALIDATE_EMAIL);


                if (!$valid_email_address) $error_list[] = ___('Invalid email address!');
                if (!$flood_control_validated) $error_list[] = sprintf(___('Double submit detected - please wait for %d seconds before posting again!'), $this->wait_time_in_seconds_before_next_contact);                
                if (!$captcha_validated) $error_list[] = ___('Invalid Captcha!');
                if (!$input_name) $error_list[] = ___('Please enter your name');
                if (!$input_subject) $error_list[] = ___('Please enter the subject of the message');
                if (!$input_message) $error_list[] = ___('Please enter the message');

                if (empty($error_list))
                {
                    
                    ////////////////
                    
                    $datavalues = [
                        'contact_id' => $id,
                        'name' => $input_name,
                        'email' => $input_email,
                        'phone' => $input_phone,
                        'subject' => $input_subject,
                        'message' => $input_message,
                        'ip_address' => get_user_real_ip_address(),
                        'user_agent' => $_SERVER['HTTP_USER_AGENT'],
                        'date_created' => get_current_datetime()];


                    $SystemDB->simpleInsertInto($this->table_messages,$datavalues);
                    // add $sender_email
                    $datavalues['sender_email'] = $input_email;
                    $datavalues['browser'] = $datavalues['user_agent'];
                    $sendmail_result = $SystemMail->sendEmailWithTemplateName(SCHLIX_MAIL_DEFAULT_SENDER, SCHLIX_MAIL_DEFAULT_EMAIL, $item['title'], $item['email'], '', '', $input_name, $input_email, 'contact-form', $datavalues);                    
                    ////////////////
                    
                    
                    if (!$sendmail_result) 
                        $error_list[] = ___('Error while using this SMTP server');
                }
            } else $error_list[] = ___('Invalid contact ID');
        } else $error_list[] = ___('Contact form is disabled');
        // Now reply
        if (empty($error_list))
        {
            return ajax_reply (200, ___('Thank you. Your message has been sent'));
        } else
            return ajax_reply(300,$error_list);
    }
            

    //_______________________________________________________________________________________________________________//
    public function Run($command) {
        switch ($command['action']) {
            case 'sm': return ajax_echo($this->ajaxSubmitMessage());
            default: return parent::Run($command);
        }
        return false;
    }


}

