/* 
 * Copyright (C) 2015 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.html for details
 */

/**
 * View Controller class
 */
SCHLIX.CMS.MacrosAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("macros");
    };

    onDOMReady ()
    {
        this.initMacroEditor();
        SCHLIX.Event.on('zipfileupload', 'change', this.previewFile, this, true);        
    }; // end func
    ////////////////////////////////////////////////////////////
    
    previewFile ()
    {
        var file_input = document.getElementById('zipfileupload');
        var file_info = file_input.files[0];
        var file_name_placeholder = document.getElementById('zip_filename');
        var file_size_placeholder = document.getElementById('__file_size');
        var file_size_display=  document.getElementById('zip_filesize');
        
        file_size_placeholder.value = file_info.size;
        file_name_placeholder.innerHTML = '<i class="far fa-file-archive"></i> ' + file_info.name;
        
        var max_file_size = parseInt(document.getElementById('__max_file_size').value, 10);
        if (file_info.size > max_file_size)
        {
            var max_file_size_formatted = this.formatFileSize(max_file_size);
            SCHLIX.Alert.warning('File upload will fail since the file is larger than the maximum upload size on this server (' +max_file_size_formatted + ')');
        }
        
        file_size_display.innerHTML = '<strong>File size</strong>: ' + format_file_size(file_info.size);
    }; // end func
     
    
    initMacroEditor ()
    {
        if (SCHLIX.Dom.get('schlix-macros-admin') && SCHLIX.Dom.get('apps_select_all'))
        {
            var display_in_apps = document.forms.macro_item_editor.display_in_apps_workaround.value;
            var display_in_blocks = document.forms.macro_item_editor.display_in_blocks_workaround.value;
            SCHLIX.Event.on('apps_select_all', "click", this.hideAppSelections, this, true);
            SCHLIX.Event.on('apps_select_some', "click", this.showAppSelections, this, true);
            SCHLIX.Event.on('blocks_select_all', "click", this.hideBlockSelections, this, true);
            SCHLIX.Event.on('blocks_select_some', "click", this.showBlockSelections, this, true);

            if (display_in_apps == 1)
                this.hideAppSelections();
            if (display_in_blocks == 1)
                this.hideBlockSelections();
            console.log('display_in_apps ' + display_in_apps);
            console.log('display_in_blocks ' + display_in_blocks);
        }
    }; // end func
///////////////////////////////////////////////////////////
    hideAppSelections ()
    {
        var div = document.getElementById('app_selections');
        div.style.display = 'none';
    }; // end func
///////////////////////////////////////////////////////////
    showAppSelections (str)
    {
        var div = document.getElementById('app_selections');
        div.style.display = 'block';
    }; // end func
///////////////////////////////////////////////////////////
    hideBlockSelections ()
    {
        var div = document.getElementById('block_selections');
        div.style.display = 'none';
    }; // end func
///////////////////////////////////////////////////////////
    showBlockSelections (str)
    {
        var div = document.getElementById('block_selections');
        div.style.display = 'block';
    }; // end func
    showInstallerWindow ()
    {
        var obj = SCHLIX.CMS.getObject('schlix-cms-package-installer');
        obj.show();
    }; // end func
    submitForm ()
    {
        /* var obj = SCHLIX.CMS.getObject('schlix-cms-package-installer');
         var form = obj.__getFirstFoundForm();
         console.log(form);*/
        return RESOLVE_TO_PARENT;
    }; // end func
    /**
     * Run command
     * @param {type} command
     * @returns {unresolved}
     */
    runCommand (command, evt)
    {
        switch (command)
        {
            case 'install':
                this.showInstallerWindow();
                break;
            case 'dialog-submit':
                return this.submitForm();
                break;
            case 'config':
                this.redirectToCMSCommand("editconfig");
                return true;
                break;
            case 'form-submit-ok':
                this.cms_control.refreshControls();
                break;
            default:
                return super.runCommand(command, evt);
                break;
        }
    }
};