<?php

/* 
 * Copyright (C) 2016 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.txt for details
 */

namespace App;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++//

class Menus extends \SCHLIX\cmsApplication_CategorizedList
{

    protected $element_ul_class;
    protected $element_li;
    protected $element_ul;
    protected $submenu_icon;
    protected $tmpstore = [];

    public function __construct()
    {
        global $CurrentUser;

        $this->disable_frontend_runtime = true; // not enabled for public
        parent::__construct('Menus', 'gk_menu_items', 'gk_menu_categories');
        $this->backend_mode = ($CurrentUser != null);
        $this->use_menu_human_readable_css = true; // $this->getConfig('int_menu_human_readable_css');
        $this->setOutputHTMLElements();
        $this->refreshAllMenuLinks(false);
    }

    //_______________________________________________________________________________________________________________//
    protected function setOutputHTMLElements()
    {
        if ($this->getConfig('int_menu_html_element') == 1 && !$this->backend_mode) {
            $this->element_ul = 'div';
            $this->element_li = 'div';
        }
        else {
            $this->element_ul = 'ul';
            $this->element_li = 'li';
        }
        $this->element_ul_class = "dropdown-menu";
    }

    //_______________________________________________________________________________________________________________//
    public function refreshAllMenuLinks($force = false) // new, IIS fix only
    {
        global $SystemDB, $SystemConfig;

        $httpbase = $SystemConfig->get('system', 'str_httpbase'); // first time install only?
        if (($httpbase === null) || $force) {
            $sql = "SELECT * FROM {$this->table_items} order by id";
            $menus = $SystemDB->getQueryResultArray($sql);
            foreach ($menus as $menu)
            {
                switch ($menu['application'])
                {
                    case 'home': if (SCHLIX_SITE_HTTPBASE)
                            $sefurl = SCHLIX_SITE_HTTPBASE;
                        else
                            $sefurl = '/';break;
                    case 'external_link':$sefurl = $menu['customurl'];
                        break;
                    default:
                        if (get_system_or_user_file("/apps/{$menu['application']}/{$menu['application']}.class.php", false))
                        {
                            $the_app_name = '\\App\\'.$menu['application'];
                            $temp_app = new $the_app_name;
                            $sefurl = $temp_app->createFriendlyURL($menu['internalurl']);
                        }
                        break;
                }
                $sql = "UPDATE {$this->table_items} SET sefurl = '{$sefurl}' WHERE id = {$menu['id']}";
                $SystemDB->query($sql);
            }
            $SystemConfig->set('system', 'str_httpbase', SCHLIX_SITE_HTTPBASE);
        }
    }



    //_______________________________________________________________________________________________________________//
    /**
     * Get the current menu ID based on the URL entered, e.g. http://website.com/html/test123.html and if it's in the menu it will return
     * that ID from gk_menu_items table
     *
     * @return -1 if it's not anywhere, or an integer of the menu id
     */
    public static function getCurrentMenuID()
    {
        $menu_info = menus::getCurrentMenuInformation();
        return ($menu_info) ? $menu_info['id'] : NULL;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Get the current menu ID based on the URL entered, e.g. http://website.com/html/test123.html and if it's in the menu it will return
     * that ID from gk_menu_items table
     *
     * @return -1 if it's not anywhere, or an integer of the menu id
     */
    public static function getCurrentMenuInformation()
    {
        // TODO: expand/include subchild - Prana - Oct 1, 2010
        global $SystemDB;

        $current_menu_info = \SCHLIX\cmsContextCache::get('menu','current_menu_info');
        if ($current_menu_info)
        {
            return $current_menu_info;
        } else
        {
            $base = "'" . SCHLIX_SITE_HTTPBASE . "'";
            if (SCHLIX_SITE_HTTPBASE == '')
                $base = "'/'";
            $url = sanitize_string($_SERVER['REQUEST_URI']);

            if ($_SERVER['REQUEST_URI'] == SCHLIX_SITE_HTTPBASE . '/')
                $url = $base; // special exception for base Oct 11, 2010
            $sql = "SELECT * FROM gk_menu_items WHERE sefurl = {$url} OR internalurl = {$url}";
            $menu = $SystemDB->getQueryResultSingleRow($sql, true);
            if (!$menu) {
                // test
                if ($_SERVER['REQUEST_URI'] == '/')
                    $sql = "SELECT * FROM gk_menu_items WHERE sefurl='/'";
                else
                    $sql = "SELECT * FROM gk_menu_items WHERE (INSTR({$url},sefurl) > 0) AND (sefurl <> {$base}) ";
                $menu = $SystemDB->getQueryResultSingleRow($sql, true);
                $menu['exact_url'] = false;
                $menu['inside_page'] = true;
            } else {
                
                $menu['exact_url'] = true;
                $menu['inside_page'] = false;
            }
             \SCHLIX\cmsContextCache::set('menu','current_menu_info', $menu);
            return $menu;
        }
    }

//_________________________________________________________________________//
    public function traverseMenuItem($category_id, $item_parent_id, $display_menu_function, $display_invisible = false)
    {
        global $SystemDB, $CurrentUser;


        $output = '';
        $category_id = (int) $category_id;
        $item_parent_id = intval($item_parent_id);
        if ($item_parent_id > 0) {
            if ($display_invisible == false)
                $s = " AND status > 0";
            else
                $s = '';

            $items = $this->tmpstore[$item_parent_id];
            
            if ($items) {
                $output.= "\n<{$this->element_ul} class=\"{$this->element_ul_class}\">"; //UL_start();
                foreach ($items as $item)
                {
                    ///	$can_read = $CurrentUser->hasReadPermission($item['permission_read']);
                    $can_read = ($this->backend_mode) ? true : $CurrentUser->hasReadPermission($item['permission_read']);
                    if ($can_read) {
                        $child_output = trim($this->traverseMenuItem($category_id, $item[$this->field_id], $display_menu_function));
                        $has_submenu = !empty($child_output);
                        //$menulink_output = $display_menu_function($item, $has_submenu);					
                        $menulink_output = call_user_func($display_menu_function, $item, $has_submenu);
                        $li_class = empty($child_output) ? "" : " class=\"dropdown-submenu\" ";
                        $output.="\t<{$this->element_li}{$li_class}>{$menulink_output}{$child_output}<{$this->element_li}>\n";
                    }
                }
                $output.= "</{$this->element_ul}>\n\n";
            }
        }
        return $output;
    }

    /**
     * TODO: fix this - just a temporary solution to reduce sql queries
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param type $category_id
     */
    protected function storeMenuParents($category_id, $extra_qry)
    {
        global $SystemDB;
        
        $sql = "SELECT * FROM {$this->table_items} where category_id = {$category_id} {$extra_qry} order by sort_order"; // fix sept 5, 2010
        $items = $SystemDB->getQueryResultArray($sql, !$display_invisible);
        foreach ($items as $item)
        {
            $item_parent_id = $item['parent_id'];
            if (!array_key_exists($item_parent_id, $this->tmpstore))
            {
                $this->tmpstore[$item_parent_id] = [];
            }
            $this->tmpstore[$item_parent_id][] = $item;
        }
    }
    //_________________________________________________________________________//
    public function displayMenuByCategoryID($category_id, $display_menu_function, $display_invisible = false, $display_block_name = '', $menu_css_class = '')
    {
        global $SystemDB, $CurrentUser;
        // Get Menu Block
        $category_id = (int) $category_id;
        if ($display_invisible == false)
            $extra_qry = " AND status > 0";
        else
            $extra_qry = '';
        $sql = "SELECT * FROM {$this->table_items} where parent_id = 0 and category_id = {$category_id} {$extra_qry} order by sort_order"; // fix sept 5, 2010
        // Nov 29, 2011 - only enable cache if display invisible = true (meaning it's being accessed from the admin)
        $main_menus = $SystemDB->getQueryResultArray($sql, !$display_invisible);
        $this->storeMenuParents($category_id, $extra_qry);
        $css_id = (!empty($display_block_name) && $this->use_menu_human_readable_css) ? $display_block_name : "menublock_{$category_id}";
        $css_class = !empty($menu_css_class) ? $menu_css_class : "nav navbar-nav";
        if ($main_menus) {
            $output = "\n<{$this->element_ul} id=\"{$css_id}\" class=\"{$css_class}\">"; //UL_start('menublock_'.$category_id,'menublock');
            foreach ($main_menus as $menu)
            {
                $can_read = ($this->backend_mode) ? true : $CurrentUser->hasReadPermission($menu['permission_read']);
                if ($can_read) {
                    $child_output = trim($this->traverseMenuItem($category_id, $menu[$this->field_id], $display_menu_function, $display_invisible));
                    $has_submenu = !empty($child_output);
                    $menu['_block_name'] = $display_block_name;
                    //$menulink_output = $display_menu_function($menu, $has_submenu);	
                    $menulink_output = call_user_func($display_menu_function, $menu, $has_submenu);
                    $li_class = empty($child_output) ? " class=\"nav-item\" " : " class=\"dropdown\" ";
                    $li_id = " id=\"block_{$display_block_name}_li_{$menu['id']}\"";
                    $output.="\t<{$this->element_li}{$li_id}{$li_class}>{$menulink_output}{$child_output}</{$this->element_li}>\n";
                    /*
                      $menulink_output = $display_menu_function($menu);
                      $child_output =$this->traverseMenuItem($category_id, $menu['id'],$display_menu_function,$display_invisible);
                      $output.="<{$this->element_li}>{$menulink_output}{$child_output}</{$this->element_li}>\n"; */
                }
            }
            $output.= "</{$this->element_ul}>\n"; //UL_end();
        }
        else
            $output = "This menu block is still empty.";
        return $output;
    }

    //_________________________________________________________________________//
    public function getMenuArrayByCategoryID($category_id, $parent_id = 0)
    {
        global $SystemDB, $CurrentUser;
        // Get Menu Block
        $category_id = (int) $category_id;
        $parent_id = (int) $parent_id;
        $sql = "SELECT * FROM {$this->table_items} where parent_id = {$parent_id} and category_id = {$category_id} {$s} order by sort_order"; // fix sept 5, 2010
        // Nov 29, 2011 - only enable cache if display invisible = true (meaning it's being accessed from the admin)
        $main_menu_items = $SystemDB->getQueryResultArray($sql, !$display_invisible);
        if ($main_menu_items) {
            foreach ($main_menu_items as $menu)
            {
                $child_menu_items = $this->getMenuArrayByCategoryID($category_id, $menu[$this->field_id]);
                if (!empty($child_menu_items))
                    $main_menu_items = array_merge($main_menu_items, $child_menu_items);
            }
        }
        return $main_menu_items;
    }

    /**
     * Before save item
     * @param array $datavalues
     * @return array
     */
    public function modifyDataValuesBeforeSaveItem($datavalues) {
        $datavalues = parent::modifyDataValuesBeforeSaveItem($datavalues);
        
        
        // Process menu types
        $datavalues['open_in_new_window'] = (int) $datavalues['open_in_new_window'];
        $datavalues['status'] = (int) $datavalues['status']; //Sept 5, 2010
        $datavalues['use_font_icon_only_no_text'] = (int) $datavalues['use_font_icon_only_no_text']; //Sept 5, 2010

        if( $datavalues['subtemplate'] === '')
            $datavalues['subtemplate'] = 'index';
            
        if( $datavalues['subtemplate_mobile'] === '')
            $datavalues['subtemplate_mobile'] = $datavalues['subtemplate'];        
        if( $datavalues['subtemplate_inner_pages'] === '')
            $datavalues['subtemplate_inner_pages'] = $datavalues['subtemplate'];
        if ($datavalues['application'] == 'home') {
            $datavalues['internalurl'] = '/' . SCHLIX_SITE_HTTPBASE;
            $datavalues['sefurl'] = '/' . SCHLIX_SITE_HTTPBASE;
        }
        else
        if ($datavalues['application'] == 'external_link') {
            $datavalues['internalurl'] = $datavalues['customurl'];
            $datavalues['sefurl'] = $datavalues['customurl'];
        }
        else
        if ($datavalues['application'] == 'menu_divider' || $datavalues['application'] == 'menu_placeholder') {
            $datavalues['internalurl'] = '';
            $datavalues['sefurl'] = 'javascript:void(0)';
        }
        
        else {
            ////// ??
            $main_class = $datavalues['application'];
            $child_class = '';
            if (strpos($datavalues['menuaction'],':'))
            {
                $childaction = explode(':', $datavalues['menuaction']);
                $datavalues['application'] =  $datavalues['application'].'.'.$childaction[0];
                $datavalues['menuaction'] = $childaction[1];
                $child_class = '_'.$childaction[0];
            }
            //// ??
            switch ($datavalues['menuaction'])
            {
                case 'standard_main_app':
                    $datavalues['internalurl'] = "app={$datavalues['application']}";
                    $datavalues['menuitem'] = '';
                    break;
                case 'standard_browse':
                    $itemid = intval(substr($datavalues['menuitem'], 1, strlen($datavalues['menuitem']) - 1));
                    if ($datavalues['menuitem'][0] == 'c')
                        $action = "viewcategory&cid={$itemid}";
                    else if ($datavalues['menuitem'][0] == 'i')
                        $action = "viewitem&id={$itemid}";
                    $datavalues['internalurl'] = "app={$datavalues['application']}&action={$action}";
                    break;
                default:
                    $datavalues['internalurl'] = "app={$datavalues['application']}&action={$datavalues['menuaction']}";
                    break;
            }
            // load temp
           // include_app_class($datavalues['application']);
            $the_app_name = '\\App\\'.$datavalues['application'];
            $temp_app = new  $the_app_name; //$datavalues['application'];

            // fix up quirks for non-SEF URL (internal)
            parse_str($datavalues['internalurl'], $tmp_url_array);
            $array_tmp_keys = array_keys($tmp_url_array); // separate it
            foreach ($array_tmp_keys as $key)
            {
                $value = trim($tmp_url_array[$key]);
                if (empty($value))
                    unset($tmp_url_array[$key]);
            }
            $datavalues['internalurl'] = http_build_query($tmp_url_array);
            ////////////

            $datavalues['sefurl'] = $temp_app->createFriendlyURL($datavalues['internalurl']);
        }
        // Fix - Nov 3, 2013
        if ($datavalues['application'] != 'external_link') {
            $datavalues['internalurl'] = remove_multiple_slashes($datavalues['internalurl']);
            $datavalues['sefurl'] = remove_multiple_slashes($datavalues['sefurl']);
        }

        
        return $datavalues;
    }
    
    
    /**
     * Modify some stuff
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param array $datavalues
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        global $SystemDB;
        
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        $item_id = (int) $retval['id'];
        if ($retval['is_new'])
        {
            $sql = "UPDATE {$this->table_items} SET sort_order=id WHERE {$this->field_id} = '{$item_id}';";
            $SystemDB->query($sql);
        }
        
    }

}