/* 
 * Copyright (C) 2015 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.html for details
 */
/**
 * Element class
 */

SCHLIX.CMS.UserExplorer = class extends SCHLIX.CMS.DataExplorerManyToMany  {  
    /**
     * Constructor
     */
    constructor (el, options)
    {
        super(el, options);
        this.field_category_id = 'cid';
        this.field_category_title = 'title';
        this.field_item_title = 'username';
        
        this.__first_time_load = true;
    };

    /**
     * Datatable row format: item status
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @returns {undefined}
     */
    formatDataTableCell_Status (elCell, oRecord, oColumn, oData) {
        var button_name = '';
        var color = '';

        switch (oData)
        {
            case -2:
                button_name = "question-circle";
                color = 'orange';
                break;
                // more ...
            case 1:
                button_name = "check-circle";
                color = 'green';
                break;
                // more ...
                
            default:
                button_name = "times-circle";
                color = 'red';
                break;
        }
        elCell.innerHTML = '<i class="fa fa-' + button_name + '" style="color:' + color + '"></i>';
    }; // end func 

    // --------------------------------- VARIABLES ------------------------------- //
    formatUserName (elCell, oRecord, oColumn, oData) {

        var field_title = SCHLIX.Util.isUndefined(this.parentControl.element_data["field-item_title"]) ? "title" : this.parentControl.element_data["field-item_title"];

        var odata_id = oRecord.getData("id");
        var odata_cid = oRecord.getData("cid");
        var app_name = this.parentControl.app_name;
        var is_category = oRecord.getData("cid") > 0;
        var the_id = is_category ? odata_cid : odata_id;
        var itemTitle = is_category ? oRecord.getData("groupname") : oRecord.getData(field_title);
        if (itemTitle == null) itemTitle = '???';
        itemTitle = (itemTitle.length > 40) ? itemTitle.substr(0, 40) + '...' : (itemTitle === '') ? '(Untitled)' : itemTitle;
        var editIcon = is_category ?  "<i class=\"fas fa-pencil-alt\"></i> " : '';
        var icon = is_category ? "<i class =\"fa fa-folder fa-2x\"></i>" : "<i class =\"fa fa-user fa-2x\"></i>";      
        var theValue = is_category ? 'c' + the_id : 'i' + the_id;
        var action_link = is_category ? 'editcategory' : 'edititem';
        var itemLink = site_httpbase + this.parentControl.schlix_application_url + 'action=' + action_link +  '&id=' + the_id;
        if (itemTitle)
            itemTitle = SCHLIX.Util.escapeHTML(itemTitle);
        if (is_category)
            elCell.innerHTML = '<div style="float:left" >' + '<a  title="Click here to go into this category" class="datatable-folder dragdrop" href="#"  data-category-id="' + the_id + '" data-dragdrop-id="' + theValue + '" id="' + app_name + "-lnk-" + theValue + '">' + " " + icon + " " + itemTitle + '</a>' +
                    '</div>  <div style="float:right;margin-left:4em"><a title="Edit this category" href="' + itemLink + '">' + editIcon + '</a></div>';
        else
            elCell.innerHTML = '<a class="dragdrop" id="' + app_name + "-lnk-" + theValue + '" data-dragdrop-id="' + theValue + '" title="Click here to edit this item" href="' + itemLink + '">' + " " + icon + " " + itemTitle + '</a>'; 
    };
};

SCHLIX.CMS.registerComponent('schlix-user-explorer', SCHLIX.CMS.UserExplorer);


/****************************************************/
/**
 * View Controller class
 */
SCHLIX.CMS.UsersAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("users");
    };
    
    static formatHistoryDescription (elCell, oRecord, oColumn, oData) {

        var itemTitle =  oRecord.getData('description');
        if (itemTitle == null) itemTitle = '???';
        itemTitle = (itemTitle.length > 100) ? itemTitle.substr(0, 100) + '...' : (itemTitle === '') ? '(Untitled)' : itemTitle;
       
        elCell.innerHTML = itemTitle;
    };
    
    
    setSendPasswordResetLinkButtonVisibility(state)
    {
        var send_password_reset = SCHLIX.Dom.get('send_password_reset');
        if (send_password_reset)
        {
            send_password_reset.style.display = state ? 'inline-block' : 'none' ;
        }        
    }
    
    setChangePasswordFieldVisibility(state)
    {
        SCHLIX.Dom.get('change_password_div').style.display = state ? 'block' : 'none';       
        if (state)
        {
            SCHLIX.Dom.get('change_email').checked = false;
            SCHLIX.Dom.get('change_email_opt_div').style.visibility = 'hidden';
            this.setChangeEmailFieldVisibility(false);
        } else
        {
            SCHLIX.Dom.get('change_email_opt_div').style.visibility = 'visible';
        }
    }

    setChangeEmailFieldVisibility(state)
    {
        SCHLIX.Dom.get('change_email_div').style.display = state ? 'block' : 'none';
        if (state)
        {
            SCHLIX.Dom.get('change_password').checked = false;
            SCHLIX.Dom.get('change_password_opt_div').style.visibility = 'hidden';            
            this.setChangePasswordFieldVisibility(false);
        } else
        {
            SCHLIX.Dom.get('change_password_opt_div').style.visibility = 'visible';            
        }
        
    }
    
    onChangePasswordClick(e)
    {
        var is_checked = SCHLIX.Dom.get('change_password').checked;
        this.setChangePasswordFieldVisibility(is_checked);
        this.setSendPasswordResetLinkButtonVisibility(!is_checked);
    }
    
    onChangeEmailClick(e)
    {
        var is_checked = SCHLIX.Dom.get('change_email').checked;
        this.setChangeEmailFieldVisibility(is_checked);
        this.setSendPasswordResetLinkButtonVisibility(!is_checked);        
    }

    onPasswordResetSent(o)
    {
        var response = ajax_parse(o.responseText);
        if (o.status == 200)
        {
            SCHLIX.Alert.info(response);
        }
    }
    
    
    sendPasswordResetLink(e)
    {
        var email_address =SCHLIX.Dom.get('original_email_address');
        if (email_address)
        {
            var callback = {success: this.onPasswordResetSent, failure: this.handleFailure, scope: this};
            var the_id = SCHLIX.Dom.get('id').value;
            var the_email = SCHLIX.Dom.get('original_email_address').value;
            var postData = "id=" + the_id + '&email=' + the_email ;
            var request = SCHLIX.Ajax.POST( site_httpbase + "/admin/app/" + this.app_name + "?action=sendpasswordreset", callback, postData);
                    
            
        } else SCHLIX.Alert('Something is wrong - email address not found');
    }
    
    
    previewAvatar () {
        var input = document.getElementById('user_avatar');
        if (input.files && input.files[0]) {
            var reader = new FileReader();

            reader.onload = function (e) {
                var preview = document.getElementById('preview_avatar');
                preview.src = e.target.result;
                preview.style.display = '';
            };

            reader.readAsDataURL(input.files[0]);
        }
    }    
    
    
    onDOMReady()
    {
        SCHLIX.Event.on('user_avatar', 'change', this.previewAvatar, this, true);        
        SCHLIX.Event.on('change_email','change',this.onChangeEmailClick, this, true);
        SCHLIX.Event.on('change_password','change',this.onChangePasswordClick, this, true);
        SCHLIX.Event.on('btn_send_password_reset','click',this.sendPasswordResetLink, this, true);
     //   SCHLIX.Event.on(SCHLIX.Dom.get('{.confirm_status}'),'click',this.onChangeUserStatus);
        
    }

    runCommand  (command, evt)
    {
            return super.runCommand(command, evt);
    };
    
} // end class