<?php

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// TODO - April 14, 2012 - Fully separate the language strings to lang_en_us.php .. clean up code (especially securimage, etc), lots of new addition and although functional some are spaghetti code
namespace App;


define('USER_REQUIRES_ACTIVATION', -2);
define('USER_ACTIVATED_IMMEDIATELY', 1);

class Users extends \SCHLIX\cmsApplication_ManyToMany {

    private $time, $currentUserID;
    private $backend_mode = false;
    protected $field_password = 'password'; // TODO
    protected $field_username = 'username';  // TODO
    protected $field_groupname = 'groupname';
    protected $minimum_password_length = 7;
    
    protected $data_directories = array(
      'avatar_original' => '/data/user/avatars/original',
      'avatar_small' => '/data/user/avatars/small',
      'avatar_medium' => '/data/user/avatars/medium',      
      'avatar_large' => '/data/user/avatars/large',      
      'user_public_images' => '/data/user/public/images',
      'user_public_files' => '/data/user/public/files',
    );

    //_________________________________________________________________________//
    public function __construct($backend_mode = false) {
        parent::__construct('Users', 'gk_user_items', 'gk_user_categories','gk_user_categories_items');
        // 2016 - addition
        $this->field_item_title = $this->field_username;
        $this->field_category_title = $this->field_groupname;
                
        // end addition
        $this->time = time();
        $this->backend_mode = $backend_mode;
        $this->verifyCurrentSession();
        
    }

    /**
     * Returns the username field name in the item table
     * @return string
     */
    public function getFieldUsername()
    {
        return $this->field_username;
    }
    /**
     * Returns the groupname field name in the category table
     * @return string
     */
    public function getFieldGroupName()
    {
        return $this->field_groupname;        
    }
    /**
     * Returns total user count
     * @return type
     */
    public function getRegisteredUsersCount()
    {
        return $this->getTotalItemCount("status = 1");
    }
    //_______________________________________________________________________________________________________________//
    public function createFriendlyURL($str) {

        if (SCHLIX_SEF_ENABLED) {
            $command_array = [];
            parse_str($str, $command_array);
            if ($command_array['action'] == 'viewitem') {
                if (intval($command_array['id']) > 0) {
                    $user = $this->getUserByID($command_array['id']);
                    if ($user) {

                        $username = $user['username'];
                        $final_url.="/{$username}.html";
                    } else
                        $final_url.="/__unknown.html";
                } else
                    $final_url.="/__unknown.html";
            } else {
                return parent::createFriendlyURL($str);
            }
            $final_url = SCHLIX_SITE_HTTPBASE . '/' . $this->app_name . $final_url;
        } else
            return parent::createFriendlyURL($str);
        return remove_multiple_slashes($final_url);
    }

    //_________________________________________________________________________//
    public function getPasswordFieldName() {
        return $this->field_password;
    }

    //_________________________________________________________________________//
    public function getUsernameFieldName() {
        return $this->field_username;
    }

    //_______________________________________________________________________________________________________________//
    public function getFieldItemTitle() {
        return $this->field_username;
    }

    //_______________________________________________________________________________________________________________//
    protected function _internalCreateUser($datavalues, $creategroup = true) {
        global $SystemDB, $SystemConfig;

        $default_group_id = $SystemConfig->get($this->app_name, 'int_default_newuser_group_id');
        $SystemDB->query("LOCK TABLES `{$this->table_items}` WRITE");
        $SystemDB->simpleInsertInto($this->table_items, $datavalues);        
        $new_user_id = $SystemDB->getLastInsertID();
        $SystemDB->query("UNLOCK TABLES");
        if ($creategroup) {
            $this->setItemCategory($new_user_id, $default_group_id, true);
        }
    }

    //_______________________________________________________________________________________________________________//
    public function getItemsByCategoryID($id, $fields = '*', $extra_criteria = '', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC', $from_cache = false) {
        global $SystemDB;
        if ($fields == '*')
            $fieldnames = $this->getItemFieldNames();
        else
            $fieldnames = explode(',', $fields);
        $total_fieldname_count = ___c($fieldnames);
       /* for ($i = 0; $i < $total_fieldname_count; $i++)
            $fieldnames[$i] = $this->table_items . '.' . $fieldnames[$i];*/
        $fields_tobe_selected = implode(',', $fieldnames);

        if (!empty($extra_criteria))
            $criteria_txt = " AND {$extra_criteria}";
        else
            $criteria_txt = '';

        if ($id == 0) /* $the_criteria = "RIGHT OUTER JOIN {$this->table_categories_items} ON {$this->table_categories_items}.{$this->field_id} != {$this->table_items}.{$this->field_id} {$criteria_txt}"; */
            $the_criteria = " WHERE {$this->table_items}.{$this->field_id} NOT IN (SELECT {$this->field_id} FROM {$this->table_categories_items})";
        else
            $the_criteria = "LEFT JOIN {$this->table_categories_items} ON {$this->table_categories_items}.{$this->field_id} = {$this->table_items}.{$this->field_id} WHERE {$this->field_category_id} = '" . intval($id) . "' {$criteria_txt}";
        $sql = $SystemDB->generateSelectSQLStatement($this->table_items, $fields_tobe_selected, $the_criteria, $start, $end, $sortby, $sortdirection, false, SCHLIX_SQL_ENFORCE_ROW_LIMIT);
            
        $items = $SystemDB->getQueryResultArray($sql, $from_cache);

        return $items;
    }

    //_________________________________________________________________________//
    public function getUserByUsernameAndPassword($input_username, $input_password, $input_groups = null) {
        global $SystemDB;

        $input_username = (string) trim($input_username);
        $input_password = (string) trim($input_password);
        $enable_login_by_email = $this->getConfig('bool_allow_login_by_email');
        $existing_user = ($enable_login_by_email && strpos($input_username, '@') !== false) ? $this->getUserByEmailAddress($input_username) : $this->getUserByUserName($input_username);
        if ($existing_user) {
            $salt = $existing_user['salt'];
            
            $password_match = (!empty($salt)) ? (crypt($input_password, $salt) === $existing_user['password']) :
                    ( (sha1($input_username . $input_password) === $existing_user['password']) || (sha1(strtolower($input_username) . $input_password) === $existing_user['password']));

            if (!$password_match)
                return false;
        } else
            return false;
        // at this point, if password doesn't match it won't be here
        // TODO - this is inefficient - FIX this later!
        $username = sanitize_string($existing_user['username']);
        $group_str = '';
        if (___c($input_groups) > 0) {
            $group_ids = implode(', ', $input_groups);
            $group_str = " AND {$this->table_categories_items}.{$this->field_category_id} IN ({$group_ids})";
        }
        $selected_fields = $this->getNonAmbiguousFieldNames($this->table_items);
        
        $sql = "SELECT {$selected_fields} FROM {$this->table_items} INNER JOIN {$this->table_categories_items} ON {$this->table_items}.{$this->field_id} = {$this->table_categories_items}.{$this->field_id} WHERE (LOWER({$this->field_username}) = LOWER({$username})) {$group_str}";
        
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    }

    //_________________________________________________________________________//
    protected function recordUserActivity($user_id, $description) {
        global $SystemDB;

        $datavalues = [];

        $datavalues['user_id'] = intval($user_id);
        $datavalues['description'] = $description;
        $datavalues['ip_address'] = get_user_real_ip_address();
        $datavalues['referrer'] = $_SERVER['HTTP_REFERER'];
        $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
        $datavalues['request_uri'] = $_SERVER['REQUEST_URI'];

        
        $SystemDB->simpleInsertInto('gk_user_history', $datavalues);        
    }

    //_________________________________________________________________________//
    public function recordCurrentUserActivity($description) {
        $current_user_id = intval($_SESSION['userid']);

        $this->recordUserActivity($current_user_id, $description);
    }

    //_________________________________________________________________________//
    public function verifyUserNamePassword($input_username, $input_password) {
        global $SystemDB, $SystemConfig;

        $admin_group_check = '';
        if ($this->backend_mode == 1) {
            $admin_group = $this->getGroupIDByGroupName(SCHLIX_DEFAULT_ADMIN_GROUP);
            $array_groups_allowed_for_backend_login = $SystemConfig->get($this->app_name, 'array_groups_allowed_for_backend_login');
            if (!is_array($array_groups_allowed_for_backend_login))
                $array_groups_allowed_for_backend_login = array($admin_group);
        }
        $result = $this->getUserByUsernameAndPassword($input_username, $input_password, $array_groups_allowed_for_backend_login);
        if (!$result) {
            if ($this->userNameExists($input_username)) {
                $this->recordFailedLoginAttempt($input_username);
                $this->recordCurrentUserActivity("LOGIN FAILED for non-existant user '{$input_username}'");
            } else {
                $this->recordCurrentUserActivity("'LOGIN FAILED for user '{$input_username}'");
            }
        } else {
            $this->recordCurrentUserActivity("LOGIN SUCCESS for user '{$input_username}'");
        }
        return $result;
    }

    //_________________________________________________________________________//
    public function userNameExists($input_username) {
        global $SystemDB;

        $username = sanitize_string(strtolower($input_username));
        $sql = "SELECT * from {$this->table_items} where LOWER({$this->field_username}) = {$username}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        $thecount = $result ? ___c($result) : 0;
        return ($thecount > 0);
    }

    //_________________________________________________________________________//
    public function getAllUserGroupNames($guest_included = true) {
        global $SystemDB;

        $sql = "SELECT {$this->field_category_id}, {$this->field_groupname} from {$this->table_categories} ORDER by {$this->field_category_id}";

        $result = $SystemDB->getQueryResultArray($sql);
        if ($guest_included) {
            $guest_user[] = array("{$this->field_category_id}" => 0, groupname => "Everyone");
            if ($result)
                $result = array_merge($guest_user, $result);
        }
        return $result;
    }

    //_________________________________________________________________________//
    public function getFailedLoginAttemptCount($input_username) {
        global $SystemDB;

        $username = sanitize_string($input_username);
        $sql = "SELECT total_failed_login_attempt from {$this->table_items} where {$this->field_username} = {$username}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        if ($result)
            $thecount = $result['total_failed_login_attempt'];
        else
            $result = 0;
        return $thecount;
    }

    //_________________________________________________________________________//

    public function recordFailedLoginAttempt($input_username) {
        global $SystemDB;

        $username = sanitize_string($input_username);
        $x = $this->getFailedLoginAttemptCount($input_username);
        $x++;
        $sql = "UPDATE {$this->table_items} SET date_last_failed_login_attempt= NOW(), total_failed_login_attempt={$x} WHERE {$this->field_username} = {$username}";
        $SystemDB->query($sql);
    }

    //_________________________________________________________________________//
    /**
     * Performs authentication. Call this function when verifying username/password from input.
     *
     * @param string $username
     * @param string $password
     * @param bool $rememberpassword
     */
    public function performAuthentication($username, $password, $rememberpassword = 0) {
        $user = $this->verifyUserNamePassword($username, $password);
        if ($user) {
            if ($user['status'] > 0) {
                $this->setUserSessionInformation($user, $rememberpassword);
                unset($_SESSION['login_error']);
                return true;
            } else
            {
                $_SESSION['login_error'] = ___('Your account is marked as inactive. If you have just recently registered, please activate your account.');
                return false;
            }
        }
        else {
            $_SESSION['login_error'] = ___('Invalid username/password');
            return false;
        }
    }

    //_________________________________________________________________________//
    protected function setUserSessionInformation(array $user, $rememberpassword) {
        global $SystemDB;

        $this->currentUserID = $user[$this->field_id];
        $_SESSION['userid'] = $user[$this->field_id];
        $_SESSION['username'] = $user[$this->field_username];
        $_SESSION['authenticated'] = true;
        $_SESSION['groupids'] = $this->getCurrentUserGroupIDs();
        $_SESSION['login_error'] = '';
        $_SESSION['logged_in_since'] = time();
        $cookiestr = "''"; // no cookie
        if ($rememberpassword) {
            $login_time = defined('SCHLIX_SESSION_LOGIN_TIME') && is_integer(SCHLIX_SESSION_LOGIN_TIME) ? SCHLIX_SESSION_LOGIN_TIME : (60 * 60 * 24); // (1 day)
            srand((double) microtime() * 1000000);
            $cvx = sha1(uniqid(mt_rand(), true));
            setcookie('cvx', $cvx, time() + $login_time);
            setcookie('uid', sha1($_SESSION['username']), time() + $login_time);
            $cookiestr = "'{$cvx}'";
        }
        $session = sanitize_string(session_id());
        $ip = sanitize_string(get_user_real_ip_address());
        $sql = "UPDATE {$this->table_items} SET date_last_logged_in= NOW(), last_ip_address = {$ip}, total_failed_login_attempt=0 WHERE id = {$user[$this->field_id]}";
        $SystemDB->query($sql);

        $sql = "INSERT IGNORE INTO gk_user_sessions (user_id, session_time, ip_address, session_string, session_cookie) VALUES ({$user[$this->field_id]}, NOW(),$ip,{$session},{$cookiestr})";

        $SystemDB->query($sql);
    }

    //_________________________________________________________________________//
    /**
     * Get uid(username)/cvx (session name) from $_cookie, validate it, and return the user
     * @global \App\type $SystemDB
     * @return array
     */
    public function getUserByUserCookies() {
        global $SystemDB;

        if (array_key_exists('cvx', $_COOKIE) && array_key_exists('uid', $_COOKIE)) {
            $cvx = sanitize_string($_COOKIE['cvx']);
            $uid = sanitize_string($_COOKIE['uid']);
            $sql = "SELECT {$this->table_items}.{$this->field_id}, {$this->field_username}, status FROM  {$this->table_items} INNER JOIN gk_user_sessions ON user_id = {$this->table_items}.{$this->field_id} WHERE SHA1({$this->field_username}) = {$uid} AND session_cookie={$cvx}";

            $result = $SystemDB->getQueryResultSingleRow($sql);
            return $result;
        } else
            return null;
    }

    //_________________________________________________________________________//
    public function verifyCurrentSession() {
        global $SystemDB, $SystemLog;

        $valid_session = false;
        $session = sanitize_string(session_id());
        $ip = sanitize_string(get_user_real_ip_address());
        $sql = "SELECT {$this->table_items}.{$this->field_id}, status FROM  {$this->table_items} INNER JOIN gk_user_sessions ON user_id = {$this->table_items}.{$this->field_id} WHERE  (session_string = {$session}) AND (ip_address = {$ip})";
        $user = $SystemDB->getQueryResultSingleRow($sql);
        if (!$user) {
            $user = $this->getUserByUserCookies();
            if ($user) {
                $this->setUserSessionInformation($user, 1);
                $valid_session = true;
            } else {
                $userid = $_SESSION['userid'];
                if ($userid > 0)
                    $_SESSION['login_error'] = ___('Invalid Session or IP Address does not match with that of the session');
            }
        } else
            $valid_session = true;

        if ($user && $user['status'] < 1) {
            $valid_session = false;
            $_SESSION['login_error'] = ___('ERROR: User is inactive or unactivated');
        }
        //if (!$valid_session) $this->resetSession();
        $_SESSION['authenticated'] = $valid_session;
        return $valid_session;
    }

    //_________________________________________________________________________//
    public function getUserNameByID($id, $from_cache = false) {
        $user = $this->getUserByID($id, $from_cache);
        return ($user != null) ? $user[$this->field_username] : null;
    }

    //_________________________________________________________________________//
    public function getUserByID($id, $from_cache = false) {
        global $SystemDB;
        $id = (int) $id;
        $user = $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE {$this->field_id} = {$id}");
        return $user;
    }

    //_________________________________________________________________________//
    public function getUserInfoByID($id, $from_cache = false) { // backward compatibility
        return $this->getUserByID($id, $from_cache);
    }

    //_________________________________________________________________________//
    public function getUserInfoByUserName($username) {// backward compatibility
        return $this->getUserByUserName($username);
    }

    //_________________________________________________________________________//
    public function getUserByUserName($username) {
        global $SystemDB;
        $username = sanitize_string($username);
        $sql = "SELECT * FROM {$this->table_items} where LOWER({$this->field_username}) = LOWER({$username})";
        $user = $SystemDB->getQueryResultSingleRow($sql);
        return $user;
    }

    //_________________________________________________________________________//
    public function getUserByEmailAddress($input_email) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $email = sanitize_string($input_email);
        $sql = "SELECT * from {$this->table_items} where email_address <> '' AND  LOWER(email_address) = LOWER({$email})";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    }

    //_________________________________________________________________________//
    public function viewAuthenticatedUserMainPage() {
        $this->setPageTitle(___('Members Page'));
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.main.members', $local_variables);
    }

    //_________________________________________________________________________//
    public function getCurrentUserInfo() {
        if ($_SESSION['userid'] > 0)
            $user = $this->getUserByID($_SESSION['userid']);
        else
            $user = false;
        return $user;
    }

    //_________________________________________________________________________//

    public function getCurrentUserID() {
        if ($_SESSION['userid'] > 0)
            return $_SESSION['userid'];
        else
            return false;
    }

    //_________________________________________________________________________//
    public function getCurrentUserGroups() {
        $user = $this->getCurrentUserInfo();

        if ($user) {
            $group_infos = $this->getItemCategoriesByItemID($user[$this->field_id]);
            if ($group_infos) {
                return $group_infos;
            } else
                return false;
        }

        return false;
    }

    //_________________________________________________________________________//
    public function getCurrentUserGroupNames() {
        $user = $this->getCurrentUserInfo();
        if ($user) {
            $group_infos = $this->getItemCategoriesByItemID($user[$this->field_id]);
            if ($group_infos) {
                $group_names = [];
                foreach ($group_infos as $info)
                    $group_names[] = $info[$this->field_groupname];
                return $group_names;
            } else
                return false;
        }

        return false;
    }

    //_______________________________________________________________________________________________________________//
    public function isCurrentUserMemberOfGroupName($groupname) {
        $groupnames = $this->getCurrentUserGroupNames();
        return (is_array($groupnames) && ___c($groupnames) > 0) ? in_array($groupname, $groupnames) : FALSE;
    }

    //_________________________________________________________________________//
    public function getCurrentUserGroupIDs() {
        $user = $this->getCurrentUserInfo();
        if ($user)
            return $this->getItemCategoryIDsByItemID($user[$this->field_id]);
        else
            return false;
    }

    //_________________________________________________________________________//
    public function getCurrentUserName() {
        $user = $this->getCurrentUserInfo();
        if ($user)
            return $user[$this->field_username];
        else
            return false;
    }

    //_________________________________________________________________________//
    public function displayUserInfoByID($user_id) {

        $user = $this->getUserByID($user_id);
        $this->setPageTitle($user['username'] . ' ' . ___('Profile'));
        $template_name = ($this->getConfig('bool_allow_public_profile_view')) ? 'view.userinfo' : 'view.userinfo.disabled';

        $this->loadTemplateFile($template_name, get_defined_vars());
    }

    //_________________________________________________________________________//
    public function getGroupIDByGroupName($catname) {
        global $SystemDB;
        $catname = sanitize_string($catname);
        $sql = "SELECT {$this->field_category_id} from {$this->table_categories} where {$this->field_groupname} = {$catname}";
        $usergroup = $SystemDB->getQueryResultSingleRow($sql);
        if ($usergroup)
            return $usergroup[$this->field_category_id];
        else
            return 0;
    }

    //_________________________________________________________________________//
    public function getGroupNameByGroupID($id) {
        global $SystemDB;
        $id = (int) $id;
        $sql = "SELECT {$this->field_groupname} from {$this->table_categories} where {$this->field_category_id} = {$id}";
        $usergroup = $SystemDB->getQueryResultSingleRow($sql);
        if ($usergroup)
            return $usergroup['groupname'];
        else
            return 0;
    }

    //_________________________________________________________________________//
    public function validateUserNameString($str) {
        return preg_match('/^[a-zA-Z][\w\.-]*[a-zA-Z0-9]$/i', $str);
    }

    //_________________________________________________________________________//
    public function validateEmailAddressString($str) {
        return preg_match("/^[a-zA-Z][\w\.-]*[a-zA-Z0-9]@[a-zA-Z][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]$/i", $str);
    }

    //_________________________________________________________________________//
    public function resetSession() {
        global $SystemDB;

        $session = sanitize_string(session_id());
        $sql = "DELETE FROM gk_user_sessions WHERE session_string = {$session}";
        $SystemDB->query($sql);
        unset($_SESSION['authenticated']);
        unset($_SESSION['userid']);
        unset($_SESSION['username']);
    }

    //_________________________________________________________________________//
    /**
     * For CRON
     * @global type $SystemDB
     */
    public static function processRunDeleteExpiredSessions() {
        global $SystemDB;

        $maxtime = (defined('SCHLIX_SESSION_LOGIN_TIME') && intval(SCHLIX_SESSION_LOGIN_TIME) > 0) ? SCHLIX_SESSION_LOGIN_TIME : 1209600;
        $maxtime = $maxtime / (24 * 60 * 60);
        if ($maxtime < 2)
            $maxtime = 10;
        $sql = "DELETE FROM gk_user_sessions WHERE ((session_cookie = '') AND (DATEDIFF(NOW(), session_time) > 2)) OR (DATEDIFF(NOW(), session_time) > {$maxtime});";
        $SystemDB->query($sql);
    }

//_________________________________________________________________________//
    public function debugSession() {
        print "DEBUG Session ID " . session_id() . ": ";
        //debug_array($_SESSION);
        print "<br/>";
        print "DEBUG Cookies: ";
        debug_array($_COOKIE);
        print "<br/>";
    }

//_________________________________________________________________________//
    public function logout() {
        $_SESSION['backend_login'] = false;
        $this->recordCurrentUserActivity("LOGOUT");

        setcookie("cvx", '', $this->time - 3600 * 3600);   // Sets the cookie username
        setcookie("uid", '', $this->time - 3600 * 3600); // Sets the cookie password

        $this->resetSession();

        session_unset();
        session_destroy();
    }

//_________________________________________________________________________//
    public function adminAuthenticated() {
        return $_SESSION['authenticated'];
    }

//_________________________________________________________________________//

    public function authenticated() {
        return $_SESSION['authenticated'];
    }

//_______________________________________________________________________________________________________________//
    public function viewMainPage() {
        if ($this->authenticated()) {
            $this->viewAuthenticatedUserMainPage();
        } else
            $this->viewLoginPage();
    }

    //_______________________________________________________________________________________________________________//
    public function generateRandomPasswordSalt($hash_type, $work_factor = 9) {
        if ($work_factor < 4 || $work_factor > 31)
            $work_factor = 9;
        $rand = function_exists('openssl_random_pseudo_bytes') ? openssl_random_pseudo_bytes(16) : sha1(mt_rand(0, 32768) . microtime(true), true);
        $silly_string = substr(strtr(base64_encode($rand), '+', '.'), 0, 22);
        switch ($hash_type) {
            case 'sha256': $salt = '$5$rounds=' . str_pad($work_factor * 10000, 6, '0', STR_PAD_LEFT) . '$' . substr($silly_string, 0, 16);
                break;
            case 'sha512': $salt = '$6$rounds=' . str_pad($work_factor * 10000, 6, '0', STR_PAD_LEFT) . '$' . substr($silly_string, 0, 16);
                break;
            case 'blowfish':
            default: $salt = '$2y$' . str_pad($work_factor, 2, '0', STR_PAD_LEFT) . '$' . $silly_string;
                break;
        }
        return $salt;
    }

    //_______________________________________________________________________________________________________________//
    public function generateNewSaltAndHashedPassword($password, $salt = '') {
        global $SystemConfig;

        $password = trim($password);
        /* TODO remove salt strength hardcoding in v1.2.6 - no time right now (Dec 21, 2012) */
        if (empty($salt))
            $salt = $this->generateRandomPasswordSalt('blowfish', 5);
        /* $SystemConfig->get('system', 'salt_type') $this->getConfig('system', 'salt_strength') */
        $hashed_password = crypt($password, $salt);
        return array($salt, $hashed_password);
    }
    /**
     * Converts a string to an alphanumeric one (dash and underscores allowed)
     * @param string $str
     * @param bool $allow_utf8
     * @return string
     */
    public function convertIntoValidUserName($str) {
        
        if (!$str)
            return null;
        $pattern = '/[^A-za-z0-9_-]/';
        $str = preg_replace($pattern, '-', $str);
        $str = preg_replace('/-+/', "-", $str);

        return trim($str, ' -');
    }

    /**
     * Verify the user email address
     * @internal
     * @param string $email
     */
    protected function getUserRealEmailAddress($email)
    {
        $real_email = strtolower(trim($email));
        list($username, $domain) = explode('@', $email);
        if ($domain == 'gmail.com' || $domain == 'googlemail.com')
        {
            $username = str_replace('.','', $username);
            $pluspos = strpos($username, '+');
            if ($pluspos !== FALSE)
            {
                $username = substr($username,0 , $pluspos);
            }
            return "{$username}@{$domain}";
        }
        return $real_email;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * 
     * @global \App\cmsLogger $SystemLog
     * @param array $datavalues
     * @return string
     */
    protected function modifyDataValuesBeforeSaveItem($datavalues)
    {
        global $SystemLog;
        
        $datavalues = parent::modifyDataValuesBeforeSaveItem($datavalues);
        foreach ($datavalues as $k => $v)
        {
            if (is_string($v))
                $datavalues[$k] = trim($v);
        }
        //$datavalues = array_map('trim', $datavalues);
        if ($datavalues['id'] == 'new')
            $datavalues['date_created'] = date('Y-m-d');
        if (intval($datavalues['id'] > 0)) {
            $existing_user = $this->getUserByID(intval($datavalues['id']));
            $datavalues['username'] = $existing_user['username']; // this is so the user can't override it
            $datavalues['guid'] = $existing_user['guid'];
        }
        $datavalues['real_email_address'] = $this->getUserRealEmailAddress($datavalues['email_address']);
        $datavalues['username'] = $this->convertIntoValidUserName($datavalues['username']);
        if (array_key_exists('password', $datavalues)) {
            if ($datavalues['password'] != '') {
                $datavalues['_plaintext_password_length'] = strlen($datavalues['password']);
                $datavalues['_plaintext_password'] = $datavalues['password'];
                $datavalues['_plaintext_password_verify'] = $datavalues['password_verify'];
                list ($datavalues['salt'], $datavalues['password']) = $this->generateNewSaltAndHashedPassword($datavalues['password'], $existing_user['salt']);
            } else {
                unset($datavalues['password']);
            }
        }
        
        // User picture
        $new_avatar_filename_only = $datavalues['guid'];
        $file_avatar = $_FILES['user_avatar'];
        if (is_uploaded_file($_FILES['user_avatar']['tmp_name'])) {
            
            
            if ($file_avatar['error'] == UPLOAD_ERR_OK) {
                $avatar_file_name = $_FILES['user_avatar']['name'];
                $new_avatar_filename =  pathinfo($new_avatar_filename_only, PATHINFO_FILENAME) . '.' . pathinfo($avatar_file_name, PATHINFO_EXTENSION);
                // name the avatar the same as the vfname
                $datavalues['avatar'] = $new_avatar_filename;
                $valid_image = is_valid_uploaded_image_file($file_avatar['tmp_name'], $new_avatar_filename);
                if ($valid_image)
                {
                    move_uploaded_file($file_avatar['tmp_name'], $this->getDataFileFullPath('avatar_original', $new_avatar_filename));


                    $small_width = $this->getConfig('int_small_width', 32);
                    $small_height = $this->getConfig('int_small_height', 32);
                    $medium_width = $this->getConfig('int_medium_width', 128);
                    $medium_height = $this->getConfig('int_medium_height', 128);                
                    $large_width = $this->getConfig('int_large_width', 256);
                    $large_height = $this->getConfig('int_large_height', 256);                

                    create_image_thumbnail($this->getDataFileFullPath('avatar_original', $new_avatar_filename), $this->getDataFileFullPath('avatar_small', $new_avatar_filename), $small_width, $small_height);
                    create_image_thumbnail($this->getDataFileFullPath('avatar_original', $new_avatar_filename), $this->getDataFileFullPath('avatar_medium', $new_avatar_filename), $medium_width, $medium_height);
                    create_image_thumbnail($this->getDataFileFullPath('avatar_original', $new_avatar_filename), $this->getDataFileFullPath('avatar_large', $new_avatar_filename), $large_width, $large_height);
                } else
                {
                    $SystemLog->record("Invalid image type being uploaded from ". get_user_real_ip_address());
                    
                }
                
            }
        }
        
        return $datavalues;        
    }
 //_______________________________________________________________________________________________________________//
    public function getValidationErrorListBeforeSaveItem($data) {
        
        $error_list =  parent::getValidationErrorListBeforeSaveItem($data);
        if (!isset($data['username']) || (isset($data['username']) && $data['username'] == ''))
            $error_list[] = ___('Username is empty');
        if (!empty($data['password']) && ($data['_plaintext_password_length'] < $this->minimum_password_length))
            $error_list[] = ___('ERROR: Your password is too short. Minimum length: ') . $this->minimum_password_length;

        return $error_list;
    }

    //_______________________________________________________________________________________________________________//
    public function findDuplicateUsers($data) {
        global $SystemDB;

        $current_id = intval($data[$this->field_id]);
        $sql = "SELECT * from {$this->table_items} WHERE ({$this->field_username} = '{$data['{$this->field_username}']}')";
        if (intval($current_id) != 0)
            $sql.= " AND (id != '{$current_id}')";

        $resultx = $SystemDB->getQueryResultArray($sql);
        return $resultx;
    }

    //______________________________________________________________________________________________________________//
    public function validateNewUserRegistration() {// custom function, can be inherited and changed
        global $SystemConfig, $SystemDB;


        $securimage = new \securimage();
        $securimage->session_name = SCHLIX_SESSION_NAME;

        $error_list = [];
        $enable_captcha_user_registration = $SystemConfig->get($this->app_name, 'bool_enable_captcha_user_registration');
        if ($enable_captcha_user_registration && $securimage->check($_POST['verification_code']) == false) {
           $error_list[] = ___('Invalid verification code');
        }
        if (empty($_POST['username'])) {
           $error_list[] = ___('Please fill in the username');
        }
        if (empty($_POST['email_address'])) {
           $error_list[] = ___('Please fill in the email address');
        } // only return for these 2 checks only
        // 1. Check if username is valid
        if (!$this->validateUserNameString($_POST['username']))
           $error_list[] = ___('Username contains an invalid character. Please use alphanumeric and underscore characters only.');
        // 2. Check if email address is valid
        if (!$this->validateEmailAddressString($_POST['email_address']))
            $error_list[] =___('Your e-mail address is invalid') ;
        // 3. Check if email is registered
        $the_email = sanitize_string($_POST['email_address']);
        $the_username = sanitize_string($_POST['username']);
        $thecount = ___c($this->getUserByEmailAddress($_POST['email_address']));

        if ($thecount > 0)
            $error_list[] =___('The email address has already been registered');
        // 3. Check if username is taken
        if ($this->userNameExists($_POST['username']))
            $error_list[] =___('The username has already been taken');
        // 4. Check if password = password_verify
        if ($_POST['password'] != $_POST['password_verify'])
           $error_list[] = ___('Your password does not match its verification');
        // 5. Check if password length < 5
        if (strlen($_POST['password']) < $this->minimum_password_length)
            $error_list[] =___('Your password is too short');
        if ($this->getConfig('bool_require_gender') && ($_POST['gender'] < 1 || $_POST['gender'] > 3) )
        {
            $error_list[] =___('Please specify your gender');
        }
        
        if ($this->getConfig('bool_require_gender') && ($_POST['gender'] < 1 || $_POST['gender'] > 3) )
        {
            $error_list[] =___('Please specify your gender');
        }        
        
        if ($this->getConfig('bool_require_dob'))
        {
            $user_input_date = $_POST['dob_year'].'-'.str_pad($_POST['dob_month'],2,'0',STR_PAD_LEFT).'-'.str_pad($_POST['dob_day'],2,'0',STR_PAD_LEFT);
            if (is_date($user_input_date,'Y-m-d'))
            {    
                $_POST['date_of_birth'] = $user_input_date;
            } else
            {
                $error_list[] = ___('Invalid date of birth').' ('.$user_input_date.')';
            }
            

            $minimum_age = $this->getConfig('int_minimum_age_registration');
            if ($minimum_age > 0)
            {
                $birthdate = new DateTime($user_input_date);
                $now = new DateTime();
                $interval = $now->diff($birthdate);
                $age = $interval->y;
                if ($age < $minimum_age)
                {
                    $error_list[] = ___('You do not meet the minimum age requirement to register for this site');
                }
            }            
        }
        
        return $error_list;
    }

    //_________________________________________________________________________//
    public function viewUserRegistrationPage() {
        global $SystemConfig, $SystemDB;

        $display_registration_form = true;
        $success_list = [];
        $enable_user_registration = $SystemConfig->get($this->app_name, 'bool_enable_registration');
        $enable_captcha_user_registration = $SystemConfig->get($this->app_name, 'bool_enable_captcha_user_registration');
        $default_group_id = $SystemConfig->get($this->app_name, 'int_default_newuser_group_id');
        $config_requires_activation = $this->getConfig('bool_requires_activation_by_email');
        if (!$default_group_id) {
            $this->setPageTitle(___('User Registration - System Error'));
            echo \__Html::H1(___('Error')) . \__Html::P(___('Default group for new user registration has not been set in the Administration area.'));
            return false;
        }
        if ($enable_user_registration != 1) {
            $this->setPageTitle(___('User Registration Disabled'));
            $this->loadTemplateFile('view.registration.disabled', get_defined_vars());
            return false;
        }
        if (!empty($_SESSION['registration_success']))
        {
            $success_list[] = $_SESSION['registration_success'];
            $display_registration_form = FALSE;
            $_SESSION['registration_success']= NULL;
        }
        if (is_postback()) {
            $current_time = date('Y/m/j h:i:s A');
            $activation_string =  md5($current_time . fpost_string('username') . mt_rand());
            foreach (array_keys($_POST) as $input)
                $_POST[$input] = trim($_POST[$input]);
            $datavalues = select_http_post_variables('username', 'password', 'password_verify', 'firstname', 'lastname', 'email_address','date_of_birth','gender');
            $datavalues[$this->field_username] = fpost_string('username');
            $datavalues[$this->field_password] = fpost_string('password');
            $error_list = $this->validateNewUserRegistration();
            if (___c($error_list) ==  0) {
// Register User
                //	$datavalues['category_id'] = $default_group_id;
                $plaintextpasswd = $datavalues['password'];
                list ($datavalues['salt'], $datavalues[$this->field_password]) = $this->generateNewSaltAndHashedPassword($datavalues[$this->field_password]);

                $datavalues['date_created'] = get_current_datetime();
                $datavalues['last_ip_address'] = get_user_real_ip_address();
                $datavalues['source'] = 'normalregistration';
                $datavalues['guid'] = new_uuid_v4();
                if ($config_requires_activation) {
                    $datavalues['status'] = USER_REQUIRES_ACTIVATION;
                    $datavalues['activation_string'] = $activation_string;
                } else {
                    $datavalues['status'] = USER_ACTIVATED_IMMEDIATELY;
                }
                unset($datavalues['password_verify']);

                $this->_internalCreateUser($datavalues, true);
                //
                $last_created_user = $this->getUserByUserName($datavalues[$this->field_username]);
                $last_created_user_id = $last_created_user['id'];
                //
                if ($config_requires_activation) {
                    $activation_url = SCHLIX_SITE_URL . $this->createFriendlyURL("action=activation&id={$last_created_user_id}&token={$activation_string}");
                    $this->sendRegistrationEmailToUserByID($last_created_user_id, 'new-user-activation-required', array('plaintextpassword' => $plaintextpasswd, 'activation_string' => $activation_string, 'activation_url' => $activation_url));
                    
                    $success_msg = ___('Thank you, you have been registered on our site. Please activate your account by clicking the link sent to your email');
                    
                } else {
                    $this->sendRegistrationEmailToUserByID($last_created_user_id, 'new-user-registration-success', array('password' => $plaintextpasswd));
                    $success_msg = ___('Thank you, you have been registered on our site. You may now login to access your account');
                }
                $_SESSION['registration_success'] = $success_msg;
                $this->redirectToOtherAction('action=register');
            }
        } 
        $this->setPageTitle(___('User Registration'));
        //$introtext = $SystemConfig->get($this->app_name, 'txt_registration_intro');
        $this->loadTemplateFile('view.registration', get_defined_vars());
    }

    //_______________________________________________________________________________________________________________//
    public function sendRegistrationEmailToUserByID($user_id, $email_template_name, $vars = []) {
        global $SystemMail;

        $ip_address = get_user_real_ip_address();

        $user = $this->getItemByID($user_id, true); // cached
        $email_vars = array('firstname' => $user['firstname'], 'site_url' => SCHLIX_SITE_URL, 'site_name' => SCHLIX_SITE_NAME, 'ip_address' => $ip_address, 'username' => $user['username']);
        if (is_array($vars) && ___c($vars) > 0)
            $email_vars = array_merge($email_vars, $vars);

        if (!$SystemMail->sendEmailFromWebMasterToUserByUserIDWithTemplateName($user_id, $email_template_name,  $email_vars)) {
            display_schlix_alert(___('SMTP/Local mail send failed! Please notify the website owner!').$user_id.' '.$email_template_name);
            $this->recordLog("SMTP/Local mail send failed - Cannot send a registration email to {$user['email_address']}");
        }
    }

    //_______________________________________________________________________________________________________________//
    public function verifyActivationRequest($userid, $token, $activate_immediately = true) {
        global $SystemDB;

        $userid = intval($userid);
        $token = sanitize_string($token);
        $sql = "SELECT * from {$this->table_items} WHERE ({$this->field_id} = {$userid}) AND (activation_string = {$token}) AND (activation_string <> '') AND (status = -2)";
        $user = $SystemDB->getQueryResultSingleRow($sql);
        if ($user && $activate_immediately) {
            $sql = "UPDATE {$this->table_items} SET status = 1, activation_string = '' WHERE ({$this->field_id} = {$userid}) AND (status = -2)";
            $SystemDB->query($sql);
        }
        return $user;
    }

    //_______________________________________________________________________________________________________________//
    public function processActivationRequest($userid, $token) {
        $userid = intval($userid);
        $this->setPageTitle(___('User Activation'));
        $activation_error = '';
        $user_verified = false;
        $enable_captcha_user_activation = $this->getConfig('bool_enable_captcha_user_activation');
        // Process User Input
        if ($_POST['activation_string']) {
            $userid = intval($_POST['userid']);
            $token = $_POST['activation_string'];
            //validation
            if (!is_valid_csrf())
                $activation_error.= ___('Invalid CSRF Verification Token') . \__Html::BR();
            if ($enable_captcha_user_activation && !is_captcha_verification_valid())
                $activation_error.=___('Invalid Captcha. Cannot proceed with activation') . \__Html::BR();;
            if ($userid == 0)
                $activation_error = ___('UID is zero. Cannot proceed with activation') . \__Html::BR();
            if (empty($token))
                $activation_error.= ___('Token is empty. Cannot proceed with activation') . \__Html::BR();
            if (empty($activation_error))
                $user_verified = $this->verifyActivationRequest($userid, $token); // final check
                
/////
            if ($user_verified) {
                //exit here, all good
                $this->loadTemplateFile('view.activation.complete', get_defined_vars());
                return true;
                /////
            } else
                $activation_error.= ___('No match between user ID and token') . \__Html::BR();
        } else {
            //////// No POST
            if ($userid == 0)
                $activation_error = ___('UID is zero. Cannot proceed with activation') . \__Html::BR();
            if (empty($token))
                $activation_error.= ___('Token is empty. Cannot proceed with activation') . \__Html::BR();
        }
        $this->loadTemplateFile('view.activation.request', get_defined_vars());
    }

    //_______________________________________________________________________________________________________________//
    public function getItemOrCategoryToViewFromFullVirtualFilename($url, $enable_redirect_no_trailingslash_folder = false) {
        $request_filename = basename($url);
        $file_path_info = null;
        if ($request_filename)
            $file_path_info = pathinfo($request_filename);
        if (empty($url) || $url == 'index.html') {
            $command['action'] = 'main';
            return $command;
        }
        if ($file_path_info['extension'] == 'html') {
            $username = $file_path_info['filename'];
            $user = $this->getItemByVirtualFilename($username);

            $command['action'] = 'viewitem';
            $command['id'] = $user[$this->field_id];
            return $command;
        }
        $command['action'] = '404error';
        return $command;
    }

    //____________________________________________________________________//

    public function generatePassword($length = 6, $strength = 0) {
        $vowels = 'aieouy';
        $consonants = 'bdghjmnpqrstvz';
        if ($strength & 1) {
            $consonants .= 'BDGHJLMNPQRSTVWXZ';
        }
        if ($strength & 2) {
            $vowels .= "AEUY";
        }
        if ($strength & 4) {
            $consonants .= '23456789';
        }
        if ($strength & 8) {
            $consonants .= '@#$%';
        }

        $password = '';
        $alt = time() % 2;
        for ($i = 0; $i < $length; $i++) {
            if ($alt == 1) {
                $password .= $consonants[(rand() % strlen($consonants))];
                $alt = 0;
            } else {
                $password .= $vowels[(rand() % strlen($vowels))];
                $alt = 1;
            }
        }
        return $password;
    }
    /**
     * Change a user's password
     * @param int $id
     * @param string $newpassword
     * @return boolean
     */
    protected function changePassword($id, $newpassword)
    {
        global $SystemDB;
        $id = (int) $id;        
        $userinfo = $this->getItemByID($id);
        if ($userinfo)
        {
            $datavalues = [];
            list ($datavalues['salt'], $datavalues[$this->field_password]) = $this->generateNewSaltAndHashedPassword($newpassword, $userinfo['salt']);
            $datavalues['date_modified'] = get_current_datetime();
            $datavalues['last_ip_address'] = get_user_real_ip_address();
            
            $SystemDB->simpleUpdate($this->table_items, $datavalues, $this->field_id, $id);
            
            return TRUE;
        }
        return FALSE;
    }
    
    protected function checkPasswordQuality($password, $password_verify)
    {
        $error_list = [];

        if (strlen($password) < 6)
            $error_list[] = ___('Password is too short');
        if (strlen($password) > 63)
            $error_list[] = ___('Password is too long');
        if ($password !== $password_verify)
            $error_list[] = ___('Password confirmation is invalid');
        return $error_list;
    }
    //_______________________________________________________________________________________________________________//    
    public function viewPasswordChangePage()
    {
        global $CurrentUser;
        
        $this->setPageTitle(___('Change Password'));
        $error_list = [];
        $success_list = [];
        $display_password_change_form = true;
        if ($_POST['submit'])
        {
            $error_list = $this->checkPasswordQuality($_POST['password'], $_POST['password_verify']);
            if (!is_valid_csrf()) 
                $error_list[] = ___('Invalid CSRF Verification Token');
            
            $user = $this->verifyUserNamePassword($_SESSION['username'], $_POST['oldpassword']);
            if (!$user)
                $error_list[] = ___('You have entered an invalid current password');
            
            if (___c($error_list) == 0)
            {
                if ($this->changePassword($CurrentUser->getCurrentUserID(), $_POST['password']))
                {
                    $success_list[] = sprintf(___('Your password has been changed.'), $login_url);
                    $display_password_change_form = false;
                } else
                    $error_list[] = ___('Unable to change password. Unknown error');
            }
        }
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.password_change', $local_variables);        
    }
    //_______________________________________________________________________________________________________________//
   
    public function viewPasswordResetPage($id, $hash)
    {
        $this->setPageTitle(___($id.$hash.'Password Reset'));
        $expire_days = (int) $this->getConfig('int_days_forgot_password') ; 
        if ($expire_days == 0) $expire_days = 2;
        $password_reset = $this->getExistingPasswordResetRequestByIDAndHash($id, $hash, $expire_days);
        $error_list = [];
        $success_list = [];
        if ($password_reset)
        {
            $display_password_reset_form = true;
            if ($_POST['submit'])
            {
                $error_list = $this->checkPasswordQuality($_POST['password'], $_POST['password_verify']);
                if (!is_valid_csrf()) 
                    $error_list[] = ___('Invalid CSRF Verification Token');
                
                if ($password_reset['status'] == 1)
                {
                    $error_list[] = ___('Password has just been reset');
                    $display_password_reset_form = false;
                }
                if (___c($error_list) == 0)
                {
                    if ($this->changePassword($password_reset['user_id'], $_POST['password']))
                    {
                        $login_url = $this->createFriendlyURL("action=login");
                        $this->markPasswordResetRequestComplete($id);
                        $success_list[] = sprintf(___('Your password has been changed. You may now <a href="%s">login with your new password</a>.'), $login_url);
                        $display_password_reset_form = false;
                    } else
                        $error_list[] = ___('Unable to change password. Unknown error');
                }
            }
        } else
        {
            $error_list[] = ___('The password reset link that you have supplied is either invalid or has expired');
        }
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.password_reset', $local_variables);        
    }
    
    //_______________________________________________________________________________________________________________//
    public function getExistingPasswordResetRequestByIDAndHash($id, $hash, $days_difference = 1)
    {
        global $SystemDB;
        
        $id = (int) $id;
        $hash = sanitize_string($hash);
        $sql = "SELECT *  FROM `gk_user_password_reset` WHERE `id` = {$id} AND `hash` = {$hash} AND DATEDIFF(NOW(), date_created) <= {$days_difference}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    }
    //_______________________________________________________________________________________________________________//
    public function getExistingPasswordResetRequestByUserID($id, $days_difference = 1)
    {
        global $SystemDB;
        
        $id = (int) $id;
        $hash = sanitize_string($hash);
        $sql = "SELECT * FROM `gk_user_password_reset` WHERE `user_id` = {$id} AND (DATEDIFF(NOW(), date_created) < {$days_difference})";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result ? $result['id'] : 0;
    }
    
    //_______________________________________________________________________________________________________________//
    public function sendPasswordResetRequest($id)
    {
        global $SystemDB, $SystemMail;
        
        $id = (int) $id;
        $sql = "SELECT * FROM `gk_user_password_reset`  INNER JOIN `{$this->table_items}` ON `user_id` = `{$this->table_items}`.`id` WHERE `gk_user_password_reset`.`id` = {$id}";
        $password_request = $SystemDB->getQueryResultSingleRow($sql);
        $error_list = [];
        if ($password_request)
        {                    
            // update sent count
            $email_sent_count = $password_request['email_sent_count'] + 1;
            // prevent abusers
            if ($email_sent_count < 10)
            {
                $reset_url = SCHLIX_SITE_HTTPS_URL ? SCHLIX_SITE_HTTPS_URL.$this->createFriendlyURL("action=passwordreset&hash={$password_request['hash']}&xt={$id}") : SCHLIX_SITE_HTTP_URL.$this->createFriendlyURL("action=passwordreset&hash={$password_request['hash']}&xt={$id}");
                $expire_hours = ((int) $this->getConfig('int_days_forgot_password') * 24);
                $email_vars = array('firstname' => $password_request['firstname'], 
                        'reset_url' => $reset_url, 
                        'site_name' => CURRENT_SUBSITE_NAME, 
                        'ip_address' => get_user_real_ip_address(),
                        'expire_hours' => $expire_hours);
                
                if (!$SystemMail->sendEmailFromWebMasterToUserByUserIDWithTemplateName($password_request['user_id'], 'forgot-password',  $email_vars))
                    $error_list[] = ___('Cannot send the password reset email');
                // update email sent count
                $sql = "UPDATE `gk_user_password_reset`  SET email_sent_count = {$email_sent_count}";
                $password_request = $SystemDB->query($sql);
            } else $error_list[] = sprintf( ___('Too many retry attempt. Please wait for %d days'), $this->getConfig('int_days_forgot_password'));
        } else $error_list[] = ___('Cannot find password reset information');
        
        return $error_list;
    }
    //_______________________________________________________________________________________________________________//
    protected function markPasswordResetRequestComplete($id)
    {
        global $SystemDB, $SystemMail;
        
        $id = (int) $id;
        $sql = "SELECT * FROM `gk_user_password_reset`  INNER JOIN `{$this->table_items}` ON `user_id` = `{$this->table_items}`.`id` WHERE `gk_user_password_reset`.`id` = {$id}";
        $password_request = $SystemDB->getQueryResultSingleRow($sql);
        
        if ($password_request)
        {
            $datavalues = [];
            $datavalues['status'] = 1;
            $datavalues['date_modified'] = date ('Y-m-d H:i:s');
            $datavalues['confirm_ip_address'] = get_user_real_ip_address();
            $datavalues['confirm_user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            
            $SystemDB->simpleUpdate('gk_user_password_reset', $datavalues, 'id', $id);

            $email_vars = array('firstname' => $password_request['firstname'], 
                    'date_reset' => $datavalues['date_modified'], 
                    'site_name' => CURRENT_SUBSITE_NAME, 
                    'ip_address' => get_user_real_ip_address(),
                    'email' => $password_request['email']);
            if (!$SystemMail->sendEmailFromWebMasterToUserByUserIDWithTemplateName($password_request['user_id'], 'forgot-password',  $email_vars))
            {
                $error_list[] = ___('Cannot send the password reset email');
            }
            return TRUE;
        } else return FALSE;
    }
            
    //_______________________________________________________________________________________________________________//
    /**
     * Create a password reset request
     * @global \App\type $SystemDB
     * @param array $user
     * @return int
     */
    public function createPasswordResetRequest($user)
    {
        global $SystemDB;
        
        $datavalues = [];
        $datavalues['user_id'] = $user['id'];
        $datavalues['status'] = 0;
        $datavalues['date_created'] = date ('Y-m-d H:i:s');
        $datavalues['ip_address'] = get_user_real_ip_address();
        $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
        $datavalues['email'] = $user['email_address'];
        $datavalues['hash'] = sha1($_SERVER['HTTP_USER_AGENT'].$datavalues['ip_address'].$user['email_address'].rand(0,65535).new_uuid_v4());
        $datavalues['email_sent_count'] = 0;
        
        $SystemDB->simpleInsertInto('gk_user_password_reset', $datavalues);        
        
        
        $new_password_reset_id = $SystemDB->getLastInsertID();
        return $new_password_reset_id;
        //$datavalues['date_modified'] = date ('Y-m-d H:i:s');
    }
    //_______________________________________________________________________________________________________________//
    public function viewForgotPasswordPage() {
        global $SystemDB, $site_infomail, $site_name, $SystemMail;
        
        $this->setPageTitle(___('Forgot Your Password?'));
        $error_list = [];
        $success_list = [];
        
        $display_forgot_password_form = true;
        if (!empty($_SESSION['forgot_pw_success']))
        {
            $success_list[] = $_SESSION['forgot_pw_success'];
            $display_forgot_password_form = FALSE;
            $_SESSION['forgot_pw_success']= NULL;
        }
        
        if (is_postback()) {
            if ($this->getConfig('bool_enable_captcha_forgot_password') && !is_captcha_verification_valid()) 
                $error_list[] = ___('Invalid Captcha Verification Token');
            if (!is_valid_csrf()) 
                $error_list[] = ___('Invalid CSRF Verification Token');
            if (!filter_var($_POST['email'], FILTER_VALIDATE_EMAIL))
                $error_list[] = ___('Invalid e-mail address');
            $forgot_password_user = $this->getUserByEmailAddress(fpost_string('email'));
            
            if ($forgot_password_user['status'] == -2) 
                $error_list[] = ___('Your account has not been activated yet. Please activate your account first');
            if (___c($error_list) == 0 && ($forgot_password_user != NULL)) {
                $days = $this->getConfig('int_days_forgot_password');
                if ($days < 1) $days = 1;
                $password_request_id = $this->getExistingPasswordResetRequestByUserID($forgot_password_user[$this->field_id],$days);
                if ($password_request_id == 0)
                {
                    $password_request_id = $this->createPasswordResetRequest($forgot_password_user);
                }
                if ($password_request_id > 0)
                {
                    $additional_errors = $this->sendPasswordResetRequest($password_request_id);
                    $error_list = array_merge($error_list, $additional_errors);
                } else 
                {
                    $error_list[] = ___('Strange error - password request ID is NULL');
                }
               
            } else {
                $error_list[] = ___('Sorry, we cannot find your identity based on the supplied information');
            }
            if (___c($error_list) === 0)
            {
                $_SESSION['forgot_pw_success'] = ___('Your password reset activation link has been sent to').': '.___h($_POST['email']);
                $this->redirectToOtherAction("action=forgotpassword");
            }
            
        }
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.password_forgot', $local_variables);
        return TRUE;
    }

//_______________________________________________________________________________________________________________//
    public function viewLoginPage() {

        $force_ssl = $this->getConfig('bool_force_ssl_authentication');
        $login_url = $this->createFriendlyURL("action=login");
        $invalid_retries_before_captcha = $this->getConfig('int_number_of_login_retry_before_captcha', 30);
        $error_list = [];
        if ($force_ssl)
            $login_url = force_https_url() . $login_url;
        if (fpost_bool('submit')) {
            // We didn't check $_POST['password'], it could be anything the user wanted! For example:


            //if ($_SESSION['invalid_password_retry'] > $invalid_retries_max)
                //$error_list[] = ___('You have exceeded the maximum number of retry');
            
/*            if ($this->getConfig('bool_enable_captcha_forgot_password') && !is_captcha_verification_valid()) 
                $error_list[] = ___('Invalid Captcha Verification Token');*/

            if (($_SESSION['invalid_password_retry'] > $invalid_retries_before_captcha) && !is_captcha_verification_valid())
                $error_list[] = ___('Invalid captcha');
            
            if (!is_valid_csrf())
                $error_list[] = ___('Invalid CSRF Verification Token');
            

            if (___c($error_list) == 0)
            {
                $this->performAuthentication(fpost_string('username'), fpost_string('password'), fpost_bool('remember'));
                if ($this->authenticated() === true) {
                    $_SESSION['invalid_password_retry'] = 0;
                    $url = $this->getLoginRedirectURL();
                    $this->setLoginRedirectURL('');
                    if (empty($url))
                        $this->redirectToOtherAction('', OPT_REDIRECT_HTTP);
                    else
                        redirect_url($url);
                } else {
                    if (array_key_exists('invalid_password_retry', $_SESSION))
                        $_SESSION['invalid_password_retry']+=1;
                    else
                        $_SESSION['invalid_password_retry'] = 1;
                    
                    $error_list[] = $_SESSION['login_error']; //___('Invalid username or password - Please verify that the information you specified is correct. Please ensure that CAPS Lock is turned off');
                    sleep(1); // sleep, delay 1 second
                }                
            }
        }
        $this->setPageTitle(___('Login'));
        $this->loadTemplateFile('view.login', get_defined_vars());
    }

    //______________________________________________________________________________________________________________//
    public function setLoginRedirectURL($url) {
        $_SESSION['url_redirect_after_login'] = $url;
    }

    //______________________________________________________________________________________________________________//
    public function getLoginRedirectURL() {
        return $_SESSION['url_redirect_after_login'];
    }

    //______________________________________________________________________________________________________________//
    public function validateEditProfile() {// custom function, can be inherited and changed
        global $SystemDB;

        $error_list = [];
        $current_user_id = intval($_SESSION['userid']);
        // 2. Check if email address is valid
        if (empty($_POST['email_address'])) {
            $error_list[] = ___('Please fill in the email address');
        } // only return for these 2 checks only
        if (!$this->validateEmailAddressString($_POST['email_address']))
            $error_list[] = ___('Your e-mail address is invalid');
        
        
        if (strlen(trim($_POST['firstname'])) < 1)
            $error_list[] = ___('Invalid first name');
        
        if (strlen(trim($_POST['lastname'])) < 1)
            $error_list[] = ___('Invalid last name');
            
        if ($this->getConfig('bool_require_dob'))
        {
            $user_input_date = $_POST['dob_year'].'-'.str_pad($_POST['dob_month'],2,'0',STR_PAD_LEFT).'-'.str_pad($_POST['dob_day'],2,'0',STR_PAD_LEFT);
            if (is_date($user_input_date,'Y-m-d'))
            {    
                $_POST['date_of_birth'] = $user_input_date;
            } else
            {
                $error_list[] = ___('Invalid date of birth').' ('.$user_input_date.')';
            }
        }
        
        if ($this->getConfig('bool_require_gender'))
        {
            if ($_POST['gender'] < 1 || $_POST['gender'] > 3)
            {    
                $error_list[] = ___('Invalid gender');
            }
        }        
                
        
        // 3. Check if email is registered
        $the_email = sanitize_string($_POST['email_address']);
        // prevent overwriting other people's e-mail address
        $sql = "SELECT {$this->field_id} FROM {$this->table_items} WHERE (email_address = {$the_email}) AND (email_address <> '') AND {$this->field_id} <> {$current_user_id} ";
        $results = $SystemDB->getQueryResultArray($sql);
        $thecount = ___c($results);
        if ($thecount > 0)
            $error_list[] = ___('The email address you entered belongs to someone else');
 
        return $error_list;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Edit user profile
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @global \App\Users $CurrentUser
     */
    public function viewEditMyProfilePage() {

        global $CurrentUser;
        
        $allowable_html_tags = '<br><hr /><br /><br><p><b><strong><em><i><h1><h2><h3><h4><h5><h6><div>';
        $userinfo = $CurrentUser->getCurrentUserInfo();
        if ($userinfo[$this->field_id] == 0)
        {
            $this->redirectToOtherAction('action=login');
            return;
        }
        if (is_postback()) {
            $error_list = $this->validateEditProfile();
            if (___c($error_list) == 0) {
                // filter user editable fields
                $datavalues = select_http_post_variables('gender', 'date_of_birth', 'firstname', 'lastname', 'display_name', 'email_address','summary','description');
                if ($datavalues['summary'])
                    $datavalues['summary'] = real_strip_tags ($datavalues['summary'],$allowable_html_tags);
                if ($datavalues['description'])
                    $datavalues['description'] = real_strip_tags ($datavalues['description'],$allowable_html_tags);                
                $datavalues['status'] = 1;// $userinfo['status'];
                $datavalues['date_modified'] = get_current_datetime();
                $datavalues['last_ip_address'] = get_user_real_ip_address();
                $datavalues[$this->field_id] = $this->getCurrentUserID();
                //old way
                //$userid = (int) $_SESSION['userid'];
                //$SystemDB->simpleUpdate($this->table_items, $datavalues, $this->field_id, $userid);                
                $this->saveItem($userinfo[$this->field_id], $datavalues);
                
                $_SESSION['user_profile_update_status'] = ___('Your profile has been updated');
                $this->redirectToOtherAction('action=myprofile');
                
                return;
            }
        }

        $this->setPageTitle(___('Edit Profile'));
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.myprofile', $local_variables);
    }

    //_______________________________________________________________________________________________________________//
    public function getItemByVirtualFilename($input_filename, $category_id = -1) {
        // FOR COMPATIBILITY - THIS FUNCTION IS TO BE REMOVED
        $x = $this->getItemsByVirtualFilename($input_filename, $category_id);
        return $x[0];
    }

    //_______________________________________________________________________________________________________________//
    public function getItemsByVirtualFilename($input_filename, $category_id = -1) {
        global $SystemDB;

        if (!empty($input_filename)) {

            $str = "";
            $filename = sanitize_string($input_filename);
            if ($category_id >= 0)
                $str = " AND category_id = {$category_id}";
            $fields = $this->getNonAmbiguousFieldNames($this->table_items);

            $sql = "SELECT {$fields} FROM {$this->table_items} WHERE {$this->field_username} = {$filename}{$str}";
            $SystemDB->query($sql);
            $items = $SystemDB->getQueryResultArray($sql);
            return $items;
        } else
            return false;
    }

    //_______________________________________________________________________________________________________________//
    public function getCategoriesByVirtualFilename($input_filename, $parent_id = -1) {
        global $SystemDB;

        if (!empty($input_filename)) {
            $filename = sanitize_string($input_filename);
            $str = "";
            if ($parent_id >= 0)
                $str = " AND parent_id = {$parent_id}";
            $sql = "SELECT * FROM {$this->table_categories} WHERE {$this->field_groupname} = {$filename}{$str}";
            $categories = $SystemDB->getQueryResultArray($sql);
            return $categories;
        } else
            return false;
    }

    //_______________________________________________________________________________________________________________//

    public function hasAdministrationPermission() {
        global $SystemDB;

        $userid = intval($_SESSION['userid']);
        $admingroupname = sanitize_string(SCHLIX_DEFAULT_ADMIN_GROUP);
        $sql = "SELECT {$this->table_categories}.{$this->field_category_id} FROM {$this->table_categories} INNER JOIN {$this->table_categories_items} ON {$this->table_categories}.{$this->field_category_id} = {$this->table_categories_items}.{$this->field_category_id} WHERE {$this->table_categories_items}.{$this->field_id} = {$userid} AND {$this->table_categories}.{$this->field_groupname} = {$admingroupname}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        return ($result != null);
    }

    //_______________________________________________________________________________________________________________//
    public function hasPermission($permission_str) {
        global $SystemDB;

        $userid = intval($_SESSION['userid']);
        $admingroupname = sanitize_string(SCHLIX_DEFAULT_ADMIN_GROUP);
        $extra_criteria = '';
        $extra_bracket = '';
        if (!empty($permission_str)) {
            $permission_array = (is_string($permission_str)) ? @unserialize($permission_str) : $permission_str;
            if (!is_array($permission_array))
                return false;
            $permission_list = implode(', ', $permission_array);
            $extra_bracket = '(';
            $extra_criteria = " OR {$this->table_categories}.{$this->field_category_id} in ({$permission_list}))";
        }

        $sql = "SELECT {$this->table_categories}.{$this->field_category_id} FROM {$this->table_categories} INNER JOIN {$this->table_categories_items} ON {$this->table_categories}.{$this->field_category_id} = {$this->table_categories_items}.{$this->field_category_id} WHERE {$this->table_categories_items}.{$this->field_id} = {$userid} AND {$extra_bracket} {$this->table_categories}.{$this->field_groupname} = {$admingroupname} {$extra_criteria}";
        $cache_key = 'user_has_permission_'.sha1($sql);
        
        $result = \SCHLIX\cmsContextCache::get('users',$cache_key);
        if (!$result)
        {
            $result = $SystemDB->getQueryResultSingleRow($sql);
            \SCHLIX\cmsContextCache::set('users',$cache_key, $result);
        }
        
        return ($result != null);
    }

    //_______________________________________________________________________________________________________________//
    public function hasReadPermission($permission_str) {
        $permission = @unserialize($permission_str);
        if ($permission_str == '' || $permission == 'everyone')
            return true;
        else
            return $this->hasPermission($permission_str);
    }

    //_______________________________________________________________________________________________________________//
    public function hasWritePermission($permission_str) {
        return $this->hasPermission($permission_str) || $this->isCurrentUserMemberOfGroupName(SCHLIX_DEFAULT_ADMIN_GROUP);
    }

    //_______________________________________________________________________________________________________________//
    public function getGroupIDArrayForPermission() {
        $choices_array = [];
        $groups = $this->getAllCategories();
        foreach ($groups as $group)
            $choices_array[] = array('value' => $group['cid'], 'label' => $group[$this->field_groupname]);
        return $choices_array;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Deprecated. Alias of \INPUT::UserGroupReadPermissionCheckboxGroup but with echo
     * @deprecated since version 2.0
     * @param string $permission_str
     */
    public function draw_permission_read_checkboxgroup($permission_str) {
        echo \INPUT::UserGroupReadPermissionCheckboxGroup('read', $permission_str);
    }

    /**
     * Depcreated. Alias of \INPUT::UserNameSelectionField but with echo
     * @deprecated since version 2.0
     * @param string $fieldname
     * @param string $value
     * @param string $label
     * @param bool $allow_empty
     */
    public function draw_username_selection_field($fieldname, $value, $label = '', $allow_empty = false) {
        echo \INPUT::UserNameSelectionField($fieldname, $value, $label, array('class' => 'form-control'));
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Deprecated. Alias of \INPUT::UserGroupWritePermissionCheckboxGroup
     * @deprecated since version 2.0
     * @param type $permission_str
     * @param type $current_user_groups
     * @param type $enable_current_user_group
     */
    public function draw_permission_write_checkboxgroup($permission_str, $current_user_groups, $enable_current_user_group = true) {
        echo \INPUT::UserGroupWritePermissionCheckboxGroup('write', $permission_str, $current_user_groups, $enable_current_user_group);
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Display welcome page for authenticated users
     * @param string $app_name
     */
    public function displayAuthenticatedUserMainByApp($app_name) {
        $found = false;
        $requested_app = null;
        $all_frontend_apps = $this->getListofFrontendApplicationsWithUserMainPage();
        foreach ($all_frontend_apps as $app) {
            $found = ($app['name'] == $app_name);
            if ($found)
                break;
        }
        if ($found) {
            $requested_app = new $app_name;
        }
        $this->setPageTitle(___('Members Page'));
        $local_variables = compact(array_keys(get_defined_vars()));
        $this->loadTemplateFile('view.members.main', $local_variables);
    }

    //_______________________________________________________________________________________________________________//
    public function getListofFrontendApplicationsWithUserMainPage($from_cache = true, $cache_time_in_minutes = 5) {
        $all_frontend_apps = get_list_of_all_apps($from_cache, $cache_time_in_minutes);
        $all_frontend_apps_with_user_main_page = [];
        foreach ($all_frontend_apps as $frontend_app_name) {
            if ($frontend_app_name != SCHLIX_DEFAULT_USER_CLASS) {
                $full_frontend_app_name = '\\App\\'.$frontend_app_name;
                $app = new  $full_frontend_app_name;
                if ($app->hasAuthenticatedUserMainPage())
                    $all_frontend_apps_with_user_main_page[] = array('name' => $app->getApplicationName(), 'description' => $app->getApplicationDescription());
                unset($app);
            }
        }
        return $all_frontend_apps_with_user_main_page;
    }
    
    /* Resolve hostnames from ip_address field
     * @global SCHLIX\cmsDatabase $SystemDB
     */
    public function updateLogHostnames()
    {
        global $SystemDB;
        $count = 0;
        
        $sql = "SELECT id, ip_address FROM `gk_user_history` WHERE host_name IS NULL";
        $unprocessed_logs = $SystemDB->getQueryResultArray($sql);
        if ($unprocessed_logs)
        {
            foreach ($unprocessed_logs as $log) {
                $log_id = $log['id'];
                $host_name = @gethostbyaddr($log['ip_address']);
                if ($host_name == $log_id || empty($host_name))
                    $host_name =  '-';
                else
                    $count++;
                $data_update = array('host_name' => $host_name);
                $SystemDB->simpleUpdate($this->table_name,$data_update,'id',$log_id);
                $data_update = null;
            }
            //if ($count > 0)
              //  $this->record ("{$count} IP addresses resolved");
        }
    }
    
    /**
     * Returns a user's avatar URL. Valid size: small, medium, large.
     * @param int $id
     * @param string $size
     * @return string
     */
    public function getUserAvatarURLByID($id, $size='small')
    {
        $user = $this->getItemByID($id);
        if ($user)
        {
            return $this->getUserAvatarURLByExistingUserInfo($user, $size);
        } else return NULL;
    }
    
    /**
     * Returns a user's avatar URL. $he User parmaeter is an array containing user data that has been previously loaded. Valid size: small, medium, large.
     * @param array $user
     * @param string $size
     * @return string
     */
    public function getUserAvatarURLByExistingUserInfo($user, $size='small')
    {
        $valid_sizes = array('small','medium','large');
        if (!in_array($size, $valid_sizes))
            return false;
        if ($user && array_key_exists('avatar', $user))
        {
            $default_user_avatar_url = SCHLIX_SYSTEM_URL_PATH.'/images/default_user.png';
            $avatar_size = 'avatar_'.$size;
            if (is_file($this->getDataFileFullPath($avatar_size, $user['avatar'])))
                $avatar_url = $this->getDataFileURLPath($avatar_size, $user['avatar']);

            return $avatar_url ? $avatar_url : $default_user_avatar_url;
        } else return NULL;
    }    
    //_______________________________________________________________________________________________________________//
    /**
     * CRON Scheduler Method - resolve hostnames in log
     * @global cmsLogger $SystemLog
     */    
    public static function processRunResolveUserHistoryHostnames()
    {
        global $SystemLog;

        $SystemLog->updateLogHostnames();        
    }
    
    //_______________________________________________________________________________________________________________//
    public function Run($command) {
        global $SystemDB;

        if ($this->backend_mode != 1) {
            if ($this->authenticated() === true) {
                switch ($command['action']) {
                    case 'userpage': $this->displayAuthenticatedUserMainByApp($command['userapp']);
                        break;
                    case 'viewitem': $this->displayUserInfoByID($command[$this->field_id]);
                        break;
                    case 'myprofile': $this->viewEditMyProfilePage();
                        break;
                    case 'changepassword': $this->viewPasswordChangePage();
                        break;                    
                    case 'logout': $this->logout();
                        $this->redirectToOtherAction('');
                        break;
                    default: $this->viewMainPage();
                        break;
                }
            } else {
                switch ($command['action']) {
                    case 'viewitem': $this->displayUserInfoByID($command[$this->field_id]);
                        break;
                    case 'passwordreset': $this->viewPasswordResetPage($command['xt'], $command['hash']);
                        break;
                    case 'activation': $this->processActivationRequest($command['id'], $command['token']);
                        break;
                    case 'forgotpassword': $this->viewForgotPasswordPage();
                        break;
                    case 'register': $this->viewUserRegistrationPage();
                        break;
                    case 'login': $this->viewLoginPage();
                        break;
                    default: $this->viewMainPage();
                        break;
                }
            }
        }
        return true;
    }

}

?>