<?php

namespace App;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
//_______________________________________________________________________________________________________________//

class versioning extends \SCHLIX\cmsApplication_List {

    private $header;
    private $guid;

    public function __construct() {
        parent::__construct('Versioning', 'gk_global_version_items');
        $this->disable_frontend_runtime = true;
    }

    //_______________________________________________________________________________________________________________//
    public function getItemArchiveByVersion($app_name, $guid, $version) {

        $version = sanitize_string(floatval($version));
        $app_name = sanitize_string($app_name);
        $guid = sanitize_string($guid);
        $itemdata = $this->table_items->selectOne('*', "(app_name = {$app_name}) AND (guid = {$guid}) AND (version = {$version})");
        if ($itemdata) {
            $unserialized_data = @unserialize($itemdata['content']);
            return $unserialized_data;
        } else
            return NULL;
    }

    //_______________________________________________________________________________________________________________//
    public function getItemArchiveMaxVersion($app_name, $guid) {
        $app_name = sanitize_string($app_name);
        $guid = sanitize_string($guid);
        $itemdata = $this->table_items->selectOne('MAX(version) as max_version', "(app_name = {$app_name}) AND (guid = {$guid})");
        return $itemdata['max_version'];
    }

    //_______________________________________________________________________________________________________________//

    public function getItemsByApplicationName($app_name) {
        $app_name = sanitize_string($app_name);
        return $this->table_items->selectAll('*', "(app_name = {$app_name})");
    }

/*
      //_______________________________________________________________________________________________________________//
      public function getAllItems($fields, $extra_criteria, $start, $end, $sortby, $sortdirection, $from_cache)
      {
      } */

    //_______________________________________________________________________________________________________________//
    /**
     * Saves data to versioning table
     * @param cmsApplication_List $data
     * @param array $data
     * @global Database $SystemDB
     * @global Users $CurrentUser
     *
     */
    public function save($app, array $data) {
        global $CurrentUser, $CurrentUser, $SystemDB;

        $WhichUser = ($CurrentUser == null) ? $CurrentUser : $CurrentUser;

        if ($app->hasVersioning()) {
            // only if it has guid and version
            if (array_key_exists('guid', $data) && array_key_exists('version', $data)) {
                $guid = sanitize_string($data['guid']);
                $created_by_id = sanitize_string($WhichUser->getCurrentUserID());
                $version = sanitize_string($data['version']);
                $app_name = sanitize_string($app->getApplicationName());
                $serialized_obj = @serialize($data);
                $current = sanitize_string(get_current_datetime());
                if ($serialized_obj) {
                    $sanitized_serialized_obj = sanitize_string($serialized_obj);
                    $sqltxt = "INSERT INTO {$this->table_items} (guid, version, app_name, date_created, created_by_id, content) VALUES ({$guid}, {$version}, {$app_name}, {$current}, {$created_by_id}, {$sanitized_serialized_obj})";
                    $SystemDB->query($sqltxt);
                    return true;
                }
            }
        }
        return false;
    }

}

?>