/* 
 * Copyright (C) 2015 SCHLIX WEB INC.
 *
 * This software is licensed under SHARED SOURCE; you can redistribute
 * but forking or creating a derivative is prohibited 
 *
 * Please read the LICENSE.html for details
 */

/**
 * View Controller class
 */
SCHLIX.CMS.WysiwygEditorsAdmin = class extends SCHLIX.CMS.BaseController  {  
    /**
     * Constructor
     */
    constructor ()
    {
        super("wysiwygeditors");
    };

    showInstallerWindow ()
    {
        var obj = SCHLIX.CMS.getObject('schlix-cms-package-installer');
        obj.show();
    }; // end func
    
    onDOMReady()
    {
        
        SCHLIX.Event.on('zipfileupload', 'change', this.previewFile, this, true);
        
    }; // end func
    ////////////////////////////////////////////////////////////
    
    previewFile ()
    {
        var file_input = document.getElementById('zipfileupload');
        var file_info = file_input.files[0];
        var file_name_placeholder = document.getElementById('zip_filename');
        var file_size_placeholder = document.getElementById('__file_size');
        var file_size_display=  document.getElementById('zip_filesize');
        
        file_size_placeholder.value = file_info.size;
        file_name_placeholder.innerHTML = '<i class="far fa-file-archive"></i> ' + file_info.name;
        
        var max_file_size = parseInt(document.getElementById('__max_file_size').value, 10);
        if (file_info.size > max_file_size)
        {
            var max_file_size_formatted = this.formatFileSize(max_file_size);
            SCHLIX.Alert.warning('File upload will fail since the file is larger than the maximum upload size on this server (' +max_file_size_formatted + ')');
        }
        
        file_size_display.innerHTML = '<strong>File size</strong>: ' + format_file_size(file_info.size);
    }; // end func 

    /**
     * Called before submitting to parent
     * @returns {unresolved}
     */
    submitForm ()
    {
        /* var obj = SCHLIX.CMS.getObject('schlix-cms-package-installer');
         var form = obj.__getFirstFoundForm();
         console.log(form);*/
        return RESOLVE_TO_PARENT;
    }; // end func
    
    

    /**
     * Datatable row format: item title
     * @param {type} elCell
     * @param {type} oRecord
     * @param {type} oColumn
     * @param {type} oData
     * @param {type} oDataTable
     * @returns {undefined}
     */
    static formatDataTableCell_ProfileTitleColumn  (elCell, oRecord, oColumn, oData) {

        var field_title = SCHLIX.Util.isUndefined(this.parentControl.element_data["field-item_title"]) ? "title" : this.parentControl.element_data["field-item_title"];
        var odata_id = oRecord.getData("id");
        var odata_cid = oRecord.getData("cid");
        var itemTitle = oRecord.getData(field_title); // oRecord.getData("title");
        var itemLink = site_httpbase;
        if (itemTitle == null)
        {
            SCHLIX.log("formatDataTableCell_DefaultTitleColumn cannot process a null title", 'warn');
            return;
        }
        if (itemTitle.length > 40)
            itemTitle = itemTitle.substr(0, 40) + '...';
        var app_name = this.parentControl.app_name;
            // console.log(oRecord);
            var the_id = oRecord.getData("id");
            var theValue = 'i' + the_id;
            if (itemTitle === '')
                itemTitle = '(Untitled)';
            //itemLink = _schlix_app_controller + oColumn.app_name + '&action=edititem&id=' + oRecord.getData("id");
            var icon = "<i class =\"far fa-file-alt fa-2x\"></i>";
            // var checkbox = '<input type="checkbox" class="selections" value="' + theValue + '"  />';            

            itemLink += this.parentControl.schlix_application_url + 'action=edititem&id=' + the_id;
            var item_class = (oRecord.getData('id')==1 || oRecord.getData('virtual_filename') == 'home') ? 'html_home' : 'dragdrop';
            var preview_link = oRecord.getData("preview_link");
            var display_preview_link = '';
            if (SCHLIX.Util.isString(preview_link))
            {
                var preview_icon = "<i class=\"fa fa-eye\"></i> ";
                display_preview_link = '<div style="float:right;margin-left:4em">' +  '<a target="_blank" href="' + preview_link + '">' + preview_icon +  '</a>' + '</div>';
            }

            elCell.innerHTML = '<a class="' + item_class + '" id="' + app_name + "-lnk-" + theValue + '" data-dragdrop-id="' + theValue + '" title="Click here to edit this item" href="' + itemLink + '">' + " " + icon + " " + itemTitle + '</a>' + display_preview_link;
    };    
    
    static formatProfile_CheckBox (elCell, oRecord, oColumn, oData) {
            if (oRecord.getData('is_system')==1)
                    elCell.innerHTML = '<i class="fa fa-2x fa-edit"></i>';
            else
            {
                var theID = '';
                var theValue = '';

                var app_name = this.parentControl.app_name;
                if (oRecord.getData("cid") > 0)
                {
                    theID = app_name + '-select-cid' + oRecord.getData("cid");
                    theValue = 'c' + oRecord.getData("cid");
                }
                else
                {
                    theID = app_name + '-select-id' + oRecord.getData("id");
                    theValue = 'i' + oRecord.getData("id");
                }

                elCell.innerHTML = '<input type="checkbox" class="' + app_name + '-chkselections" name="' + app_name + '-chkselections[]" id="' + theID + '"  value="' + theValue + '" />';

            }
    }; 
    
   redirectToOtherAppCMSCommand (app, command, open_in_new_window)
   {
       var the_url = site_httpbase + "/admin/app/" + app + "?action=" + command;

       if (open_in_new_window)
       {
           var win = window.open(the_url, 'new');
           if (win) {
               //Browser has allowed it to be opened
               win.focus();
           } else {
               //Browser has blocked it
               SCHLIX.Alert.warning('Cannot open the command in a new window since a popup blocker blocked the call. Please enable popup for this website');
           }
       } else
       {
           window.location.href = the_url;
       }

   };
    
    /**
     * Run command
     * @param {type} command
     * @returns {unresolved}
     */
    runCommand (command, evt)
    {
        var open_in_new_window = is_rightclick_event(evt);
        
        switch (command)
        {
            case 'new-item':
                var editor_name = (evt.target.getAttribute('data-wysiwyg-editor-name'));
                this.redirectToOtherAppCMSCommand('wysiwygeditors.profiles', "newitem&editorname=" + editor_name,open_in_new_window);
                return true;
                break;
            case 'install':
                this.showInstallerWindow();
                break;
            case 'dialog-submit':
                return this.submitForm();
                break;
            case 'form-submit-ok':
                this.cms_control.refreshControls();
                break;
            default:
                return super.runCommand(command, evt);
                break;
        }
    }
};
