<?php

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

namespace App;


class wysiwygEditors extends \SCHLIX\cmsApplication_List {

    /**
     * The default editor
     * @var \SCHLIX\cmsWysiwygEditor 
     */
    protected $default_editor;
    /**
     * Override the limited profile ID
     * @var int 
     */
    protected $override_limited_profile_id;
    /**
     * Override the full profile ID
     * @var int 
     */    
    protected $override_full_profile_id;
    

    public function __construct() {
        parent::__construct('Wysiwyg Editors', 'gk_wysiwyg_items');
        $this->disable_frontend_runtime = false;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Returns duplicate item, given item title
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param array $data
     * @return array
     */
    public function findDuplicateItems($data) {
        global $SystemDB;

        $current_id = (int) $data[$this->field_id];
        $the_title = sanitize_string($data['title']);
        $sql = "SELECT * from {$this->table_items} WHERE (title = {$the_title})";

        if (intval($current_id) != 0)
            $sql .= " AND (id != '{$current_id}')";

        $result = $SystemDB->getQueryResultArray($sql);

        return $result;
    } 

    //_______________________________________________________________________________________________________________//
    public function getDefaultWYSIWYGID($type = 'frontend') {
        $data = $this->getDefaultWYSIWYG();
        return ($data != NULL) ? $data[$this->field_id] : 0;
    }

    /**
     * Return the editor by virtual filename
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $virtual_filename
     * @return array
     */
    public function getWysiwygEditorByName($virtual_filename) {
        global $SystemDB;

        return $SystemDB->getQueryResultSingleRow("SELECT * FROM {$this->table_items} WHERE `title` = :virtual_filename", ['virtual_filename' => $virtual_filename]);
    } 
    
    public function getItemsByVirtualFilename($virtual_filename, $category_id = -1) {

        return $this->getWysiwygEditorByName($virtual_filename);
    } 
    
    /**
     * Loading WYSIWYG configuration script
     * @param int $profile_id
     */
    public function viewInitScript($profile_id) {
        $this->initializeMyWysiwygEditorByProfileID($profile_id);
        $this->default_editor->viewInitScript();
    }

    /**
     * Inits $this->default_editor
     * @param int $profile_id
     */
    protected function initializeMyWysiwygEditorByProfileID($profile_id) {
        $this->default_editor = \App\wysiwygEditors_Profiles::getEditorInstanceByProfileID($profile_id);
    }


    /**
     * Override the default full provilege WYSIWYG editor. Usually called by admin class
     * with specific need for a specific editor
     * @param int $profile_id
     */
    public function overrideFullPrivilegeWysiwygEditor($profile_id)
    {
        $this->override_full_profile_id = $profile_id;
    }
    
    /**
     * Override the default limited WYSIWYG editor. Usually called by admin class
     * with specific need for a specific editor
     * @param int $profile_id
     */
    public function overrideLimitedPrivilegeWysiwygEditor($profile_id)
    {
        $this->override_limited_profile_id = $profile_id;
    }
    
    /**
     * Run the default WYSIWYG profile - backend     
     * @global \SCHLIX\cmsConfigRegistry $SystemConfig
     */
    public function runDefaultLimitedPrivilegeWysiwygEditor() {

        $profile_id = $this->override_limited_profile_id ? $this->override_limited_profile_id : $this->getConfig('int_default_limited_privilege_wysiwyg_profile_id', 2);
        $this->runWysiwygEditorByProfileID($profile_id);
    }
    
    /**
     * Run the default WYSIWYG profile - backend     
     * @global \SCHLIX\cmsConfigRegistry $SystemConfig
     */
    public function runDefaultFullPrivilegeWysiwygEditor() {

        
        $profile_id = $this->override_full_profile_id ? $this->override_full_profile_id : $this->getConfig('int_default_full_privilege_wysiwyg_profile_id', 1);
        $this->runWysiwygEditorByProfileID($profile_id);
    }

    /**
     * Run the WYSIWYG Editor by ID
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @global \SCHLIX\cmsConfigRegistry $SystemConfig
     * @global \SCHLIX\cmsLogger $SystemLog
     * @param int $profile_id
     */
    public function runWysiwygEditorByProfileID($profile_id) {
        global $HTMLHeader;
        $url = $this->createFriendlyURL("action=init&id={$profile_id}");

        $this->initializeMyWysiwygEditorByProfileID($profile_id);
        if ($this->default_editor) {
            $js = $this->default_editor->getHeaderVariableInitScript($url);
            if ($js)
                $HTMLHeader->JAVASCRIPT_TEXT($js);

            $HTMLHeader->JAVASCRIPT_EXTERNAL($url);
        } else {
            $this->recordLog('Unable to load WYSIWYG editor ID#' . $profile_id);
        }
    }

    public function callEditorFunction ($command)
    {
        if ($command['editor'])
        {
            $item = $this->getWysiwygEditorByName($command['editor']);
            if ($item)
            {
                $editor_name ='\\WysiwygEditor\\'. $item['title'];
                try
                {
                $editor = new $editor_name();
                if ($editor)
                    $editor->Run($command);
                } 
                catch (\Exception $exc) {
                    $this->recordLog("callEditorFunction: Cannot find WYSIWYG editor {$editor_name}");
                }
            } else
            {
                die ("Not a valid WYSIWYG editor class: {$command['editor']}");
            }
        }
    }
    //_______________________________________________________________________________________________________________//
    public function Run($command) {
        switch ($command['action']) {
            case 'init':
                $this->viewInitScript($command['id']);
                return false;
                break;
            case 'call':
                $this->callEditorFunction($command);
                return false;
            default:
                redirect_url(SCHLIX_SITE_HTTPBASE . '/');
        }

        return false;
    }

}

//class
?>
