<?php
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

namespace App;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
class wysiwygEditors_Profiles_Admin extends \SCHLIX\cmsAdmin_List {

//_________________________________________________________________________//
    public function __construct() {
        // Data: Item
        parent::__construct(true, null);
        $this->setItemFieldNamesForAjaxListing('id', 'title', 'virtual_filename', 'date_created', 'date_modified', 'is_system');
        
    }
      
    //_________________________________________________________________________//
    public function ajaxGetProfiles($poll_id, $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC') {
        global $SystemDB;

        $poll_id = (int) $poll_id;
        if ((int) $start > (int) $end)
            return ajax_reply(400, 'Invalid Request');
        $sortby = $SystemDB->escapeString($sortby);
        $items_per_page = min(HARDCODE_MAX_ROWLIMIT, $end - $start);
        $fields = implode(',', quote_array_of_field_names_for_query($this->getItemFieldNamesForAjaxListing()));
        $total_item_count = $this->app->getTotalItemCount();
        $result_array_files = $this->app->getAllItems($fields, "editor_id = {$poll_id}", $start, $end, $sortby, $sortdirection, false);
        return ajax_datasource_reply(200, $result_array_files, $start, $end, $items_per_page, $total_item_count, $sortby, $sortdirection);
    }
            
    
    /**
     * Modify data before save item
     * @global \App\Users $CurrentUser
     * @global \App\WysiwygEditors $WYSIWYGEditor
     * @param array $datavalues
     * @return array
     */
    public function onModifyDataBeforeSaveItem($datavalues) {
        global $CurrentUser, $WYSIWYGEditor;
        
        $datavalues = parent::onModifyDataBeforeSaveItem($datavalues);
        
        if ($datavalues['permission_read_everyone'])
            $datavalues['permission_read'] = 'everyone';
        $datavalues['permission_read'] = serialize($datavalues['permission_read']);
        $datavalues['permission_write'] = serialize($datavalues['permission_write']);

        $datavalues['virtual_filename'] = alpha_numeric_with_dash_underscore($datavalues['virtual_filename']);
        $datavalues['virtual_filename'] = $this->app->preventDuplicateValueInItemTable('virtual_filename',  $datavalues['virtual_filename'],$datavalues[$this->field_id]);
        if ((int) $datavalues['id'] > 0)
        { 
            $editor = \App\wysiwygEditors_Profiles::getEditorInstanceByProfileID($datavalues['id']);
        } else
        {
            $editor_data = $WYSIWYGEditor->getWysiwygEditorByName($datavalues['wysiwyg_editor_name']);
            if ($editor_data)
            {
                $datavalues['editor_id'] = $editor_data['id'];
                $editor_name = '\\WysiwygEditor\\'.alpha_numeric_with_dash_underscore($datavalues['wysiwyg_editor_name']);
                $editor = new $editor_name();
                
            } else die (sprintf( ___('Cannot find editor %s'), $datavalues['wysiwyg_editor_name']));
            
        }
        if ($editor)
            $datavalues = $editor->onModifyDataBeforeSaveConfig ($datavalues);
        return $datavalues;
    }
    /**
     * Write config after save item
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) {
        
        
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            $editor = \App\wysiwygEditors_Profiles::getEditorInstanceByProfileID($retval['id']);
            
            $post_keys_to_ignore = array('id');


            $existing_wysiwyg_profile = $this->getItemByID(intval($retval['id']));
            $data_wysiwyg_short_code_name = $existing_wysiwyg_profile['virtual_filename'];      

            $wysiwyg_config = new \SCHLIX\cmsConfigRegistry('gk_wysiwyg_config');
            $existing_keys = $wysiwyg_config->get($data_wysiwyg_short_code_name);
            $wysiwyg_config->clearCache($data_wysiwyg_short_code_name);
            $wysiwyg_config->deleteSection($data_wysiwyg_short_code_name);
            $data_for_wysiwyg = $_POST;

            foreach ($post_keys_to_ignore as $ignore)
                 unset($data_for_wysiwyg[$ignore]);
            if ((int) $original_datavalues['reset_to_default'] == 1)
            {
                $data_for_wysiwyg = $editor->getDefaultProfileResetSettings($original_datavalues['reset_to_default_profile']);
            } else
            {
                if ($existing_keys)
                {
                    foreach ($existing_keys as $ex_key => $ex_value) {
                        if (!array_key_exists($ex_key, $data_for_wysiwyg))
                            $data_for_wysiwyg[$ex_key] = NULL;
                    }
                }
            }

            // handle checkbox save because it won't show up in post
            foreach ($data_for_wysiwyg as $key => $value) {
                $wysiwyg_config->set($data_wysiwyg_short_code_name, $key, $value);
            }
            
            $editor->onAfterSaveConfig ($retval['id']);
            $editor->reloadConfigFromDatabase();
            $generated_script = $editor->getUncachedConfigurationScript();
            
            $wysiwyg_config->set($data_wysiwyg_short_code_name,'str_generated_config', $generated_script);
            
        }
    }
    //_________________________________________________________________________//
    public function Run() {
        switch (fget_alphanumeric('action')) {
            case 'getprofiles': 
                return ajax_echo($this->ajaxGetProfiles(fget_string('id',9), fget_int('start'), fget_int('end'), fget_string_noquotes_notags('sortby'), fget_string_noquotes_notags('sortdirection')));
                break; 
            
            default:return parent::Run();
        }
    }


}

?>