<?php

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

namespace App;

class wysiwygEditors_Profiles extends \SCHLIX\cmsApplication_List {

    protected $default_editor;

    public function __construct() {
        parent::__construct('Wysiwyg Profiles', 'gk_wysiwyg_profiles');
        $this->disable_frontend_runtime = true;
    }

    //_______________________________________________________________________________________________________________//
    public function viewMainPage() {
        return false;
    }

    /**
     * Returns an editor instance
     * @param int $id
     * @return \SCHLIX\cmsWysiwygEditor
     */
    public static function getEditorInstanceByProfileID($id) {
        $editor_name = self::getClassNameByProfileID($id);

        if ($editor_name) {
            $editor_class_name = '\\WysiwygEditor\\' . $editor_name;
            return new $editor_class_name($id);
        }
        return NULL;
    }

    /**
     * Returns the editor name by profile id
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $profile_id
     * @return string
     */
    public static function getProfileNameByID($profile_id) {
        global $SystemDB;

        $row = $SystemDB->getQueryResultSingleRow("SELECT gk_wysiwyg_profiles.*, gk_wysiwyg_items.title as editor_name FROM gk_wysiwyg_profiles INNER JOIN `gk_wysiwyg_items` ON editor_id = gk_wysiwyg_items.id WHERE gk_wysiwyg_profiles.id = :id", ['id' => $profile_id]);
        return $row['virtual_filename'];
    }

    /**
     * Returns the editor name by profile id
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $profile_id
     * @return string
     */
    public static function getClassNameByProfileID($profile_id) {
        global $SystemDB;

        if (!$SystemDB->tableExists('gk_wysiwyg_profiles')) return; // workaround for v2.1.1-5
        $row = $SystemDB->getQueryResultSingleRow("SELECT gk_wysiwyg_profiles.*, gk_wysiwyg_items.title as editor_name FROM gk_wysiwyg_profiles INNER JOIN `gk_wysiwyg_items` ON editor_id = gk_wysiwyg_items.id WHERE gk_wysiwyg_profiles.id = :id", ['id' => $profile_id]);
        return $row['editor_name'];
    }

    /**
     * Return the editor by virtual filename
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $editor_name
     * @return array
     */
    public static function getAllWysiwygProfiles() {
        global $SystemDB;

        if (!$SystemDB->tableExists('gk_wysiwyg_profiles')) return; // workaround for v2.1.1-5
        return $SystemDB->getQueryResultArray("SELECT gk_wysiwyg_profiles.*, gk_wysiwyg_items.title as editor_name FROM gk_wysiwyg_profiles INNER JOIN `gk_wysiwyg_items` ON editor_id = gk_wysiwyg_items.id");
    }

    /**
     * Return the editor by virtual filename
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $editor_name
     * @return array
     */
    public function getProfilesByEditorName($editor_name) {
        global $SystemDB;

        return $SystemDB->getQueryResultArray("SELECT {$this->table_items}.*, gk_wysiwyg_items.title as editor_name FROM {$this->table_items} INNER JOIN `gk_wysiwyg_items` ON editor_id = gk_wysiwyg_items.id WHERE gk_wysiwyg_items.`title` = :editor_name", ['editor_name' => $editor_name]);
    }

    /**
     * Create an empty editor profile and populate it with data
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @global type $WYSIWYGEditor
     * @param string $editor_name
     * @param string $profile_code_name
     * @param string $profile_name
     * @param bool $is_system
     * @param array $config
     * @return boolean
     */
    public function createEmptyProfile($editor_name, $profile_code_name, $profile_name, $is_system, $config) {
        global $SystemDB, $WYSIWYGEditor;

        $editor = $WYSIWYGEditor->getWysiwygEditorByName($editor_name);
        if ($editor) {
            $data = ['editor_id' => $editor['id'], 'title' => $profile_name, 'virtual_filename' => $profile_code_name, 'is_system' => $is_system];
            $SystemDB->query("INSERT INTO {$this->table_items} (editor_id, title, virtual_filename, date_created, date_modified, is_system) VALUES (:editor_id, :title, :virtual_filename, NOW(), NOW(), :is_system) ", $data);
            if (is_array($config))
            {
                $wysiwyg_config = new \SCHLIX\cmsConfigRegistry('gk_wysiwyg_config');
                $wysiwyg_config->deleteSection($profile_code_name);
                // handle checkbox save because it won't show up in post
                foreach ($config as $key => $value) {
                    $wysiwyg_config->set($profile_code_name, $key, $value);
                }
                return true;
            }
        }
        return false;
    }

    //_______________________________________________________________________________________________________________//
    public function Run($command) {
        redirect_url(SCHLIX_SITE_HTTPBASE . '/');
        return false;
    }

}

//class
?>
