<?php

namespace SCHLIX;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
// SCHLIX WEB CONTENT MANAGEMENT SYSTEM - Copyright (C) SCHLIX WEB INC.
// This is a SHARED SOURCE
// You may use this software commercially, but you are not allowed to create a fork or create a derivative of this software
// Please read the license for details
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//

/**
 * class SCHLIX\cmsDatabase is about to be replaced in v2.1 
 * it will use PDO instead of MySQLi
 */
class cmsDatabase extends \MySQLi {

    const _debug_sql_ = 0;
//________________________________________________________________________//
    /**
     * Is it initialized?
     * @var bool
     */
    protected $is_initalized = false;
    
    /**
     * Server hostname, e.g. localhost, 127.0.0.1
     * @var string
     */
    protected $server;
    /**
     * Database login - username
     * @var string
     */
    protected $username;
    /**
     * Database login - password
     * @var string
     */
    protected $password;
    /**
     * Database name
     * @var string 
     */
    protected $db;
    /**
     * Last Query string
     * @var strnig
     */
    protected $last_query;
    /**
     * Last query result
     * @var mixed
     */
    protected $last_query_result;
    
    private $has_warning = false;
    
    private $cacheSystem;
    
    private $__dbg_ts__;
    private $__dbg_qc__;
    
//________________________________________________________________________//
    public function __construct($server, $db, $username, $password) {

        if (defined('SCHLIX_SQL_CACHE_ENABLED') && SCHLIX_SQL_CACHE_ENABLED == true)
            $this->cacheSystem = new \SCHLIX\cmsFSCache('sql');
        $this->server = $server;
        $this->username = $username;
        $this->password = $password;
        $this->db = $db;
        $_tmp = error_reporting();
        error_reporting(0);
        $result = parent::__construct($this->server, $this->username, $this->password, $this->db);


        if ($this->connect_error) {
            if ($this->isAjaxRequest())
                return ajax_echo(ajax_reply('141', "Cannot connect to the database"));
            else
                $error_message = '###[SCHLIX Database offline.]###<br />We will be back shortly to fix this.';
            include('view.offline.template.php');
            exit;
        }
        error_reporting($_tmp);
        $this->set_charset('utf8'); // ALWAYS SET to utf8 
        $this->is_initalized = true;
        if (self::_debug_sql_)
        {
            $this->__dbg_ts__ = microtime(true);
            $this->__dbg("==================BEGIN @{$this->__dbg_ts__} ================\n");
        }
        return $result;
    }

    /**
     * Returns the query builder
     * @return \SCHLIX\cmsSQLQueryBuilder
     */
    public function q()
    {
        return new cmsSQLQueryBuilder($this);
    }
    //________________________________________________________________________//

    private function isAjaxRequest() {
        return ((!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') && ($_GET['ajax'] == 1));
    }

    //________________________________________________________________________//
    public function reselectDatabase() {
        if ($this->db && !$this->connect_error)
            $this->select_db($this->db);
    }

    //________________________________________________________________________//
    /**
     * Escape string, without the quote
     * @param string $s
     * @return string
     */
    public function escapeString($s) {
        return $this->real_escape_string($s);
    }

    //________________________________________________________________________//
    /**
     * Escape string and add a quote by default. Set to FALSE so it won't add a quote
     * @param string $s
     * @param bool $add_quote
     * @return type
     */
    public function sanitizeString($s, $add_quote = true) {
        $val = $this->real_escape_string($s);
        return $add_quote ? "'{$val}'" : $val;
    }

    //________________________________________________________________________//
    public function getTableColumns($tablename) {
        $sql = "SHOW COLUMNS FROM `{$tablename}`";
        $columns = $this->getQueryResultArray($sql);
        if ($columns)
        {
            $this->_cache_table_columns[$tablename] = $columns;
            return $columns;
        }
        else
            return false;

    }

    //________________________________________________________________________//
    /**
     * Returns true if a table has the specified column
     * @param string $tablename Table Name
     * @param string $tablecolumn Column Name
     * @return boolean
     */
    public function tableColumnExists($tablename, $tablecolumn) {
        $tablecolumn = $this->sanitizeString($tablecolumn);
        $sql = "SHOW COLUMNS FROM `{$tablename}` LIKE {$tablecolumn}";
        $columns = $this->getQueryResultArray($sql);
        return (___c($columns) > 0);
    }

    //________________________________________________________________________//
    /**
     * Returns true if table exists. Use runtime (this script only
     * @param string $tablename
     * @param bool $current_runtime_cache
     * @return boolean
     */
    public function tableExists($tablename) {

        $tablename = $this->sanitizeString($tablename);
        $sql = "SHOW TABLES LIKE {$tablename}";
        $columns = $this->getQueryResultArray($sql);
        return (___c($columns) > 0); 
    } 
    
    /**
     * Executes an SQL Query. If $data is specified, the variables will be bound
     * to the specified query and escaped properly with mysqli_real_escape_string.
     * If you are using a paremterized query, the default syntax is named statements
     * like the PDO one, not the MySQLi one. 
     * 
     * e.g.   INTO tbl_example (id,value) VALUES (:id,:value)
     * 
     * Although it's doable, DO NOT mix sql with the value, e.g. if you want
     * to do a non-paramterized statement, then don't mix it with the one
     * that uses paramaterized statment
     * 
     * This is wrong: INTO tbl_example (id,value) VALUES (:id,'Sanitized string')
     * 
     * Although it will work it can have an unexpected outcome if the string contains
     * variable name. e.g. ' .... String string string :example ....text'
     * 
     * @param string $sql SQL Query String
     * @param array $data parameters in an associative array that needs to be bound
     * @return mixed
     */
    public function query($sql, $data = NULL) 
    {              
        if (is_array($data)) {
            $unmatched = [];
            preg_match_all('/:([a-zA-Z_][a-zA-Z0-9_]+)/', $sql, $prms);
            foreach ($data as $k => $v)
            {
                $data[':'.$k] = $this->sanitizeString($v);
                unset ($data[$k]);
            }
            foreach ($prms[0] as $k)
                if (!array_key_exists($k, $data))
                        $unmatched[] = $k;
            $sql = strtr($sql, $data);
            if (!empty($unmatched))
                die (sprintf("Error: SQL Parameter [%s] was unspecified", implode(',', $unmatched)));
        }
        if (self::_debug_sql_)
        {
            $this->__dbg_qc__++;
            $caller = get_calling_class_function();
            $this->__dbg($this->__dbg_qc__."\t".$caller['class'].'->'.$caller['function']."\t".$sql."\n");
        }
        $this->last_query_result = parent::query($sql);
        $this->last_query = $sql;
        if ($this->error) { {
                $error_txt = 'Could not run query: ' . $sql . "\n Error:" . $this->error;
                if ($this->isAjaxRequest()) {
                    return ajax_echo(ajax_reply('151', "SQL Query Error: {$sql}.\n{$error_txt}"));
                } else {
                    echo '<div style="background:pink; border:1px solid orange">DB Error - ' . $error_txt . '</div>';
                }
            }
            exit;
        }
        return $this->last_query_result;
    }
    /**
     * @internal
     * @param string $text
     */
    private function __dbg($text)
    {
        file_put_contents(CURRENT_SUBSITE_PATH.'/data/private/logs/sql-'.date('Y-m-d').'.log', $text, FILE_APPEND);
    }
    /**
     * Fetch the result of the last query and return it as an associative array
     * Do not call this method directly
     * @return array
     */
    protected function fetch_result_as_array() {
        if (method_exists('mysqli_result', 'fetch_all')) {
            $array = null;
            if ($this->last_query_result)
            {
                $array = $this->last_query_result->fetch_all(MYSQLI_ASSOC);
            }
        } else {
            $array = [];
            while ($row = $this->last_query_result->fetch_assoc())
                $array[] = $row;
        }
        $this->last_query_result->close();

        return $array;
    }

    /**
     * Returns a cached query result as an associative array (data row collection)
     * @param int $hours
     * @param int $minutes
     * @param int $seconds
     * @param string $sql
     * @param array $data
     * @return array
     */
    public function getTimedCachedQueryResultArray($hours, $minutes, $seconds, $sql, $data = NULL) {

        if (SCHLIX_SQL_CACHE_ENABLED && !empty($sql)) {
            $cache_key = sha1($sql); // sha1(sql);            
            $cached_result = $this->cacheSystem->get($cache_key);
            if ($cached_result !== false) {
                return $cached_result;
            } else {
                $this->query($sql, $data);
                $array = $this->fetch_result_as_array();
                $this->cacheSystem->set($cache_key, $array, $hours, $minutes, $seconds);
                return $array;
            }
        } else {
            $this->query($sql, $data);
            return $this->fetch_result_as_array();
        }
    }

    /**
     * Returns a cached query result as an associative array (data row collection)
     * @param string $sql
     * @param array $data
     * @return array Associative Array
     */
    public function getCachedQueryResultArray($sql, $data = NULL) {

        if (SCHLIX_SQL_CACHE_ENABLED && ((int) SCHLIX_SQL_CACHE_TIME > 0) && isset($sql)) {
            return $this->getTimedCachedQueryResultArray(0, 0, SCHLIX_SQL_CACHE_TIME, $sql);
        } else {
            $this->query($sql, $data);
            return $this->fetch_result_as_array();
        }
    }

    //________________________________________________________________________//
    /**
     * Returns a single row from cache
     * @param string $sql SQL Statement
     * @param array $data Parameters that needs to be bound
     * @return array Associative Array
     */
    public function getCachedQueryResultSingleRow($sql, $data = NULL) {
        $result = $this->getCachedQueryResultArray($sql, $data);
        if ($result)
            return $result[0];
    }

    //________________________________________________________________________//
    /**
     * Get the first result of a query in (a single data row)
     * @param string $sql
     * @param array $data
     * @return array
     */
    public function getQueryResultSingleRow($sql, $data = NULL) {
        $result = $this->getQueryResultArray($sql, $data);
        if ($result)
            return $result[0];
    }

    //________________________________________________________________________//
    /**
     * Get SQL query result as an associative array (data row collections)
     * @param string $sql
     * @param array $data
     * @return array
     */
    public function getQueryResultArray($sql, $data = NULL) {

        $this->query($sql, $data);
        return $this->fetch_result_as_array();
    }    
    
    //________________________________________________________________________//    
    /**
     * Returns the last insert ID
     * @return int
     */
    public function getLastInsertID() {
        return $this->insert_id;
    }

    public function getLastQueryResult()
    {
        return $this->last_query_result;
    }
    //________________________________________________________________________//
    public function simpleInsertInto($table_name, $array) {
        foreach ($array as $key => $value) {
            $temp_keys[] = '`' . $key . '`';
            $temp_values[] = $this->sanitizeString($value);
        }
        $str_keys = implode(', ', $temp_keys);
        $str_values = implode(', ', $temp_values);

        $sql = "INSERT INTO `{$table_name}` ($str_keys) VALUES ($str_values)";
        return $this->query($sql);
    }

    //________________________________________________________________________//
    public function simpleUpdate($table_name, $array, $where_key, $where_value) {
        $string = '';
        $keys = array_keys($array);
        foreach ($keys as $key) {
            if ($key != $where_key)
                $string.= $key . '=' . $this->sanitizeString($array[$key]) . ', ';
        }
        $sql_set_cmd = substr($string, 0, strlen($string) - 2); // take out the last comma
        $where_value = $this->sanitizeString($where_value);
        $sql = "UPDATE {$table_name} SET " . $sql_set_cmd . " WHERE `{$where_key}` = $where_value;";
        return $this->query($sql);
    }

    /**
     * Performs search and replace
     * @param string $table_name
     * @param string $field_name
     * @param string $search
     * @param string $replace
     * @param string $where_condition
     */
    public function searchAndReplace($table_name, $field_name, $search, $replace, $where_condition = NULL)
    {
        $str_search = $this->sanitizeString($search);
        $str_replace = $this->sanitizeString($replace);
        $str_where = empty($where_condition) ? '' : " WHERE {$where_condition}";
        return $this->query("UPDATE `{$table_name}` SET `{$field_name}` = REPLACE(`{$field_name}`, '{$str_search}', '{$str_replace}') {$str_where}");
    }
    
    //_______________________________________________________________________________________________________________//
    public function getAllRecordsFromTable($table_name, $data_fields, $extra_criteria, $fields = '*', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC', $from_cache = false) {
        global $SystemDB;

        if ($sortby != '')
            if (strpos($sortby, ',') === false)
                if (!array_key_exists($sortby, $data_fields))
                    $sortby = '';
        $sql = $SystemDB->generateSelectSQLStatement($table_name, $fields, $extra_criteria, $start, $end, $sortby, $sortdirection, true, SCHLIX_SQL_ENFORCE_ROW_LIMIT);

        $search_result = $SystemDB->getQueryResultArray($sql);
        return $search_result;
    }
    
    //_______________________________________________________________________________________________________________//
    function generateSQLSelectParameters($tablename, $fields = '*', $extra_criteria = '', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC', $force_limit = false) {
        global $SystemDB;

        $start = intval($start);
        $end = intval($end);
        $sort_criteria = '';
        $additional_criteria = '';
        $sortby = $SystemDB->escapeString(preg_replace("/[^a-z.,_\d]/i", '', $sortby)); // further sanitize to preventy sql injection
        if (empty($fields))
            $fields = '*'; // double check
        if ($start > $end)
            $start = $end = 0; // autocorrect

        $fields = trim($fields);

        if (!is_array($fields) && $fields !== '*' && strpos($fields, ',') !== FALSE) {
            $fields_array = explode(',', $fields);
            $fields_count = ___c($fields_array);
            for ($i = 0; $i < $fields_count; $i++) {
                $tmp_field = str_replace('`', '', $fields_array[$i]);
                $fields_array[$i] = "`{$tablename}`.`{$tmp_field}`";
            }
            $fields = implode(',', $fields_array);
        } else if ($fields === '*') {
            $fields = "{$tablename}.*";
        }
        if (!empty($sortby) && !empty($sortdirection)) {
            //$sortby = quote_field_name_for_query($sortby); if there's more than one sort, this is invalid - dec 8, 2011
            $sortdirection = (strtoupper($sortdirection) == 'DESC') ? 'DESC' : 'ASC';
            $sort_criteria = " ORDER BY {$sortby} {$sortdirection}";
        }

        if ($force_limit)
            $limiter = HARDCODE_MAX_ROWLIMIT;
        else
            $limiter = 0;
        $item_start = max(0, $start);
        $item_limit = min($end - $start, $limiter);
        if ($force_limit == true && $item_limit == 0 && $item_start == 0)
            $item_limit = HARDCODE_MAX_ROWLIMIT;
        if (!empty($extra_criteria))
            $additional_criteria = " {$extra_criteria} ";

        return array('fields' => $fields, 'tablename' => $tablename, 'criteria' => $additional_criteria, 'sort_criteria' => $sort_criteria, 'start' => $item_start, 'limit' => $item_limit);
    }

//_______________________________________________________________________________________________________________//
    function generateSelectSQLStatement($tablename, $fields = '*', $extra_criteria = '', $start = 0, $end = 0, $sortby = '', $sortdirection = 'ASC', $add_where_clause = true, $force_limit = false) {
        
        $limit_criteria = '';

        $start = intval($start);
        $end = intval($end);

        if ($add_where_clause && !empty($extra_criteria))
            $criteria = " WHERE {$extra_criteria} ";
        else
            $criteria = $extra_criteria;
        $params = $this->generateSQLSelectParameters($tablename, $fields, $criteria, $start, $end, $sortby, $sortdirection, $force_limit);
        if ($params) {
            if ($params['start'] >= 0 && $params['limit'] > 0)
                $limit_criteria = "	LIMIT {$params['start']}, {$params['limit']}";
            $sql = "SELECT {$params['fields']} FROM {$params['tablename']} {$params['criteria']} {$params['sort_criteria']} {$limit_criteria}";
            return $sql;
        } else
            return false;
    }

//________________________________________________________________________//
    function __destruct() {
        if ($this->is_initalized)
        {            
            $this->close();
            if (self::_debug_sql_)
            {
                $date_end = get_current_datetime();
                $diff = round(microtime(true) - $this->__dbg_ts__, 3);
                $this->__dbg("\n========= END @ {$date_end} ~ {$diff} seconds ================\n");
            }        
            
        }
    }

//________________________________________________________________________//
}

class cmsSessionHandler {

    private $session_table = 'gk_session_handler';
    private $db;
    private $debug_filename;

    public function __construct() {
        $this->debug_filename = SCHLIX_SITE_PATH . '/debug_session.txt';
        $this->db = new cmsDatabase(SCHLIX_DB_HOST, SCHLIX_DB_DATABASE, SCHLIX_DB_USERNAME, SCHLIX_DB_PASSWORD);
        //@ini_set('session.use_only_cookies', '1');
        //global $SystemDB;
        // Register this object as the session handler
        /*    session_set_save_handler(
          array( &$this, "open" ),
          array( &$this, "close" ),
          array( &$this, "read" ),
          array( &$this, "write"),
          array( &$this, "destroy"),
          array( &$this, "gc" )
          ); */
        $save_handler_result = session_set_save_handler(
            array($this, "open"), array($this, "close"), array($this, "read"), array($this, "write"), array($this, "destroy"), array($this, "gc")
        );

        if (!$save_handler_result)
            die('Failure when calling session_set_save_handler() ');
        // the following prevents unexpected effects when using objects as save handlers
        register_shutdown_function('session_write_close');
    }

    /**
     * Open the session
     * @return bool
     */
    public function open($save_path, $session_name) {
        // $limit = time() - (3600 * 24);
        //  $this->db->query("DELETE FROM {$this->session_table} WHERE timestamp < {$limit}");
        return true;
    }

    /**
     * Close the session
     * @return bool
     */
    public function close() {
        return true;
    }

    /**
     * Read the session
     * @param int session id
     * @return string string of the sessoin
     */
    public function read($id) {

        $sanitized_id = $this->db->sanitizeString($id);
        $sql = "SELECT data FROM {$this->session_table} WHERE id =  {$sanitized_id}";
        $result = $this->db->getQueryResultSingleRow($sql);

        return $result ? $result['data'] : ''; // PHP 7.1.x fix - OCt 2, 2017
    }

    /**
     * Get a list of unique IP addresses within x days
     * @param int $days
     * @return type
     */
    public function getUniqueIPAddressWithinDay($days) {
        $days = (int) $days;
        $sql = "SELECT COUNT(DISTINCT(`ip_address`)) AS session_count FROM `gk_session_handler` WHERE DATEDIFF(NOW(),`timestamp`) <= {$days}";
        $result = $this->db->getQueryResultSingleRow($sql);
        return $result['session_count'];
    }

    /**
     * Return the total number of active sessions within x seconds
     * @param int $seconds
     * @return int
     */
    public function getCountWithinSeconds($seconds) {
        $seconds = (int) $seconds;
        $sql = "SELECT COUNT(*) as session_count FROM `gk_session_handler` WHERE TIMESTAMPDIFF(SECOND, `timestamp`,NOW()) < {$seconds}";
        $result = $this->db->getQueryResultSingleRow($sql);
        return $result['session_count'];
    }

    /**
     * Write the session
     * @param int session id
     * @param string data of the session
     * @return bool
     */
    public function write($id, $data) {
        global $SystemLog;

        if ($id != '' /* && $data != '' */ && $this->db != NULL) {
            $sanitized_id = "'" . $this->db->real_escape_string($id) . "'";
            $sanitized_data = "'" . $this->db->real_escape_string($data) . "'";
            $ip_addr = "'" . $this->db->real_escape_string(get_user_real_ip_address()) . "'";
            $sanitized_user_agent =  "'" . $this->db->real_escape_string(str_limit($_SERVER['HTTP_USER_AGENT'], 255)) . "'";
            $sql = "REPLACE INTO {$this->session_table} (`id`,`data`,`ip_address`,`user_agent`) VALUES({$sanitized_id}, {$sanitized_data}, {$ip_addr}, {$sanitized_user_agent})";
            $this->db->query($sql);
            return true; // fix for PHP7 - Oct 5, 2016
        } else {
            debug_backtrace();
            $error_message = "Error when trying to write session";
            $SystemLog->record($error_message);
            die($error_message);
        }
    }

    /**
     * Destroy the session
     * @param int session id
     * @return bool
     */
    public function destroy($id) {
        $sanitized_id = $this->db->sanitizeString($id);
        $this->db->query("DELETE FROM {$this->session_table} WHERE `id` = {$sanitized_id}");
        setcookie(session_name(), "", time() - 3600);
        return true;
    }

    /**
     * Garbage Collector
     * @param int life time (sec.)
     */
    public function gc($maxlifetime) {
        $sql = "DELETE FROM {$this->session_table} WHERE timestamp < DATE_SUB(NOW(), INTERVAL " . $maxlifetime . " SECOND)";
        $this->db->query($sql);
    }

}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//
class cmsLogger {

    protected $table_name;

    public function __construct($table_name) {
        $this->table_name = $table_name;
    }

    //________________________________________________________________________//
    public function record($error, $module = 'system', $type = 'info') {
        global $SystemDB;

        // create new data - will be sanitized with InsertSQL
        $datavalues = [];
        $datavalues['ip_address'] = get_user_real_ip_address();
        $datavalues['module'] = $module;
        $datavalues['type'] = $type;
        $datavalues['referrer'] = $_SERVER['HTTP_REFERER'];
        $datavalues['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
        $datavalues['request_uri'] = $_SERVER['REQUEST_URI'];
        $datavalues['description'] = $error;
        $SystemDB->simpleInsertInto($this->table_name, $datavalues);
    }

    //________________________________________________________________________//
    public function error($error, $module = 'system') {

        $this->record($error, $module, 'error');
    }

    public function warn($error, $module = 'system') {

        $this->record($error, $module, 'warn');
    }

    public function info($error, $module = 'system') {

        $this->record($error, $module, 'info');
    }

    public function debug($error, $module = 'system') {

        $this->record($error, $module, 'debug');
    }

    //________________________________________________________________________//
    /**
     * Returns the textual data of log by month and year
     * @param int $month
     * @param int $year
     */
    private function saveLogDataByMonthAndYear($year, $month) {
        global $SystemDB;

        $month = (int) $month;
        $year = (int) $year;
        $sql = "SELECT * FROM {$this->table_name}  WHERE MONTH(`date_created`) = {$month} AND YEAR(`date_created`) = {$year} ORDER BY `date_created`";
        //$data = $SystemDB->getQueryResultArray($sql);
        $data = $SystemDB->query($sql);
        $result = '';

        $logfilename = SCHLIX_SITE_PATH . "/data/private/logs/log-{$year}-{$month}.txt";
        $file = @fopen($logfilename, 'w');
        if ($file) {
            while ($log = $SystemDB->getLastQueryResult()->fetch_assoc())
            {
                $logline = "#{$log['id']}|{$log['date_created']}|Module:{$log['module']}|Type:{$log['type']}|{$log['description']}|Info: {$log['ip_address']} - {$log['user_agent']} - {$log['referrer']} - {$log['request_uri']}\n";
                fwrite($file, $logline);
            }
            $SystemDB->getLastQueryResult()->close();
            $data = NULL;
            fclose($file);
        } else {
            $this->record("Unable to save {$logfilename}");
        }
    }

    //________________________________________________________________________//
    /**
     * Archive all the logs separated by month and year, e.g. 2010-10.txt, etc
     */
    public function Archive() {
        global $SystemDB;
        $sql = "SELECT DISTINCT YEAR(`date_created`) AS log_year, MONTH(`date_created`) AS log_month FROM {$this->table_name}  WHERE MONTH(`date_created`) <> MONTH(NOW());";
        $yearmonths = $SystemDB->getQueryResultArray($sql);
        foreach ($yearmonths as $yearmonth) {
            $log_year = $yearmonth['log_year'];
            $log_month = $yearmonth['log_month'];
            $this->saveLogDataByMonthAndYear($log_year, $log_month);
        }
        $sql = "DELETE FROM {$this->table_name}  WHERE MONTH(`date_created`) <> MONTH(NOW());;";
        $SystemDB->query($sql);
    }

    /**
     * Resolve hostnames from ip_address field
     * @global SCHLIX\cmsDatabase $SystemDB
     */
    public function updateLogHostnames() {
        global $SystemDB;
        $count = 0;

        $sql = "SELECT id, ip_address FROM {$this->table_name} WHERE host_name IS NULL";
        $unprocessed_logs = $SystemDB->getQueryResultArray($sql);
        if ($unprocessed_logs) {
            foreach ($unprocessed_logs as $log) {
                $log_id = $log['id'];
                $host_name = @gethostbyaddr($log['ip_address']);
                if ($host_name == $log_id || empty($host_name))
                    $host_name = '-';
                else
                    $count++;
                $data_update = array('host_name' => $host_name);
                $SystemDB->simpleUpdate($this->table_name, $data_update, 'id', $log_id);
                $data_update = null;
            }
            //if ($count > 0)
              //  $this->record("{$count} IP addresses resolved");
        }
    }

    //_______________________________________________________________________________________________________________//
    /**
     * CRON Scheduler Method - resolve hostnames in log
     * @global cmsLogger $SystemLog
     */
    public static function processRunResolveHostnames() {
        global $SystemLog;

        $SystemLog->updateLogHostnames();
    }

    //_______________________________________________________________________________________________________________//
    /**
     * CRON Scheduler Method
     * @global cmsLogger $SystemLog
     */
    public static function processRunCleanupLog() {
        global $SystemLog;

        $SystemLog->Archive();
    }

}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
/**
 * A helper class that represents an SQL table
 */
class cmsSQLTable {

    private $table_name;
    private $table_exists = false;
    private $columns = NULL;
    private $primary_key_names;
    private $enable_cache;
    private $db; // database - so users can reuse this class for another db

    //________________________________________________________________________//

    /**
     * Construct
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $table_name
     * @param \SCHLIX\cmsDatabase $from_database
     * @param bool $populate_field_names
     */
    public function __construct($table_name, cmsDatabase $from_database = null, $populate_field_names = true) {
        global $SystemDB;

        if (!empty($table_name)) {
            $this->table_name = $table_name;
            $this->db = ($from_database == null) ? $SystemDB : $from_database;
            $this->table_exists = $this->tableExists($this->table_name); // $this->tableExists();
        } else {
            ob_start();
            debug_print_backtrace();
            $debug_content = ob_get_contents();
            ob_end_clean();
            $debug_content = nl2br($debug_content);
            die("Empty Table Name<br />" . $debug_content);
        }
        /* if ($populate_field_names)
          return $this->tableExists(true); */
    }

    //________________________________________________________________________//
    public function __toString() {
        return $this->table_name;
    }

    //________________________________________________________________________//
    /**
     * Verify if this table exists
     * @param bool $from_cache
     * @return bool
     */
    public function tableExists() 
    {
        return \SCHLIX\cmsContextCache::get('db_table_exists',$this->table_name);
        //return $this->table_exists ? true : $this->db->tableExists($this->table_name);
    }

    public function q()
    {
        global $SystemDB;
        
        return $SystemDB->q()->from($this->table_name);
    }
    //________________________________________________________________________//
    /**
     * Get Table columns as array - this is the raw output from MySQL
     * @param bool $from_cache
     * @return array
     */
    public function getFields($from_cache = false) {
        if ($this->table_exists)
        {
            $fields = $this->db->getTableColumns($this->table_name);
            \SCHLIX\cmsContextCache::set('tablecolumns',$this->table_name, $fields);
            return $fields;
        } else return NULL;
        return ($this->table_exists) ? $this->db->getTableColumns($this->table_name, $from_cache) : NULL;
    }

    /**
     * Returns true if fieldname exists
     * @param string $fieldname
     * @return bool
     */
    public function fieldExists($fieldname)
    {
        $all_field_names = $this->getFieldNamesAsArray();
        return is_array($all_field_names) && in_array($fieldname, $all_field_names);
    }
    
    //________________________________________________________________________//
    /**
     * Returns this table's column name as array from cache
     * @return array
     */
    public function getFieldNamesAsArray() {

        global $__schlix_table_columns;
        if ($__schlix_table_columns[$this->table_name] != NULL)
        {
            return $__schlix_table_columns[$this->table_name];
        } else
        {
            $columns_array = $this->getFields();
            $column_names = [];
            if ($columns_array) {
                foreach ($columns_array as $column) {
                    $column_names[] = $column['Field'];
                }
                return $column_names;
            }
            return NULL;
        }
    }

    /**
     * Returns this table's column name as array - set to key 
     * @param bool $from_cache
     * @return array
     */
    public function getFieldNamesAsArrayKeys() {
        $columns_array = $this->getFieldNamesAsArray();
        return ($columns_array !== NULL) ? array_fill_keys($columns_array, NULL) : NULL;
    }

    //________________________________________________________________________//
    /**
     * Returns the list of field/column names
     * @return string
     */
    public function getFieldNamesAsString() {
        $column_names = $this->getFieldNamesAsArray();
        return implode(',', $column_names);
    }

    //________________________________________________________________________//
    /**
     * Returns the primary key in array form
     * @return array
     */
    public function getPrimaryKeysAsArray() {
        if ($this->table_exists) {
                $sql = "SHOW KEYS FROM {$this->table_name} WHERE key_name = 'PRIMARY'";
                return $this->db->getQueryResultArray($sql);
        } else
            return NULL;
    }

    //________________________________________________________________________//
    /**
     * Returns a comma separated field names
     * @param bool $from_cache
     * @return string
     */
    public function getPrimaryKeyFieldNames() {
        $result = $this->getPrimaryKeyFieldNamesAsArray();
        return (___c($result) > 0) ? implode(',', $result) : null;
    }

    //________________________________________________________________________//
    /**
     * Returns an array containing field names
     * @param bool $from_cache
     * @return array
     */
    public function getPrimaryKeyFieldNamesAsArray() {
        global $__schlix_table_pks;
        if ($__schlix_table_pks[$this->table_name] != NULL)
        {
            return $__schlix_table_pks[$this->table_name];
        } else
        {
            if ($this->table_exists)
            {
                $primary_keys_array = $this->getPrimaryKeysAsArray();
                if ($primary_keys_array)
                {
                    return array_column($primary_keys_array,'Column_name');
                }
            }
        }  
        return NULL;
    }

    //________________________________________________________________________//
    /**
     * Executes SELECT * FROM table_name WHERE ...
     *
     * @param string $fields
     * @param string $extra_criteria
     * @param string $sortby
     * @param string $sortdirection
     * @param int $start
     * @param int $end
     * @param bool $add_where_clause
     * @param bool $force_limit
     * @param bool $from_cache
     * @return array
     */
    public function selectAll($fields = '*', $extra_criteria = '', $sortby = '', $sortdirection = 'ASC', $start = 0, $end = 0, $add_where_clause = true, $force_limit = false, $from_cache = false) {
        $start = (int) $start;
        $end = (int) $end;

        $sql = $this->generateSQLSelectStatement($this->table_name, $fields, $extra_criteria, $sortby, $sortdirection, $start, $end, true, SCHLIX_SQL_ENFORCE_ROW_LIMIT);
        $search_result = $this->db->getQueryResultArray($sql, $from_cache);
        return $search_result;
    }

    //________________________________________________________________________//
    /**
     * Returns a single row from SELECT * FROM table_name WHERE ...
     *
     * @param string $fields
     * @param string $extra_criteria
     * @param string $sortby
     * @param string $sortdirection 
     * @param bool $add_where_clause
     * @param bool $force_limit
     * @param bool $from_cache
     * @return array
     */
    public function selectOne($fields = '*', $extra_criteria = '', $sortby = '', $sortdirection = 'ASC', $add_where_clause = true, $force_limit = false, $from_cache = false) {
        $result = $this->selectAll($fields, $extra_criteria, 0, 1, $sortby, $sortdirection, $add_where_clause, $force_limit, $from_cache);
        return (($result != NULL) && (___c($result) > 0)) ? $result[0] : NULL;
    }

    //________________________________________________________________________//
    /**
     * Given a comma separated fieldnames, return them escaped with backtick
     * @param string $fieldnames
     * @param bool $check_if_field_exists
     * @return string
     */
    public function escapeFieldNames($fieldnames, $check_if_field_exists = false) {
        if ($check_if_field_exists)
            $table_fields = $this->getFieldNamesAsArray();
        if (strpos($fieldnames, ',') !== FALSE) {
            $fieldnames_array = explode(',', $fieldnames);
            $fieldnames_result = [];
            $count = ___c($fieldnames_array);
            // reduce if in loop
            if ($check_if_field_exists) {
                for ($i = 0; $i < $count; $i++) {
                    $field_name = trim($fieldnames_array[$i]);
                    if (in_array($field_name, $table_fields))
                        $fieldnames_result[] = "`{$field_name}`";
                }
            } else {
                for ($i = 0; $i < $count; $i++) {
                    $field_name = trim($fieldnames_array[$i]);
                    $fieldnames_result[] = "`{$field_name}`";
                }
            }
            return implode(',', $fieldnames_result);
        } else {
            $fieldnames = trim($fieldnames);
            if ($check_if_field_exists)
                return (in_array($field_name, $fieldnames_result)) ? "`{$fieldnames}`" : EMPTY_STRING;
            else
                return "`{$fieldnames}`";
        }
    }

    //________________________________________________________________________//
    /**
     * An alias of SELECT COUNT(fieldnames) FROM (this->table) WHERE (criteria)
     * @param type $field_names
     * @param type $criteria
     * @param type $from_cache
     * @return int
     */
    public function count($field_names = '*', $criteria = '', $from_cache = false) {
        $criteria_txt = '';
        if (!empty($criteria))
            $criteria_txt = " WHERE {$criteria}";
        $field_names = $this->escapeFieldNames($field_names);
        $sql = "SELECT COUNT({$field_names}) as total_item_count FROM {$this->table_name} {$criteria_txt}";
        $result = $this->db->getQueryResultSingleRow($sql, $from_cache);
        return $result['total_item_count'];
    }

    //________________________________________________________________________//
    /**
     * Executes INSERT INTO table_name VALUES ( ... ) ON DUPLICATE KEY UPDATE ...
     * @param array $datavalues
     * @param string $insert_options
     * @param bool $update_if_duplicate
     * @param array $update_datavalues
     * @return query result
     */
    public function quickInsert(array $datavalues, $insert_options = EMPTY_STRING, $update_if_duplicate = false, array $update_datavalues = []) {
        $valid_options = array('', 'LOW_PRIORITY', 'HIGH_PRIORITY', 'DELAYED', 'IGNORE');

        $sql = "INSERT {$insert_options} INTO {$this->table_name} " . $this->convertDataValuesArrayIntoSQLForInsertOperation($datavalues);
        if ($update_if_duplicate && is_array($update_datavalues) && ___c($update_datavalues) > 0) {
            $sql.= " ON DUPLICATE KEY UPDATE " . $this->convertDataValuesArrayIntoSQLForUpdateOperation($update_datavalues);
        }
        return $this->db->query($sql);
    }

    //________________________________________________________________________//
    /**
     * Executes UPDATE table_name SET field=value WHERE ...
     * @param array $datavalues
     * @param string $unsanitized_where_condition
     * @return query result
     */
    public function quickUpdate(array $datavalues, $unsanitized_where_condition) {
        $sql = "UPDATE {$this->table_name} SET " . $this->convertDataValuesArrayIntoSQLForUpdateOperation($datavalues);
        if ($unsanitized_where_condition !== EMPTY_STRING) {
            $sql.=" WHERE " . $unsanitized_where_condition;
        }
        return $this->db->query($sql);
    }

    /**
     * Performs search and replace
     * @param string $field_name
     * @param string $search
     * @param string $replace
     * @param string $where_condition
     */
    public function searchAndReplace($field_name, $search, $replace, $where_condition = NULL)
    {
        if ($this->fieldExists($field_name))
        {
            $str_search = $this->sanitizeString($search);
            $str_replace = $this->sanitizeString($replace);
            $str_where = empty($where_condition) ? '' : " WHERE {$where_condition}";
            return $this->query("UPDATE `{$this->table_name}` SET `{$field_name}` = REPLACE(`{$field_name}`, '{$str_search}', '{$str_replace}') {$str_where}");
        } else return false;
    }
    
    //________________________________________________________________________//
    /**
     * Executes DELETE FROM table_name WHERE ....
     * @param string $unsanitized_where_condition
     * @return query result
     */
    public function quickDelete($unsanitized_where_condition) {
        $sql = "DELETE FROM {$this->table_name}";
        if ($unsanitized_where_condition !== EMPTY_STRING) {
            $sql.=" WHERE " . $unsanitized_where_condition;
        }
        return $this->db->query($sql);
    }

    //________________________________________________________________________//
    /**
     * Executes TRUNCATE TABLE table_name
     * @return array
     */
    public function truncate() {
        $sql = "TRUNCATE TABLE {$this->table_name}";
        return $this->db->query($sql);
    }

    //________________________________________________________________________//
    /**
     * Optimize MySQL table
     * @param type $unsanitized_options
     * @return array
     */
    public function optimize($unsanitized_options = EMPTY_STRING) {
        $sql = "OPTIMIZE {$unsanitized_options} TABLE {$this->table_name}";
        return $this->db->getQueryResultSingleRow($sql);
    }

    //________________________________________________________________________//
    /**
     * Executes ANALYZE TABLE table_name
     * @param type $unsanitized_options
     * @return array
     */
    public function analyze($unsanitized_options = EMPTY_STRING) {
        $sql = "ANALYZE {$unsanitized_options} TABLE {$this->table_name}";
        return $this->db->getQueryResultSingleRow($sql);
    }

    //________________________________________________________________________//
    /**
     * Returns the CREATE TABLE syntax of this table
     * @return string
     */
    public function showCreateTable() {
        $sql = "SHOW CREATE TABLE {$this->table_name}";
        $result = $this->db->getQueryResultSingleRow($sql);
        return is_array($result) ? $result['Create Table'] : NULL;
    }

    //________________________________________________________________________//
    /**
     * Given a key/value pair array, returns the SQL for INSERT statement
     * @param array $array
     * @param bool $dont_sanitize
     * @return string
     */
    public function convertDataValuesArrayIntoSQLForInsertOperation(array $array, $dont_sanitize = false) {

        foreach ($array as $key => $value) {
            $temp_keys[] = "`{$key}`";
            $temp_values[] = ($dont_sanitize ? $value : sanitize_string($value));
        }
        $str_keys = implode(', ', $temp_keys);
        $str_values = implode(', ', $temp_values);
        $string = "($str_keys) VALUES ($str_values)";
        return $string;
    }

    //________________________________________________________________________//
    /**
     * Given a key/value pair array, returns the SQL for UPDATE statement
     * @param array $array
     * @param bool $dont_sanitize
     * @return string
     */
    public function convertDataValuesArrayIntoSQLForUpdateOperation(array $array, $dont_sanitize = false) {
        $string = EMPTY_STRING;
        // remove primary key from update statement
        $primary_keys = $this->getPrimaryKeyFieldNamesAsArray();
        foreach ($primary_keys as $primary_key) {
            if (array_key_exists($primary_key, $array))
                unset($array[$primary_key]);
        }
        // merge
        foreach ($array as $key => $value) {
            $update_val = ($dont_sanitize ? $value : sanitize_string($value));
            $string.= "`{$key}`={$update_val}, ";
        }
        $string = substr($string, 0, strlen($string) - 2); // take out the last comma

        return $string;
    }

    //_______________________________________________________________________________________________________________//
    public function generateSQLSelectStatement($tablename, $fields = '*', $extra_criteria = '', $sortby = '', $sortdirection = 'ASC', $start = 0, $end = 0, $add_where_clause = true, $force_limit = false) {
        // 1) fields cannot be empty
        if (empty($fields))
            $fields = '*'; // double check

            
// 2) sanitize integer value
        $start = max(0, (int) $start);
        $end = (int) $end;
        if ($start >= 0 && $end > 0) {
            if ($start > $end)
                $start = $end = 0; // autocorrect, now guaranteed that $end >= $start
            $item_start = $start;
            $item_limit = $end - $start;
            if ($force_limit === true) {
                if (($item_limit === 0 && $item_start === 0) || ($item_limit > HARDCODE_MAX_ROWLIMIT))
                    $item_limit = HARDCODE_MAX_ROWLIMIT;
            }
            $statement_limit_criteria = " LIMIT {$item_start}, {$item_limit}";
        } else {
            $statement_limit_criteria = EMPTY_STRING;
        }
        // 3) Add where clause?
        $statement_criteria = ($add_where_clause && !empty($extra_criteria)) ? " WHERE {$extra_criteria} " : $extra_criteria;
        // 4) Sort By
        if (!empty($sortby) && !empty($sortdirection)) {
            //$sortby = quote_field_name_for_query($sortby); if there's more than one sort, this is invalid - dec 8, 2011
            $fixed_sort_by = $this->escapeFieldNames($sortby);
            $fixed_sort_direction = (strtoupper($sortdirection) == 'DESC') ? 'DESC' : 'ASC';
            $statement_sort_criteria = " ORDER BY {$fixed_sort_by} {$fixed_sort_direction}";
        } else {
            $statement_sort_criteria = EMPTY_STRING;
        }
        // DO NOT sanitize table field names by default, e.g. COUNT(*) as table_name can be correct
        // $statement_table_name = $this->escapeFieldNames($tablename);
        $sql = "SELECT {$fields} FROM {$tablename} {$statement_criteria} {$statement_sort_criteria} {$statement_limit_criteria}";
        return $sql;
    }

}

/**
 * cmsConfigRegistry - stores config in the database
 */
class cmsConfigRegistry {

    protected $table_name;

    /**
     * Please specify an existing table name where the config is stored
     * @param string $table_name
     */
    public function __construct($table_name) {

        $this->table_name = $table_name;
    }

    //________________________________________________________________________//
    /**
     * Check if a config key exists or not
     * @global SCHLIX\cmsDatabase $SystemDB
     * @param string $section
     * @param string $key
     * @param boolean $from_cache
     * @return boolean
     */
    public function exists($section, $key) {
        global $SystemDB;

        $sql = "SELECT * FROM {$this->table_name} WHERE section='{$section}' AND `key`='{$key}'";
        $results_array = $SystemDB->getQueryResultArray($sql);
        return (___c($results_array) > 0);
    }
    
    private function sanitizeResult($c)
    {
        $config_key = $c['key'];
        $config_prefix = '';
        $prefix_pos = strpos($config_key, '_');
        if ($prefix_pos !== FALSE) {
            $config_prefix = substr($config_key, 0, $prefix_pos);
        }
        switch ($config_prefix) {
            case 'array':
                $result = unserialize($c['value']);
                break;
            case 'float':
                $result = (float) $c['value'];
                break;
            case 'double':
                $result = (double) $c['value'];
                break;
            case 'int':
                $result = (int) $c['value'];
                break;
            case 'bool':
                if ($c['value'] === '' || $c['value'] === NULL)
                    $result = NULL;
                else
                {
                    $result = ((bool) $c['value']) ? 1: 0;
                }
                
                break;
            case 'str':
            case 'string':
            default:
                $result = strval($c['value']);
                break;
        }
        return $result;
    }
    
    /**
     * Clear context cache
     * @param string $section
     * @param string $key
     */
    public function clearCache($section, $key = '')
    {
        $cx_key = "_config_{$section}_{$key}";      
        cmsContextCache::purge("__config__", $cx_key);
    }
     //________________________________________________________________________//
    /**
     * Get configuration content by key
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $section
     * @param string $key
     * @param boolean $use_cached_result
     * @return boolean|int|double|float|string|array
     */
    public function get($section, $key = '', $use_cached_result = false) {
        global $SystemDB;

        $cx_key = "_config_{$section}_{$key}";
        $context_cache = cmsContextCache::get("__config__", $cx_key);
        if ($context_cache)
            return $context_cache;
        else
        {
            $sanitized_section = sanitize_string($section);        
            $sql = "SELECT * FROM {$this->table_name} WHERE section={$sanitized_section} ";
            if (!empty($key))
            {
                $sanitized_key = sanitize_string($key);
                $sql.= " AND `key`={$sanitized_key}";
            }
            $results_array = $use_cached_result ? $SystemDB->getCachedQueryResultArray($sql) : $SystemDB->getQueryResultArray($sql);
            $config = [];
            foreach ($results_array as $result) {
                $config_key = $result['key'];
                $config[$config_key] = $this->sanitizeResult($result);
            }
            if (!empty($key) && array_key_exists($key, $config))
            {
                cmsContextCache::set("__config__", $cx_key, $config[$key]);
                return $config[$key];
            }
            else {
                cmsContextCache::set("__config__", $cx_key, $config);
                return (empty($results_array)) ? null : $config;
            }
        }
    }

    /**
     * Delete all keys within this section
     * @param string $section
     */
    public function deleteSection($section)
    {
        global $SystemDB;
        
        $SystemDB->query("DELETE FROM {$this->table_name} WHERE `section` = :section", ['section'=> $section]);
    }
    //________________________________________________________________________//
    /**
     *  Returns multiple sections
     * @global \SCHLIX\SCHLIX\cmsDatabase $SystemDB
     * @param array $sections
     * @return array
     */
    public function getMultipleSections(array $sections) {
        global $SystemDB;

        if (!is_array($sections) || ___c($sections) == 0)
            return NULL;
        
        $sanitized_sections_array = [];
        foreach ($sections as $section)
            $sanitized_sections_array[] = sanitize_string($section);
        
        $sanitized_sections_string = implode(',', $sanitized_sections_array);
        $sql = "SELECT * FROM {$this->table_name} WHERE section IN ($sanitized_sections_string) ";
        $results_array = $SystemDB->getQueryResultArray($sql);
        
        $config = [];
        for ($i = 0, $count = ___c($results_array); $i < $count; $i++) {
            
            $section = $results_array[$i]['section'];
            $key = $results_array[$i]['key'];
            $config[$section][$key] =  $this->sanitizeResult($results_array[$i]);
        }
        return $config;

    }

    //________________________________________________________________________//
    /**
     * Prevents a key name collision in another section
     * @global \SCHLIX\SCHLIX\cmsDatabase $SystemDB
     * @param string $section
     * @param string $key
     * @param string $value
     * @return string
     */
    public function preventFaultyAliasName($section, $key, $value) {
        global $SystemDB;


        $id = (int) $id;
        $key = sanitize_string($key);
        $section = sanitize_string($section);
        $str_value = sanitize_string($value);
        $sql = "SELECT * FROM {$this->table_name} WHERE section <> {$section} AND `key`={$key} AND value = {$str_value}";
        $results = $SystemDB->getQueryResultArray($sql);
        foreach ($results as $result)
            $existing_items_with_the_same_name[] = $result['value'];
        if ($existing_items_with_the_same_name) {
            foreach ($existing_items_with_the_same_name as $name) {
                foreach ($results as $result)
                    $existing_items_with_the_same_name[] = $result['value'];
                $name = unserialize(stripslashes($value));
                $suggested = unserialize(stripslashes($value));
                $i = 1;
                while (in_array($suggested, $existing_items_with_the_same_name)) {
                    $suggested = $name . '-' . $i;
                    $i++;
                }
            }
            return convert_into_sef_friendly_title($suggested);
        } else {
            return $value;
        }
    }

    //________________________________________________________________________//
    /**
     * Set a config key. The key must have prefix 'array_','int_','string_',
     * 'str_','bool_','double_','float_'. Otherwise it won't be set
     * @global \SCHLIX\SCHLIX\cmsDatabase $SystemDB
     * @param string $section
     * @param string $key
     * @param string $value
     * @return boolean
     */
    public function set($section, $key, $value) {
        global $SystemDB;

        $datavalues = [];
        if (is_string($value))
            $value = trim($value);
        $original_value = $value; // before serialized

        if (get_magic_quotes_gpc())
            $value = addslashes($value);


        $config_prefix = '';
        $prefix_pos = strpos($key, '_');
        if ($prefix_pos !== FALSE) {
            $config_prefix = substr($key, 0, $prefix_pos);
        }
        switch ($config_prefix) {
            case 'array':
                if (!is_array($value))
                    $value = [];
                $value = serialize($value);
                break;
            case 'int':
                $value = (int) $value;
                break;
            case 'bool':
                $value = ((bool) $value) ? 1: 0;
                break;
            case 'str':
            case 'string':
                $value = strval($value);
                break;
            default:
                //$value = strval($value);
                // REJECT all without prefix
                return false;
                break;
        }

        if ($key === 'str_alias' || $key === 'str_app_description') {
            if (empty($original_value)) {
                // name it as the original app name or uppercase first letter if it's a description
                $value = ($key === 'str_alias') ? $section : ucwords($section);
            }
            if ($key === 'str_alias')
                $value = $this->preventFaultyAliasName($section, $key, $value);
        }
        if (!$this->exists($section, $key)) {

            // create new data
            $datavalues['section'] = $section;
            $datavalues['key'] = $key;
            $datavalues['value'] = $value;
            $SystemDB->simpleInsertInto($this->table_name, $datavalues);
            return true;
        } else {
            // update existing data
            $newValue = sanitize_string($value);
            $sql = "UPDATE {$this->table_name} SET `value` = {$newValue} WHERE `section`='{$section}' AND `key`='{$key}'";
            $SystemDB->query($sql);

            return true;
        }
    }

    /**
     * Parse attributes from a taag, e.g.
     * <schlix-config config-key="int_number" value="test" label="This Key" config-default-value="123" />
     * @param type $str_attribute
     * @return type
     */
    protected function parseAttributes($str_attribute) {

        // v2.1.0-2 $regex = "@(\S+)=(\"|'|)(.*)(\"|'|>)@isU";
        // v2.1.1-1 update regex
        $regex = "@(?:(\S+)\s*=\s*(\"[^\"]*\"|'[^']*'|[^\"'<>\s]+)\s+)+@isU";        
        preg_match_all($regex, $str_attribute, $matches);
        $match_count = ___c($matches[0]);
        $attrs = [];
        if ($match_count > 0) {
            for ($i = 0; $i < $match_count; $i++) {
                $quoted_value = $matches[2][$i];
                $unquoted_value = substr($quoted_value,1,strlen($quoted_value)-2);
                $key = $matches[1][$i];
                $value = $unquoted_value; // $matches[3][$i];
                $attrs[$key] = $value;
            }
        }
        $final_attrs = [];
        $label = '';
        $name = '';
        foreach ($attrs as $key => $value) {
            switch ($key) {
                case 'label':
                    $label = $value;
                    break;
                case 'config-default-value':
                    $default_value = $value;
                    break;
                case 'config-key':
                    $final_attrs['id'] = $value;
                    $name = $value;
                    break;
                case 'config-fonticon':
                    $fonticon = $value;
                    break;
                default:
                    $final_attrs[$key] = $value;
                    break;
            }
        }
        $result = array('label' => $label, 'attributes' => $final_attrs, 'name' => $name, 'default_value' => $default_value, 'fonticon' => $fonticon);
        
        return $result;
    }
    

    /**
     * Encodes an array and return it as a string
     * @param array $array
     * @return string
     */
    public static function encodeArray($array)
    {
        return base64_encode(json_encode($array));
    }
    
    /**
     * Decodes a base-64, JSON-encoded array
     * @param string $str
     * @return array
     */
    public static function decodeArray($str)
    {
        return json_decode(base64_decode($str), true);
    }

    protected function parseConfigTag($app_name, $tag_type, $attrs) {
        $tag_conf = $this->parseAttributes($attrs);
        
        if ($tag_type == 'app_alias')
            $value = $this->get($app_name, 'str_alias');
        else
        if ($tag_type != 'app_description')
            $value = $this->get($app_name, $tag_conf['name']);
        $default_value = $tag_conf['default_value'];

        $formgroup_css_class = 'form-group';
        $wrapper_warning = '';
        $original_value = $value;
        if (($value == NULL || $value == '') && !empty($default_value)) {            
            $value = $default_value;
           
            $formgroup_css_class.= ' has-warning';
            $wrapper_warning = ' '.\__HTML::I('',array('class' => 'fa fa-exclamation-triangle'));
            $this->has_warning = true;
        }
        
        $fg = \__HTML::DIV_start(array('class' => $formgroup_css_class));
        $fg_end = \__HTML::DIV_end();
        
        $needs_input_group = !empty($tag_conf['fonticon']);
        
        $ig = \__HTML::DIV_start(array('class' => 'input-group'));
        $ig_end = \__HTML::DIV_end();
        
        $iga = \__HTML::DIV_start(array('class' => 'input-group-addon'));
        $iga_end = \__HTML::DIV_end();
        
        
        switch ($tag_type) {
            case 'app_alias' :
                $value = $this->get($app_name, 'str_alias');

                $label = ___('Application Alias');
                $tag_conf['attributes']['id'] = 'str_alias';
                $tag_conf['attributes']['placeholder'] = $label;
                $tag_conf['attributes']['required'] = 'required';
                $tag_conf['attributes']['pattern'] = '^[a-zA-Z][a-zA-Z0-9-_\.]{1,50}$';
                if (empty($value))
                    $value = $default_value;
                
                $str_result =  $ig.$iga.\__HTML::I_start(['class'=>'fa fa-puzzle-piece']).\__HTML::I_end().$iga_end;
                $str_result.= \INPUT::TEXT('str_alias', $value, $tag_conf['attributes']);
                $str_result.= $ig_end;
                
                
                $str_result = $fg.\__Html::LABEL_FOR($label.$wrapper_warning, 'str_alias') . $str_result. $fg_end;
                break;
            case 'app_description':
                $value = $this->get($app_name, 'str_app_description');
                $label = ___('Application Description');
                $tag_conf['attributes']['id'] = 'str_app_description';
                $tag_conf['attributes']['placeholder'] = $label;
                //$tag_conf['attributes']['required'] = 'required';
                $tag_conf['attributes']['pattern'] = '.{1,50}';
                $str_result = $fg;
                if ($tag_conf['label'])
                    $str_result.= \__Html::LABEL_FOR($tag_conf['label'].$wrapper_warning, $tag_conf['name']);
                else
                    $str_result.= \__Html::LABEL_FOR($label.$wrapper_warning, 'str_app_description');
                $str_result.=  $ig.$iga.\__HTML::I_start(['class'=>'far fa-file-alt']).\__HTML::I_end().$iga_end;                
                $str_result.= \INPUT::TEXT('str_app_description', $value, $tag_conf['attributes']);
                $str_result.= $ig_end;                
                $str_result.= $fg_end;
                break;
            case 'meta_options': 
                $item_per_column = $tag_conf['attributes']['item-per-column'] ?  $tag_conf['attributes']['item-per-column'] : 1;
                $column_count =  $tag_conf['attributes']['column-count'] ? $tag_conf['attributes']['column-count'] : 3;
                
                $arr_opt = self::decodeArray($tag_conf['attributes']['config-options']);
                $arr_values =  is_string($value) ? '' : @serialize($value);
                
                $arr_default_options = self::decodeArray($default_value);
                $str_result.=  \INPUT::CMSObjectOptions($tag_conf['name'], $arr_opt, $arr_values, $arr_default_options, $item_per_column, $column_count);
                
                break;
            case 'integerbox':
                $str_result = $fg;
                if ($tag_conf['label'])
                    $str_result.= \__Html::LABEL_FOR($tag_conf['label'].$wrapper_warning, $tag_conf['name']);
                
                $tag_conf['attributes']['class'] = $tag_conf['attributes']['class'] . ' numericbox';
                $str_result.= \INPUT::NUMBER($tag_conf['name'], $value, $tag_conf['attributes']);
                $str_result.= $fg_end;
                break;
            case 'imageselector':
                $str_result = $fg;
                if ($tag_conf['label'])
                    $str_result.= \__Html::LABEL_FOR($tag_conf['label'].$wrapper_warning, $tag_conf['name']);
                $str_result.= \__HTML::DIV_start(['class' => 'input-group']);
                $tag_conf['attributes']['class'] = $tag_conf['attributes']['class'] . ' imagebox';
                $str_result.= \INPUT::TEXT($tag_conf['name'], $value, $tag_conf['attributes']);
                $str_result.= \__HTML::SPAN_start(['class'=>'input-group-addon']);
                $str_result.= \__HTML::A(___('Select an image'),'javascript:void(0)',
                    ['class' => 'schlix-media-select', 
                    'data-target'=> $tag_conf['name']]);
                $str_result.= \__HTML::SPAN_end();
                $str_result.= \__HTML::DIV_end();
                $str_result.= $fg_end;
                break;
                
            case 'colorpicker':
                $tag_conf['attributes']['class'] = $tag_conf['attributes']['class'] . ' short-text';
                $str_result = \INPUT::COLOR($tag_conf['name'], $value, $tag_conf['attributes']);

                if ($tag_conf['label'])
                    $str_result = \__Html::LABEL_FOR($tag_conf['label'], $tag_conf['name']) . $str_result;
                break;
            case 'fonticon':
                $str_result = $fg;
                if ($tag_conf['label'])
                    $str_result.= \__Html::LABEL_FOR($tag_conf['label'].$wrapper_warning, $tag_conf['name']);
                
                $tag_conf['attributes']['class'] = $tag_conf['attributes']['class'] . ' fonticon-button-selector';
                // data-iconset="fontawesome" data-icon
                $tag_conf['attributes']['data-icon'] = $value;
                $tag_conf['attributes']['data-iconset'] = 'fontawesome';
                $tag_conf['attributes']['role'] = 'iconpicker';
                //echo \INPUT::BUTTON($tag_conf['name'], $value, $tag_conf['attributes']);;die;
                $str_result.= \INPUT::BUTTON($tag_conf['name'], '', $tag_conf['attributes']);
                $str_result.= $fg_end;
                break;
            case 'hidden':
                $str_result.= \INPUT::HIDDEN($tag_conf['name'], $value, $tag_conf['attributes']);                
                break;
            case 'textbox':
                $str_result = $fg;
                if ($tag_conf['label'])
                    $str_result.= \__Html::LABEL_FOR($tag_conf['label'].$wrapper_warning, $tag_conf['name']);
                if ($needs_input_group)
                    $str_result.=  $ig.$iga.\__HTML::I_start(['class'=>$tag_conf['fonticon']]).\__HTML::I_end().$iga_end;
                $str_result.= \INPUT::TEXT($tag_conf['name'], $value, $tag_conf['attributes']);
                if ($needs_input_group)
                    $str_result.= $ig_end;                
                $str_result.= $fg_end;
                
                break;
            case 'textarea':
                $str_result = \INPUT::TEXTAREA($tag_conf['name'], "<![CDATA[\n" . $value . "]]>", $tag_conf['attributes']);
                if ($tag_conf['label'])
                    $str_result = \__Html::LABEL_FOR($tag_conf['label'], $tag_conf['name']) . $str_result;
                break;
            case 'checkbox':
                $real_warning = ($this->has_warning && $original_value===NULL);
                if ($original_value === NULL && $default_value)
                    $original_value = true; // what a mess
                $extra_warning = $real_warning && $value == false ? ' ('.___('unchecked by default').')' : '';
                $str_checkbox = \INPUT::HIDDEN($tag_conf['name'], '0') . \INPUT::CHECKBOX($tag_conf['name'], '1', $original_value, $tag_conf['attributes']);
                if ($tag_conf['label'])
                    $str_result = \__Html::LABEL($tag_conf['label'].$extra_warning, $str_checkbox, false);
                
                $checkbox_class = $real_warning ? 'checkbox '.$formgroup_css_class : 'checkbox';
                $str_result = \__Html::DIV_start(array('class' => $checkbox_class)) . $str_result . \__Html::DIV_end();
                break;
            default: return '';
        }

        return $str_result;
    }

    //_____________________________________//
    protected function getSpecificTag($tagname, $source) {
        $regex = '/<' . $tagname . '\\ (.*)>(.+?)<\/' . $tagname . '>/s';
        preg_match_all($regex, $source, $matches);
        return $matches;
    }

    private function getNewAttributes($node) {
        $final_attrs = [];
        $default_value = NULL;
        if ($node->hasAttributes()) {
            foreach ($node->attributes as $attr) {
                switch ($attr->nodeName) {
                    case 'label':
                        $label = $attr->nodeValue;
                        break;
                    case 'config-default-value':
                        $default_value = $attr->nodeValue;
                        break;

                    case 'config-key':
                        $final_attrs['id'] = $attr->nodeValue;
                        $name = $attr->nodeValue;
                        break;
                    default:
                        $final_attrs[$attr->nodeName] = $attr->nodeValue;
                        break;
                }
            }
        }
        
        $result = array('label' => $label, 'attributes' => $final_attrs, 'name' => $name);
        if ($default_value)
            $result['default_value'] = $default_value;
        return $result;
    }

    /**
     * @internal
     * @param string $input
     * @param array $attrs
     * @param type $node_val
     * @return \DomNode
     */
    private function createDOMElement($input, Array $attrs = NULL, $node_val = NULL) {

        $newnode = $this->config_document->createElement($input, $node_val);
        if ($attrs != null) {
            foreach ($attrs as $key => $val) {
                $newnode->setAttribute($key, $val);
            }
        }
        return $newnode;
    }

    private function generateRealHTMLElement($app_name, \DOMNode $old_dom_node) {
        global $CurrentUser;

        $new_node_options = $this->getNewAttributes($old_dom_node);
        $attr = $new_node_options['attributes'];
        $config_key = $new_node_options['name'];
        /* $newnode= $this->config_document->createElement('a');
          $newnode->nodeValue = 'a';
          $newnode->setAttribute('class', 'player');
          $newnode->setAttribute('href', 'http://www.yahoo.com');
          $newnode->setAttribute('style', "display: block"); */
        if ($config_key != 'app_alias' && $config_key != 'app_description')
            $config_value = $this->get($app_name, $config_key);
        switch ($old_dom_node->localName) {

            case 'dropdownlist':
                $parent = $this->createDOMElement('div', array(
                  'id' => 'select_' . $new_node_options['name'],
                  'class' => 'select'));

                $select_elem = $this->createDOMElement('select', array(
                  'id' => $new_node_options['name'],
                  'name' => $new_node_options['name'])
                );
                foreach ($attr as $key => $value) {
                    $select_elem->setAttribute($key, $value);
                }
                $multi_select = isset($attr['multiple']);
                if ($old_dom_node->hasChildNodes()) {
                    foreach ($old_dom_node->childNodes as $old_child_node) {
                        if ($old_child_node->nodeName === 'schlix-config:option') {
                            $newnode = $this->createDOMElement('option', array(
                              'value' => $old_child_node->getAttribute('value')), $old_child_node->nodeValue);

                            $is_selected = $multi_select ? (is_array($config_value) && in_array($old_child_node->getAttribute('value'), $config_value)) : ($config_value == $old_child_node->getAttribute('value'));
                            if ($is_selected) {
                                $newnode->setAttribute('selected', 'selected');
                            }


                            $select_elem->appendChild($newnode);
                        }
                    }
                    // Now the legend and fieldset

                    if ($new_node_options['label']) {
                        $newnode_label = $this->createDOMElement('label', array('for' => $new_node_options['name']), $new_node_options['label']);
                        $parent->appendChild($newnode_label);
                    }
                    $parent->appendChild($select_elem);
                    return $parent;
                }
                break;

            case 'radiogroup':
                $parent_class = 'radiogroup';
                $option_item_wrapper_class = 'radio';
                $multi_select = false;
                $option_tag = 'input';
                $option_tag_type = 'radio';
                $__not_done = true;
                $option_item = [];
                foreach ($old_dom_node->childNodes as $old_child_node) {
                    if ($old_child_node->nodeName === 'schlix-config:option')
                        $option_item[$old_child_node->getAttribute('value')] = $old_child_node->nodeValue;
                }
                break;
            case 'checkboxgroup':
                $parent_class = 'checkboxgroup';
                $option_item_wrapper_class = 'checkbox';
                $option_tag = 'input';
                $option_tag_type = 'checkbox';
                $multi_select = true;
                $__not_done = true;
                $option_item = [];
                foreach ($old_dom_node->childNodes as $old_child_node) {
                    if ($old_child_node->nodeName === 'schlix-config:option')
                        $option_item[$old_child_node->getAttribute('value')] = $old_child_node->nodeValue;
                }

                break;
            case 'permission-usergroups':
                $parent_class = 'checkboxgroup';
                $option_item_wrapper_class = 'checkbox';
                $option_tag = 'input';
                $option_tag_type = 'checkbox';
                $multi_select = true;
                $__not_done = true;
                $option_item = [];
                $user_groups = $CurrentUser->getGroupIDArrayForPermission();
                foreach ($user_groups as $user_group) {
                    $option_item[$user_group['value']] = $user_group['label'];
                }
                $__not_done = true;
                break;
        }
        if ($__not_done) { // for checkboxgroup & radiogroup
            ///// begin /////
            $parent = $this->createDOMElement('div', array(
              'id' => $new_node_options['name'],
              'class' => $parent_class));
            foreach ($attr as $key => $value) {
                $parent->setAttribute($key, $value);
            }
            if (___c($option_item) > 0) {
                $legend = $this->createDOMElement('legend', null, $new_node_options['label']);
                $fieldset = $this->createDOMElement('fieldset');
                $fieldset->appendChild($legend);                
                $multi_default_value = [];
                if ($multi_select && (array_key_exists('default_value', $new_node_options)))
                {
                    // for checkboxgroup default values
                    $trimmed_default_Value = trim($new_node_options['default_value']);
                    if ($trimmed_default_Value)
                        $trimmed_default_Value = preg_replace('#\s{1,}#', '', $trimmed_default_Value);
                    $multi_default_value = explode(',', trim($trimmed_default_Value));
                }
                foreach ($option_item as $option_item_value => $option_item_label) {
                    $checkbox_wrapper = $this->createDOMElement('div', array('class' => $option_item_wrapper_class));
                    $newnode_label = $this->createDOMElement('label');
                    $newnode_label_text = $this->createDOMElement('span', NULL, $option_item_label);
                    $item_opt_name = ($multi_select) ? $new_node_options['name'] . '[]' : $new_node_options['name'];
                    
                    $is_selected = $multi_select ? empty($config_value) ? (___c($multi_default_value) > 0 ? in_array($option_item_value, $multi_default_value) : false) : (is_array($config_value) && in_array($option_item_value, $config_value)) : 
                        (empty($config_value) ? (array_key_exists('default_value', $new_node_options) && $new_node_options['default_value'] == $option_item_value) : ($config_value == $option_item_value));
                    $new_node_attrs = ['type' => $option_tag_type,'name' => $item_opt_name, 'value' => $option_item_value];
                    if ($is_selected)
                        $new_node_attrs['checked'] = 'checked';
                    $newnode = $this->createDOMElement('input', $new_node_attrs);
                    
                    $newnode_label->appendChild($newnode);
                    $newnode_label->appendChild($newnode_label_text);
                    $checkbox_wrapper->appendChild($newnode_label);
                    $fieldset->appendChild($checkbox_wrapper);
                }
                // Now the legend and fieldset

                $parent->appendChild($fieldset);
                return $parent;
            }
        }
    }

    private function replaceWithNode($app_name, $node) {

        $dom_list = $this->config_document->getElementsByTagNameNS('schlix-config', $node); //all namespaces, all local names        

        $i = $dom_list->length - 1;
        while ($i > -1) {
            $element = $dom_list->item($i);
            $ignore = false;

            $new_node = $this->generateRealHTMLElement($app_name, $element);
            if ($new_node != null)
                $element->parentNode->replaceChild($new_node, $element);
            $i--;
        }
    }

    /**
     * Load config file for components
     * @param string $component_name
     * @param string $item
     * @param string $item_name
     * @param string $instance_name
     * @return string
     */
    public static function loadConfigFile($component_name, $item, $item_name, $instance_name)
    {
        // $item is used in the config file, do not delete from the parameter despite netbeans indicating that it's not being used
        $tbla = ['wysiwygeditors' =>'gk_wysiwyg_config', 'macros' => 'gk_macro_config', 'blocks' => 'gk_block_config', 'themes' => 'gk_theme_config'];
        $tbl = $tbla[$component_name];
        if (!$tbl)
            die (___('Error: could not determine the table name for this configuration set'));
        $item_name = convert_to_safe_filename($item_name);
        $filename = get_system_or_user_file("/{$component_name}/{$item_name}/config.template.php",false); 
        if (file_exists($filename)) {
            
            $admin_language = SCHLIX_DEFAULT_LANGUAGE;
            if ($admin_language != 'en_us')
            {
                $language_filename = get_system_or_user_file("/{$component_name}/{$item_name}/languages/{$admin_language}.lang.php", FALSE); 
                if (file_exists($language_filename))
                    include_once($language_filename);
            }
            
            $component_config = new \SCHLIX\cmsConfigRegistry($tbl);
            // ugh - backward compatibility. these variables will be required in the template file
            switch ($component_name)
            {
                case 'wysiwygeditors': $wysiwyg_config = $component_config; break;
                case 'blocks': $block_config = $component_config; break;
                case 'macros': $macro_config = $compnent_config; break;
                case 'themes': $theme_config = $compnent_config; break;
            }

            ob_start();
            // $item will be needed here
            include_once ($filename);
            $config_html = ob_get_contents();
            ob_end_clean();
            return $component_config->processConfigTags($instance_name, $config_html);
        } else
            return \__Html::P(sprintf(___('No configuration file can be found for %s'), ___h($item_name)));
    }
    /**
     * Process schlix-config tags
     * @param string $app_name
     * @param striing $html
     * @return string
     */
    public function processConfigTags($app_name, $html) {

        $element_tobe_replaced = array('radiogroup', 'checkboxgroup', 'permission-usergroups', 'dropdownlist' );

        $xml_header = "<?xml version='1.0' encoding='utf-8'?>";

        $utf8_header = '<meta http-equiv="content-type" content="text/html; charset=utf-8">';
        $xml_prefix = '<schlix-config xmlns:schlix-config="schlix-config">';
        $xml_suffix = '</schlix-config>';
        // Fix utf-8 issues? Nope. This was because SAFE_HTML should be outside the ___()
        // DONT USE $html = html_entity_decode($html, ENT_QUOTES, "utf-8");

        // 2.0.3 preg_match_all('#<schlix-config:((?:[a-z][a-z0-9_]*))\ (.*)\/>#iU', $html, $matches);
        // 2.0.4 preg_match_all('#<schlix-config:((?:[a-z][a-z0-9_]*))\ (.*)\/>#iU', $html, $matches);
        // fix for v2.0.6 - carriage return and tab in between elements        
        preg_match_all('#<schlix-config:((?:[a-z][a-z0-9_]*))\s(((?!</schlix-config:).)*)\/>#iUs', $html, $matches);        
        $match_count = ___c($matches[0]);
        if ($match_count > 0) {
            for ($i = 0; $i < $match_count; $i++) {
                $full_tag = $matches[0][$i];
                $tag_type = $matches[1][$i];
                $attrs = $matches[2][$i];
                $result = $this->parseConfigTag($app_name, $tag_type, $attrs);
                $html = str_replace($full_tag, $result, $html);
            }
        }
        // $html = utf8_encode(html_entity_decode($html));
        $config_doc = $xml_header . $xml_prefix . $html . $xml_suffix;
        //libxml_use_internal_errors(true);
        $opt = LIBXML_ERR_WARNING | LIBXML_PARSEHUGE; // set to 0 to see config parse errors
        $xml = new \SimpleXMLElement($config_doc, $opt, false, 'schlix-config'); // simplexml_load_string($config_doc);

        $this->config_document = new \DOMDocument();
        $str = $xml->asXML();
        $this->config_document->loadXML($str);

        if ($element_tobe_replaced)
            foreach ($element_tobe_replaced as $el)
                $this->replaceWithNode($app_name, $el);

        $final = $this->config_document->saveHTML();
        $final = str_replace('><', ">\n<", $final); //debug
        if ($this->has_warning)
        {
            $warning = '<div class="alert alert-danger"><i class="fa fa-exclamation-triangle"></i> '.___('Some values have not been set and may cause some errors. Please configure it and click Save when done').'</div>';
            $final = $warning.$final;
        }
        $final = str_replace($xml_prefix,'', $final);
        $final = str_replace($xml_suffix,'',$final);
        return $final;
        //return $html;
    }

    //________________________________________________________________________//
}

/**
 * Filter $_POST
 */
class cmsHttpInputFilter {

    /**
     * Return string value
     * @param array $_arr
     * @param type $varname
     * @param type $limit
     * @return string
     */
    public static function string($_arr, $varname, $trim = true, $limit = 0) {
        if (!array_key_exists($varname, $_arr)) return '';
        $s = $_arr[$varname];
        if ($trim)
            $s = trim($s);
        return str_limit($s, $limit);
    }

    /**
     * Return string value
     * @param array $_arr
     * @param type $varname
     * @param type $limit
     * @return string
     */
    public static function alphanumeric($_arr, $varname, $limit = 0) {
        $s = self::string($_arr, $varname, true, $limit);        
        return alpha_numeric_only($s,'');
    }    
    /**
     * Returns boolean value
     * @param array $_arr
     * @param type $varname
     * @return boolean
     */
    public static function bool($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return FALSE;
        $val = $_arr[$varname];
        $strval = strtolower(strval($val));
        if (strlen($strval > 1)) {
            if ($strval == 'true')
                return TRUE;
            else
                return FALSE;
        } else
            return (bool) $_arr[$varname];
    }

    /**
     * Return integer value
     * @param array $_arr
     * @param type $varname
     * @return int
     */
    public static function int($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return 0;
        return (int) $_arr[$varname];
    }

    /**
     * Returns unsigned int
     * @param array $_arr
     * @param type $varname
     * @return int
     */
    public static function uint($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return 0;
        $x = (int) $_arr[$varname];
        if ($x < 0)
            $x = 0;
        return $x;
    }

    /**
     * Returns float value
     * @param array $_arr
     * @param type $varname
     * @return float
     */
    public static function float($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return 0;
        return (float) $_arr[$varname];
    }

    /**
     * Returns double value
     * @param array $_arr
     * @param type $varname
     * @return double
     */
    public static function double($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return 0;
        return (double) $_arr[$varname];
    }

    /**
     * Returns a sanitized filename
     * @param array $_arr
     * @param type $varname
     * @return string
     */
    public static function filename_only($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return '';
        $invalid_chars = array(">", "<", "~", "../", '"', "'", "&", "/", "\\", "?", "#");
        return str_replace($invalid_chars, '_', trim($_arr[$varname]));
    }

    /**
     * Returns a sanitized full path
     * @param array $_arr
     * @param type $varname
     * @return string
     */
    public static function filename_fullpath($_arr, $varname) {
        if (!array_key_exists($varname, $_arr)) return '';
        $invalid_chars = array(">", "<", "~", "../", '"', "'", "&", "\\", "?", "#");

        $string = str_replace($invalid_chars, '_', trim($_arr[$varname]));
        $string = preg_replace('/[\x00-\x1F\x7F-\x9F]/u', '', $string);                
        return $string;
    }

    /**
     * returns a sanitized string with no tags
     * @param array $_arr
     * @param type $varname
     * @param type $limit
     * @return string
     */
    public static function string_notags($_arr, $varname, $limit = 0) {
        if (!array_key_exists($varname, $_arr)) return '';
        return str_limit(strip_tags($_arr[$varname]),$limit);
    }
    
    /**
     * returns a sanitized string with no tags and no quotes
     * @param array $_arr
     * @param type $varname
     * @param type $limit
     * @return string
     */
    public static function string_noquotes_notags($_arr, $varname, $limit = 0) {
        $value = preg_replace("/[\'\")(;|`,<>]/", "", trim($_arr[$varname])); //FIX - potential bug
        return str_limit($value, $limit);
    }
    
    /**
     * Returns array var
     * @param type $_arr
     * @param type $varname
     * @return type
     */
    public static function array_any($_arr, $varname)
    {
        return (is_array($_arr[$varname])) ?  $_arr[$varname] : NULL;
    }

    /**
     * Returns a sanitized array of integer
     * @param type $_arr
     * @param type $varname
     * @return boolean
     */
    public static function array_int($_arr, $varname)
    {
        $v_ar = $_arr[$varname];
        if(is_array($v_ar))
        {
            foreach ($v_ar as $key => $value)
            {
                $v_ar[$key] = (int) $value;
            }         
            return $v_ar;
        }
        return NULL;
    }    
}
                

/* 
 * Adapted from Doctrine DBAL QueryBuilder
 * licensed under the MIT license. For more information, see
 * <http://www.doctrine-project.org>.
 * 
 */
class cmsSQLQueryBuilder
{
    /*
     * The query types.
     */
    const SELECT = 0;
    const DELETE = 1;
    const UPDATE = 2;
    const INSERT = 3;

    /*
     * The builder states.
     */
    const STATE_DIRTY = 0;
    const STATE_CLEAN = 1;

    /**
     * The DBAL Connection.
     *
     * @var \SCHLIX\cmsDatabase
     */
    private $connection;

    /**
     * @var array The array of SQL parts collected.
     */
    private $sqlParts = array(
        'select'  => [],
        'from'    => [],
        'join'    => [],
        'set'     => [],
        'where'   => null,
        'groupBy' => [],
        'having'  => null,
        'orderBy' => [],
        'values'  => [],
    );

    /**
     * The complete SQL string for this query.
     *
     * @var string
     */
    private $sql;

    /**
     * The query parameters.
     *
     * @var array
     */
    private $params = [];

    /**
     * The parameter type map of this query.
     *
     * @var array
     */
    private $paramTypes = [];

    /**
     * The type of query this is. Can be select, update or delete.
     *
     * @var integer
     */
    private $type = self::SELECT;

    /**
     * The state of the query object. Can be dirty or clean.
     *
     * @var integer
     */
    private $state = self::STATE_CLEAN;

    /**
     * The index of the first result to retrieve.
     *
     * @var integer
     */
    private $firstResult = null;

    /**
     * The maximum number of results to retrieve.
     *
     * @var integer
     */
    private $maxResults = null;

    /**
     * The counter of bound parameters used with {@see bindValue).
     *
     * @var integer
     */
    private $boundCounter = 0;

    /**
     * Initializes a new <tt>QueryBuilder</tt>.
     *
     * @param \SCHLIX\cmsDatabase
     */
    public function __construct($connection = null)
    {
        global $SystemDB;
        
        if ($connection == NULL && $SystemDB instanceof \SCHLIX\cmsDatabase)
        {
            $this->connection = $SystemDB;
        } else $this->connection = $connection;
    } 

    /**
     * Gets the type of the currently built query.
     *
     * @return integer
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * Gets the associated DBAL Connection for this query builder.
     *
     * @return \SCHLIX\cmsDatabase
     */
    public function getConnection()
    {
        return $this->connection;
    }

    /**
     * Gets the state of this query builder instance.
     *
     * @return integer Either QueryBuilder::STATE_DIRTY or QueryBuilder::STATE_CLEAN.
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * Executes this query using the bound parameters and their types.
     *
     * @param array $data parameters in an associative array that needs to be bound
     * @return mixed
     */
    public function execute($data = NULL)
    {
        if ($this->type !== self::SELECT) {
            return $this->connection->query($this->getSQL(), $data);
        }
        return null;
    }
    /**
     * Executes this query using the bound parameters and their types.
     *
     * @param array $data parameters in an associative array that needs to be bound
     * @return mixed
     */
    public function getQueryResultArray($data = NULL)
    {
        if ($this->type === self::SELECT) {
            return $this->connection->getQueryResultArray($this->getSQL(), $data);
        }   
        return null;
    }
    
    /**
     * Returns the number of rows
     * @return int
     */
    public function getQueryResultCount($data = NULL)
    {
        if ($this->type === self::SELECT) {
            $this->select('COUNT(*) AS total_item_count');
            $result = $this->connection->getQueryResultSingleRow($this->getSQL(), $data);
            return $result['total_item_count'];
        }        
        return null;
    }

    /**
     * Executes this query using the bound parameters and their types.
     *
     * @param array $data parameters in an associative array that needs to be bound
     * @return mixed
     */
    public function getQueryResultSingleRow($data = NULL)
    {
        if ($this->type === self::SELECT) {
            return $this->connection->getQueryResultSingleRow($this->getSQL(), $data);
        } 
        return null;
        
    }

    /**
     * Gets the complete SQL string formed by the current specifications of this QueryBuilder.
     *
     * <code>
     *     $qb = $em->createQueryBuilder()
     *         ->select('u')
     *         ->from('User', 'u')
     *     echo $qb->getSQL(); // SELECT u FROM User u
     * </code>
     *
     * @return string The SQL query string.
     */
    public function getSQL()
    {
        if ($this->sql !== null && $this->state === self::STATE_CLEAN) {
            return $this->sql;
        }

        switch ($this->type) {
            case self::INSERT:
                $sql = 'INSERT INTO ' . $this->sqlParts['from']['table'] .
                ' (' . implode(', ', array_keys($this->sqlParts['values'])) . ')' .
                ' VALUES(' . implode(', ', $this->sqlParts['values']) . ')';
                if (array_key_exists('onDuplicateKeyUpdate', $this->sqlParts))
                {
                    $sql.= ' ON DUPLICATE KEY UPDATE '.implode(", ", $this->sqlParts['onDuplicateKeyUpdate']);
                }
                break;
            case self::DELETE:
                
                $table = $this->sqlParts['from']['table'] . ($this->sqlParts['from']['alias'] ? ' ' . $this->sqlParts['from']['alias'] : '');
                $sql = 'DELETE FROM ' . $table . ($this->sqlParts['where'] !== null ? ' WHERE ' . ((string) $this->sqlParts['where']) : '');

                break;

            case self::UPDATE:
                
                $table = $this->sqlParts['from']['table'] . ($this->sqlParts['from']['alias'] ? ' ' . $this->sqlParts['from']['alias'] : '');
                $sql = 'UPDATE ' . $table
                    . ' SET ' . implode(", ", $this->sqlParts['set'])
                    . ($this->sqlParts['where'] !== null ? ' WHERE ' . ((string) $this->sqlParts['where']) : '');


                
                break;

            case self::SELECT:
            default:
                $sql = 'SELECT ' . implode(', ', $this->sqlParts['select']);

                $sql .= ($this->sqlParts['from'] ? ' FROM ' . implode(', ', $this->getFromClauses()) : '')
                    . ($this->sqlParts['where'] !== null ? ' WHERE ' . ((string) $this->sqlParts['where']) : '')
                    . ($this->sqlParts['groupBy'] ? ' GROUP BY ' . implode(', ', $this->sqlParts['groupBy']) : '')
                    . ($this->sqlParts['having'] !== null ? ' HAVING ' . ((string) $this->sqlParts['having']) : '')
                    . ($this->sqlParts['orderBy'] ? ' ORDER BY ' . implode(', ', $this->sqlParts['orderBy']) : '')
                    . $this->getLimitOffset();
                
                if ($this->isLimitQuery()) {
                    return $this->connection->getDatabasePlatform()->modifyLimitQuery(
                        $sql,
                        $this->maxResults,
                        $this->firstResult
                    );
                }

                
                break;
        }

        $this->state = self::STATE_CLEAN;
        $this->sql = $sql;

        return $sql;
    }
    
    private function getLimitOffset()
    {        
        return  (array_key_exists('limitOffset',$this->sqlParts)) ? " LIMIT {$this->sqlParts['limitOffset']['offset']}, {$this->sqlParts['limitOffset']['limit']}" : '';        
    }
    /**
     * Set the start & end
     * @param int $start
     * @param int $end
     * @param bool $force_limit
     */
    public function startEnd($start, $end, $force_limit = true)
    {
        $start = max(0, (int) $start);
        $end = (int) $end;
        if ($start >= 0 && $end > 0) {
            if ($start > $end)
                $start = $end = 0; // autocorrect, now guaranteed that $end >= $start
            $item_start = $start;
            $item_limit = $end - $start;
            if ($force_limit === true) {
                if (($item_limit === 0 && $item_start === 0) || ($item_limit > HARDCODE_MAX_ROWLIMIT))
                    $item_limit = HARDCODE_MAX_ROWLIMIT;
            }
        } 
        $this->add('limitOffset',['limit' => $item_limit, 'offset' => $item_start], false);
        return $this;
    }
    
    /**
     * Set the limit & offset
     * @param int $limit
     * @param int $offset
     * @param bool $force_limit
     */
    public function limitOffset($limit, $offset = 0, $force_limit = true)
    {
        $limit = max (0, (int) $limit);
        $offset = max (0, (int) $offset);
        if ($force_limit === true && (($limit === 0 && $offset === 0) || ($limit > HARDCODE_MAX_ROWLIMIT))) 
            $limit = HARDCODE_MAX_ROWLIMIT;
        $this->add('limitOffset',['limit' => $limit, 'offset' => $offset], false);
        return $this;
    }

    /**
     * Sets the maximum number of results to retrieve (the "limit").
     *
     * @param integer $maxResults The maximum number of results to retrieve.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function setMaxResults($maxResults)
    {
        $this->state = self::STATE_DIRTY;
        $this->maxResults = $maxResults;

        return $this;
    }

    /**
     * Gets the maximum number of results the query object was set to retrieve (the "limit").
     * Returns NULL if {@link setMaxResults} was not applied to this query builder.
     *
     * @return integer The maximum number of results.
     */
    public function getMaxResults()
    {
        return $this->maxResults;
    } 
    /**
     * Either appends to or replaces a single, generic query part.
     *
     * The available parts are: 'select', 'from', 'set', 'where',
     * 'groupBy', 'having' and 'orderBy'.
     *
     * @param string  $sqlPartName
     * @param string  $sqlPart
     * @param boolean $append
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function add($sqlPartName, $sqlPart, $append = false)
    {
        $isArray = is_array($sqlPart);
        $isMultiple = is_array($this->sqlParts[$sqlPartName]);

        if ($isMultiple && !$isArray) {
            $sqlPart = array($sqlPart);
        }

        $this->state = self::STATE_DIRTY;

        if ($append) {
            if ($sqlPartName == "orderBy" || $sqlPartName == "groupBy" || $sqlPartName == "select" || $sqlPartName == "set") {
                foreach ($sqlPart as $part) {
                    $this->sqlParts[$sqlPartName][] = $part;
                }
            } elseif ($isArray && is_array($sqlPart[key($sqlPart)])) {
                $key = key($sqlPart);
                $this->sqlParts[$sqlPartName][$key][] = $sqlPart[$key];
            } elseif ($isMultiple) {
                $this->sqlParts[$sqlPartName][] = $sqlPart;
            } else {
                $this->sqlParts[$sqlPartName] = $sqlPart;
            }

            return $this;
        }

        $this->sqlParts[$sqlPartName] = $sqlPart;

        return $this;
    }

    /**
     * Specifies an item that is to be returned in the query result.
     * Replaces any previously specified selections, if any.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.id', 'p.id')
     *         ->from('users', 'u')
     *         ->leftJoin('u', 'phonenumbers', 'p', 'u.id = p.user_id');
     * </code>
     *
     * @param mixed $select The selection expressions.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function select($select = null)
    {
        $this->type = self::SELECT;
        if ($select)
        {
            $selects = is_array($select) ? $select : func_get_args();
            $this->add('select', $selects, false);
        }
        return $this;
    }


    /**
     * Turns the query being built into a bulk delete query that ranges over
     * a certain table.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->delete('users', 'u')
     *         ->where('u.id = :user_id');
     *         ->setParameter(':user_id', 1);
     * </code>
     *
     * @param string $delete The table whose rows are subject to the deletion.
     * @param string $alias  The table alias used in the constructed query.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function delete($delete = null, $alias = null)
    {
        
        $prev_type = $this->type;
        $this->type = self::DELETE;
        if ($delete)
        {
                $this->add('from', ['table' => $delete,'alias' => $alias]);

        } else
        {
            if ($prev_type === self::SELECT)
            {                
                if (is_array($this->sqlParts['from']))
                {                    
                    $this->add('from',['table' => $this->sqlParts['from'][0]['table'],'alias' =>  $this->sqlParts['from'][0]['alias']  ]);
                }
            } else throw new \Exception ('Table name must be specified for DELETE');
        }
        
        return $this;        
    }
    
    /**
     * Turns the query being built into a bulk update query that ranges over
     * a certain table
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->update('users', 'u')
     *         ->set('u.password', md5('password'))
     *         ->where('u.id = ?');
     * </code>
     *
     * @param string $update The table whose rows are subject to the update.
     * @param string $alias  The table alias used in the constructed query.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function update($update = null, $alias = null)
    {
        
        $prev_type = $this->type;
        $this->type = self::UPDATE;
        if ($update)
        {
                $this->add('from', ['table' => $update,'alias' => $alias]);

        } else
        {
            if ($prev_type === self::SELECT)
            {                
                if (is_array($this->sqlParts['from']))
                {                    
                    $this->add('from',['table' => $this->sqlParts['from'][0]['table'],'alias' =>  $this->sqlParts['from'][0]['alias']  ]);
                }
            } else throw new \Exception ('Table name must be specified for UPDATE');
        }
        
        return $this;        
    }
    /**
     * Escape array to prevent SQL injection
     * @param array $key_values
     * @return string
     */
    protected function escapeArrayForInsert(array $key_values)
    {
        $kv = [];
        foreach ($key_values as $key => $value)
           $kv[$this->field($key)] = $this->escapeValue ($value); 
        unset($key_values);
        return $kv;

    }
    /**
     * Escape array to prevent SQL injection
     * @param array $key_values
     * @return string
     */
    protected function escapeArrayForUpdate(array $key_values, $escape_value = true)
    {
        $kv = [];
        foreach ($key_values as $key => $value)
           $kv[] = $this->field($key).' = '.($escape_value ? $this->escapeValue ($value) : $value); 
        unset($key_values);
        return $kv;

    }
    /**
     * Escape string if it's avalue, don't escape it if it's a variable starting
     * with ':'.
     * @param string $s
     * @return string
     */
    protected function escapeValue($s)
    {
        return ($s[0] === ':') ? $s : "'" . $this->connection->escapeString($s) . "'";
    }
    
    /**
     * insertinto('table', ['id' => 'value'], true/false)
     * if automatic escape is specified, then it will automatically escape it
     * @param string $table
     * @param array $values
     * @param bool $automatic_escape
     * @return $this
     */
    public function insertInto($table = null)
    {
        $prev_type = $this->type;
        $this->type = self::INSERT;
        if ($table)
        {
            $this->add('from', ['table' => $table]);
        } else
        {
            if ($prev_type === self::SELECT)
            {                
                if (is_array($this->sqlParts['from']))
                {                    
                    $this->add('from',['table' => $this->sqlParts['from'][0]['table']]);
                }
            } else throw new \Exception ('Table name must be specified for INSERT');
        }
        return  $this;
    }
    /**
     * if automatic escape is specified, then it will automatically escape it
     * @param array $values
     * @param bool $automatic_escape
     * @return $this
     */    
    public function values(array $values, $automatic_escape = true)
    {
        if ($this->type == self::INSERT && is_array($values))
        {
            $this->add('values', $automatic_escape ? $this->escapeArrayForInsert($values) : $values);
        } else throw new \Exception('Invalid insert syntax');
        return  $this;        
    }
    
    public function onDuplicateKeyUpdate(array $values, $automatic_escape = true)
    {
        if ($this->type == self::INSERT)
        {
            $this->add('onDuplicateKeyUpdate', $this->escapeArrayForUpdate($values, $automatic_escape));
        } else throw new \Exception('Invalid insert syntax');
        return  $this;        
    }

    /**
     * Creates and adds a query root corresponding to the table identified by the
     * given alias, forming a cartesian product with any existing query roots.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.id')
     *         ->from('users', 'u')
     * </code>
     *
     * @param string      $from  The table.
     * @param string|null $alias The alias of the table.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function from($from, $alias = null)
    {
        
        return $this->add('from', ['table' => $from,'alias' =>  $alias], true);
    } 

    /**
     * Creates and adds a join to the query.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.name')
     *         ->from('users', 'u')
     *         ->innerJoin('u', 'phonenumbers', 'p', 'p.is_primary = 1');
     * </code>
     *
     * @param string $fromAlias The alias that points to a from clause.
     * @param string $join      The table name to join.
     * @param string $alias     The alias of the join table.
     * @param string $condition The condition for the join.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function innerJoin($fromAlias, $join, $alias, $condition = null)
    {
        return $this->add('join', array(
            $fromAlias => array(
                'joinType'      => 'inner',
                'joinTable'     => $join,
                'joinAlias'     => $alias,
                'joinCondition' => $this->composite($condition)
            )
        ), true);
    }

    /**
     * Creates and adds a left join to the query.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.name')
     *         ->from('users', 'u')
     *         ->leftJoin('u', 'phonenumbers', 'p', 'p.is_primary = 1');
     * </code>
     *
     * @param string $fromAlias The alias that points to a from clause.
     * @param string $join      The table name to join.
     * @param string $alias     The alias of the join table.
     * @param string $condition The condition for the join.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function leftJoin($fromAlias, $join, $alias, $condition = null)
    {
        return $this->add('join', array(
            $fromAlias => array(
                'joinType'      => 'left',
                'joinTable'     => $join,
                'joinAlias'     => $alias,
                'joinCondition' => $this->composite($condition)
            )
        ), true);
    }

    /**
     * Creates and adds a right join to the query.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.name')
     *         ->from('users', 'u')
     *         ->rightJoin('u', 'phonenumbers', 'p', 'p.is_primary = 1');
     * </code>
     *
     * @param string $fromAlias The alias that points to a from clause.
     * @param string $join      The table name to join.
     * @param string $alias     The alias of the join table.
     * @param string $condition The condition for the join.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function rightJoin($fromAlias, $join, $alias, $condition = null)
    {
        return $this->add('join', array(
            $fromAlias => array(
                'joinType'      => 'right',
                'joinTable'     => $join,
                'joinAlias'     => $alias,
                'joinCondition' => $this->composite($condition)
            )
        ), true);
    }

    /**
     * Sets a new value for a column in a bulk update query.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->update('users', 'u')
     *         ->set(['u.password', md5('password')])
     *         ->where('u.id = ?');
     * </code>
     *
     * @param string $key_values   The column to set.
     * @param bool $automatic_escape automatically escape (default = true).
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    function set(array $key_values, $automatic_escape = true)
    {
        if (is_array($key_values))
        {
            
            $kv = $this->escapeArrayForUpdate($key_values, $automatic_escape);
            return $this->add('set', implode(', ', $kv), true);
        } else throw  new Exception ('Invalid key/value array');
    }    


    /**
     * Evaluate where condition
     * @param string|array $stmt
     * @return string
     * @throws Exception
     */
    private function composite($stmt)
    {
        $cond = '';
        if ($stmt === null)
            return '';
        if (is_string($stmt))
            $cond = '('.$stmt.')';
        else if (is_array($stmt))
        {
            $ar_k = array_keys ($stmt);
            $count = ___c($ar_k);
            if ($count % 2 != 1)
            {
                print_r($stmt);
                throw new \Exception('Invalid composite SQL condition specified');
            }
            $all_where = [];
            for ($i =0; $i < $count; $i ++ )
            {
                $cur_stmt = $stmt[$i];
                // if it's a statement (must be even or 0)
                if (($i+1) % 2 == 1)
                {
                    if (is_string($cur_stmt))
                        $all_where[] = $cur_stmt;
                    elseif (is_array($cur_stmt))
                        $all_where[] = $this->composite($cur_stmt);
                    else
                        throw new \Exception ('Condition must be either array or string');
                } else // if it's OR/AND (must be odd)
                $all_where[] = $cur_stmt;

            }
            $cond = '('.implode(' ', $all_where).')';
        } else if (is_numeric ($stmt))
            return '('.$stmt.')';
        return $cond;
    }
    
    /**
     * Specifies one or more restrictions to the query result.
     * Replaces any previously specified restrictions, if any.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.name')
     *         ->from('users', 'u')
     *         ->where('u.id = ?');
      *
     *     $qb->update('users', 'u')
     *         ->set('u.password', md5('password'))
     *         ->where(['x = 1', 'AND', 
            ['id = :id','AND','x = 1', 'AND','x = z'],
            'OR', 'x = 1', 'AND',['x = 1','AND','z = 5']
        ]);
     * </code>
     *
     * @param mixed $predicates The restriction predicates.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function where($predicates)
    {
        $to_add = $this->composite($predicates);
        if ($this->sqlParts['where'])
            $to_add = "{$this->sqlParts['where']} AND {$to_add}";
        return $this->add('where', $to_add, true);
    }
            
    /**
     * Specifies a grouping over the results of the query.
     * Replaces any previously specified groupings, if any.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->select('u.name')
     *         ->from('users', 'u')
     *         ->groupBy('u.id');
     * </code>
     *
     * @param mixed $groupBy The grouping expression.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function groupBy($groupBy)
    {
        if ($groupBy)
        {
            $g = is_array($groupBy) ? $groupBy : func_get_args();
            $this->add('groupBy', $g, false);
        }
        return $this;
    } 
    /**
     * Sets a value for a column in an insert query.
     *
     * <code>
     *     $qb = $conn->createQueryBuilder()
     *         ->insert('users')
     *         ->values(
     *             array(
     *                 'name' => '?'
     *             )
     *         )
     *         ->setValue('password', '?');
     * </code>
     *
     * @param string $column The column into which the value should be inserted.
     * @param string $value  The value that should be inserted into the column.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function setValue($column, $value)
    {
        $this->sqlParts['values'][$column] = $value;

        return $this;
    }

    /**
     * Specifies a restriction over the groups of the query.
     * Replaces any previous having restrictions, if any.
     *
     * @param mixed $having The restriction over the groups.
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function having($having)
    {
        return $this->add('having', $this->composite($having));
    } 
    
    public function field($fieldname)
    {
        return "`{$fieldname}`";
    }

    /**
     * Specifies an ordering for the query results.
     * Parameter must be in the form of key => sort direction.
     * For example: orderBy('id' => 'DESC', 'name' => 'ASC')
     * 
     * @param array $opt
     * @return $this
     */
    public function orderBy(array $opt)
    {
        foreach ($opt as $key => $sort_dir)
        {
            $sort_dir_fixed = strtoupper($sort_dir) == 'DESC' ? 'DESC' : 'ASC';              
            $this->add('orderBy', $this->field($key).' '.$sort_dir_fixed, true);
        } 
        return $this;
        //return $this->add('orderBy', $sort . ' ' . (! $order ? 'ASC' : $order), false);
    } 
    
    /**
     * Gets a query part by its name.
     *
     * @param string $queryPartName
     *
     * @return mixed
     */
    public function getPart($queryPartName)
    {
        return $this->sqlParts[$queryPartName];
    }

    /**
     * Gets all query parts.
     *
     * @return array
     */
    public function getParts()
    {
        return $this->sqlParts;
    }

    /**
     * Resets SQL parts.
     *
     * @param array|null $queryPartNames
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function reset($queryPartNames = null)
    {
        if (is_null($queryPartNames))
            $queryPartNames = array_keys($this->sqlParts);
        foreach ($queryPartNames as $queryPartName)
            $this->resetQueryPart($queryPartName);
        return $this;
    }

    /**
     * Resets a single SQL part.
     *
     * @param string $queryPartName
     *
     * @return $this \SCHLIX\cmsSQLQueryBuilder instance.
     */
    public function resetQueryPart($queryPartName)
    {
        $this->sqlParts[$queryPartName] = is_array($this->sqlParts[$queryPartName])
            ? [] : null;

        $this->state = self::STATE_DIRTY;

        return $this;
    }
            

    /**
     * @return string[]
     */
    private function getFromClauses()
    {
        $fromClauses = array();
        $knownAliases = array();

        // Loop through all FROM clauses
        foreach ($this->sqlParts['from'] as $from) {
            if ($from['alias'] === null) {
                $tableSql = $from['table'];
                $tableReference = $from['table'];
            } else {
                $tableSql = $from['table'] . ' ' . $from['alias'];
                $tableReference = $from['alias'];
            }

            $knownAliases[$tableReference] = true;

            $fromClauses[$tableReference] = $tableSql . $this->getSQLForJoins($tableReference, $knownAliases);
        }

        $this->verifyAllAliasesAreKnown($knownAliases);

        return $fromClauses;
    }

    /**
     * @param array $knownAliases
     *
     * @throws QueryException
     */
    private function verifyAllAliasesAreKnown(array $knownAliases)
    {
        foreach ($this->sqlParts['join'] as $fromAlias => $joins) {
            if ( ! isset($knownAliases[$fromAlias])) {
                throw new \Exception('Unknown alias '.$fromAlias); //QueryException::unknownAlias($fromAlias, array_keys($knownAliases));
            }
        }
    }


    /**
     * Returns true if array is associative, false if it's sequential
     * @param array $arr
     * @return boolean
     */
    private function isAssociativeArray(array $arr)
    {
        if ([] === $arr) return false;
        return array_keys($arr) !== range(0, ___c($arr) - 1);
    }
    /**
     * @return bool
     */
    private function isLimitQuery()
    {
        return $this->maxResults !== null || $this->firstResult !== null;
    }
     /**
     * Gets a string representation of this QueryBuilder which corresponds to
     * the final SQL query being constructed.
     *
     * @return string The string representation of this QueryBuilder.
     */
    public function __toString()
    {
        return $this->getSQL();
    }
            

    /**
     * @param string $fromAlias
     * @param array  $knownAliases
     *
     * @return string
     */
    private function getSQLForJoins($fromAlias, array &$knownAliases)
    {
        $sql = '';

        if (isset($this->sqlParts['join'][$fromAlias])) {
            foreach ($this->sqlParts['join'][$fromAlias] as $join) {
                if (array_key_exists($join['joinAlias'], $knownAliases)) {
                    throw QueryException::nonUniqueAlias($join['joinAlias'], array_keys($knownAliases));
                }
                $sql .= ' ' . strtoupper($join['joinType'])
                    . ' JOIN ' . $join['joinTable'] . ' ' . $join['joinAlias']
                    . ' ON ' . ((string) $join['joinCondition']);
                $knownAliases[$join['joinAlias']] = true;
            }

            foreach ($this->sqlParts['join'][$fromAlias] as $join) {
                $sql .= $this->getSQLForJoins($join['joinAlias'], $knownAliases);
            }
        }

        return $sql;
    }

}
