<?php
namespace SCHLIX;

/**
 * Animated GIF Resizer utility.
 */
class AnimatedGIFResizer {

    private $gif_buffer;
    private $pointer = 0;
    private $gif_header;
    private $global_rgb;
    private $gl_mod;
    private $gl_mode;
    private $width;
    private $height;
    private $au = 0;
    private $nt = 0;
    private $correct_gif = false;
    private $frame_loop = 0;
    private $array_frames = [];
    private $gn_fld = [];
    private $dl_frmf = [];
    private $dl_frms = [];

    public function __construct($src, $optimize = FALSE) {
        
        if (!is_file($src) && !is_readable($src))
            return FALSE;
        $this->gif_buffer = file_get_contents($src);
        $this->gif_header = $this->getByte(13);
        if (substr($this->gif_header, 0, 6) != 'GIF87a' && substr($this->gif_header, 0, 6) != 'GIF89a') {
            return false;
        }        
        $this->correct_gif = true;
        
        $this->width = $this->rlInt($this->gif_header[6] . $this->gif_header[7]);
        $this->height = $this->rlInt($this->gif_header[8] . $this->gif_header[9]);
        
        if (($vt = ord($this->gif_header[10])) & 128 ? 1 : 0) {
            $this->global_rgb = $this->getByte(pow(2, ($vt & 7) + 1) * 3);
        }
        $buffer_add = '';
        if ($this->gif_buffer[$this->pointer] == "\x21") {
            while ($this->gif_buffer[$this->pointer + 1] != "\xF9" && $this->gif_buffer[$this->pointer] != "\x2C") 
            {
                switch ($this->gif_buffer[$this->pointer + 1]) {
                    case "\xFE":
                        $sum = 2;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
                        break;
                    case "\xFF":
                        $sum = 14;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        $buffer_add .= $this->getByte($sum + 1);
                        break;
                    case "\x01":
                        $sum = 15;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
                }
            }
            $this->gl_mod = $buffer_add;
        }
        while ($this->gif_buffer[$this->pointer] != "\x3B" && $this->gif_buffer[$this->pointer + 1] != "\xFE" && $this->gif_buffer[$this->pointer + 1] != "\xFF" && $this->gif_buffer[$this->pointer + 1] != "\x01") {
            $lc_mod = $lc_palet = $header = $gr_mod = null;
            $pnts = [];
            $this->frame_loop++;
            while ($this->gif_buffer[$this->pointer] != "\x2C") {
                switch ($this->gif_buffer[$this->pointer + 1]) {
                    case "\xF9":
                        $this->gn_fld[] = $this->gif_buffer[$this->pointer + 3];
                        $this->dl_frmf[] = $this->gif_buffer[$this->pointer + 4];
                        $this->dl_frms[] = $this->gif_buffer[$this->pointer + 5];
                        $gr_mod = $buffer_add = $this->getByte(8);
                        break;
                    case "\xFE":
                        $sum = 2;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
                        break;
                    case "\xFF":
                        $sum = 14;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        if (substr($tmp_buffer = $this->getByte($sum + 1), 3, 8) == "NETSCAPE") 
                        {
                            if (!$this->nt) {
                                $this->nt = 1;
                                $this->gl_mod .= $tmp_buffer;
                            }
                        } else 
                        $buffer_add .= $tmp_buffer;

                        break;
                    case "\x01":
                        $sum = 15;
                        while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                            $sum += $lc_i + 1;
                        }
                        $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
                }
            }
            $lc_mod = $buffer_add;
            $pnts[] = $this->msInt(1, 2);
            $pnts[] = $this->msInt(3, 2);
            $pnts[] = $this->msInt(5, 2);
            $pnts[] = $this->msInt(7, 2);
            $header = $this->getByte(10);
            if ((($pnts[0] + $pnts[2]) - $this->width) > 0) {
                $header[1] = $header[2] = $header[6] = "\x00";
                $header[5] = $this->asciiToChar($this->width);

                $pnts[0] = 0;
                $pnts[2] = $this->width;
            }
            if ((($pnts[1] + $pnts[3]) - $this->height) > 0) {
                $header[3] = $header[4] = $header[8] = "\x00";
                $header[7] = $this->asciiToChar($this->height);
                $pnts[1] = 0;
                $pnts[3] = $this->height;
            }
            if ((ord($this->gif_buffer[$this->pointer - 1]) & 128 ? 1 : 0)) {
                $lc_i = pow(2, (ord($this->gif_buffer[$this->pointer - 1]) & 7) + 1) * 3;
                $lc_palet = $this->getByte($lc_i);
            }
            $sum = 0;
            $this->pointer++;
            while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                $sum += $lc_i + 1;
            }
            $this->pointer--;
            $the_frame = [];
            $the_frame['lc_mod'] = $lc_mod;
            $the_frame['lc_palet'] = $lc_palet;
            $the_frame['lmage'] = $this->getByte($sum + 2);
            $the_frame['header'] = $header;
            $the_frame['x'] = $pnts[0];
            $the_frame['y'] = $pnts[1];
            $the_frame['frame_width'] = $pnts[2];
            $the_frame['frame_height'] = $pnts[3];
            $the_frame['gr_mod'] = $gr_mod;
            $the_frame['tr_frame'] = ord($gr_mod[3]) & 1 ? 1 : 0;
            $this->array_frames[] = $the_frame;
        }
        
        $buffer_add = "";
        while ($this->gif_buffer[$this->pointer] != "\x3B") {
            switch ($this->gif_buffer[$this->pointer + 1]) {
                case "\xFE":
                    $sum = 2;
                    while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                        $sum += $lc_i + 1;
                    }
                    $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
                    $this->au = ($sum == 17);
                    break;
                case "\xFF":
                    $sum = 14;
                    while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                        $sum += $lc_i + 1;
                    }$buffer_add .= $this->getByte($sum + 1);
                    break;
                case "\x01":
                    $sum = 15;
                    while (($lc_i = ord($this->gif_buffer[$this->pointer + $sum])) != 0x00) {
                        $sum += $lc_i + 1;
                    }
                    $optimize ? $this->getByte($sum + 1) : $buffer_add .= $this->getByte($sum + 1);
            }
        }
        $this->gl_mode = $buffer_add;
        $this->gif_buffer = "";
    }

    private function getByte($n) {
        $b = substr($this->gif_buffer, $this->pointer, $n);
        $this->pointer += $n;
        return $b;
    }

    private function rlInt($hw) {
        $z = ord($hw[1]) << 8;
        $c = ord($hw[0]);
        $x = $z | $c;
        return $x;
    }

    private function msInt($g_f, $g_s) {
        return $this->rlInt(substr($this->gif_buffer, $this->pointer + $g_f, $g_s));
    }

    private function asciiToChar($t) {
        return chr($t & 0xFF) . chr(($t & 0xFF00) >> 8);
    }

    private function createImage($i) {
        return $this->gif_header . $this->global_rgb . $this->gl_mod . $this->array_frames[$i]['lc_mod'] . $this->array_frames[$i]['header'] . $this->array_frames[$i]['lc_palet'] . $this->array_frames[$i]['lmage'] . "\x3B";
    }

    /**
     * Resize frame, returns it as a string
     * @param string $buffer
     * @param int $frame_num
     * @param array $des
     * @return string
     */
    private function resizeFrame($buffer, $frame_num, $des) {
        $buf_n = round($this->array_frames[$frame_num]['frame_width'] * $des[0]);
        $n_width = $buf_n ? $buf_n : 1;
        $buf_n = round($this->array_frames[$frame_num]['frame_height'] * $des[1]);
        $n_height = $buf_n ? $buf_n : 1;
        $n_pos_x = round($this->array_frames[$frame_num]['x'] * $des[0]);
        $n_pos_y = round($this->array_frames[$frame_num]['y'] * $des[1]);
        $this->array_frames[$frame_num]['off_xy'] = $this->asciiToChar($n_pos_x) . $this->asciiToChar($n_pos_y);
        $str_img = @imagecreatefromstring($buffer);
        if ($this->frame_loop == 1 || $des[3]) {
            $img_s = @imagecreatetruecolor($n_width, $n_height);
        } else {
            $img_s = @imagecreate($n_width, $n_height);
        }if ($this->array_frames[$frame_num]['tr_frame']) {
            $in_trans = @imagecolortransparent($str_img);
            if ($in_trans >= 0 && $in_trans < @imagecolorstotal($img_s)) {
                $tr_clr = @imagecolorsforindex($str_img, $in_trans);
            }if ($this->frame_loop == 1 || $des[3]) {
                $n_trans = @imagecolorallocatealpha($img_s, 255, 255, 255, 127);
            } else {
                $n_trans = @imagecolorallocate($img_s, $tr_clr['red'], $tr_clr['green'], $tr_clr['blue']);
            }
            @imagecolortransparent($img_s, $n_trans);
            @imagefill($img_s, 0, 0, $n_trans);
        }
        @imagecopyresampled($img_s, $str_img, 0, 0, 0, 0, $n_width, $n_height, $this->array_frames[$frame_num]['frame_width'], $this->array_frames[$frame_num]['frame_height']);
        @ob_start();
        @imagegif($img_s);
        $t_img = ob_get_clean();
        @ob_end_clean();
        @imagedestroy($str_img);
        @imagedestroy($img_s);

        return $t_img;
    }

    private function encodeFrame($str_img, $gr_i) {
        $hd = $offset = 13 + pow(2, (ord($str_img[10]) & 7) + 1) * 3;
        $palet = "";
        $i_hd = 0;
        $m_off = 0;
        for ($i = 13; $i < $offset; $i++) {
            $palet .= $str_img[$i];
        }
        if ($this->array_frames[$gr_i]['tr_frame']) 
        {
            while ($str_img[$offset + $m_off] != "\xF9") $m_off++;
            $str_img[$offset + $m_off + 2] = $this->gn_fld[$gr_i];
            $str_img[$offset + $m_off + 3] = $this->dl_frmf[$gr_i];
            $str_img[$offset + $m_off + 4] = $this->dl_frms[$gr_i];
        }
        while ($str_img[$offset] != "\x2C") {
            $offset = $offset + $this->rlInt($str_img[$offset + 2]) + 4;
            $i_hd = $i_hd + $this->rlInt($str_img[$offset + 2]) + 8;
        }
        $str_img[$offset + 1] = $this->array_frames[$gr_i]['off_xy'][0];
        $str_img[$offset + 2] = $this->array_frames[$gr_i]['off_xy'][1];
        $str_img[$offset + 3] = $this->array_frames[$gr_i]['off_xy'][2];
        $str_img[$offset + 4] = $this->array_frames[$gr_i]['off_xy'][3];
        $str_img[$offset + 9] = chr($str_img[$offset + 9] | 0x80 | (ord($str_img[10]) & 0x7));
        $ms1 = substr($str_img, $hd, $i_hd + 10);
        if (!$this->array_frames[$gr_i]['tr_frame']) {
            $ms1 = $this->array_frames[$gr_i]['gr_mod'] . $ms1;
        }
        return $ms1 . $palet . substr(substr($str_img, $offset + 10), 0, -1);
    }

    /**
     * Resize GIF
     * @param string $destination
     * @param int $thumb_width
     * @param int $thumb_height
     * @param bool $keep_aspect_ratio
     * @param bool $resampled
     * @return boolean
     */
    public function resize($destination, $thumb_width, $thumb_height, $keep_aspect_ratio = TRUE, $resampled = FALSE) {
        if (!$this->correct_gif)
            return false;
        if ($thumb_width == 0 || $thumb_height == 0) {
            $thumb_width = 10;
            $thumb_height = 10;
        }
        
        $xcr = array(0, 0, 0);
        $f_buf = '';
        
        $xcr[3] = $resampled;
        $xcr[0] = $thumb_width / $this->width;
        $xcr[1] = $thumb_height / $this->height;
        if ($keep_aspect_ratio) {
            $rt = min($xcr[0], $xcr[1]);
            $xcr[0] == $rt ? $xcr[1] = $rt : $xcr[0] = $rt;
        }
        
        for ($i = 0; $i < $this->frame_loop; $i++) {
            $f_buf .= $this->encodeFrame($this->resizeFrame($this->createImage($i), $i, $xcr), $i);
        }
        $gm = $this->gif_header;
        $gm[10] = $gm[10] & 0x7F;
        $bf_t = round($this->width * $xcr[0]);
        $t = $this->asciiToChar($bf_t ? $bf_t : 1);
        $gm[6] = $t[0];
        $gm[7] = $t[1];
        $bf_t = round($this->height * $xcr[1]);
        $t = $this->asciiToChar($bf_t ? $bf_t : 1);
        $gm[8] = $t[0];
        $gm[9] = $t[1];
        if (strlen($this->gl_mode)) {
            $comment_block = $this->gl_mode . "\x3B";
        } else {
            $comment_block = "\x3B";
        }
        if (!$this->au) {
            $comment_block = "\x21\xFE\x0EXILHCS\x00" . $comment_block;
        }
        return file_put_contents($destination, $gm . $this->gl_mod . $f_buf . (iconv_strlen($comment_block) >= 19 ? $comment_block : "\x21"));
    }

}
