<?php
namespace SCHLIX;



class cmsDBCache
{
    private static $cache_content = [];
    /**
     * Set DB cache
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $cache_key
     * @param mixed $content
     * @param int $hours
     * @param int $minutes
     * @param int $seconds
     */
    public static function set($cache_key, $content, $hours = 0, $minutes = 10, $seconds = 0)
    {
        global $SystemDB;
        
        $next_expiry_seconds = ($hours * 3600) + ($minutes * 60) + $seconds;
        $sanitized_expiry = sanitize_string(date("Y-m-d H:i:s", time() + $next_expiry_seconds));
        $sanitized_cache_key = sanitize_string($cache_key);
        if (is_array($content))
            $content = serialize($content);
        $sanitized_content = sanitize_string($content);
        $sql = "REPLACE INTO `gk_cache_items` (`cache_key`,`cache_content`,`date_expiry`) VALUES({$sanitized_cache_key}, {$sanitized_content}, {$sanitized_expiry})";
        $SystemDB->query($sql);

        self::$cache_content[$section][$cache_key] = $content;
    }
    
    /**
     * Get DB cache
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $cache_key
     * @return mixed
     */
    public static function get($cache_key)
    {
        global $SystemDB;
        
        $now = sanitize_string(get_current_datetime());
        $sanitized_cache_key = sanitize_string($cache_key);
        $sql = "SELECT `cache_content` FROM `gk_cache_items` WHERE `cache_key` = {$sanitized_cache_key} AND date_expiry > {$now}";
        $result = $SystemDB->getQueryResultSingleRow($sql);
        if ($result)
        {
            $cache_content = $result['cache_content'];
            return (is_serialized($cache_content)) ? unserialize($cache_content) : $cache_content;
        } else return NULL;
    }
    /**
     * CRON command - delete expired cache
     * @global SCHLIX\cmsLogger $SystemLog
     */
    public static function processRunPurgeExpiredCache() 
    {
        global $SystemDB;
        
        $now = sanitize_string(get_current_datetime());
        $sql = "DELETE FROM `gk_cache_items` WHERE `date_expiry` < {$now}";
        $SystemDB->query($sql);
    }    
}
/**
 * Just cache for current request
 */
class cmsContextCache
{
    private static $cache_content = [];
    
    public static function set($section, $cache_key, $content)
    {
        self::$cache_content[$section][$cache_key] = $content;
    }
    
    public static function purge($section, $cache_key)
    {
        if (self::$cache_content[$section][$cache_key])
        {
            unset(self::$cache_content[$section][$cache_key]);
        }        
    }
    public static function get($section, $cache_key)
    {
        if (self::$cache_content[$section][$cache_key])
        {
            return self::$cache_content[$section][$cache_key];
        }
        return NULL;
    }
    
    public static function dump()
    {
        return self::$cache_content;
    }
}
/**
 * CMS Filesystem Cache
 */
class cmsFSCache
{
    private $dir;
    private $cache_dir;
    
    public function __construct($dir) {
        $this->dir = $dir;
        $this->ext = 'cache';
        $this->cache_dir = SCHLIX_FS_CACHE_FOLDER.'/'.$dir;
        if (!is_dir($this->cache_dir  ))
        {
            if (!mkdir($this->cache_dir, 0755))
            {
                die('Cannot create '.$this->cache_dir);
            }
        }
    }
    
    private function getCacheExpiryFileName($cache_key)
    {
        $cache_key = strtolower(str_replace('../','', $cache_key));
        $filename = $this->cache_dir.'/'.$cache_key.'.expiry';
        return $filename;
    }
    
    private function getCacheContentFileName($cache_key)
    {
        $cache_key = strtolower(str_replace('../','', $cache_key));
        $filename = $this->cache_dir.'/'.$cache_key.'.'.$this->ext;
        return $filename;
    }

    /**
     * Set cache content
     * @param string $cache_key
     * @param mixed $content
     * @param int $hours
     * @param int $minutes
     * @param int $seconds
     * @return type
     */
    public function set($cache_key, $content, $hours = 1, $minutes = 0, $seconds = 0)
    {        
        $next_expiry_seconds = ($hours * 3600) + ($minutes * 60) + $seconds;
        $expiry_time = time() + $next_expiry_seconds;
        file_put_contents($this->getCacheContentFileName($cache_key), @serialize($content));
        return file_put_contents($this->getCacheExpiryFileName($cache_key), $expiry_time);
    }
    
    /**
     * Get cache content
     * @param type $cache_key
     * @return boolean
     */
    public function get($cache_key)
    {
        $file_content = $this->getCacheContentFileName($cache_key);
        $file_expiry = $this->getCacheExpiryFileName($cache_key);
        if (file_exists($file_content) & file_exists($file_expiry))
        {
            $expiry = file_get_contents($file_expiry);
            if ($expiry > time())
            {                
                return @unserialize(file_get_contents($file_content));
            } else
            {
                unlink($file_content);
                unlink($file_expiry);
                return false;
            }
        }
        return false;
    }
    
    /**
     * CRON command - delete expired cache
     * @global SCHLIX\cmsLogger $SystemLog
     */
    public static function processRunPurgeExpiredCache() 
    {
        global $SystemLog;
        
        $count_deleted = 0;        
        $files_cannot_be_deleted = [];
        $items = cmsDirectoryFilter::getRecursiveDirectoryIterator(SCHLIX_FS_CACHE_FOLDER, cmsDirectoryFilter::FILTER_FILE_ONLY, array('web.config','.htaccess','README.txt'));
        foreach ($items as $item) {
            if ($item->isFile()) {
                $ext = $item->getExtension();
                if ($ext == 'expiry')
                {
                    $path =  $item->getPath();
                    $expiry_time = (int) file_get_contents($path.'/'.$item->getFileName());
                    if (time() - $expiry_time > 0)
                    {
                        $cache_file = $path.'/'.$item->getBaseName(".{$ext}").'.array';
                        if (file_exists($cache_file))
                        {
                            if (!@unlink($cache_file))
                                $files_cannot_be_deleted[] = $cache_file;
                            else
                                $count_deleted++;
                        }
                    }
                }
            }
        }

        $str = "{$count_deleted} expired cache file(s) have been deleted.";
        if (___c($files_cannot_be_deleted) > 0)
            $str.="Error while trying to delete:\n".implode("\n", $files_cannot_be_deleted);
        $SystemLog->record($str);
    }

}
/* * ******************************************************** */

/**
 * This class controls locking for process. 
 */
class cmsProcessMutex {

    //_____________________________________________________//
    private static function setLockStatus($process_name, $lockstatus) {
        global $SystemDB;

        $process_name = alpha_numeric_with_dash_underscore($process_name); // sanitize, just in case
        $lockstatus = ($lockstatus != 0) ? 1 : 0; // sanitize
        $sql = "INSERT INTO gk_mutexlock_items (process_name, status) VALUES ('{$process_name}', {$lockstatus}) ON DUPLICATE KEY UPDATE status={$lockstatus}";
        /* $locking = $lockstatus ? 'LOCK' : 'UNLOCK';
          echo $locking.' '.$process_name.br(); */
        $SystemDB->query($sql);
    }

    public static function refreshStalledEntries($maxlocktime_before_unlock = HARDCODE_MAX_MUTEX_LOCKTIME) {
        global $SystemDB;

        // minimum 1 minute
        $maxlocktime_before_unlock = $maxlocktime_before_unlock < 60 ? 60 : $maxlocktime_before_unlock;
        $sql = "update gk_mutexlock_items SET status = 0 WHERE TIME_TO_SEC(TIMEDIFF(NOW(), timestamp_last_touched)) > {$maxlocktime_before_unlock}";
        $SystemDB->query($sql);
    }

    //_____________________________________________________//
    public static function getLockStatus($process_name, $maxlocktime_before_unlock = HARDCODE_MAX_MUTEX_LOCKTIME) {
        global $SystemDB;

        self::refreshStalledEntries($maxlocktime_before_unlock);
        $sql = "SELECT status FROM gk_mutexlock_items WHERE process_name = " . sanitize_string($process_name);
        $item = $SystemDB->getQueryResultSingleRow($sql, false);
        if ($item) {
            /* if (($item['timestamp_last_touched'] > 0) && (time() - $item['timestamp_last_touched'] > $maxlocktime_before_unlock)) {
              echo "Unlocking ".$process_name.' exceeds time: '.time().' - '.$item['timestamp_last_touched'].br();
              ProcessMutex::Unlock($process_name);
              return false;
              }
              else
              return $item; */
            return $item['status'];
        } else
            return false;
    }

    //_____________________________________________________//
    public static function Lock($process_name) {
        self::setLockStatus($process_name, 1);
    }

    //_____________________________________________________//
    public static function Unlock($process_name) {
        self::setLockStatus($process_name, 0);
    }

}

/* * ******************************************************** */
/**
 * Class for directory iterator
 */
class cmsDirectoryFilter extends \FilterIterator {

    private $_ext;
    private $_options;
    private $_it;
    private $_forbidden_listing;

    const FILTER_NONE = 0;
    const FILTER_FILE_ONLY = 1;
    const FILTER_DIR_ONLY = 2;

    //_______________________________________________________________________________________________________________//

    public function __construct(\Iterator $it, $options, $forbidden_listing = []) {
        parent::__construct($it);
        $this->_it = $it;
        $this->_options = $options;
        $this->_forbidden_listing = (is_array($forbidden_listing)) ? $forbidden_listing : [];
    }

    //_______________________________________________________________________________________________________________//
    public function accept() {
        $filename = $this->_it->getFileName();
        if (($filename[0] == '.') || (in_array($filename, $this->_forbidden_listing)))
            return false;
        if ($this->_options == cmsDirectoryFilter::FILTER_NONE)
            return true;
        if ($this->_options == cmsDirectoryFilter::FILTER_DIR_ONLY && $this->_it->isDir())
            return true;
        if ($this->_options == cmsDirectoryFilter::FILTER_FILE_ONLY && $this->_it->isFile())
            return true;
        return false;
    }

    //_______________________________________________________________________________________________________________//
    public static function getRecursiveDirectoryIterator($path, $filter, $forbidden_listing = []) {
        $realpath = str_replace('\\', '/', realpath($path)); // for security, prevent path traversal in case developer forgets
        if (strpos($realpath, SCHLIX_ROOT_PATH) === false) {
            'Path input is insecure';
            return false;
        }
        return new cmsDirectoryFilter(new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($realpath, \RecursiveDirectoryIterator::UNIX_PATHS), \RecursiveIteratorIterator::SELF_FIRST), $filter, $forbidden_listing);
    }
    //_______________________________________________________________________________________________________________//

    public static function getDirectoryIterator($path, $filter, $forbidden_listing = []) {
        global $SystemLog;
        $realpath = str_replace('\\', '/', realpath($path)); // for security, prevent path traversal in case developer forgets
        if (strpos($realpath, SCHLIX_ROOT_PATH) === FALSE) {

            $SystemLog->record("cmsDirectoryFilter - requested path input does not exist or is insecure: {$path}");
            //die('Insecure path or path does not exist');
            return false;
        }
        return new cmsDirectoryFilter(new \DirectoryIterator($realpath), $filter, $forbidden_listing);
    }

}


/**
 * Class for displaying HTML Page Header
 */
class cmsHTMLPageHeader {

    private $header = [];
    private $header_scripts = [];
    private $called = [];
     //_______________________________________________________________________________________________________________//
    /**
     * Adds a text to the header
     * @param string $str
     */
    public function add($str) {
        $txt = trim($str);
        $key = md5($str);
        if (!empty($txt) )
        {
            if (str_starts_with($txt, "<script"))
                $this->header_scripts[$key] = $txt;
            else
                $this->header[$key]=$txt;
        }
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Adds a locally hosted CSS relative to the httpbase
     * @param string $src
     * @param string $media
     */
    public function CSS($src, $media = '') {
        $attr = [];
        if ($media)
            $attr['media'] = $media;
        $csstext = \__HTML::CSS($src,$attr);
        
        $this->add($csstext);
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Adds CSS Text to the page header
     * @param string $str
     */
    public function CSS_TEXT($str) {
        $this->add(\__HTML::CSS($str));
    }
    //_______________________________________________________________________________________________________________//
    /**
     * Adds CSS Text to the page header
     * @param string $src
     */
    public function CSS_EXTERNAL($src, $media) {
        $attr = [];
        if ($media)
            $attr['media'] = $media;
        $csstext = \__HTML::CSS_EXTERNAL($src,$attr);
        
        $this->add(\__HTML::CSS_EXTERNAL($csstext));
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Adds a locally hosted Javascript file relative to the URL of this site
     * @param string $str
     * @param bool $no_http_base
     */
    public function JAVASCRIPT($str, $no_http_base = false) {
        $base = $no_http_base ? '' : SCHLIX_SITE_HTTPBASE;
        $jstext = "<script type=\"text/javascript\" src=\"" . $base . "{$str}\"></script>"; //JAVASCRIPT($str);
        $this->add($jstext);
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Adds a Javascript text to the page header
     * @param string $str
     */
    public function JAVASCRIPT_TEXT($str) {
        $this->add(\__HTML::JAVASCRIPT_TEXT($str));
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Adds an externally hosted Javascript file to the header
     * @param type $str
     */
    public function JAVASCRIPT_EXTERNAL($str) {
        $this->add(\__HTML::JAVASCRIPT_EXTERNAL($str));
    }

    /**
     * get JQuery 1.12
     * @param bool $return_as_string
     * @return bool|string
     */    
    public function JAVASCRIPT_JQUERY1($return_as_string = false) {
        $jquery_url = SCHLIX_SYSTEM_URL_PATH . '/js/jquery/jquery-1.12.4.min.js';
        $js_str = "<script type=\"text/javascript\" src=\"" . $jquery_url . "\"></script>";
        if ($return_as_string)
            return $js_str;
        else {
            $this->JAVASCRIPT($jquery_url, true);
            return false;
        }
        
    }
    
    
    /**
     * get JQuery 2
     * @param bool $return_as_string
     * @return bool|string
     */
    public function JAVASCRIPT_JQUERY2($return_as_string = false) {
        $jquery_url = SCHLIX_SYSTEM_URL_PATH . '/js/jquery/jquery-2.2.4.min.js';
        $js_str = "<script type=\"text/javascript\" src=\"" . $jquery_url . "\"></script>";
        if ($return_as_string)
            return $js_str;
        else {
            $this->JAVASCRIPT($jquery_url, true);
            return false;
        }
        
    }
    
    /**
     * Set Canonical URL of this page. If you set $absolute to true, it will not
     * add the HTTPS or HTTP URL
     * @param string $url
     * @param bool $absolute
     */
    public function setCanonicalURL($url, $absolute = false)
    {
        $url_host = SCHLIX_SITE_SSL_ENABLED ? SCHLIX_SITE_HTTPS_URL : SCHLIX_SITE_HTTP_URL;
        $full_canonical_url =  $absolute ? $url_host. remove_multiple_slashes('/' .$url) : $url;

        $this->add('<link rel="canonical" href="'.$full_canonical_url.'" />');
    }
    /**
     * get JQuery 3
     * @param bool $return_as_string
     * @return bool|string
     */    
    public function JAVASCRIPT_JQUERY3($return_as_string = false) {
        $jquery_url = SCHLIX_SYSTEM_URL_PATH . '/js/jquery/jquery-3.2.1.min.js';
        $js_str = "<script type=\"text/javascript\" src=\"" . $jquery_url . "\"></script>";
        if ($return_as_string)
            return $js_str;
        else {
            $this->JAVASCRIPT($jquery_url, true);
            return false;
        }
        
    }
    
    //_______________________________________________________________________________________________________________//
    /**
     * Adds SCHLIX UI Javascript to the page header. If return_as_string is specified 
     * then it will won't be added to the the header
     * @param bool $return_as_string
     * @param int $js_type
     * @return bool
     */
    public function JAVASCRIPT_SCHLIX_UI($return_as_string = false, $js_type = 0) {
        if ($this->called['schlix_ui'])
            return NULL;        
        if ($js_type == 0)
            $js_type = SCHLIX_JS_MODE;
        
        $schlix_ui_url = NULL;
        $file_found = false;

        switch ($js_type) {
            case JS_SCHLIX_GZIP:
            case JS_SCHLIX_MIN_ONLY:
            case JS_SCHLIX_DEBUG: //deprecated
                $schlix_ui_file = 'schlix-ui.minified.js';
                break;
        }

        $file_found = file_exists(SCHLIX_SYSTEM_PATH . '/js/schlix-ui/' . $schlix_ui_file);
        if (!$schlix_ui_url)
            $schlix_ui_url = SCHLIX_SYSTEM_URL_PATH . '/js/schlix-ui/' . $schlix_ui_file;

        $js_str = ($file_found) ? "<script type=\"text/javascript\" src=\"" . $schlix_ui_url . "\"></script>" : self::JAVASCRIPT_TEXT('alert("Javascript file ' . $schlix_ui_file . ' not found");');
        $this->called['schlix_ui'] = true;
        
        if ($return_as_string)
            return $js_str;
        else {
            $this->JAVASCRIPT($schlix_ui_url, true);
            return false;
        }
    }

    /**
     * Adds SCHLIX init script to the page header. If return_as_string is specified 
     * then it will won't be added to the the header
     * @param bool $return_as_string
     * @return string|bool
     */
   public function JAVASCRIPT_SCHLIX_INIT($return_as_string = false) {
        if ($this->called['schlix_cms_init'])
            return NULL;
        
        $schlix_cms_init_url = SCHLIX_SYSTEM_URL_PATH.'/js/schlix-cms/initjs.php';
        $this->called['schlix_cms_init'] = true;
        
        $js_str.="<script type=\"text/javascript\" src=\"" . $schlix_cms_init_url . "\"></script>";
        if ($return_as_string)
            return $js_str . "\n";
        else {
            $this->Add($js_str);
            return false;
        }

        
   }    
    //_________________________________________________________________________//
    /**
     * Adds SCHLIX CMS Javascript to the page header. If return_as_string is specified 
     * then it will won't be added to the the header
     * @param bool $return_as_string
     * @param int $js_type
     * @return string|bool
     */
   public function JAVASCRIPT_SCHLIX_CMS($return_as_string = false, $js_type = 0) {
        if ($this->called['schlix_cms'])
            return NULL;

        if ($js_type == 0)
            $js_type = SCHLIX_JS_MODE;
        $schlix_cms_url = '';
        $file_found = false;
        $js_str = "\n";
        switch ($js_type) {

            case JS_SCHLIX_GZIP:

            case JS_SCHLIX_MIN_ONLY:
                $schlix_cms_file = 'schlix-cms.minified.js';
                $file_found = file_exists(SCHLIX_SYSTEM_PATH . '/js/schlix-cms/' . $schlix_cms_file);
                $schlix_cms_url = SCHLIX_SYSTEM_URL_PATH . '/js/schlix-cms/' . $schlix_cms_file;
                break;
            case JS_SCHLIX_DEBUG:
                $schlix_cms_file = 'schlix-cms.debug.js';
                $f_iterator = new \GlobIterator(SCHLIX_SYSTEM_PATH . '/js/schlix-cms/t*.js', \FilesystemIterator::KEY_AS_FILENAME);
                foreach ($f_iterator as $it) {
                    $js_array[] = $it->getFileName();
                }
                natsort($js_array);

                break;
        } 
        $schlix_cms_init_url = SCHLIX_SYSTEM_URL_PATH.'/js/schlix-cms/initjs.php';
        $this->called['schlix_cms_init'] = true;
        
        $js_str.="<script type=\"text/javascript\" src=\"" . $schlix_cms_init_url . "\"></script>";
       if ($js_type != JS_SCHLIX_DEBUG) {
            $js_str.= $file_found ? 
                    "<script type=\"text/javascript\" src=\"" . $schlix_cms_url . "\"></script>" : 
                self::JAVASCRIPT_TEXT('alert("Javascript file ' . $schlix_cms_file . ' not found");');
        } else {
            if (___c($js_array) > 1) {
                foreach ($js_array as $js) {
                    $js_str.="<script type=\"text/javascript\" src=\"" . SCHLIX_SYSTEM_URL_PATH . '/js/schlix-cms/' . $js . "\"></script>\n";
                }
            } else {
                $js_str = self::JAVASCRIPT_TEXT('alert("SCHLIX CMS Debug files can not be found");');
            }
        }
        $this->called['schlix_cms'] = true;
        
        if ($return_as_string)
            return $js_str . "\n";
        else {
            $this->Add($js_str);
            return false;
        }
    }
    //_______________________________________________________________________________________________________________//
    /**
     * 
     * @return string
     */
    public function getAll() {

        if (is_array($_SESSION['schlix_internal_messages']) && ___c($_SESSION['schlix_internal_messages']) > 0) {
            $this->JAVASCRIPT_SCHLIX_UI();
            $this->JAVASCRIPT_SCHLIX_CMS();
            $this->add(static::JAVASCRIPT_TEXT('_schlix_internal_messages = ' . json_encode($_SESSION['schlix_internal_messages']) . ';'));
            // This is done so that when there's a redirect, message box will still be displayed;
            if ($_SESSION['schlix_internal_redirect'] == 1) {
                unset($_SESSION['schlix_internal_redirect']);
            } else {
                unset($_SESSION['schlix_internal_messages']);
            }
        }
        if ($this->header || $this->header_scripts)
        {
            $result = "<!-- schlix header -->\n";
            if ($this->header)
                $result.= implode("\n",$this->header);            
            if ($this->header_scripts)
                $result.= implode("\n",$this->header_scripts);
            $result.= "\n<!-- end of schlix header -->\n";
         
        } else $result = '';
        return $result;        
    }
    
    public function getHeaderTagsNonScriptsOnly()
    {
        $result = '';
        if ($this->header )
        {
            $result = "<!-- schlix header ns -->\n";
            if ($this->header)
                $result.= implode("\n",$this->header);            
            $result.= "\n<!-- end of schlix header ns -->\n";
         
        } else $result = '';
        return $result;        
    }
    
    public function getHeaderTagsScriptsOnly()
    {
        if (is_array($_SESSION['schlix_internal_messages']) && ___c($_SESSION['schlix_internal_messages']) > 0) {
            $this->JAVASCRIPT_SCHLIX_UI();
            $this->JAVASCRIPT_SCHLIX_CMS();
            $this->add(static::JAVASCRIPT_TEXT('_schlix_internal_messages = ' . json_encode($_SESSION['schlix_internal_messages']) . ';'));
            // This is done so that when there's a redirect, message box will still be displayed;
            if ($_SESSION['schlix_internal_redirect'] == 1) {
                unset($_SESSION['schlix_internal_redirect']);
            } else {
                unset($_SESSION['schlix_internal_messages']);
            }
        }
        if ($this->header_scripts)
        {
            $result = "<!-- schlix header s -->\n";
            if ($this->header_scripts)
                $result.= implode("\n",$this->header_scripts);
            $result.= "\n<!-- end of schlix header s -->\n";
         
        } else $result = '';
        return $result;        
    }
    

//_______________________________________________________________________________________________________________//
}

/**
 * Class to display HTML output in the theme
 */
class cmsPageOutput {

    /**
     * Outputs HTML string
     */
    public static function ApplicationBody() {
        global $__html_output;
        echo $__html_output;
    }

    /**
     * Returns a string of HTML Header
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @return string
     */
    public static function HTMLHeader()
    {
        global $HTMLHeader;

        $basehref = SCHLIX_SITE_URL . remove_multiple_slashes(SCHLIX_SITE_HTTPBASE . '/');
        $s = '<meta name="generator" content="SCHLIX" />' . "\n";
        $s.= '<base href="' . $basehref . '" />' . "\n";
        $s.= $HTMLHeader->getAll();        
        return $s;
    }


    /**
     * Returns a string of HTML Header
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @return string
     */
    public static function HTMLHeaderNonScripts()
    {
        global $HTMLHeader;

        $basehref = SCHLIX_SITE_URL . remove_multiple_slashes(SCHLIX_SITE_HTTPBASE . '/');
        $s = '<meta name="generator" content="SCHLIX" />' . "\n";
        $s.= '<base href="' . $basehref . '" />' . "\n";
        $s.= $HTMLHeader->getHeaderTagsNonScriptsOnly();        
        return $s;
    }
    
    /**
     * Returns a string of HTML Header
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @return string
     */
    public static function HTMLFooterScripts()
    {
        global $HTMLHeader;
        return $HTMLHeader->getHeaderTagsScriptsOnly();
    }
    
                        
    
    //_________________________________________________________________________//
    /**
     * Returns the currently set page title 
     * @global \SCHLIX\cmsApplication_Basic $Application
     * @return string
     */
    public static function HTMLPageTitle() {
        global $Application;
        return $Application->getPageTitle();
    }

    //_________________________________________________________________________//
    /**
     * Returns the meta description
     * @global \SCHLIX\cmsApplication_Basic $Application
     * @return string
     */
    public static function HTMLMetaDescription() {
        global $Application;
        return $Application->getPageMetaDescription();
    }

    //_________________________________________________________________________//
    /**
     * Returns the meta keyword
     * @global \SCHLIX\cmsApplication_Basic $Application
     * @return string
     */    
    public static function HTMLMetaKeywords() {
        global $Application;
        return $Application->getPageMetaKeywords();
    }

    //_________________________________________________________________________//
    /**
     * Returns the breadcrumb (usually set by the app)
     * @global \SCHLIX\cmsApplication_Basic $Application
     * @return string
     */    
    public static function BreadCrumbs() {
        global $Application;

        return $Application->displayBreadCrumbs();
    }

    //_________________________________________________________________________//
    /**
     * Returns the output of all blocks in the specified category
     * @global \App\Block $Blocks
     * @param string $position
     * @return string
     */
    public static function BlockCategory($position) {
        global $Blocks, $CurrentUser;
        
        /*$s = '<div style="border:1px solid orange;display:block;">';
        $s.='<div style="background:orange;color:white">'.___h($position).'</div>';
        $s.=$Blocks->getBlocksByCategoryName($position);
        $s.= '</div>';
        return  $s;
         * 
         */
         $prefix = '';
         $suffix = '';
         if ($_COOKIE['schlix_frontendedit_control_showblock'] == 2)
         {
             if ($CurrentUser->authenticated())
             {
                $prefix = \__HTML::DIV_start(array('class'=>'schlix_frontendedit_control-blockcategory-container'));
                $prefix.= \__HTML::DIV_start(array('class'=>'schlix_frontendedit_control-blockcategory-title')).
                    \__HTML::I('',array('class'=>'fa fa-th-large')).' '.
                    $position.
                    \__HTML::DIV_end();
                $suffix = \__HTML::DIV_end();
             }
         }
        return $prefix.$Blocks->getBlocksByCategoryName($position).$suffix;
    }

    //_________________________________________________________________________//
    /**
     * Returns the number of blocks in a block category. Useful if you
     * want to check whether or not a certain tag to be dispalyed or not
     * @global \App\Block $Blocks
     * @param string $position
     * @return int
     */
    public static function BlockCountInCategory($position) {
        global $Blocks;

        return $Blocks->getBlockCountInCategory($position);
    }

    //_________________________________________________________________________//
    /**
     * Returns the output of a single block
     * @global \App\Block $Blocks
     * @param string $block_name
     * @return string
     */
    public static function displayBlock($block_name) {
        global $Blocks;

        return $Blocks->getSingleBlock($block_name);
    }

}

